#pragma once
#include "../../geometry/polygon.h"

namespace geomodcore {

	class Curve 
	{

	public:
		/**
		* Erzeugt eine quidistante Diskretisierung der Kurve.
		* Die Sampling-Gre darf nicht negativ sein. 
		* \param n Abtastschritte
		*/
		virtual Polygon* getDiscretization(const unsigned int n) const = 0;

		/**
		* Berechnet eine Nherung fr die Lnge der Kurve anhand einer Diskretisierung der Kurve
		* mit Parameterbereich L und der daraus resultierenden chordalen Lnge.
		* Die Methode kann berschrieben werden, sofern eine vollstndige Berechnung der Bogenlnge mglich ist.
		* \param n Anzahl der Schritte
		*/
		virtual double getLength(const unsigned int n) const = 0;

		/**
		* Bestimmung der abgeleiteten Kurve.
		*/
		Curve* getDerivation(void) const { };

		/**
		* Berechnet die Bogenlnge der Kurve aus der Ableitung.
		*/
		virtual double getArcLength(void) const = 0;
					
	};
}