// Kratos Multi-Physics
//
// Copyright (c) 2016 Pooyan Dadvand, Riccardo Rossi, CIMNE (International Center for Numerical Methods in Engineering)
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
// 	-	Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// 	-	Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer
// 		in the documentation and/or other materials provided with the distribution.
// 	-	All advertising materials mentioning features or use of this software must display the following acknowledgement:
// 			This product includes Kratos Multi-Physics technology.
// 	-	Neither the name of the CIMNE nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED ANDON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
// THE USE OF THISSOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// System includes

// External includes
#include <boost/python.hpp>


// Project includes
#include "includes/define.h"
#include "includes/ublas_interface.h"
#include "containers/data_value_container.h"
//#include "containers/hash_data_value_container.h"
#include "containers/variables_list_data_value_container.h"
#include "containers/fix_data_value_container.h"
#include "containers/vector_component_adaptor.h"
#include "containers/flags.h"
//#include "containers/all_variables_data_value_container.h"
// #include "includes/kratos_flags.h"
#include "includes/deprecated_variables.h"
#include "includes/constitutive_law.h"
#include "python/variable_indexing_python.h"
#include "python/vector_python_interface.h"
#include "python/vector_scalar_operator_python.h"
#include "python/vector_vector_operator_python.h"
#include "python/bounded_vector_python_interface.h"

#include "includes/convection_diffusion_settings.h"
#include "includes/radiation_settings.h"
#include "utilities/timer.h"



#ifdef KRATOS_REGISTER_IN_PYTHON_FLAG_IMPLEMENTATION
#undef KRATOS_REGISTER_IN_PYTHON_FLAG_IMPLEMENTATION
#endif
#define KRATOS_REGISTER_IN_PYTHON_FLAG_IMPLEMENTATION(flag) \
 scope().attr(#flag) = boost::ref(flag)      \

#ifdef KRATOS_REGISTER_IN_PYTHON_FLAG
#undef KRATOS_REGISTER_IN_PYTHON_FLAG
#endif
#define KRATOS_REGISTER_IN_PYTHON_FLAG(flag) \
    KRATOS_REGISTER_IN_PYTHON_FLAG_IMPLEMENTATION(flag);   \
    KRATOS_REGISTER_IN_PYTHON_FLAG_IMPLEMENTATION(NOT_##flag)

#ifdef KRATOS_REGISTER_IN_PYTHON_VARIABLE
#undef KRATOS_REGISTER_IN_PYTHON_VARIABLE
#endif
#define KRATOS_REGISTER_IN_PYTHON_VARIABLE(variable) \
    scope().attr(#variable) = boost::ref(variable);


#ifdef KRATOS_REGISTER_IN_PYTHON_3D_VARIABLE_WITH_COMPONENTS
#undef KRATOS_REGISTER_IN_PYTHON_3D_VARIABLE_WITH_COMPONENTS
#endif
#define KRATOS_REGISTER_IN_PYTHON_3D_VARIABLE_WITH_COMPONENTS(name) \
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(name) \
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(name##_X) \
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(name##_Y) \
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(name##_Z)

namespace Kratos
{
//KRATOS_CREATE_FLAG(STRUCTURE,   63);

namespace Python
{
using namespace boost::python;

void  AddDeprecatedVariablesToPython()
{
          
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(  IS_INACTIVE )

    //for Level Set application:
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(  IS_DUPLICATED )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(  SPLIT_ELEMENT )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(  SPLIT_NODAL )

    //for PFEM fluids application:
    KRATOS_REGISTER_IN_PYTHON_VARIABLE( IS_JACK_LINK )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE( IMPOSED_PRESSURE )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE( IMPOSED_VELOCITY_X )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE( IMPOSED_VELOCITY_Y )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE( IMPOSED_VELOCITY_Z )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE( IMPOSED_ANGULAR_VELOCITY_X )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE( IMPOSED_ANGULAR_VELOCITY_Y )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE( IMPOSED_ANGULAR_VELOCITY_Z )
            
    //For the DEM Application:
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(IMPOSED_VELOCITY_X_VALUE)
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(IMPOSED_VELOCITY_Y_VALUE)
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(IMPOSED_VELOCITY_Z_VALUE)
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(IMPOSED_ANGULAR_VELOCITY_X_VALUE)
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(IMPOSED_ANGULAR_VELOCITY_Y_VALUE)
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(IMPOSED_ANGULAR_VELOCITY_Z_VALUE)

    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   IS_INLET )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE (   IS_INTERFACE )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE (   IS_VISITED )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE ( IS_EROSIONABLE )

    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   IS_STRUCTURE )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   IS_POROUS )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   IS_WATER )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   IS_FLUID )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   IS_BOUNDARY )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   IS_FREE_SURFACE )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   IS_AIR_EXIT )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   IS_LAGRANGIAN_INLET )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   IS_WATER_ELEMENT )


    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   IS_BURN )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   IS_DRIPPING )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   IS_PERMANENT )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   IS_WALL )

    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   Ypr ) //var name does not follow standard
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   Yox )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   Yfuel )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   Hfuel )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   Hpr )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   Hpr1 )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   Hox )

    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   RADIATIVE_INTENSITY )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   RADIATIVE_INTENSITY_1 )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   RADIATIVE_INTENSITY_2 )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   RADIATIVE_INTENSITY_3 )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   RADIATIVE_INTENSITY_4 )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   RADIATIVE_INTENSITY_5 )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   RADIATIVE_INTENSITY_6 )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   RADIATIVE_INTENSITY_7 )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   RADIATIVE_INTENSITY_8 )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   RADIATIVE_INTENSITY_9 )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   RADIATIVE_INTENSITY_10 )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   RADIATIVE_INTENSITY_11 )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   RADIATIVE_INTENSITY_12 )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   RADIATIVE_INTENSITY_13 )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   RADIATIVE_INTENSITY_14 )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   RADIATIVE_INTENSITY_15 )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   RADIATIVE_INTENSITY_16 )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   RADIATIVE_INTENSITY_17 )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   RADIATIVE_INTENSITY_18 )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   RADIATIVE_INTENSITY_19 )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   RADIATIVE_INTENSITY_20 )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   RADIATIVE_INTENSITY_21 )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   RADIATIVE_INTENSITY_22 )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   RADIATIVE_INTENSITY_23 )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   RADIATIVE_INTENSITY_24 )

    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   rhoD )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   xi )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   a )
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   b )


    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   IS_SLIP )

    //for Level Set application:
    KRATOS_REGISTER_IN_PYTHON_VARIABLE(   IS_DIVIDED )

    KRATOS_REGISTER_IN_PYTHON_3D_VARIABLE_WITH_COMPONENTS( xi_c )

}
}  // namespace Python.
} // Namespace Kratos

#undef KRATOS_REGISTER_IN_PYTHON_FLAG
#undef KRATOS_REGISTER_IN_PYTHON_VARIABLE
#undef KRATOS_REGISTER_IN_PYTHON_3D_VARIABLE_WITH_COMPONENTS
