// Kratos Multi-Physics
//
// Copyright (c) 2016 Pooyan Dadvand, Riccardo Rossi, CIMNE (International Center for Numerical Methods in Engineering)
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
// 	-	Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
// 	-	Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer
// 		in the documentation and/or other materials provided with the distribution.
// 	-	All advertising materials mentioning features or use of this software must display the following acknowledgement:
// 			This product includes Kratos Multi-Physics technology.
// 	-	Neither the name of the CIMNE nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDERS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED ANDON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
// THE USE OF THISSOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "integration/line_gauss_legendre_integration_points.h"
#include "integration/triangle_gauss_legendre_integration_points.h"
#include "integration/triangle_gauss_radau_integration_points.h"
#include "integration/quadrilateral_gauss_legendre_integration_points.h"
#include "integration/tetrahedron_gauss_legendre_integration_points.h"
#include "integration/hexahedron_gauss_legendre_integration_points.h"
#include "integration/prism_gauss_legendre_integration_points.h"

#include "integration/line_gauss_lobatto_integration_points.h"
#include "integration/quadrilateral_gauss_lobatto_integration_points.h"
#include "integration/hexahedron_gauss_lobatto_integration_points.h"
#include "integration/prism_gauss_lobatto_integration_points.h"

#define tet10_a 0.108103018168070
#define tet10_b 0.445948490915965
#define tet10_c 0.816847572980459

//LINE:

//Gauss-Legendre

namespace Kratos
{
LineGaussLegendreIntegrationPoints1::IntegrationPointsArrayType LineGaussLegendreIntegrationPoints1::msIntegrationPoints =
{
    {
        IntegrationPointType(0.00, 2.00)
    }
};

LineGaussLegendreIntegrationPoints2::IntegrationPointsArrayType LineGaussLegendreIntegrationPoints2::msIntegrationPoints =
{
    {
        IntegrationPointType(-std::sqrt(1.00 / 3.00), 1.00),
        IntegrationPointType( std::sqrt(1.00 / 3.00), 1.00)
    }
};

LineGaussLegendreIntegrationPoints3::IntegrationPointsArrayType LineGaussLegendreIntegrationPoints3::msIntegrationPoints =
{
    {
        IntegrationPointType(-std::sqrt(3.00 / 5.00), 5.00 / 9.00),
        IntegrationPointType( 0.00                  , 8.00 / 9.00),
        IntegrationPointType( std::sqrt(3.00 / 5.00), 5.00 / 9.00)
    }
};

LineGaussLegendreIntegrationPoints4::IntegrationPointsArrayType LineGaussLegendreIntegrationPoints4::msIntegrationPoints =
{
    {
        IntegrationPointType(-std::sqrt(3.00/7.00 + 2.00/7.00*std::sqrt(6.00/5.00)), (18.00 - std::sqrt(30.00))/36.00),
        IntegrationPointType(-std::sqrt(3.00/7.00 - 2.00/7.00*std::sqrt(6.00/5.00)), (18.00 + std::sqrt(30.00))/36.00),
        IntegrationPointType(std::sqrt(3.00/7.00 - 2.00/7.00*std::sqrt(6.00/5.00)), (18.00 + std::sqrt(30.00))/36.00),
        IntegrationPointType(std::sqrt(3.00/7.00 + 2.00/7.00*std::sqrt(6.00/5.00)), (18.00 - std::sqrt(30.00))/36.00)
    }
};

LineGaussLegendreIntegrationPoints5::IntegrationPointsArrayType LineGaussLegendreIntegrationPoints5::msIntegrationPoints =
{
    {
        IntegrationPointType(-std::sqrt(5.00 + 2.00*std::sqrt(10.00/7.00))/3.00, (322.00 - 13.00*std::sqrt(70.00))/900.00),
        IntegrationPointType(-std::sqrt(5.00 - 2.00*std::sqrt(10.00/7.00))/3.00, (322.00 + 13.00*std::sqrt(70.00))/900.00),
        IntegrationPointType(0.000000000000000, 128.00/225.00),
        IntegrationPointType(std::sqrt(5.00 - 2.00*std::sqrt(10.00/7.00))/3.00, (322.00 + 13.00*std::sqrt(70.00))/900.00),
        IntegrationPointType(std::sqrt(5.00 + 2.00*std::sqrt(10.00/7.00))/3.00, (322.00 - 13.00*std::sqrt(70.00))/900.00)
    }
};

LineGaussLegendreIntegrationPoints6::IntegrationPointsArrayType LineGaussLegendreIntegrationPoints6::msIntegrationPoints =
{
    {
        IntegrationPointType(-0.9324695142031520278123, 0.1713244923791703450403),
        IntegrationPointType(-0.661209386466264513661, 0.3607615730481386075698),
        IntegrationPointType(-0.2386191860831969086305, 0.4679139345726910473899),
        IntegrationPointType(0.2386191860831969086305, 0.4679139345726910473899),
        IntegrationPointType(0.661209386466264513661, 0.3607615730481386075698),
        IntegrationPointType(0.9324695142031520278123, 0.1713244923791703450403)
    }
};

LineGaussLegendreIntegrationPoints7::IntegrationPointsArrayType LineGaussLegendreIntegrationPoints7::msIntegrationPoints =
{
    {
        IntegrationPointType(-0.9491079123427585245262, 0.1294849661688696932706),
        IntegrationPointType(-0.7415311855993944398639, 0.2797053914892766679015),
        IntegrationPointType(-0.4058451513773971669066, 0.38183005050511894495),
        IntegrationPointType(0, 0.417959183673469387755),
        IntegrationPointType(0.4058451513773971669066, 0.38183005050511894495),
        IntegrationPointType(0.7415311855993944398639, 0.2797053914892766679015),
        IntegrationPointType(0.9491079123427585245262, 0.1294849661688696932706)
    }
};

LineGaussLegendreIntegrationPoints8::IntegrationPointsArrayType LineGaussLegendreIntegrationPoints8::msIntegrationPoints =
{
    {
        IntegrationPointType(-0.9602898564975362316836, 0.1012285362903762591525),
        IntegrationPointType(-0.7966664774136267395916, 0.2223810344533744705444),
        IntegrationPointType(-0.5255324099163289858177, 0.313706645877887287338),
        IntegrationPointType(-0.1834346424956498049395, 0.3626837833783619829652),
        IntegrationPointType( 0.1834346424956498049395, 0.3626837833783619829652),
        IntegrationPointType( 0.5255324099163289858177, 0.313706645877887287338),
        IntegrationPointType( 0.7966664774136267395916, 0.2223810344533744705444),
        IntegrationPointType( 0.9602898564975362316836, 0.1012285362903762591525)
    }
};

LineGaussLegendreIntegrationPoints9::IntegrationPointsArrayType LineGaussLegendreIntegrationPoints9::msIntegrationPoints =
{
    {
        IntegrationPointType(-0.9681602395076260898356, 0.0812743883615744119719),
        IntegrationPointType(-0.8360311073266357942994, 0.1806481606948574040585),
        IntegrationPointType(-0.6133714327005903973087, 0.2606106964029354623187),
        IntegrationPointType(-0.3242534234038089290385, 0.312347077040002840069),
        IntegrationPointType(0, 0.330239355001259763165),
        IntegrationPointType( 0.3242534234038089290385, 0.312347077040002840069),
        IntegrationPointType( 0.6133714327005903973087, 0.2606106964029354623187),
        IntegrationPointType( 0.8360311073266357942994, 0.1806481606948574040585),
        IntegrationPointType( 0.9681602395076260898356, 0.0812743883615744119719)
    }
};

LineGaussLegendreIntegrationPoints10::IntegrationPointsArrayType LineGaussLegendreIntegrationPoints10::msIntegrationPoints =
{
    {
        IntegrationPointType(-0.973906528517171720078, 0.0666713443086881375936),
        IntegrationPointType(-0.8650633666889845107321, 0.149451349150580593146),
        IntegrationPointType(-0.6794095682990244062343, 0.219086362515982043996),
        IntegrationPointType(-0.4333953941292471907993, 0.2692667193099963550912),
        IntegrationPointType(-0.1488743389816312108848, 0.2955242247147528701739),
        IntegrationPointType( 0.1488743389816312108848, 0.2955242247147528701739),
        IntegrationPointType( 0.4333953941292471907993, 0.2692667193099963550912),
        IntegrationPointType( 0.6794095682990244062343, 0.219086362515982043996),
        IntegrationPointType( 0.8650633666889845107321, 0.149451349150580593146),
        IntegrationPointType( 0.973906528517171720078, 0.0666713443086881375936)
    }
};

//Gauss-Lobatto

LineGaussLobattoIntegrationPoints1::IntegrationPointsArrayType LineGaussLobattoIntegrationPoints1::msIntegrationPoints =
{
    {
        IntegrationPointType(0.00, 2.00)
    }
};

LineGaussLobattoIntegrationPoints2::IntegrationPointsArrayType LineGaussLobattoIntegrationPoints2::msIntegrationPoints =
{
    {
        IntegrationPointType(-1.00, 1.00),
        IntegrationPointType( 1.00, 1.00)
    }
};

LineGaussLobattoIntegrationPoints3::IntegrationPointsArrayType LineGaussLobattoIntegrationPoints3::msIntegrationPoints =
{
    {
        IntegrationPointType(-1.00, 1.00 / 3.00),
        IntegrationPointType( 0.00, 4.00 / 3.00),
        IntegrationPointType( 1.00, 1.00 / 3.00)
    }
};

LineGaussLobattoIntegrationPoints4::IntegrationPointsArrayType LineGaussLobattoIntegrationPoints4::msIntegrationPoints =
{
    {
        IntegrationPointType(-1.00, 1.00 / 6.00),
        IntegrationPointType(-std::sqrt(5.00) / 5.00, 5.00 / 6.00),
        IntegrationPointType( std::sqrt(5.00) / 5.00, 5.00 / 6.00),
        IntegrationPointType( 1.00, 1.00 / 6.00)
    }
};

LineGaussLobattoIntegrationPoints5::IntegrationPointsArrayType LineGaussLobattoIntegrationPoints5::msIntegrationPoints =
{
    {
        IntegrationPointType(-1.00, 0.10),
        IntegrationPointType(-std::sqrt(21.00) / 7.00, 49.00 / 90.00),
        IntegrationPointType( 0.00, 32.00 / 45.00),
        IntegrationPointType( std::sqrt(21.00) / 7.00, 49.00 / 90.00),
        IntegrationPointType( 1.00, 0.10)
    }
};

LineGaussLobattoIntegrationPoints6::IntegrationPointsArrayType LineGaussLobattoIntegrationPoints6::msIntegrationPoints =
{
    {
        IntegrationPointType(-1.00, 1.00 / 15.00),
        IntegrationPointType(-std::sqrt((7.00+2.00*std::sqrt(7)) / 21.00), (14.00-std::sqrt(7)) / 30.00),
        IntegrationPointType(-std::sqrt((7.00-2.00*std::sqrt(7)) / 21.00), (14.00+std::sqrt(7)) / 30.0),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7)) / 21.00), (14.00+std::sqrt(7)) / 30.00),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7)) / 21.00), (14.00-std::sqrt(7)) / 30.00),
        IntegrationPointType( 1.00, 1.00 / 15.00)
    }
};

//TRIANGLE:

//Gauss-Legendre

TriangleGaussLegendreIntegrationPoints1::IntegrationPointsArrayType TriangleGaussLegendreIntegrationPoints1::msIntegrationPoints =
{
    {
        IntegrationPointType( 1.00 / 3.00 , 1.00 / 3.00 , 1.00 / 2.00 )
    }
};

TriangleGaussLegendreIntegrationPoints2::IntegrationPointsArrayType TriangleGaussLegendreIntegrationPoints2::msIntegrationPoints =
{

    {
        IntegrationPointType( 1.00 / 6.00 , 1.00 / 6.00 , 1.00 / 6.00 ),
        IntegrationPointType( 2.00 / 3.00 , 1.00 / 6.00 , 1.00 / 6.00 ),
        IntegrationPointType( 1.00 / 6.00 , 2.00 / 3.00 , 1.00 / 6.00 )
    }
};

TriangleGaussLegendreIntegrationPoints3::IntegrationPointsArrayType TriangleGaussLegendreIntegrationPoints3::msIntegrationPoints =
{
    {
        IntegrationPointType( 1.00 / 5.00 , 1.00 / 5.00 , 25.00 / 96.00 ),
        IntegrationPointType( 3.00 / 5.00 , 1.00 / 5.00 , 25.00 / 96.00 ),
        IntegrationPointType( 1.00 / 5.00 , 3.00 / 5.00 , 25.00 / 96.00 ),
        IntegrationPointType( 1.00 / 3.00 , 1.00 / 3.00 , -27.00 / 96.00 )
    }
};

#define KRATOS_TRI_G4_wa 0.109951743655322
#define KRATOS_TRI_G4_wb 0.223381589678011
#define KRATOS_TRI_G4_Na1 0.816847572980459
#define KRATOS_TRI_G4_Nb1 0.108103018168070
#define KRATOS_TRI_G4_Na2 0.091576213509771
#define KRATOS_TRI_G4_Nb2 0.445948490915965
TriangleGaussLegendreIntegrationPoints4::IntegrationPointsArrayType TriangleGaussLegendreIntegrationPoints4::msIntegrationPoints =
{
    {
        IntegrationPointType( KRATOS_TRI_G4_Na2, KRATOS_TRI_G4_Na2, KRATOS_TRI_G4_wa ),
        IntegrationPointType( KRATOS_TRI_G4_Na1, KRATOS_TRI_G4_Na2, KRATOS_TRI_G4_wa ),
        IntegrationPointType( KRATOS_TRI_G4_Na2, KRATOS_TRI_G4_Na1, KRATOS_TRI_G4_wa ),
        IntegrationPointType( KRATOS_TRI_G4_Nb2, KRATOS_TRI_G4_Nb2, KRATOS_TRI_G4_wb ),
        IntegrationPointType( KRATOS_TRI_G4_Nb1, KRATOS_TRI_G4_Nb2, KRATOS_TRI_G4_wb ),
        IntegrationPointType( KRATOS_TRI_G4_Nb2, KRATOS_TRI_G4_Nb1, KRATOS_TRI_G4_wb )
    }
};
#undef KRATOS_TRI_G4_wa
#undef KRATOS_TRI_G4_wb
#undef KRATOS_TRI_G4_Na1
#undef KRATOS_TRI_G4_Na2
#undef KRATOS_TRI_G4_Nb1
#undef KRATOS_TRI_G4_Nb2


TriangleGaussRadauIntegrationPoints1::IntegrationPointsArrayType TriangleGaussRadauIntegrationPoints1::msIntegrationPoints =
{
    {
        IntegrationPointType( 0.5 , 0.5 , 1.00 / 3.00 ),
        IntegrationPointType( 0.5 , 0.0 , 1.00 / 3.00 ),
        IntegrationPointType( 0.0 , 0.5 , 1.00 / 3.00 )
    }
};

TriangleGaussRadauIntegrationPoints2::IntegrationPointsArrayType TriangleGaussRadauIntegrationPoints2::msIntegrationPoints =
{
    {
        IntegrationPointType( 1.00 / 3.00 , 1.00 / 3.00 , -0.562500000000000 ),
        IntegrationPointType( 0.6 , 0.2 , 0.520833333333333 ),
        IntegrationPointType( 0.2 , 0.6 , 0.520833333333333 ),
        IntegrationPointType( 0.2 , 0.2 , 0.520833333333333 )
    }
};

TriangleGaussRadauIntegrationPoints3::IntegrationPointsArrayType TriangleGaussRadauIntegrationPoints3::msIntegrationPoints =
{
    {
        IntegrationPointType( 0.816847572980459 , 0.091576213509771 , 0.109951743655322 ),
        IntegrationPointType( 0.091576213509771 , 0.816847572980459 , 0.109951743655322 ),
        IntegrationPointType( 0.091576213509771 , 0.091576213509771 , 0.109951743655322 ),
        IntegrationPointType( 0.108103018168070 , 0.445948490915965 , 0.223381589678011 ),
        IntegrationPointType( 0.445948490915965 , 0.108103018168070 , 0.223381589678011 ),
        IntegrationPointType( 0.445948490915965 , 0.445948490915965 , 0.223381589678011 )
    }
};

TriangleGaussRadauIntegrationPoints4::IntegrationPointsArrayType TriangleGaussRadauIntegrationPoints4::msIntegrationPoints =
{
    {
        IntegrationPointType( 1.00 / 3.00 , 1.00 / 3.00 , 0.225000000000000 ),
        IntegrationPointType( 0.797426985353087 , 0.101286507323456 , 0.125939180544827 ),
        IntegrationPointType( 0.101286507323456 , 0.797426985353087 , 0.125939180544827 ),
        IntegrationPointType( 0.101286507323456 , 0.101286507323456 , 0.125939180544827 ),
        IntegrationPointType( 0.470142064105115 , 0.059715871789770 , 0.132394152788506 ),
        IntegrationPointType( 0.059715871789770 , 0.470142064105115 , 0.132394152788506 ),
        IntegrationPointType( 0.470142064105115 , 0.470142064105115 , 0.132394152788506 )
    }
};

TriangleGaussRadauIntegrationPoints5::IntegrationPointsArrayType TriangleGaussRadauIntegrationPoints5::msIntegrationPoints =
{
    {
        IntegrationPointType( 0.873821971016996 , 0.063089014491502 , 0.050844906370207 ),
        IntegrationPointType( 0.063089014491502 , 0.873821971016996 , 0.050844906370207 ),
        IntegrationPointType( 0.063089014491502 , 0.063089014491502 , 0.050844906370207 ),
        IntegrationPointType( 0.501426509658179 , 0.249286745170910 , 0.116786275726379 ),
        IntegrationPointType( 0.249286745170910 , 0.501426509658179 , 0.116786275726379 ),
        IntegrationPointType( 0.249286745170910 , 0.249286745170910 , 0.116786275726379 ),
        IntegrationPointType( 0.636502499121399 , 0.310352451033785 , 0.082851075618374 ),
        IntegrationPointType( 0.636502499121399 , 0.053145049844816 , 0.082851075618374 ),
        IntegrationPointType( 0.310352451033785 , 0.636502499121399 , 0.082851075618374 ),
        IntegrationPointType( 0.310352451033785 , 0.053145049844816 , 0.082851075618374 ),
        IntegrationPointType( 0.053145049844816 , 0.636502499121399 , 0.082851075618374 ),
        IntegrationPointType( 0.053145049844816 , 0.310352451033785 , 0.082851075618374 )
    }
};

TriangleGaussRadauIntegrationPoints6::IntegrationPointsArrayType TriangleGaussRadauIntegrationPoints6::msIntegrationPoints =
{
    {
        IntegrationPointType( 1.00 / 3.00 , 1.00 / 3.00 , -0.149570044467670 ),
        IntegrationPointType( 0.479308067841923 , 0.260345966079038 , 0.175615257433204 ),
        IntegrationPointType( 0.260345966079038 , 0.479308067841923 , 0.175615257433204 ),
        IntegrationPointType( 0.260345966079038 , 0.260345966079038 , 0.175615257433204 ),
        IntegrationPointType( 0.869739794195568 , 0.065130102902216 , 0.053347235608839 ),
        IntegrationPointType( 0.065130102902216 , 0.869739794195568 , 0.053347235608839 ),
        IntegrationPointType( 0.065130102902216 , 0.065130102902216 , 0.053347235608839 ),
        IntegrationPointType( 0.638444188569809 , 0.312865496004875 , 0.077113760890257 ),
        IntegrationPointType( 0.638444188569809 , 0.048690315425316 , 0.077113760890257 ),
        IntegrationPointType( 0.312865496004875 , 0.638444188569809 , 0.077113760890257 ),
        IntegrationPointType( 0.312865496004875 , 0.048690315425316 , 0.077113760890257 ),
        IntegrationPointType( 0.048690315425316 , 0.638444188569809 , 0.077113760890257 ),
        IntegrationPointType( 0.048690315425316 , 0.312865496004875 , 0.077113760890257 )
    }
};


//QUADRILATERAL:

//Gauss-Legendre

QuadrilateralGaussLegendreIntegrationPoints1::IntegrationPointsArrayType
QuadrilateralGaussLegendreIntegrationPoints1::msIntegrationPoints =
{
    {
        IntegrationPointType( 0.00 , 0.00 , 4.00 )
    }
};

QuadrilateralGaussLegendreIntegrationPoints2::IntegrationPointsArrayType
QuadrilateralGaussLegendreIntegrationPoints2::msIntegrationPoints =
{
    {
        IntegrationPointType( -1.00/std::sqrt(3.0) , -1.00/std::sqrt(3.0), 1.00 ),
        IntegrationPointType(  1.00/std::sqrt(3.0) , -1.00/std::sqrt(3.0), 1.00 ),
        IntegrationPointType(  1.00/std::sqrt(3.0) ,  1.00/std::sqrt(3.0), 1.00 ),
        IntegrationPointType( -1.00/std::sqrt(3.0) ,  1.00/std::sqrt(3.0), 1.00 )
    }
};

QuadrilateralGaussLegendreIntegrationPoints3::IntegrationPointsArrayType
QuadrilateralGaussLegendreIntegrationPoints3::msIntegrationPoints =
{
    {
        IntegrationPointType( -std::sqrt(3.00/5.00) , -std::sqrt(3.00/5.00), 25.00/81.00 ),
        IntegrationPointType(             0.00 , -std::sqrt(3.00/5.00), 40.00/81.00 ),
        IntegrationPointType(  std::sqrt(3.00/5.00) , -std::sqrt(3.00/5.00), 25.00/81.00 ),

        IntegrationPointType( -std::sqrt(3.00/5.00) ,             0.00, 40.00/81.00 ),
        IntegrationPointType(             0.00 ,             0.00, 64.00/81.00 ),
        IntegrationPointType(  std::sqrt(3.00/5.00) ,             0.00, 40.00/81.00 ),

        IntegrationPointType( -std::sqrt(3.00/5.00) ,  std::sqrt(3.00/5.00), 25.00/81.00 ),
        IntegrationPointType(             0.00 ,  std::sqrt(3.00/5.00), 40.00/81.00 ),
        IntegrationPointType(  std::sqrt(3.00/5.00) ,  std::sqrt(3.00/5.00), 25.00/81.00 )
    }
};


QuadrilateralGaussLegendreIntegrationPoints4::IntegrationPointsArrayType
QuadrilateralGaussLegendreIntegrationPoints4::msIntegrationPoints =
{
    {
        IntegrationPointType( -std::sqrt( (3.0 + 2.0 * std::sqrt(6.0/5.0)) / 7.0 ), -std::sqrt( (3.0 + 2.0 * std::sqrt(6.0/5.0)) / 7.0 ), (0.5 - std::sqrt(5.0/6.0)/6.0)*(0.5 - std::sqrt(5.0/6.0)/6.0)),
        IntegrationPointType( -std::sqrt( (3.0 + 2.0 * std::sqrt(6.0/5.0)) / 7.0 ), -std::sqrt( (3.0 - 2.0 * std::sqrt(6.0/5.0)) / 7.0 ), (0.5 - std::sqrt(5.0/6.0)/6.0)*(0.5 + std::sqrt(5.0/6.0)/6.0)),
        IntegrationPointType( -std::sqrt( (3.0 + 2.0 * std::sqrt(6.0/5.0)) / 7.0 ),  std::sqrt( (3.0 - 2.0 * std::sqrt(6.0/5.0)) / 7.0 ), (0.5 - std::sqrt(5.0/6.0)/6.0)*(0.5 + std::sqrt(5.0/6.0)/6.0)),
        IntegrationPointType( -std::sqrt( (3.0 + 2.0 * std::sqrt(6.0/5.0)) / 7.0 ),  std::sqrt( (3.0 + 2.0 * std::sqrt(6.0/5.0)) / 7.0 ), (0.5 - std::sqrt(5.0/6.0)/6.0)*(0.5 - std::sqrt(5.0/6.0)/6.0)),

        IntegrationPointType( -std::sqrt( (3.0 - 2.0 * std::sqrt(6.0/5.0)) / 7.0 ), -std::sqrt( (3.0 + 2.0 * std::sqrt(6.0/5.0)) / 7.0 ), (0.5 + std::sqrt(5.0/6.0)/6.0)*(0.5 - std::sqrt(5.0/6.0)/6.0)),
        IntegrationPointType( -std::sqrt( (3.0 - 2.0 * std::sqrt(6.0/5.0)) / 7.0 ), -std::sqrt( (3.0 - 2.0 * std::sqrt(6.0/5.0)) / 7.0 ), (0.5 + std::sqrt(5.0/6.0)/6.0)*(0.5 + std::sqrt(5.0/6.0)/6.0)),
        IntegrationPointType( -std::sqrt( (3.0 - 2.0 * std::sqrt(6.0/5.0)) / 7.0 ),  std::sqrt( (3.0 - 2.0 * std::sqrt(6.0/5.0)) / 7.0 ), (0.5 + std::sqrt(5.0/6.0)/6.0)*(0.5 + std::sqrt(5.0/6.0)/6.0)),
        IntegrationPointType( -std::sqrt( (3.0 - 2.0 * std::sqrt(6.0/5.0)) / 7.0 ),  std::sqrt( (3.0 + 2.0 * std::sqrt(6.0/5.0)) / 7.0 ), (0.5 + std::sqrt(5.0/6.0)/6.0)*(0.5 - std::sqrt(5.0/6.0)/6.0)),

        IntegrationPointType(  std::sqrt( (3.0 - 2.0 * std::sqrt(6.0/5.0)) / 7.0 ), -std::sqrt( (3.0 + 2.0 * std::sqrt(6.0/5.0)) / 7.0 ), (0.5 + std::sqrt(5.0/6.0)/6.0)*(0.5 - std::sqrt(5.0/6.0)/6.0)),
        IntegrationPointType(  std::sqrt( (3.0 - 2.0 * std::sqrt(6.0/5.0)) / 7.0 ), -std::sqrt( (3.0 - 2.0 * std::sqrt(6.0/5.0)) / 7.0 ), (0.5 + std::sqrt(5.0/6.0)/6.0)*(0.5 + std::sqrt(5.0/6.0)/6.0)),
        IntegrationPointType(  std::sqrt( (3.0 - 2.0 * std::sqrt(6.0/5.0)) / 7.0 ),  std::sqrt( (3.0 - 2.0 * std::sqrt(6.0/5.0)) / 7.0 ), (0.5 + std::sqrt(5.0/6.0)/6.0)*(0.5 + std::sqrt(5.0/6.0)/6.0)),
        IntegrationPointType(  std::sqrt( (3.0 - 2.0 * std::sqrt(6.0/5.0)) / 7.0 ),  std::sqrt( (3.0 + 2.0 * std::sqrt(6.0/5.0)) / 7.0 ), (0.5 + std::sqrt(5.0/6.0)/6.0)*(0.5 - std::sqrt(5.0/6.0)/6.0)),

        IntegrationPointType(  std::sqrt( (3.0 + 2.0 * std::sqrt(6.0/5.0)) / 7.0 ), -std::sqrt( (3.0 + 2.0 * std::sqrt(6.0/5.0)) / 7.0 ), (0.5 - std::sqrt(5.0/6.0)/6.0)*(0.5 - std::sqrt(5.0/6.0)/6.0)),
        IntegrationPointType(  std::sqrt( (3.0 + 2.0 * std::sqrt(6.0/5.0)) / 7.0 ), -std::sqrt( (3.0 - 2.0 * std::sqrt(6.0/5.0)) / 7.0 ), (0.5 - std::sqrt(5.0/6.0)/6.0)*(0.5 + std::sqrt(5.0/6.0)/6.0)),
        IntegrationPointType(  std::sqrt( (3.0 + 2.0 * std::sqrt(6.0/5.0)) / 7.0 ),  std::sqrt( (3.0 - 2.0 * std::sqrt(6.0/5.0)) / 7.0 ), (0.5 - std::sqrt(5.0/6.0)/6.0)*(0.5 + std::sqrt(5.0/6.0)/6.0)),
        IntegrationPointType(  std::sqrt( (3.0 + 2.0 * std::sqrt(6.0/5.0)) / 7.0 ),  std::sqrt( (3.0 + 2.0 * std::sqrt(6.0/5.0)) / 7.0 ), (0.5 - std::sqrt(5.0/6.0)/6.0)*(0.5 - std::sqrt(5.0/6.0)/6.0)),
    }
};


QuadrilateralGaussLegendreIntegrationPoints5::IntegrationPointsArrayType
QuadrilateralGaussLegendreIntegrationPoints5::msIntegrationPoints =
{
    {
        IntegrationPointType( -0.906179845938664, -0.906179845938664, 0.236926885056189 * 0.236926885056189),
        IntegrationPointType( -0.906179845938664, -0.538469310105683, 0.236926885056189 * 0.478628670499366),
        IntegrationPointType( -0.906179845938664, 0.000000000000000, 0.236926885056189 * 0.568888888888889),
        IntegrationPointType( -0.906179845938664, 0.538469310105683, 0.236926885056189 * 0.478628670499366),
        IntegrationPointType( -0.906179845938664, 0.906179845938664, 0.236926885056189 * 0.236926885056189),

        IntegrationPointType( -0.538469310105683, -0.906179845938664, 0.478628670499366 * 0.236926885056189),
        IntegrationPointType( -0.538469310105683, -0.538469310105683, 0.478628670499366 * 0.478628670499366),
        IntegrationPointType( -0.538469310105683, 0.000000000000000, 0.478628670499366 * 0.568888888888889),
        IntegrationPointType( -0.538469310105683, 0.538469310105683, 0.478628670499366 * 0.478628670499366),
        IntegrationPointType( -0.538469310105683, 0.906179845938664, 0.478628670499366 * 0.236926885056189),

        IntegrationPointType( 0.000000000000000, -0.906179845938664, 0.568888888888889 * 0.236926885056189),
        IntegrationPointType( 0.000000000000000, -0.538469310105683, 0.568888888888889 * 0.478628670499366),
        IntegrationPointType( 0.000000000000000, 0.000000000000000, 0.568888888888889 * 0.568888888888889),
        IntegrationPointType( 0.000000000000000, 0.538469310105683, 0.568888888888889 * 0.478628670499366),
        IntegrationPointType( 0.000000000000000, 0.906179845938664, 0.568888888888889 * 0.236926885056189),

        IntegrationPointType( 0.538469310105683, -0.906179845938664, 0.478628670499366 * 0.236926885056189),
        IntegrationPointType( 0.538469310105683, -0.538469310105683, 0.478628670499366 * 0.478628670499366),
        IntegrationPointType( 0.538469310105683, 0.000000000000000, 0.478628670499366 * 0.568888888888889),
        IntegrationPointType( 0.538469310105683, 0.538469310105683, 0.478628670499366 * 0.478628670499366),
        IntegrationPointType( 0.538469310105683, 0.906179845938664, 0.478628670499366 * 0.236926885056189),

        IntegrationPointType( 0.906179845938664, -0.906179845938664, 0.236926885056189 * 0.236926885056189),
        IntegrationPointType( 0.906179845938664, -0.538469310105683, 0.236926885056189 * 0.478628670499366),
        IntegrationPointType( 0.906179845938664, 0.000000000000000, 0.236926885056189 * 0.568888888888889),
        IntegrationPointType( 0.906179845938664, 0.538469310105683, 0.236926885056189 * 0.478628670499366),
        IntegrationPointType( 0.906179845938664, 0.906179845938664, 0.236926885056189 * 0.236926885056189),
    }
};


QuadrilateralGaussLegendreIntegrationPoints6::IntegrationPointsArrayType
QuadrilateralGaussLegendreIntegrationPoints6::msIntegrationPoints =
{
    {
        IntegrationPointType( -0.9324695142031521, -0.9324695142031521, 0.1713244923791704*0.1713244923791704 ),
        IntegrationPointType( -0.9324695142031521, -0.6612093864662645, 0.1713244923791704*0.3607615730481386 ),
        IntegrationPointType( -0.9324695142031521, -0.2386191860831969, 0.1713244923791704*0.4679139345726910 ),
        IntegrationPointType( -0.9324695142031521, 0.2386191860831969, 0.1713244923791704*0.4679139345726910 ),
        IntegrationPointType( -0.9324695142031521, 0.6612093864662645, 0.1713244923791704*0.3607615730481386 ),
        IntegrationPointType( -0.9324695142031521, 0.9324695142031521, 0.1713244923791704*0.1713244923791704 ),

        IntegrationPointType( -0.6612093864662645, -0.9324695142031521, 0.3607615730481386*0.1713244923791704 ),
        IntegrationPointType( -0.6612093864662645, -0.6612093864662645, 0.3607615730481386*0.3607615730481386 ),
        IntegrationPointType( -0.6612093864662645, -0.2386191860831969, 0.3607615730481386*0.4679139345726910 ),
        IntegrationPointType( -0.6612093864662645, 0.2386191860831969, 0.3607615730481386*0.4679139345726910 ),
        IntegrationPointType( -0.6612093864662645, 0.6612093864662645, 0.3607615730481386*0.3607615730481386 ),
        IntegrationPointType( -0.6612093864662645, 0.9324695142031521, 0.3607615730481386*0.1713244923791704 ),

        IntegrationPointType( -0.2386191860831969, -0.9324695142031521, 0.4679139345726910*0.1713244923791704 ),
        IntegrationPointType( -0.2386191860831969, -0.6612093864662645, 0.4679139345726910*0.3607615730481386 ),
        IntegrationPointType( -0.2386191860831969, -0.2386191860831969, 0.4679139345726910*0.4679139345726910 ),
        IntegrationPointType( -0.2386191860831969, 0.2386191860831969, 0.4679139345726910*0.4679139345726910 ),
        IntegrationPointType( -0.2386191860831969, 0.6612093864662645, 0.4679139345726910*0.3607615730481386 ),
        IntegrationPointType( -0.2386191860831969, 0.9324695142031521, 0.4679139345726910*0.1713244923791704 ),

        IntegrationPointType( 0.2386191860831969, -0.9324695142031521, 0.4679139345726910*0.1713244923791704 ),
        IntegrationPointType( 0.2386191860831969, -0.6612093864662645, 0.4679139345726910*0.3607615730481386 ),
        IntegrationPointType( 0.2386191860831969, -0.2386191860831969, 0.4679139345726910*0.4679139345726910 ),
        IntegrationPointType( 0.2386191860831969, 0.2386191860831969, 0.4679139345726910*0.4679139345726910 ),
        IntegrationPointType( 0.2386191860831969, 0.6612093864662645, 0.4679139345726910*0.3607615730481386 ),
        IntegrationPointType( 0.2386191860831969, 0.9324695142031521, 0.4679139345726910*0.1713244923791704 ),

        IntegrationPointType( 0.6612093864662645, -0.9324695142031521, 0.3607615730481386*0.1713244923791704 ),
        IntegrationPointType( 0.6612093864662645, -0.6612093864662645, 0.3607615730481386*0.3607615730481386 ),
        IntegrationPointType( 0.6612093864662645, -0.2386191860831969, 0.3607615730481386*0.4679139345726910 ),
        IntegrationPointType( 0.6612093864662645, 0.2386191860831969, 0.3607615730481386*0.4679139345726910 ),
        IntegrationPointType( 0.6612093864662645, 0.6612093864662645, 0.3607615730481386*0.3607615730481386 ),
        IntegrationPointType( 0.6612093864662645, 0.9324695142031521, 0.3607615730481386*0.1713244923791704 ),

        IntegrationPointType( 0.9324695142031521, -0.9324695142031521, 0.1713244923791704*0.1713244923791704 ),
        IntegrationPointType( 0.9324695142031521, -0.6612093864662645, 0.1713244923791704*0.3607615730481386 ),
        IntegrationPointType( 0.9324695142031521, -0.2386191860831969, 0.1713244923791704*0.4679139345726910 ),
        IntegrationPointType( 0.9324695142031521, 0.2386191860831969, 0.1713244923791704*0.4679139345726910 ),
        IntegrationPointType( 0.9324695142031521, 0.6612093864662645, 0.1713244923791704*0.3607615730481386 ),
        IntegrationPointType( 0.9324695142031521, 0.9324695142031521, 0.1713244923791704*0.1713244923791704 ),
    }
};


QuadrilateralGaussLegendreIntegrationPoints7::IntegrationPointsArrayType
QuadrilateralGaussLegendreIntegrationPoints7::msIntegrationPoints =
{
    {
        IntegrationPointType( -0.9491079123427585, -0.9491079123427585, 0.1294849661688697*0.1294849661688697 ),
        IntegrationPointType( -0.9491079123427585, -0.7415311855993945, 0.1294849661688697*0.2797053914892766 ),
        IntegrationPointType( -0.9491079123427585, -0.4058451513773972, 0.1294849661688697*0.3818300505051189 ),
        IntegrationPointType( -0.9491079123427585, 0.0000000000000000, 0.1294849661688697*0.4179591836734694 ),
        IntegrationPointType( -0.9491079123427585, 0.4058451513773972, 0.1294849661688697*0.3818300505051189 ),
        IntegrationPointType( -0.9491079123427585, 0.7415311855993945, 0.1294849661688697*0.2797053914892766 ),
        IntegrationPointType( -0.9491079123427585, 0.9491079123427585, 0.1294849661688697*0.1294849661688697 ),

        IntegrationPointType( -0.7415311855993945, -0.9491079123427585, 0.2797053914892766*0.1294849661688697 ),
        IntegrationPointType( -0.7415311855993945, -0.7415311855993945, 0.2797053914892766*0.2797053914892766 ),
        IntegrationPointType( -0.7415311855993945, -0.4058451513773972, 0.2797053914892766*0.3818300505051189 ),
        IntegrationPointType( -0.7415311855993945, 0.0000000000000000, 0.2797053914892766*0.4179591836734694 ),
        IntegrationPointType( -0.7415311855993945, 0.4058451513773972, 0.2797053914892766*0.3818300505051189 ),
        IntegrationPointType( -0.7415311855993945, 0.7415311855993945, 0.2797053914892766*0.2797053914892766 ),
        IntegrationPointType( -0.7415311855993945, 0.9491079123427585, 0.2797053914892766*0.1294849661688697 ),

        IntegrationPointType( -0.4058451513773972, -0.9491079123427585, 0.3818300505051189*0.1294849661688697 ),
        IntegrationPointType( -0.4058451513773972, -0.7415311855993945, 0.3818300505051189*0.2797053914892766 ),
        IntegrationPointType( -0.4058451513773972, -0.4058451513773972, 0.3818300505051189*0.3818300505051189 ),
        IntegrationPointType( -0.4058451513773972, 0.0000000000000000, 0.3818300505051189*0.4179591836734694 ),
        IntegrationPointType( -0.4058451513773972, 0.4058451513773972, 0.3818300505051189*0.3818300505051189 ),
        IntegrationPointType( -0.4058451513773972, 0.7415311855993945, 0.3818300505051189*0.2797053914892766 ),
        IntegrationPointType( -0.4058451513773972, 0.9491079123427585, 0.3818300505051189*0.1294849661688697 ),

        IntegrationPointType( 0.0000000000000000, -0.9491079123427585, 0.4179591836734694*0.1294849661688697 ),
        IntegrationPointType( 0.0000000000000000, -0.7415311855993945, 0.4179591836734694*0.2797053914892766 ),
        IntegrationPointType( 0.0000000000000000, -0.4058451513773972, 0.4179591836734694*0.3818300505051189 ),
        IntegrationPointType( 0.0000000000000000, 0.0000000000000000, 0.4179591836734694*0.4179591836734694 ),
        IntegrationPointType( 0.0000000000000000, 0.4058451513773972, 0.4179591836734694*0.3818300505051189 ),
        IntegrationPointType( 0.0000000000000000, 0.7415311855993945, 0.4179591836734694*0.2797053914892766 ),
        IntegrationPointType( 0.0000000000000000, 0.9491079123427585, 0.4179591836734694*0.1294849661688697 ),

        IntegrationPointType( 0.4058451513773972, -0.9491079123427585, 0.3818300505051189*0.1294849661688697 ),
        IntegrationPointType( 0.4058451513773972, -0.7415311855993945, 0.3818300505051189*0.2797053914892766 ),
        IntegrationPointType( 0.4058451513773972, -0.4058451513773972, 0.3818300505051189*0.3818300505051189 ),
        IntegrationPointType( 0.4058451513773972, 0.0000000000000000, 0.3818300505051189*0.4179591836734694 ),
        IntegrationPointType( 0.4058451513773972, 0.4058451513773972, 0.3818300505051189*0.3818300505051189 ),
        IntegrationPointType( 0.4058451513773972, 0.7415311855993945, 0.3818300505051189*0.2797053914892766 ),
        IntegrationPointType( 0.4058451513773972, 0.9491079123427585, 0.3818300505051189*0.1294849661688697 ),

        IntegrationPointType( 0.7415311855993945, -0.9491079123427585, 0.2797053914892766*0.1294849661688697 ),
        IntegrationPointType( 0.7415311855993945, -0.7415311855993945, 0.2797053914892766*0.2797053914892766 ),
        IntegrationPointType( 0.7415311855993945, -0.4058451513773972, 0.2797053914892766*0.3818300505051189 ),
        IntegrationPointType( 0.7415311855993945, 0.0000000000000000, 0.2797053914892766*0.4179591836734694 ),
        IntegrationPointType( 0.7415311855993945, 0.4058451513773972, 0.2797053914892766*0.3818300505051189 ),
        IntegrationPointType( 0.7415311855993945, 0.7415311855993945, 0.2797053914892766*0.2797053914892766 ),
        IntegrationPointType( 0.7415311855993945, 0.9491079123427585, 0.2797053914892766*0.1294849661688697 ),

        IntegrationPointType( 0.9491079123427585, -0.9491079123427585, 0.1294849661688697*0.1294849661688697 ),
        IntegrationPointType( 0.9491079123427585, -0.7415311855993945, 0.1294849661688697*0.2797053914892766 ),
        IntegrationPointType( 0.9491079123427585, -0.4058451513773972, 0.1294849661688697*0.3818300505051189 ),
        IntegrationPointType( 0.9491079123427585, 0.0000000000000000, 0.1294849661688697*0.4179591836734694 ),
        IntegrationPointType( 0.9491079123427585, 0.4058451513773972, 0.1294849661688697*0.3818300505051189 ),
        IntegrationPointType( 0.9491079123427585, 0.7415311855993945, 0.1294849661688697*0.2797053914892766 ),
        IntegrationPointType( 0.9491079123427585, 0.9491079123427585, 0.1294849661688697*0.1294849661688697 ),
    }
};


QuadrilateralGaussLegendreIntegrationPoints8::IntegrationPointsArrayType
QuadrilateralGaussLegendreIntegrationPoints8::msIntegrationPoints =
{
    {
        IntegrationPointType( -0.9602898564975363, -0.9602898564975363, 0.1012285362903763*0.1012285362903763 ),
        IntegrationPointType( -0.9602898564975363, -0.7966664774136267, 0.1012285362903763*0.2223810344533745 ),
        IntegrationPointType( -0.9602898564975363, -0.5255324099163290, 0.1012285362903763*0.3137066458778873 ),
        IntegrationPointType( -0.9602898564975363, -0.1834346424956498, 0.1012285362903763*0.3626837833783620 ),
        IntegrationPointType( -0.9602898564975363, 0.1834346424956498, 0.1012285362903763*0.3626837833783620 ),
        IntegrationPointType( -0.9602898564975363, 0.5255324099163290, 0.1012285362903763*0.3137066458778873 ),
        IntegrationPointType( -0.9602898564975363, 0.7966664774136267, 0.1012285362903763*0.2223810344533745 ),
        IntegrationPointType( -0.9602898564975363, 0.9602898564975363, 0.1012285362903763*0.1012285362903763 ),

        IntegrationPointType( -0.7966664774136267, -0.9602898564975363, 0.2223810344533745*0.1012285362903763 ),
        IntegrationPointType( -0.7966664774136267, -0.7966664774136267, 0.2223810344533745*0.2223810344533745 ),
        IntegrationPointType( -0.7966664774136267, -0.5255324099163290, 0.2223810344533745*0.3137066458778873 ),
        IntegrationPointType( -0.7966664774136267, -0.1834346424956498, 0.2223810344533745*0.3626837833783620 ),
        IntegrationPointType( -0.7966664774136267, 0.1834346424956498, 0.2223810344533745*0.3626837833783620 ),
        IntegrationPointType( -0.7966664774136267, 0.5255324099163290, 0.2223810344533745*0.3137066458778873 ),
        IntegrationPointType( -0.7966664774136267, 0.7966664774136267, 0.2223810344533745*0.2223810344533745 ),
        IntegrationPointType( -0.7966664774136267, 0.9602898564975363, 0.2223810344533745*0.1012285362903763 ),

        IntegrationPointType( -0.5255324099163290, -0.9602898564975363, 0.3137066458778873*0.1012285362903763 ),
        IntegrationPointType( -0.5255324099163290, -0.7966664774136267, 0.3137066458778873*0.2223810344533745 ),
        IntegrationPointType( -0.5255324099163290, -0.5255324099163290, 0.3137066458778873*0.3137066458778873 ),
        IntegrationPointType( -0.5255324099163290, -0.1834346424956498, 0.3137066458778873*0.3626837833783620 ),
        IntegrationPointType( -0.5255324099163290, 0.1834346424956498, 0.3137066458778873*0.3626837833783620 ),
        IntegrationPointType( -0.5255324099163290, 0.5255324099163290, 0.3137066458778873*0.3137066458778873 ),
        IntegrationPointType( -0.5255324099163290, 0.7966664774136267, 0.3137066458778873*0.2223810344533745 ),
        IntegrationPointType( -0.5255324099163290, 0.9602898564975363, 0.3137066458778873*0.1012285362903763 ),

        IntegrationPointType( -0.1834346424956498, -0.9602898564975363, 0.3626837833783620*0.1012285362903763 ),
        IntegrationPointType( -0.1834346424956498, -0.7966664774136267, 0.3626837833783620*0.2223810344533745 ),
        IntegrationPointType( -0.1834346424956498, -0.5255324099163290, 0.3626837833783620*0.3137066458778873 ),
        IntegrationPointType( -0.1834346424956498, -0.1834346424956498, 0.3626837833783620*0.3626837833783620 ),
        IntegrationPointType( -0.1834346424956498, 0.1834346424956498, 0.3626837833783620*0.3626837833783620 ),
        IntegrationPointType( -0.1834346424956498, 0.5255324099163290, 0.3626837833783620*0.3137066458778873 ),
        IntegrationPointType( -0.1834346424956498, 0.7966664774136267, 0.3626837833783620*0.2223810344533745 ),
        IntegrationPointType( -0.1834346424956498, 0.9602898564975363, 0.3626837833783620*0.1012285362903763 ),

        IntegrationPointType( 0.1834346424956498, -0.9602898564975363, 0.3626837833783620*0.1012285362903763 ),
        IntegrationPointType( 0.1834346424956498, -0.7966664774136267, 0.3626837833783620*0.2223810344533745 ),
        IntegrationPointType( 0.1834346424956498, -0.5255324099163290, 0.3626837833783620*0.3137066458778873 ),
        IntegrationPointType( 0.1834346424956498, -0.1834346424956498, 0.3626837833783620*0.3626837833783620 ),
        IntegrationPointType( 0.1834346424956498, 0.1834346424956498, 0.3626837833783620*0.3626837833783620 ),
        IntegrationPointType( 0.1834346424956498, 0.5255324099163290, 0.3626837833783620*0.3137066458778873 ),
        IntegrationPointType( 0.1834346424956498, 0.7966664774136267, 0.3626837833783620*0.2223810344533745 ),
        IntegrationPointType( 0.1834346424956498, 0.9602898564975363, 0.3626837833783620*0.1012285362903763 ),

        IntegrationPointType( 0.5255324099163290, -0.9602898564975363, 0.3137066458778873*0.1012285362903763 ),
        IntegrationPointType( 0.5255324099163290, -0.7966664774136267, 0.3137066458778873*0.2223810344533745 ),
        IntegrationPointType( 0.5255324099163290, -0.5255324099163290, 0.3137066458778873*0.3137066458778873 ),
        IntegrationPointType( 0.5255324099163290, -0.1834346424956498, 0.3137066458778873*0.3626837833783620 ),
        IntegrationPointType( 0.5255324099163290, 0.1834346424956498, 0.3137066458778873*0.3626837833783620 ),
        IntegrationPointType( 0.5255324099163290, 0.5255324099163290, 0.3137066458778873*0.3137066458778873 ),
        IntegrationPointType( 0.5255324099163290, 0.7966664774136267, 0.3137066458778873*0.2223810344533745 ),
        IntegrationPointType( 0.5255324099163290, 0.9602898564975363, 0.3137066458778873*0.1012285362903763 ),

        IntegrationPointType( 0.7966664774136267, -0.9602898564975363, 0.2223810344533745*0.1012285362903763 ),
        IntegrationPointType( 0.7966664774136267, -0.7966664774136267, 0.2223810344533745*0.2223810344533745 ),
        IntegrationPointType( 0.7966664774136267, -0.5255324099163290, 0.2223810344533745*0.3137066458778873 ),
        IntegrationPointType( 0.7966664774136267, -0.1834346424956498, 0.2223810344533745*0.3626837833783620 ),
        IntegrationPointType( 0.7966664774136267, 0.1834346424956498, 0.2223810344533745*0.3626837833783620 ),
        IntegrationPointType( 0.7966664774136267, 0.5255324099163290, 0.2223810344533745*0.3137066458778873 ),
        IntegrationPointType( 0.7966664774136267, 0.7966664774136267, 0.2223810344533745*0.2223810344533745 ),
        IntegrationPointType( 0.7966664774136267, 0.9602898564975363, 0.2223810344533745*0.1012285362903763 ),

        IntegrationPointType( 0.9602898564975363, -0.9602898564975363, 0.1012285362903763*0.1012285362903763 ),
        IntegrationPointType( 0.9602898564975363, -0.7966664774136267, 0.1012285362903763*0.2223810344533745 ),
        IntegrationPointType( 0.9602898564975363, -0.5255324099163290, 0.1012285362903763*0.3137066458778873 ),
        IntegrationPointType( 0.9602898564975363, -0.1834346424956498, 0.1012285362903763*0.3626837833783620 ),
        IntegrationPointType( 0.9602898564975363, 0.1834346424956498, 0.1012285362903763*0.3626837833783620 ),
        IntegrationPointType( 0.9602898564975363, 0.5255324099163290, 0.1012285362903763*0.3137066458778873 ),
        IntegrationPointType( 0.9602898564975363, 0.7966664774136267, 0.1012285362903763*0.2223810344533745 ),
        IntegrationPointType( 0.9602898564975363, 0.9602898564975363, 0.1012285362903763*0.1012285362903763 ),
    }
};


QuadrilateralGaussLegendreIntegrationPoints9::IntegrationPointsArrayType
QuadrilateralGaussLegendreIntegrationPoints9::msIntegrationPoints =
{
    {
        IntegrationPointType( -0.9681602395076261, -0.9681602395076261, 0.0812743883615744*0.0812743883615744 ),
        IntegrationPointType( -0.9681602395076261, -0.8360311073266358, 0.0812743883615744*0.1806481606948574 ),
        IntegrationPointType( -0.9681602395076261, -0.6133714327005904, 0.0812743883615744*0.2606106964029354 ),
        IntegrationPointType( -0.9681602395076261, -0.3242534234038089, 0.0812743883615744*0.3123470770400029 ),
        IntegrationPointType( -0.9681602395076261, 0.0000000000000000, 0.0812743883615744*0.3302393550012598 ),
        IntegrationPointType( -0.9681602395076261, 0.3242534234038089, 0.0812743883615744*0.3123470770400029 ),
        IntegrationPointType( -0.9681602395076261, 0.6133714327005904, 0.0812743883615744*0.2606106964029354 ),
        IntegrationPointType( -0.9681602395076261, 0.8360311073266358, 0.0812743883615744*0.1806481606948574 ),
        IntegrationPointType( -0.9681602395076261, 0.9681602395076261, 0.0812743883615744*0.0812743883615744 ),

        IntegrationPointType( -0.8360311073266358, -0.9681602395076261, 0.1806481606948574*0.0812743883615744 ),
        IntegrationPointType( -0.8360311073266358, -0.8360311073266358, 0.1806481606948574*0.1806481606948574 ),
        IntegrationPointType( -0.8360311073266358, -0.6133714327005904, 0.1806481606948574*0.2606106964029354 ),
        IntegrationPointType( -0.8360311073266358, -0.3242534234038089, 0.1806481606948574*0.3123470770400029 ),
        IntegrationPointType( -0.8360311073266358, 0.0000000000000000, 0.1806481606948574*0.3302393550012598 ),
        IntegrationPointType( -0.8360311073266358, 0.3242534234038089, 0.1806481606948574*0.3123470770400029 ),
        IntegrationPointType( -0.8360311073266358, 0.6133714327005904, 0.1806481606948574*0.2606106964029354 ),
        IntegrationPointType( -0.8360311073266358, 0.8360311073266358, 0.1806481606948574*0.1806481606948574 ),
        IntegrationPointType( -0.8360311073266358, 0.9681602395076261, 0.1806481606948574*0.0812743883615744 ),

        IntegrationPointType( -0.6133714327005904, -0.9681602395076261, 0.2606106964029354*0.0812743883615744 ),
        IntegrationPointType( -0.6133714327005904, -0.8360311073266358, 0.2606106964029354*0.1806481606948574 ),
        IntegrationPointType( -0.6133714327005904, -0.6133714327005904, 0.2606106964029354*0.2606106964029354 ),
        IntegrationPointType( -0.6133714327005904, -0.3242534234038089, 0.2606106964029354*0.3123470770400029 ),
        IntegrationPointType( -0.6133714327005904, 0.0000000000000000, 0.2606106964029354*0.3302393550012598 ),
        IntegrationPointType( -0.6133714327005904, 0.3242534234038089, 0.2606106964029354*0.3123470770400029 ),
        IntegrationPointType( -0.6133714327005904, 0.6133714327005904, 0.2606106964029354*0.2606106964029354 ),
        IntegrationPointType( -0.6133714327005904, 0.8360311073266358, 0.2606106964029354*0.1806481606948574 ),
        IntegrationPointType( -0.6133714327005904, 0.9681602395076261, 0.2606106964029354*0.0812743883615744 ),

        IntegrationPointType( -0.3242534234038089, -0.9681602395076261, 0.3123470770400029*0.0812743883615744 ),
        IntegrationPointType( -0.3242534234038089, -0.8360311073266358, 0.3123470770400029*0.1806481606948574 ),
        IntegrationPointType( -0.3242534234038089, -0.6133714327005904, 0.3123470770400029*0.2606106964029354 ),
        IntegrationPointType( -0.3242534234038089, -0.3242534234038089, 0.3123470770400029*0.3123470770400029 ),
        IntegrationPointType( -0.3242534234038089, 0.0000000000000000, 0.3123470770400029*0.3302393550012598 ),
        IntegrationPointType( -0.3242534234038089, 0.3242534234038089, 0.3123470770400029*0.3123470770400029 ),
        IntegrationPointType( -0.3242534234038089, 0.6133714327005904, 0.3123470770400029*0.2606106964029354 ),
        IntegrationPointType( -0.3242534234038089, 0.8360311073266358, 0.3123470770400029*0.1806481606948574 ),
        IntegrationPointType( -0.3242534234038089, 0.9681602395076261, 0.3123470770400029*0.0812743883615744 ),

        IntegrationPointType( 0.0000000000000000, -0.9681602395076261, 0.3302393550012598*0.0812743883615744 ),
        IntegrationPointType( 0.0000000000000000, -0.8360311073266358, 0.3302393550012598*0.1806481606948574 ),
        IntegrationPointType( 0.0000000000000000, -0.6133714327005904, 0.3302393550012598*0.2606106964029354 ),
        IntegrationPointType( 0.0000000000000000, -0.3242534234038089, 0.3302393550012598*0.3123470770400029 ),
        IntegrationPointType( 0.0000000000000000, 0.0000000000000000, 0.3302393550012598*0.3302393550012598 ),
        IntegrationPointType( 0.0000000000000000, 0.3242534234038089, 0.3302393550012598*0.3123470770400029 ),
        IntegrationPointType( 0.0000000000000000, 0.6133714327005904, 0.3302393550012598*0.2606106964029354 ),
        IntegrationPointType( 0.0000000000000000, 0.8360311073266358, 0.3302393550012598*0.1806481606948574 ),
        IntegrationPointType( 0.0000000000000000, 0.9681602395076261, 0.3302393550012598*0.0812743883615744 ),

        IntegrationPointType( 0.3242534234038089, -0.9681602395076261, 0.3123470770400029*0.0812743883615744 ),
        IntegrationPointType( 0.3242534234038089, -0.8360311073266358, 0.3123470770400029*0.1806481606948574 ),
        IntegrationPointType( 0.3242534234038089, -0.6133714327005904, 0.3123470770400029*0.2606106964029354 ),
        IntegrationPointType( 0.3242534234038089, -0.3242534234038089, 0.3123470770400029*0.3123470770400029 ),
        IntegrationPointType( 0.3242534234038089, 0.0000000000000000, 0.3123470770400029*0.3302393550012598 ),
        IntegrationPointType( 0.3242534234038089, 0.3242534234038089, 0.3123470770400029*0.3123470770400029 ),
        IntegrationPointType( 0.3242534234038089, 0.6133714327005904, 0.3123470770400029*0.2606106964029354 ),
        IntegrationPointType( 0.3242534234038089, 0.8360311073266358, 0.3123470770400029*0.1806481606948574 ),
        IntegrationPointType( 0.3242534234038089, 0.9681602395076261, 0.3123470770400029*0.0812743883615744 ),

        IntegrationPointType( 0.6133714327005904, -0.9681602395076261, 0.2606106964029354*0.0812743883615744 ),
        IntegrationPointType( 0.6133714327005904, -0.8360311073266358, 0.2606106964029354*0.1806481606948574 ),
        IntegrationPointType( 0.6133714327005904, -0.6133714327005904, 0.2606106964029354*0.2606106964029354 ),
        IntegrationPointType( 0.6133714327005904, -0.3242534234038089, 0.2606106964029354*0.3123470770400029 ),
        IntegrationPointType( 0.6133714327005904, 0.0000000000000000, 0.2606106964029354*0.3302393550012598 ),
        IntegrationPointType( 0.6133714327005904, 0.3242534234038089, 0.2606106964029354*0.3123470770400029 ),
        IntegrationPointType( 0.6133714327005904, 0.6133714327005904, 0.2606106964029354*0.2606106964029354 ),
        IntegrationPointType( 0.6133714327005904, 0.8360311073266358, 0.2606106964029354*0.1806481606948574 ),
        IntegrationPointType( 0.6133714327005904, 0.9681602395076261, 0.2606106964029354*0.0812743883615744 ),

        IntegrationPointType( 0.8360311073266358, -0.9681602395076261, 0.1806481606948574*0.0812743883615744 ),
        IntegrationPointType( 0.8360311073266358, -0.8360311073266358, 0.1806481606948574*0.1806481606948574 ),
        IntegrationPointType( 0.8360311073266358, -0.6133714327005904, 0.1806481606948574*0.2606106964029354 ),
        IntegrationPointType( 0.8360311073266358, -0.3242534234038089, 0.1806481606948574*0.3123470770400029 ),
        IntegrationPointType( 0.8360311073266358, 0.0000000000000000, 0.1806481606948574*0.3302393550012598 ),
        IntegrationPointType( 0.8360311073266358, 0.3242534234038089, 0.1806481606948574*0.3123470770400029 ),
        IntegrationPointType( 0.8360311073266358, 0.6133714327005904, 0.1806481606948574*0.2606106964029354 ),
        IntegrationPointType( 0.8360311073266358, 0.8360311073266358, 0.1806481606948574*0.1806481606948574 ),
        IntegrationPointType( 0.8360311073266358, 0.9681602395076261, 0.1806481606948574*0.0812743883615744 ),

        IntegrationPointType( 0.9681602395076261, -0.9681602395076261, 0.0812743883615744*0.0812743883615744 ),
        IntegrationPointType( 0.9681602395076261, -0.8360311073266358, 0.0812743883615744*0.1806481606948574 ),
        IntegrationPointType( 0.9681602395076261, -0.6133714327005904, 0.0812743883615744*0.2606106964029354 ),
        IntegrationPointType( 0.9681602395076261, -0.3242534234038089, 0.0812743883615744*0.3123470770400029 ),
        IntegrationPointType( 0.9681602395076261, 0.0000000000000000, 0.0812743883615744*0.3302393550012598 ),
        IntegrationPointType( 0.9681602395076261, 0.3242534234038089, 0.0812743883615744*0.3123470770400029 ),
        IntegrationPointType( 0.9681602395076261, 0.6133714327005904, 0.0812743883615744*0.2606106964029354 ),
        IntegrationPointType( 0.9681602395076261, 0.8360311073266358, 0.0812743883615744*0.1806481606948574 ),
        IntegrationPointType( 0.9681602395076261, 0.9681602395076261, 0.0812743883615744*0.0812743883615744 ),
    }
};


QuadrilateralGaussLegendreIntegrationPoints10::IntegrationPointsArrayType
QuadrilateralGaussLegendreIntegrationPoints10::msIntegrationPoints =
{
    {
        IntegrationPointType( -0.9739065285171717, -0.9739065285171717, 0.0666713443086881*0.0666713443086881 ),
        IntegrationPointType( -0.9739065285171717, -0.8650633666889845, 0.0666713443086881*0.1494513491505806 ),
        IntegrationPointType( -0.9739065285171717, -0.6794095682990244, 0.0666713443086881*0.2190863625159820 ),
        IntegrationPointType( -0.9739065285171717, -0.4333953941292472, 0.0666713443086881*0.2692667193099963 ),
        IntegrationPointType( -0.9739065285171717, -0.1488743389816312, 0.0666713443086881*0.2955242247147529 ),
        IntegrationPointType( -0.9739065285171717, 0.1488743389816312, 0.0666713443086881*0.2955242247147529 ),
        IntegrationPointType( -0.9739065285171717, 0.4333953941292472, 0.0666713443086881*0.2692667193099963 ),
        IntegrationPointType( -0.9739065285171717, 0.6794095682990244, 0.0666713443086881*0.2190863625159820 ),
        IntegrationPointType( -0.9739065285171717, 0.8650633666889845, 0.0666713443086881*0.1494513491505806 ),
        IntegrationPointType( -0.9739065285171717, 0.9739065285171717, 0.0666713443086881*0.0666713443086881 ),

        IntegrationPointType( -0.8650633666889845, -0.9739065285171717, 0.1494513491505806*0.0666713443086881 ),
        IntegrationPointType( -0.8650633666889845, -0.8650633666889845, 0.1494513491505806*0.1494513491505806 ),
        IntegrationPointType( -0.8650633666889845, -0.6794095682990244, 0.1494513491505806*0.2190863625159820 ),
        IntegrationPointType( -0.8650633666889845, -0.4333953941292472, 0.1494513491505806*0.2692667193099963 ),
        IntegrationPointType( -0.8650633666889845, -0.1488743389816312, 0.1494513491505806*0.2955242247147529 ),
        IntegrationPointType( -0.8650633666889845, 0.1488743389816312, 0.1494513491505806*0.2955242247147529 ),
        IntegrationPointType( -0.8650633666889845, 0.4333953941292472, 0.1494513491505806*0.2692667193099963 ),
        IntegrationPointType( -0.8650633666889845, 0.6794095682990244, 0.1494513491505806*0.2190863625159820 ),
        IntegrationPointType( -0.8650633666889845, 0.8650633666889845, 0.1494513491505806*0.1494513491505806 ),
        IntegrationPointType( -0.8650633666889845, 0.9739065285171717, 0.1494513491505806*0.0666713443086881 ),

        IntegrationPointType( -0.6794095682990244, -0.9739065285171717, 0.2190863625159820*0.0666713443086881 ),
        IntegrationPointType( -0.6794095682990244, -0.8650633666889845, 0.2190863625159820*0.1494513491505806 ),
        IntegrationPointType( -0.6794095682990244, -0.6794095682990244, 0.2190863625159820*0.2190863625159820 ),
        IntegrationPointType( -0.6794095682990244, -0.4333953941292472, 0.2190863625159820*0.2692667193099963 ),
        IntegrationPointType( -0.6794095682990244, -0.1488743389816312, 0.2190863625159820*0.2955242247147529 ),
        IntegrationPointType( -0.6794095682990244, 0.1488743389816312, 0.2190863625159820*0.2955242247147529 ),
        IntegrationPointType( -0.6794095682990244, 0.4333953941292472, 0.2190863625159820*0.2692667193099963 ),
        IntegrationPointType( -0.6794095682990244, 0.6794095682990244, 0.2190863625159820*0.2190863625159820 ),
        IntegrationPointType( -0.6794095682990244, 0.8650633666889845, 0.2190863625159820*0.1494513491505806 ),
        IntegrationPointType( -0.6794095682990244, 0.9739065285171717, 0.2190863625159820*0.0666713443086881 ),

        IntegrationPointType( -0.4333953941292472, -0.9739065285171717, 0.2692667193099963*0.0666713443086881 ),
        IntegrationPointType( -0.4333953941292472, -0.8650633666889845, 0.2692667193099963*0.1494513491505806 ),
        IntegrationPointType( -0.4333953941292472, -0.6794095682990244, 0.2692667193099963*0.2190863625159820 ),
        IntegrationPointType( -0.4333953941292472, -0.4333953941292472, 0.2692667193099963*0.2692667193099963 ),
        IntegrationPointType( -0.4333953941292472, -0.1488743389816312, 0.2692667193099963*0.2955242247147529 ),
        IntegrationPointType( -0.4333953941292472, 0.1488743389816312, 0.2692667193099963*0.2955242247147529 ),
        IntegrationPointType( -0.4333953941292472, 0.4333953941292472, 0.2692667193099963*0.2692667193099963 ),
        IntegrationPointType( -0.4333953941292472, 0.6794095682990244, 0.2692667193099963*0.2190863625159820 ),
        IntegrationPointType( -0.4333953941292472, 0.8650633666889845, 0.2692667193099963*0.1494513491505806 ),
        IntegrationPointType( -0.4333953941292472, 0.9739065285171717, 0.2692667193099963*0.0666713443086881 ),

        IntegrationPointType( -0.1488743389816312, -0.9739065285171717, 0.2955242247147529*0.0666713443086881 ),
        IntegrationPointType( -0.1488743389816312, -0.8650633666889845, 0.2955242247147529*0.1494513491505806 ),
        IntegrationPointType( -0.1488743389816312, -0.6794095682990244, 0.2955242247147529*0.2190863625159820 ),
        IntegrationPointType( -0.1488743389816312, -0.4333953941292472, 0.2955242247147529*0.2692667193099963 ),
        IntegrationPointType( -0.1488743389816312, -0.1488743389816312, 0.2955242247147529*0.2955242247147529 ),
        IntegrationPointType( -0.1488743389816312, 0.1488743389816312, 0.2955242247147529*0.2955242247147529 ),
        IntegrationPointType( -0.1488743389816312, 0.4333953941292472, 0.2955242247147529*0.2692667193099963 ),
        IntegrationPointType( -0.1488743389816312, 0.6794095682990244, 0.2955242247147529*0.2190863625159820 ),
        IntegrationPointType( -0.1488743389816312, 0.8650633666889845, 0.2955242247147529*0.1494513491505806 ),
        IntegrationPointType( -0.1488743389816312, 0.9739065285171717, 0.2955242247147529*0.0666713443086881 ),

        IntegrationPointType( 0.1488743389816312, -0.9739065285171717, 0.2955242247147529*0.0666713443086881 ),
        IntegrationPointType( 0.1488743389816312, -0.8650633666889845, 0.2955242247147529*0.1494513491505806 ),
        IntegrationPointType( 0.1488743389816312, -0.6794095682990244, 0.2955242247147529*0.2190863625159820 ),
        IntegrationPointType( 0.1488743389816312, -0.4333953941292472, 0.2955242247147529*0.2692667193099963 ),
        IntegrationPointType( 0.1488743389816312, -0.1488743389816312, 0.2955242247147529*0.2955242247147529 ),
        IntegrationPointType( 0.1488743389816312, 0.1488743389816312, 0.2955242247147529*0.2955242247147529 ),
        IntegrationPointType( 0.1488743389816312, 0.4333953941292472, 0.2955242247147529*0.2692667193099963 ),
        IntegrationPointType( 0.1488743389816312, 0.6794095682990244, 0.2955242247147529*0.2190863625159820 ),
        IntegrationPointType( 0.1488743389816312, 0.8650633666889845, 0.2955242247147529*0.1494513491505806 ),
        IntegrationPointType( 0.1488743389816312, 0.9739065285171717, 0.2955242247147529*0.0666713443086881 ),

        IntegrationPointType( 0.4333953941292472, -0.9739065285171717, 0.2692667193099963*0.0666713443086881 ),
        IntegrationPointType( 0.4333953941292472, -0.8650633666889845, 0.2692667193099963*0.1494513491505806 ),
        IntegrationPointType( 0.4333953941292472, -0.6794095682990244, 0.2692667193099963*0.2190863625159820 ),
        IntegrationPointType( 0.4333953941292472, -0.4333953941292472, 0.2692667193099963*0.2692667193099963 ),
        IntegrationPointType( 0.4333953941292472, -0.1488743389816312, 0.2692667193099963*0.2955242247147529 ),
        IntegrationPointType( 0.4333953941292472, 0.1488743389816312, 0.2692667193099963*0.2955242247147529 ),
        IntegrationPointType( 0.4333953941292472, 0.4333953941292472, 0.2692667193099963*0.2692667193099963 ),
        IntegrationPointType( 0.4333953941292472, 0.6794095682990244, 0.2692667193099963*0.2190863625159820 ),
        IntegrationPointType( 0.4333953941292472, 0.8650633666889845, 0.2692667193099963*0.1494513491505806 ),
        IntegrationPointType( 0.4333953941292472, 0.9739065285171717, 0.2692667193099963*0.0666713443086881 ),

        IntegrationPointType( 0.6794095682990244, -0.9739065285171717, 0.2190863625159820*0.0666713443086881 ),
        IntegrationPointType( 0.6794095682990244, -0.8650633666889845, 0.2190863625159820*0.1494513491505806 ),
        IntegrationPointType( 0.6794095682990244, -0.6794095682990244, 0.2190863625159820*0.2190863625159820 ),
        IntegrationPointType( 0.6794095682990244, -0.4333953941292472, 0.2190863625159820*0.2692667193099963 ),
        IntegrationPointType( 0.6794095682990244, -0.1488743389816312, 0.2190863625159820*0.2955242247147529 ),
        IntegrationPointType( 0.6794095682990244, 0.1488743389816312, 0.2190863625159820*0.2955242247147529 ),
        IntegrationPointType( 0.6794095682990244, 0.4333953941292472, 0.2190863625159820*0.2692667193099963 ),
        IntegrationPointType( 0.6794095682990244, 0.6794095682990244, 0.2190863625159820*0.2190863625159820 ),
        IntegrationPointType( 0.6794095682990244, 0.8650633666889845, 0.2190863625159820*0.1494513491505806 ),
        IntegrationPointType( 0.6794095682990244, 0.9739065285171717, 0.2190863625159820*0.0666713443086881 ),

        IntegrationPointType( 0.8650633666889845, -0.9739065285171717, 0.1494513491505806*0.0666713443086881 ),
        IntegrationPointType( 0.8650633666889845, -0.8650633666889845, 0.1494513491505806*0.1494513491505806 ),
        IntegrationPointType( 0.8650633666889845, -0.6794095682990244, 0.1494513491505806*0.2190863625159820 ),
        IntegrationPointType( 0.8650633666889845, -0.4333953941292472, 0.1494513491505806*0.2692667193099963 ),
        IntegrationPointType( 0.8650633666889845, -0.1488743389816312, 0.1494513491505806*0.2955242247147529 ),
        IntegrationPointType( 0.8650633666889845, 0.1488743389816312, 0.1494513491505806*0.2955242247147529 ),
        IntegrationPointType( 0.8650633666889845, 0.4333953941292472, 0.1494513491505806*0.2692667193099963 ),
        IntegrationPointType( 0.8650633666889845, 0.6794095682990244, 0.1494513491505806*0.2190863625159820 ),
        IntegrationPointType( 0.8650633666889845, 0.8650633666889845, 0.1494513491505806*0.1494513491505806 ),
        IntegrationPointType( 0.8650633666889845, 0.9739065285171717, 0.1494513491505806*0.0666713443086881 ),

        IntegrationPointType( 0.9739065285171717, -0.9739065285171717, 0.0666713443086881*0.0666713443086881 ),
        IntegrationPointType( 0.9739065285171717, -0.8650633666889845, 0.0666713443086881*0.1494513491505806 ),
        IntegrationPointType( 0.9739065285171717, -0.6794095682990244, 0.0666713443086881*0.2190863625159820 ),
        IntegrationPointType( 0.9739065285171717, -0.4333953941292472, 0.0666713443086881*0.2692667193099963 ),
        IntegrationPointType( 0.9739065285171717, -0.1488743389816312, 0.0666713443086881*0.2955242247147529 ),
        IntegrationPointType( 0.9739065285171717, 0.1488743389816312, 0.0666713443086881*0.2955242247147529 ),
        IntegrationPointType( 0.9739065285171717, 0.4333953941292472, 0.0666713443086881*0.2692667193099963 ),
        IntegrationPointType( 0.9739065285171717, 0.6794095682990244, 0.0666713443086881*0.2190863625159820 ),
        IntegrationPointType( 0.9739065285171717, 0.8650633666889845, 0.0666713443086881*0.1494513491505806 ),
        IntegrationPointType( 0.9739065285171717, 0.9739065285171717, 0.0666713443086881*0.0666713443086881 ),
    }
};

//Gauss-Lobatto

QuadrilateralGaussLobattoIntegrationPoints1::IntegrationPointsArrayType
QuadrilateralGaussLobattoIntegrationPoints1::msIntegrationPoints =
{
	{
		IntegrationPointType( -1.00 , 0.00, 1.00 ),
		IntegrationPointType(  1.00 , 0.00, 1.00 )
	}
};

QuadrilateralGaussLobattoIntegrationPoints2::IntegrationPointsArrayType
QuadrilateralGaussLobattoIntegrationPoints2::msIntegrationPoints =
{
	{
		IntegrationPointType( -1.00 , -1.00, 0.50 ),
		IntegrationPointType(  1.00 , -1.00, 0.50 ),
		IntegrationPointType(  1.00 ,  1.00, 0.50 ),
		IntegrationPointType( -1.00 ,  1.00, 0.50 )
	}
};

//TETRAHEDRON:

//Gauss-Legendre

TetrahedronGaussLegendreIntegrationPoints1::IntegrationPointsArrayType
TetrahedronGaussLegendreIntegrationPoints1::msIntegrationPoints =
{
    {
        IntegrationPointType( 0.25,0.25,0.25 , 1.00 / 6.00 )
    }
};

TetrahedronGaussLegendreIntegrationPoints2::IntegrationPointsArrayType
TetrahedronGaussLegendreIntegrationPoints2::msIntegrationPoints =
{
    {
        IntegrationPointType( 0.58541020,0.13819660,0.13819660 , 1.00 / 24.00 ),
        IntegrationPointType( 0.13819660,0.58541020,0.13819660 , 1.00 / 24.00 ),
        IntegrationPointType( 0.13819660,0.13819660,0.58541020 , 1.00 / 24.00 ),
        IntegrationPointType( 0.13819660,0.13819660,0.13819660 , 1.00 / 24.00 )
    }
};

TetrahedronGaussLegendreIntegrationPoints3::IntegrationPointsArrayType
TetrahedronGaussLegendreIntegrationPoints3::msIntegrationPoints =
{
    {
        IntegrationPointType( 0.25,0.25,0.25 , -0.1333333333333333333333333333333 ),
        IntegrationPointType( 0.5,0.1666666666666667,0.1666666666666667	, 0.075 ),
        IntegrationPointType( 0.1666666666666667,0.5,0.1666666666666667 , 0.075 ),
        IntegrationPointType( 0.1666666666666667,0.1666666666666667,0.5, 0.075 ),
        IntegrationPointType( 0.1666666666666667,0.1666666666666667,0.1666666666666667, 0.075 )
    }
};

// 	TetrahedronGaussLegendreIntegrationPoints4::IntegrationPointsArrayType
// 			TetrahedronGaussLegendreIntegrationPoints4::msIntegrationPoints=
// 	{
// 		{
// 			IntegrationPointType( 1.0/4.0,  1.0/4.0,  1.0/4.0, -4.0/30.0 ),
// 			IntegrationPointType( 1.0/6.0,  1.0/6.0,  1.0/6.0, 9.0/120.0 ),
// 			IntegrationPointType( 1.0/2.0,  1.0/6.0,  1.0/6.0, 9.0/120.0 ),
// 			IntegrationPointType( 1.0/6.0,  1.0/2.0,  1.0/6.0, 9.0/120.0 ),
// 			IntegrationPointType( 1.0/6.0,  1.0/6.0,  1.0/2.0, 9.0/120.0 )
// 		}
// 	};


TetrahedronGaussLegendreIntegrationPoints4::IntegrationPointsArrayType
TetrahedronGaussLegendreIntegrationPoints4::msIntegrationPoints=
{
    {
        IntegrationPointType( tet10_a,  tet10_a,  tet10_a, -1.0/60.0 ),
        IntegrationPointType( tet10_c,  tet10_a,  tet10_a, -1.0/60.0 ),
        IntegrationPointType( tet10_a,  tet10_c,  tet10_a, -1.0/60.0 ),
        IntegrationPointType( tet10_a,  tet10_a,  tet10_c, -1.0/60.0 ),
        IntegrationPointType( tet10_b,  tet10_a,  tet10_a, -1.0/60.0 ),
        IntegrationPointType( tet10_b,  tet10_b,  tet10_a, -1.0/60.0 ),
        IntegrationPointType( tet10_a,  tet10_b,  tet10_a, -1.0/60.0 ),
        IntegrationPointType( tet10_a,  tet10_a,  tet10_b, -1.0/60.0 ),
        IntegrationPointType( tet10_b,  tet10_a,  tet10_b, -1.0/60.0 ),
        IntegrationPointType( tet10_a,  tet10_b,  tet10_b, -1.0/60.0 ),
    }
};


TetrahedronGaussLegendreIntegrationPoints5::IntegrationPointsArrayType
TetrahedronGaussLegendreIntegrationPoints5::msIntegrationPoints=
{
    {
        IntegrationPointType(1.0/4.0, 1.0/4.0, 1.0/4.0,-74.0/5625.0 ),
        IntegrationPointType(1.0/14.0, 1.0/14.0, 1.0/14.0,343.0/45000.0 ),
        IntegrationPointType(11.0/14.0, 1.0/14.0, 1.0/14.0,343.0/45000.0 ),
        IntegrationPointType(1.0/14.0, 11.0/14.0, 1.0/14.0,343.0/45000.0 ),
        IntegrationPointType(1.0/14.0, 1.0/14.0, 11.0/14.0,343.0/45000.0 ),
        IntegrationPointType((1.0+std::sqrt(5.0/14.0))/4.0,(1.0-std::sqrt(5.0/14.0))/
        4.0, (1.0-std::sqrt(5.0/14.0))/4.0,56.0/2250.0 ),
        IntegrationPointType((1.0+std::sqrt(5.0/14.0))/4.0,(1.0+std::sqrt(5.0/14.0))/
        4.0, (1.0-std::sqrt(5.0/14.0))/4.0,56.0/2250.0 ),
        IntegrationPointType((1.0-std::sqrt(5.0/14.0))/4.0,(1.0+std::sqrt(5.0/14.0))/
        4.0, (1.0-std::sqrt(5.0/14.0))/4.0,56.0/2250.0 ),
        IntegrationPointType((1.0-std::sqrt(5.0/14.0))/4.0,(1.0-std::sqrt(5.0/14.0))/
        4.0, (1.0+std::sqrt(5.0/14.0))/4.0,56.0/2250.0 ),
        IntegrationPointType((1.0+std::sqrt(5.0/14.0))/4.0,(1.0-std::sqrt(5.0/14.0))/
        4.0, (1.0+std::sqrt(5.0/14.0))/4.0,56.0/2250.0 ),
        IntegrationPointType((1.0-std::sqrt(5.0/14.0))/4.0,(1.0+std::sqrt(5.0/14.0))/
        4.0, (1.0+std::sqrt(5.0/14.0))/4.0,56.0/2250.0 )
    }
};

//PRISM:

//Gauss-Legendre

// TODO: Check if the values are correct

PrismGaussLegendreIntegrationPoints1::IntegrationPointsArrayType
PrismGaussLegendreIntegrationPoints1::msIntegrationPoints=
{
    {
        IntegrationPointType( 2.00 / 3.00 , 1.00 / 6.00, 0.50, 1.00 / 6.00),
        IntegrationPointType( 1.00 / 6.00 , 2.00 / 3.00, 0.50, 1.00 / 6.00),
        IntegrationPointType( 1.00 / 6.00 , 1.00 / 6.00, 0.50, 1.00 / 6.00)
    }
};

PrismGaussLegendreIntegrationPoints2::IntegrationPointsArrayType
PrismGaussLegendreIntegrationPoints2::msIntegrationPoints=
{
    {
        IntegrationPointType( 2.00 / 3.00 , 1.00 / 6.00, ((-1.0/std::sqrt(3.0)+1.0)/2.0), 1.00 / 12.00),
        IntegrationPointType( 1.00 / 6.00 , 2.00 / 3.00, ((-1.0/std::sqrt(3.0)+1.0)/2.0), 1.00 / 12.00),
        IntegrationPointType( 1.00 / 6.00 , 1.00 / 6.00, ((-1.0/std::sqrt(3.0)+1.0)/2.0), 1.00 / 12.00),
        IntegrationPointType( 2.00 / 3.00 , 1.00 / 6.00, (( 1.0/std::sqrt(3.0)+1.0)/2.0), 1.00 / 12.00),
        IntegrationPointType( 1.00 / 6.00 , 2.00 / 3.00, (( 1.0/std::sqrt(3.0)+1.0)/2.0), 1.00 / 12.00),
        IntegrationPointType( 1.00 / 6.00 , 1.00 / 6.00, (( 1.0/std::sqrt(3.0)+1.0)/2.0), 1.00 / 12.00)
    }
};

PrismGaussLegendreIntegrationPoints3::IntegrationPointsArrayType
PrismGaussLegendreIntegrationPoints3::msIntegrationPoints=
{
    {
        IntegrationPointType( 2.00 / 3.00 , 1.00 / 6.00, (-std::sqrt(3.00 / 5.00)+1.0)/2.0, 5.00 / 108.00),
        IntegrationPointType( 1.00 / 6.00 , 2.00 / 3.00, (-std::sqrt(3.00 / 5.00)+1.0)/2.0, 5.00 / 108.00),
        IntegrationPointType( 1.00 / 6.00 , 1.00 / 6.00, (-std::sqrt(3.00 / 5.00)+1.0)/2.0, 5.00 / 108.00),
        IntegrationPointType( 2.00 / 3.00 , 1.00 / 6.00, 0.500000000000000000000000000000,  4.00 /  54.00),
        IntegrationPointType( 1.00 / 6.00 , 2.00 / 3.00, 0.500000000000000000000000000000,  4.00 /  54.00),
        IntegrationPointType( 1.00 / 6.00 , 1.00 / 6.00, 0.500000000000000000000000000000,  4.00 /  54.00),
        IntegrationPointType( 2.00 / 3.00 , 1.00 / 6.00, ( std::sqrt(3.00 / 5.00)+1.0)/2.0, 5.00 / 108.00),
        IntegrationPointType( 1.00 / 6.00 , 2.00 / 3.00, ( std::sqrt(3.00 / 5.00)+1.0)/2.0, 5.00 / 108.00),
        IntegrationPointType( 1.00 / 6.00 , 1.00 / 6.00, ( std::sqrt(3.00 / 5.00)+1.0)/2.0, 5.00 / 108.00)
    }
};

PrismGaussLegendreIntegrationPoints4::IntegrationPointsArrayType
PrismGaussLegendreIntegrationPoints4::msIntegrationPoints=
{
    {
        IntegrationPointType( 2.00 / 3.00 , 1.00 / 6.00, 0.06943184420297371239, 0.02898790376145448812),
        IntegrationPointType( 1.00 / 6.00 , 2.00 / 3.00, 0.06943184420297371239, 0.02898790376145448812),
        IntegrationPointType( 1.00 / 6.00 , 1.00 / 6.00, 0.06943184420297371239, 0.02898790376145448812),
        IntegrationPointType( 2.00 / 3.00 , 1.00 / 6.00, 0.33000947820757186760, 0.05434542957187884522),
        IntegrationPointType( 1.00 / 6.00 , 2.00 / 3.00, 0.33000947820757186760, 0.05434542957187884522),
        IntegrationPointType( 1.00 / 6.00 , 1.00 / 6.00, 0.33000947820757186760, 0.05434542957187884522),
        IntegrationPointType( 2.00 / 3.00 , 1.00 / 6.00, 0.66999052179242813240, 0.05434542957187884522),
        IntegrationPointType( 1.00 / 6.00 , 2.00 / 3.00, 0.66999052179242813240, 0.05434542957187884522),
        IntegrationPointType( 1.00 / 6.00 , 1.00 / 6.00, 0.66999052179242813240, 0.05434542957187884522),
        IntegrationPointType( 2.00 / 3.00 , 1.00 / 6.00, 0.93056815579702628761, 0.02898790376145448812),
        IntegrationPointType( 1.00 / 6.00 , 2.00 / 3.00, 0.93056815579702628761, 0.02898790376145448812),
        IntegrationPointType( 1.00 / 6.00 , 1.00 / 6.00, 0.93056815579702628761, 0.02898790376145448812)
    }
};

PrismGaussLegendreIntegrationPoints5::IntegrationPointsArrayType
PrismGaussLegendreIntegrationPoints5::msIntegrationPoints=
{
    {
        IntegrationPointType( 2.00 / 3.00 , 1.00 / 6.00, 0.04691007703066800360, 0.01974390708801575729),
        IntegrationPointType( 1.00 / 6.00 , 2.00 / 3.00, 0.04691007703066800360, 0.01974390708801575729),
        IntegrationPointType( 1.00 / 6.00 , 1.00 / 6.00, 0.04691007703066800360, 0.01974390708801575729),
        IntegrationPointType( 2.00 / 3.00 , 1.00 / 6.00, 0.23076534494715845448, 0.03988572254161387234),
        IntegrationPointType( 1.00 / 6.00 , 2.00 / 3.00, 0.23076534494715845448, 0.03988572254161387234),
        IntegrationPointType( 1.00 / 6.00 , 1.00 / 6.00, 0.23076534494715845448, 0.03988572254161387234),
        IntegrationPointType( 2.00 / 3.00 , 1.00 / 6.00, 0.50000000000000000000, 0.04740740740740740741),
        IntegrationPointType( 1.00 / 6.00 , 2.00 / 3.00, 0.50000000000000000000, 0.04740740740740740741),
        IntegrationPointType( 1.00 / 6.00 , 1.00 / 6.00, 0.50000000000000000000, 0.04740740740740740741),
        IntegrationPointType( 2.00 / 3.00 , 1.00 / 6.00, 0.76923465505284154552, 0.03988572254161387234),
        IntegrationPointType( 1.00 / 6.00 , 2.00 / 3.00, 0.76923465505284154552, 0.03988572254161387234),
        IntegrationPointType( 1.00 / 6.00 , 1.00 / 6.00, 0.76923465505284154552, 0.03988572254161387234),
        IntegrationPointType( 2.00 / 3.00 , 1.00 / 6.00, 0.95308992296933199640, 0.01974390708801575729),
        IntegrationPointType( 1.00 / 6.00 , 2.00 / 3.00, 0.95308992296933199640, 0.01974390708801575729),
        IntegrationPointType( 1.00 / 6.00 , 1.00 / 6.00, 0.95308992296933199640, 0.01974390708801575729)
    }
};

//Gauss-Lobatto

PrismGaussLobattoIntegrationPoints1::IntegrationPointsArrayType
PrismGaussLobattoIntegrationPoints1::msIntegrationPoints =
{
	{
		IntegrationPointType( 0.0 , 0.0 , 0.5 , 1.00 / 6.00 ),
		IntegrationPointType( 1.0 , 0.0 , 0.5 , 1.00 / 6.00 ),
		IntegrationPointType( 0.0 , 1.0 , 0.5 , 1.00 / 6.00 )
	}
};

PrismGaussLobattoIntegrationPoints2::IntegrationPointsArrayType
PrismGaussLobattoIntegrationPoints2::msIntegrationPoints =
{
	{
		IntegrationPointType( 0.0 , 0.0 , 0.0 , 1.00 / 12.00 ),
		IntegrationPointType( 1.0 , 0.0 , 0.0 , 1.00 / 12.00 ),
		IntegrationPointType( 0.0 , 1.0 , 0.0 , 1.00 / 12.00 ),
		IntegrationPointType( 0.0 , 0.0 , 1.0 , 1.00 / 12.00 ),
		IntegrationPointType( 1.0 , 0.0 , 1.0 , 1.00 / 12.00 ),
		IntegrationPointType( 0.0 , 1.0 , 1.0 , 1.00 / 12.00 )
	}
};

//HEXAHEDRON:

//Gauss-Legendre

HexahedronGaussLegendreIntegrationPoints1::IntegrationPointsArrayType
HexahedronGaussLegendreIntegrationPoints1::msIntegrationPoints =
{
    {
        IntegrationPointType( 0.00 , 0.00, 0.00 , 8.00 )
    }
};

HexahedronGaussLegendreIntegrationPoints2::IntegrationPointsArrayType
HexahedronGaussLegendreIntegrationPoints2::msIntegrationPoints =
{
    {
        IntegrationPointType( -1.00/std::sqrt(3.0) , -1.00/std::sqrt(3.0), -1.00/std::sqrt(3.0), 1.00 ),
        IntegrationPointType(  1.00/std::sqrt(3.0) , -1.00/std::sqrt(3.0), -1.00/std::sqrt(3.0), 1.00 ),
        IntegrationPointType(  1.00/std::sqrt(3.0) ,  1.00/std::sqrt(3.0), -1.00/std::sqrt(3.0), 1.00 ),
        IntegrationPointType( -1.00/std::sqrt(3.0) ,  1.00/std::sqrt(3.0), -1.00/std::sqrt(3.0), 1.00 ),
        IntegrationPointType( -1.00/std::sqrt(3.0) , -1.00/std::sqrt(3.0),  1.00/std::sqrt(3.0), 1.00 ),
        IntegrationPointType(  1.00/std::sqrt(3.0) , -1.00/std::sqrt(3.0),  1.00/std::sqrt(3.0), 1.00 ),
        IntegrationPointType(  1.00/std::sqrt(3.0) ,  1.00/std::sqrt(3.0),  1.00/std::sqrt(3.0), 1.00 ),
        IntegrationPointType( -1.00/std::sqrt(3.0) ,  1.00/std::sqrt(3.0),  1.00/std::sqrt(3.0), 1.00 )
    }
};

HexahedronGaussLegendreIntegrationPoints3::IntegrationPointsArrayType
HexahedronGaussLegendreIntegrationPoints3::msIntegrationPoints =
{
    {
        IntegrationPointType( -std::sqrt(3.00/5.00) , -std::sqrt(3.00/5.00), -std::sqrt(3.00/5.00), 125.00/729.00 ),
        IntegrationPointType(              0.0 , -std::sqrt(3.00/5.00), -std::sqrt(3.00/5.00), 200.00/729.00 ),
        IntegrationPointType(  std::sqrt(3.00/5.00) , -std::sqrt(3.00/5.00), -std::sqrt(3.00/5.00), 125.00/729.00 ),

        IntegrationPointType( -std::sqrt(3.00/5.00) ,              0.0, -std::sqrt(3.00/5.00), 200.00/729.00 ),
        IntegrationPointType(              0.0 ,              0.0, -std::sqrt(3.00/5.00), 320.00/729.00 ),
        IntegrationPointType(  std::sqrt(3.00/5.00) ,              0.0, -std::sqrt(3.00/5.00), 200.00/729.00 ),

        IntegrationPointType( -std::sqrt(3.00/5.00) ,  std::sqrt(3.00/5.00), -std::sqrt(3.00/5.00), 125.00/729.00 ),
        IntegrationPointType(              0.0 ,  std::sqrt(3.00/5.00), -std::sqrt(3.00/5.00), 200.00/729.00 ),
        IntegrationPointType(  std::sqrt(3.00/5.00) ,  std::sqrt(3.00/5.00), -std::sqrt(3.00/5.00), 125.00/729.00 ),

        IntegrationPointType( -std::sqrt(3.00/5.00) , -std::sqrt(3.00/5.00),              0.0, 200.00/729.00 ),
        IntegrationPointType(              0.0 , -std::sqrt(3.00/5.00),              0.0, 320.00/729.00 ),
        IntegrationPointType(  std::sqrt(3.00/5.00) , -std::sqrt(3.00/5.00),              0.0, 200.00/729.00 ),

        IntegrationPointType( -std::sqrt(3.00/5.00) ,              0.0,              0.0, 320.00/729.00 ),
        IntegrationPointType(              0.0 ,              0.0,              0.0, 512.00/729.00 ),
        IntegrationPointType(  std::sqrt(3.00/5.00) ,              0.0,              0.0, 320.00/729.00 ),

        IntegrationPointType( -std::sqrt(3.00/5.00) ,  std::sqrt(3.00/5.00),              0.0, 200.00/729.00 ),
        IntegrationPointType(              0.0 ,  std::sqrt(3.00/5.00),              0.0, 320.00/729.00 ),
        IntegrationPointType(  std::sqrt(3.00/5.00) ,  std::sqrt(3.00/5.00),              0.0, 200.00/729.00 ),

        IntegrationPointType( -std::sqrt(3.00/5.00) , -std::sqrt(3.00/5.00),  std::sqrt(3.00/5.00), 125.00/729.00 ),
        IntegrationPointType(              0.0 , -std::sqrt(3.00/5.00),  std::sqrt(3.00/5.00), 200.00/729.00 ),
        IntegrationPointType(  std::sqrt(3.00/5.00) , -std::sqrt(3.00/5.00),  std::sqrt(3.00/5.00), 125.00/729.00 ),

        IntegrationPointType( -std::sqrt(3.00/5.00) ,              0.0,  std::sqrt(3.00/5.00), 200.00/729.00 ),
        IntegrationPointType(              0.0 ,              0.0,  std::sqrt(3.00/5.00), 320.00/729.00 ),
        IntegrationPointType(  std::sqrt(3.00/5.00) ,              0.0,  std::sqrt(3.00/5.00), 200.00/729.00 ),

        IntegrationPointType( -std::sqrt(3.00/5.00) ,  std::sqrt(3.00/5.00),  std::sqrt(3.00/5.00), 125.00/729.00 ),
        IntegrationPointType(              0.0 ,  std::sqrt(3.00/5.00),  std::sqrt(3.00/5.00), 200.00/729.00 ),
        IntegrationPointType(  std::sqrt(3.00/5.00) ,  std::sqrt(3.00/5.00),  std::sqrt(3.00/5.00), 125.00/729.00 )
    }
};

HexahedronGaussLegendreIntegrationPoints4::IntegrationPointsArrayType
HexahedronGaussLegendreIntegrationPoints4::msIntegrationPoints =
{
    {
        IntegrationPointType(-0.86113631159405257522 , -0.86113631159405257522 , -0.86113631159405257522 , 0.04209147749053145454),
        IntegrationPointType(-0.33998104358485626480 , -0.86113631159405257522 , -0.86113631159405257522 , 0.07891151579507055098),
        IntegrationPointType(0.33998104358485626480 , -0.86113631159405257522 , -0.86113631159405257522 , 0.07891151579507055098),
        IntegrationPointType(0.86113631159405257522 , -0.86113631159405257522 , -0.86113631159405257522 , 0.04209147749053145454),
        IntegrationPointType(-0.86113631159405257522 , -0.33998104358485626480 , -0.86113631159405257522 , 0.07891151579507055098),
        IntegrationPointType(-0.33998104358485626480 , -0.33998104358485626480 , -0.86113631159405257522 , 0.14794033605678130087),
        IntegrationPointType(0.33998104358485626480 , -0.33998104358485626480 , -0.86113631159405257522 , 0.14794033605678130087),
        IntegrationPointType(0.86113631159405257522 , -0.33998104358485626480 , -0.86113631159405257522 , 0.07891151579507055098),
        IntegrationPointType(-0.86113631159405257522 , 0.33998104358485626480 , -0.86113631159405257522 , 0.07891151579507055098),
        IntegrationPointType(-0.33998104358485626480 , 0.33998104358485626480 , -0.86113631159405257522 , 0.14794033605678130087),
        IntegrationPointType(0.33998104358485626480 , 0.33998104358485626480 , -0.86113631159405257522 , 0.14794033605678130087),
        IntegrationPointType(0.86113631159405257522 , 0.33998104358485626480 , -0.86113631159405257522 , 0.07891151579507055098),
        IntegrationPointType(-0.86113631159405257522 , 0.86113631159405257522 , -0.86113631159405257522 , 0.04209147749053145454),
        IntegrationPointType(-0.33998104358485626480 , 0.86113631159405257522 , -0.86113631159405257522 , 0.07891151579507055098),
        IntegrationPointType(0.33998104358485626480 , 0.86113631159405257522 , -0.86113631159405257522 , 0.07891151579507055098),
        IntegrationPointType(0.86113631159405257522 , 0.86113631159405257522 , -0.86113631159405257522 , 0.04209147749053145454),
        IntegrationPointType(-0.86113631159405257522 , -0.86113631159405257522 , -0.33998104358485626480 , 0.07891151579507055098),
        IntegrationPointType(-0.33998104358485626480 , -0.86113631159405257522 , -0.33998104358485626480 , 0.14794033605678130087),
        IntegrationPointType(0.33998104358485626480 , -0.86113631159405257522 , -0.33998104358485626480 , 0.14794033605678130087),
        IntegrationPointType(0.86113631159405257522 , -0.86113631159405257522 , -0.33998104358485626480 , 0.07891151579507055098),
        IntegrationPointType(-0.86113631159405257522 , -0.33998104358485626480 , -0.33998104358485626480 , 0.14794033605678130087),
        IntegrationPointType(-0.33998104358485626480 , -0.33998104358485626480 , -0.33998104358485626480 , 0.27735296695391298990),
        IntegrationPointType(0.33998104358485626480 , -0.33998104358485626480 , -0.33998104358485626480 , 0.27735296695391298990),
        IntegrationPointType(0.86113631159405257522 , -0.33998104358485626480 , -0.33998104358485626480 , 0.14794033605678130087),
        IntegrationPointType(-0.86113631159405257522 , 0.33998104358485626480 , -0.33998104358485626480 , 0.14794033605678130087),
        IntegrationPointType(-0.33998104358485626480 , 0.33998104358485626480 , -0.33998104358485626480 , 0.27735296695391298990),
        IntegrationPointType(0.33998104358485626480 , 0.33998104358485626480 , -0.33998104358485626480 , 0.27735296695391298990),
        IntegrationPointType(0.86113631159405257522 , 0.33998104358485626480 , -0.33998104358485626480 , 0.14794033605678130087),
        IntegrationPointType(-0.86113631159405257522 , 0.86113631159405257522 , -0.33998104358485626480 , 0.07891151579507055098),
        IntegrationPointType(-0.33998104358485626480 , 0.86113631159405257522 , -0.33998104358485626480 , 0.14794033605678130087),
        IntegrationPointType(0.33998104358485626480 , 0.86113631159405257522 , -0.33998104358485626480 , 0.14794033605678130087),
        IntegrationPointType(0.86113631159405257522 , 0.86113631159405257522 , -0.33998104358485626480 , 0.07891151579507055098),
        IntegrationPointType(-0.86113631159405257522 , -0.86113631159405257522 , 0.33998104358485626480 , 0.07891151579507055098),
        IntegrationPointType(-0.33998104358485626480 , -0.86113631159405257522 , 0.33998104358485626480 , 0.14794033605678130087),
        IntegrationPointType(0.33998104358485626480 , -0.86113631159405257522 , 0.33998104358485626480 , 0.14794033605678130087),
        IntegrationPointType(0.86113631159405257522 , -0.86113631159405257522 , 0.33998104358485626480 , 0.07891151579507055098),
        IntegrationPointType(-0.86113631159405257522 , -0.33998104358485626480 , 0.33998104358485626480 , 0.14794033605678130087),
        IntegrationPointType(-0.33998104358485626480 , -0.33998104358485626480 , 0.33998104358485626480 , 0.27735296695391298990),
        IntegrationPointType(0.33998104358485626480 , -0.33998104358485626480 , 0.33998104358485626480 , 0.27735296695391298990),
        IntegrationPointType(0.86113631159405257522 , -0.33998104358485626480 , 0.33998104358485626480 , 0.14794033605678130087),
        IntegrationPointType(-0.86113631159405257522 , 0.33998104358485626480 , 0.33998104358485626480 , 0.14794033605678130087),
        IntegrationPointType(-0.33998104358485626480 , 0.33998104358485626480 , 0.33998104358485626480 , 0.27735296695391298990),
        IntegrationPointType(0.33998104358485626480 , 0.33998104358485626480 , 0.33998104358485626480 , 0.27735296695391298990),
        IntegrationPointType(0.86113631159405257522 , 0.33998104358485626480 , 0.33998104358485626480 , 0.14794033605678130087),
        IntegrationPointType(-0.86113631159405257522 , 0.86113631159405257522 , 0.33998104358485626480 , 0.07891151579507055098),
        IntegrationPointType(-0.33998104358485626480 , 0.86113631159405257522 , 0.33998104358485626480 , 0.14794033605678130087),
        IntegrationPointType(0.33998104358485626480 , 0.86113631159405257522 , 0.33998104358485626480 , 0.14794033605678130087),
        IntegrationPointType(0.86113631159405257522 , 0.86113631159405257522 , 0.33998104358485626480 , 0.07891151579507055098),
        IntegrationPointType(-0.86113631159405257522 , -0.86113631159405257522 , 0.86113631159405257522 , 0.04209147749053145454),
        IntegrationPointType(-0.33998104358485626480 , -0.86113631159405257522 , 0.86113631159405257522 , 0.07891151579507055098),
        IntegrationPointType(0.33998104358485626480 , -0.86113631159405257522 , 0.86113631159405257522 , 0.07891151579507055098),
        IntegrationPointType(0.86113631159405257522 , -0.86113631159405257522 , 0.86113631159405257522 , 0.04209147749053145454),
        IntegrationPointType(-0.86113631159405257522 , -0.33998104358485626480 , 0.86113631159405257522 , 0.07891151579507055098),
        IntegrationPointType(-0.33998104358485626480 , -0.33998104358485626480 , 0.86113631159405257522 , 0.14794033605678130087),
        IntegrationPointType(0.33998104358485626480 , -0.33998104358485626480 , 0.86113631159405257522 , 0.14794033605678130087),
        IntegrationPointType(0.86113631159405257522 , -0.33998104358485626480 , 0.86113631159405257522 , 0.07891151579507055098),
        IntegrationPointType(-0.86113631159405257522 , 0.33998104358485626480 , 0.86113631159405257522 , 0.07891151579507055098),
        IntegrationPointType(-0.33998104358485626480 , 0.33998104358485626480 , 0.86113631159405257522 , 0.14794033605678130087),
        IntegrationPointType(0.33998104358485626480 , 0.33998104358485626480 , 0.86113631159405257522 , 0.14794033605678130087),
        IntegrationPointType(0.86113631159405257522 , 0.33998104358485626480 , 0.86113631159405257522 , 0.07891151579507055098),
        IntegrationPointType(-0.86113631159405257522 , 0.86113631159405257522 , 0.86113631159405257522 , 0.04209147749053145454),
        IntegrationPointType(-0.33998104358485626480 , 0.86113631159405257522 , 0.86113631159405257522 , 0.07891151579507055098),
        IntegrationPointType(0.33998104358485626480 , 0.86113631159405257522 , 0.86113631159405257522 , 0.07891151579507055098),
        IntegrationPointType(0.86113631159405257522 , 0.86113631159405257522 , 0.86113631159405257522 , 0.04209147749053145454)
    }
};

HexahedronGaussLegendreIntegrationPoints5::IntegrationPointsArrayType
HexahedronGaussLegendreIntegrationPoints5::msIntegrationPoints =
{
    {
        IntegrationPointType(-0.90617984593866399280 , -0.90617984593866399280 , -0.90617984593866399280 , 0.013299736420632648092),
        IntegrationPointType(-0.53846931010568309104 , -0.90617984593866399280 , -0.90617984593866399280 , 0.026867508765371842524),
        IntegrationPointType(0 , -0.90617984593866399280 , -0.90617984593866399280 , 0.031934207352848290676),
        IntegrationPointType(0.53846931010568309104 , -0.90617984593866399280 , -0.90617984593866399280 , 0.026867508765371842524),
        IntegrationPointType(0.90617984593866399280 , -0.90617984593866399280 , -0.90617984593866399280 , 0.013299736420632648092),
        IntegrationPointType(-0.90617984593866399280 , -0.53846931010568309104 , -0.90617984593866399280 , 0.026867508765371842524),
        IntegrationPointType(-0.53846931010568309104 , -0.53846931010568309104 , -0.90617984593866399280 , 0.05427649123462815748),
        IntegrationPointType(0 , -0.53846931010568309104 , -0.90617984593866399280 , 0.06451200000000000000),
        IntegrationPointType(0.53846931010568309104 , -0.53846931010568309104 , -0.90617984593866399280 , 0.05427649123462815748),
        IntegrationPointType(0.90617984593866399280 , -0.53846931010568309104 , -0.90617984593866399280 , 0.026867508765371842524),
        IntegrationPointType(-0.90617984593866399280 , 0 , -0.90617984593866399280 , 0.031934207352848290676),
        IntegrationPointType(-0.53846931010568309104 , 0 , -0.90617984593866399280 , 0.06451200000000000000),
        IntegrationPointType(0 , 0 , -0.90617984593866399280 , 0.07667773006934522489),
        IntegrationPointType(0.53846931010568309104 , 0 , -0.90617984593866399280 , 0.06451200000000000000),
        IntegrationPointType(0.90617984593866399280 , 0 , -0.90617984593866399280 , 0.031934207352848290676),
        IntegrationPointType(-0.90617984593866399280 , 0.53846931010568309104 , -0.90617984593866399280 , 0.026867508765371842524),
        IntegrationPointType(-0.53846931010568309104 , 0.53846931010568309104 , -0.90617984593866399280 , 0.05427649123462815748),
        IntegrationPointType(0 , 0.53846931010568309104 , -0.90617984593866399280 , 0.06451200000000000000),
        IntegrationPointType(0.53846931010568309104 , 0.53846931010568309104 , -0.90617984593866399280 , 0.05427649123462815748),
        IntegrationPointType(0.90617984593866399280 , 0.53846931010568309104 , -0.90617984593866399280 , 0.026867508765371842524),
        IntegrationPointType(-0.90617984593866399280 , 0.90617984593866399280 , -0.90617984593866399280 , 0.013299736420632648092),
        IntegrationPointType(-0.53846931010568309104 , 0.90617984593866399280 , -0.90617984593866399280 , 0.026867508765371842524),
        IntegrationPointType(0 , 0.90617984593866399280 , -0.90617984593866399280 , 0.031934207352848290676),
        IntegrationPointType(0.53846931010568309104 , 0.90617984593866399280 , -0.90617984593866399280 , 0.026867508765371842524),
        IntegrationPointType(0.90617984593866399280 , 0.90617984593866399280 , -0.90617984593866399280 , 0.013299736420632648092),
        IntegrationPointType(-0.90617984593866399280 , -0.90617984593866399280 , -0.53846931010568309104 , 0.026867508765371842524),
        IntegrationPointType(-0.53846931010568309104 , -0.90617984593866399280 , -0.53846931010568309104 , 0.05427649123462815748),
        IntegrationPointType(0 , -0.90617984593866399280 , -0.53846931010568309104 , 0.06451200000000000000),
        IntegrationPointType(0.53846931010568309104 , -0.90617984593866399280 , -0.53846931010568309104 , 0.05427649123462815748),
        IntegrationPointType(0.90617984593866399280 , -0.90617984593866399280 , -0.53846931010568309104 , 0.026867508765371842524),
        IntegrationPointType(-0.90617984593866399280 , -0.53846931010568309104 , -0.53846931010568309104 , 0.05427649123462815748),
        IntegrationPointType(-0.53846931010568309104 , -0.53846931010568309104 , -0.53846931010568309104 , 0.10964684245453881967),
        IntegrationPointType(0 , -0.53846931010568309104 , -0.53846931010568309104 , 0.13032414106964827997),
        IntegrationPointType(0.53846931010568309104 , -0.53846931010568309104 , -0.53846931010568309104 , 0.10964684245453881967),
        IntegrationPointType(0.90617984593866399280 , -0.53846931010568309104 , -0.53846931010568309104 , 0.05427649123462815748),
        IntegrationPointType(-0.90617984593866399280 , 0 , -0.53846931010568309104 , 0.06451200000000000000),
        IntegrationPointType(-0.53846931010568309104 , 0 , -0.53846931010568309104 , 0.13032414106964827997),
        IntegrationPointType(0 , 0 , -0.53846931010568309104 , 0.15490078296220484370),
        IntegrationPointType(0.53846931010568309104 , 0 , -0.53846931010568309104 , 0.13032414106964827997),
        IntegrationPointType(0.90617984593866399280 , 0 , -0.53846931010568309104 , 0.06451200000000000000),
        IntegrationPointType(-0.90617984593866399280 , 0.53846931010568309104 , -0.53846931010568309104 , 0.05427649123462815748),
        IntegrationPointType(-0.53846931010568309104 , 0.53846931010568309104 , -0.53846931010568309104 , 0.10964684245453881967),
        IntegrationPointType(0 , 0.53846931010568309104 , -0.53846931010568309104 , 0.13032414106964827997),
        IntegrationPointType(0.53846931010568309104 , 0.53846931010568309104 , -0.53846931010568309104 , 0.10964684245453881967),
        IntegrationPointType(0.90617984593866399280 , 0.53846931010568309104 , -0.53846931010568309104 , 0.05427649123462815748),
        IntegrationPointType(-0.90617984593866399280 , 0.90617984593866399280 , -0.53846931010568309104 , 0.026867508765371842524),
        IntegrationPointType(-0.53846931010568309104 , 0.90617984593866399280 , -0.53846931010568309104 , 0.05427649123462815748),
        IntegrationPointType(0 , 0.90617984593866399280 , -0.53846931010568309104 , 0.06451200000000000000),
        IntegrationPointType(0.53846931010568309104 , 0.90617984593866399280 , -0.53846931010568309104 , 0.05427649123462815748),
        IntegrationPointType(0.90617984593866399280 , 0.90617984593866399280 , -0.53846931010568309104 , 0.026867508765371842524),
        IntegrationPointType(-0.90617984593866399280 , -0.90617984593866399280 , 0 , 0.031934207352848290676),
        IntegrationPointType(-0.53846931010568309104 , -0.90617984593866399280 , 0 , 0.06451200000000000000),
        IntegrationPointType(0 , -0.90617984593866399280 , 0 , 0.07667773006934522489),
        IntegrationPointType(0.53846931010568309104 , -0.90617984593866399280 , 0 , 0.06451200000000000000),
        IntegrationPointType(0.90617984593866399280 , -0.90617984593866399280 , 0 , 0.031934207352848290676),
        IntegrationPointType(-0.90617984593866399280 , -0.53846931010568309104 , 0 , 0.06451200000000000000),
        IntegrationPointType(-0.53846931010568309104 , -0.53846931010568309104 , 0 , 0.13032414106964827997),
        IntegrationPointType(0 , -0.53846931010568309104 , 0 , 0.15490078296220484370),
        IntegrationPointType(0.53846931010568309104 , -0.53846931010568309104 , 0 , 0.13032414106964827997),
        IntegrationPointType(0.90617984593866399280 , -0.53846931010568309104 , 0 , 0.06451200000000000000),
        IntegrationPointType(-0.90617984593866399280 , 0 , 0 , 0.07667773006934522489),
        IntegrationPointType(-0.53846931010568309104 , 0 , 0 , 0.15490078296220484370),
        IntegrationPointType(0 , 0 , 0 , 0.18411210973936899863),
        IntegrationPointType(0.53846931010568309104 , 0 , 0 , 0.15490078296220484370),
        IntegrationPointType(0.90617984593866399280 , 0 , 0 , 0.07667773006934522489),
        IntegrationPointType(-0.90617984593866399280 , 0.53846931010568309104 , 0 , 0.06451200000000000000),
        IntegrationPointType(-0.53846931010568309104 , 0.53846931010568309104 , 0 , 0.13032414106964827997),
        IntegrationPointType(0 , 0.53846931010568309104 , 0 , 0.15490078296220484370),
        IntegrationPointType(0.53846931010568309104 , 0.53846931010568309104 , 0 , 0.13032414106964827997),
        IntegrationPointType(0.90617984593866399280 , 0.53846931010568309104 , 0 , 0.06451200000000000000),
        IntegrationPointType(-0.90617984593866399280 , 0.90617984593866399280 , 0 , 0.031934207352848290676),
        IntegrationPointType(-0.53846931010568309104 , 0.90617984593866399280 , 0 , 0.06451200000000000000),
        IntegrationPointType(0 , 0.90617984593866399280 , 0 , 0.07667773006934522489),
        IntegrationPointType(0.53846931010568309104 , 0.90617984593866399280 , 0 , 0.06451200000000000000),
        IntegrationPointType(0.90617984593866399280 , 0.90617984593866399280 , 0 , 0.031934207352848290676),
        IntegrationPointType(-0.90617984593866399280 , -0.90617984593866399280 , 0.53846931010568309104 , 0.026867508765371842524),
        IntegrationPointType(-0.53846931010568309104 , -0.90617984593866399280 , 0.53846931010568309104 , 0.05427649123462815748),
        IntegrationPointType(0 , -0.90617984593866399280 , 0.53846931010568309104 , 0.06451200000000000000),
        IntegrationPointType(0.53846931010568309104 , -0.90617984593866399280 , 0.53846931010568309104 , 0.05427649123462815748),
        IntegrationPointType(0.90617984593866399280 , -0.90617984593866399280 , 0.53846931010568309104 , 0.026867508765371842524),
        IntegrationPointType(-0.90617984593866399280 , -0.53846931010568309104 , 0.53846931010568309104 , 0.05427649123462815748),
        IntegrationPointType(-0.53846931010568309104 , -0.53846931010568309104 , 0.53846931010568309104 , 0.10964684245453881967),
        IntegrationPointType(0 , -0.53846931010568309104 , 0.53846931010568309104 , 0.13032414106964827997),
        IntegrationPointType(0.53846931010568309104 , -0.53846931010568309104 , 0.53846931010568309104 , 0.10964684245453881967),
        IntegrationPointType(0.90617984593866399280 , -0.53846931010568309104 , 0.53846931010568309104 , 0.05427649123462815748),
        IntegrationPointType(-0.90617984593866399280 , 0 , 0.53846931010568309104 , 0.06451200000000000000),
        IntegrationPointType(-0.53846931010568309104 , 0 , 0.53846931010568309104 , 0.13032414106964827997),
        IntegrationPointType(0 , 0 , 0.53846931010568309104 , 0.15490078296220484370),
        IntegrationPointType(0.53846931010568309104 , 0 , 0.53846931010568309104 , 0.13032414106964827997),
        IntegrationPointType(0.90617984593866399280 , 0 , 0.53846931010568309104 , 0.06451200000000000000),
        IntegrationPointType(-0.90617984593866399280 , 0.53846931010568309104 , 0.53846931010568309104 , 0.05427649123462815748),
        IntegrationPointType(-0.53846931010568309104 , 0.53846931010568309104 , 0.53846931010568309104 , 0.10964684245453881967),
        IntegrationPointType(0 , 0.53846931010568309104 , 0.53846931010568309104 , 0.13032414106964827997),
        IntegrationPointType(0.53846931010568309104 , 0.53846931010568309104 , 0.53846931010568309104 , 0.10964684245453881967),
        IntegrationPointType(0.90617984593866399280 , 0.53846931010568309104 , 0.53846931010568309104 , 0.05427649123462815748),
        IntegrationPointType(-0.90617984593866399280 , 0.90617984593866399280 , 0.53846931010568309104 , 0.026867508765371842524),
        IntegrationPointType(-0.53846931010568309104 , 0.90617984593866399280 , 0.53846931010568309104 , 0.05427649123462815748),
        IntegrationPointType(0 , 0.90617984593866399280 , 0.53846931010568309104 , 0.06451200000000000000),
        IntegrationPointType(0.53846931010568309104 , 0.90617984593866399280 , 0.53846931010568309104 , 0.05427649123462815748),
        IntegrationPointType(0.90617984593866399280 , 0.90617984593866399280 , 0.53846931010568309104 , 0.026867508765371842524),
        IntegrationPointType(-0.90617984593866399280 , -0.90617984593866399280 , 0.90617984593866399280 , 0.013299736420632648092),
        IntegrationPointType(-0.53846931010568309104 , -0.90617984593866399280 , 0.90617984593866399280 , 0.026867508765371842524),
        IntegrationPointType(0 , -0.90617984593866399280 , 0.90617984593866399280 , 0.031934207352848290676),
        IntegrationPointType(0.53846931010568309104 , -0.90617984593866399280 , 0.90617984593866399280 , 0.026867508765371842524),
        IntegrationPointType(0.90617984593866399280 , -0.90617984593866399280 , 0.90617984593866399280 , 0.013299736420632648092),
        IntegrationPointType(-0.90617984593866399280 , -0.53846931010568309104 , 0.90617984593866399280 , 0.026867508765371842524),
        IntegrationPointType(-0.53846931010568309104 , -0.53846931010568309104 , 0.90617984593866399280 , 0.05427649123462815748),
        IntegrationPointType(0 , -0.53846931010568309104 , 0.90617984593866399280 , 0.06451200000000000000),
        IntegrationPointType(0.53846931010568309104 , -0.53846931010568309104 , 0.90617984593866399280 , 0.05427649123462815748),
        IntegrationPointType(0.90617984593866399280 , -0.53846931010568309104 , 0.90617984593866399280 , 0.026867508765371842524),
        IntegrationPointType(-0.90617984593866399280 , 0 , 0.90617984593866399280 , 0.031934207352848290676),
        IntegrationPointType(-0.53846931010568309104 , 0 , 0.90617984593866399280 , 0.06451200000000000000),
        IntegrationPointType(0 , 0 , 0.90617984593866399280 , 0.07667773006934522489),
        IntegrationPointType(0.53846931010568309104 , 0 , 0.90617984593866399280 , 0.06451200000000000000),
        IntegrationPointType(0.90617984593866399280 , 0 , 0.90617984593866399280 , 0.031934207352848290676),
        IntegrationPointType(-0.90617984593866399280 , 0.53846931010568309104 , 0.90617984593866399280 , 0.026867508765371842524),
        IntegrationPointType(-0.53846931010568309104 , 0.53846931010568309104 , 0.90617984593866399280 , 0.05427649123462815748),
        IntegrationPointType(0 , 0.53846931010568309104 , 0.90617984593866399280 , 0.06451200000000000000),
        IntegrationPointType(0.53846931010568309104 , 0.53846931010568309104 , 0.90617984593866399280 , 0.05427649123462815748),
        IntegrationPointType(0.90617984593866399280 , 0.53846931010568309104 , 0.90617984593866399280 , 0.026867508765371842524),
        IntegrationPointType(-0.90617984593866399280 , 0.90617984593866399280 , 0.90617984593866399280 , 0.013299736420632648092),
        IntegrationPointType(-0.53846931010568309104 , 0.90617984593866399280 , 0.90617984593866399280 , 0.026867508765371842524),
        IntegrationPointType(0 , 0.90617984593866399280 , 0.90617984593866399280 , 0.031934207352848290676),
        IntegrationPointType(0.53846931010568309104 , 0.90617984593866399280 , 0.90617984593866399280 , 0.026867508765371842524),
        IntegrationPointType(0.90617984593866399280 , 0.90617984593866399280 , 0.90617984593866399280 , 0.013299736420632648092)
    }
};

HexahedronGaussLegendreIntegrationPoints6::IntegrationPointsArrayType
HexahedronGaussLegendreIntegrationPoints6::msIntegrationPoints =
{
    {
        IntegrationPointType( -0.9324695142031520278123, -0.9324695142031520278123, -0.9324695142031520278123, 0.1713244923791703450403 * 0.1713244923791703450403 * 0.1713244923791703450403 ),
        IntegrationPointType( -0.9324695142031520278123, -0.9324695142031520278123, -0.6612093864662645136614, 0.1713244923791703450403 * 0.1713244923791703450403 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.9324695142031520278123, -0.9324695142031520278123, -0.2386191860831969086305, 0.1713244923791703450403 * 0.1713244923791703450403 * 0.46791393457269104739 ),
        IntegrationPointType( -0.9324695142031520278123, -0.9324695142031520278123, 0.2386191860831969086305, 0.1713244923791703450403 * 0.1713244923791703450403 * 0.46791393457269104739 ),
        IntegrationPointType( -0.9324695142031520278123, -0.9324695142031520278123, 0.6612093864662645136614, 0.1713244923791703450403 * 0.1713244923791703450403 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.9324695142031520278123, -0.9324695142031520278123, 0.9324695142031520278123, 0.1713244923791703450403 * 0.1713244923791703450403 * 0.1713244923791703450403 ),

        IntegrationPointType( -0.9324695142031520278123, -0.6612093864662645136614, -0.9324695142031520278123, 0.1713244923791703450403 * 0.3607615730481386075698 * 0.1713244923791703450403 ),
        IntegrationPointType( -0.9324695142031520278123, -0.6612093864662645136614, -0.6612093864662645136614, 0.1713244923791703450403 * 0.3607615730481386075698 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.9324695142031520278123, -0.6612093864662645136614, -0.2386191860831969086305, 0.1713244923791703450403 * 0.3607615730481386075698 * 0.46791393457269104739 ),
        IntegrationPointType( -0.9324695142031520278123, -0.6612093864662645136614, 0.2386191860831969086305, 0.1713244923791703450403 * 0.3607615730481386075698 * 0.46791393457269104739 ),
        IntegrationPointType( -0.9324695142031520278123, -0.6612093864662645136614, 0.6612093864662645136614, 0.1713244923791703450403 * 0.3607615730481386075698 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.9324695142031520278123, -0.6612093864662645136614, 0.9324695142031520278123, 0.1713244923791703450403 * 0.3607615730481386075698 * 0.1713244923791703450403 ),

        IntegrationPointType( -0.9324695142031520278123, -0.2386191860831969086305, -0.9324695142031520278123, 0.1713244923791703450403 * 0.46791393457269104739 * 0.1713244923791703450403 ),
        IntegrationPointType( -0.9324695142031520278123, -0.2386191860831969086305, -0.6612093864662645136614, 0.1713244923791703450403 * 0.46791393457269104739 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.9324695142031520278123, -0.2386191860831969086305, -0.2386191860831969086305, 0.1713244923791703450403 * 0.46791393457269104739 * 0.46791393457269104739 ),
        IntegrationPointType( -0.9324695142031520278123, -0.2386191860831969086305, 0.2386191860831969086305, 0.1713244923791703450403 * 0.46791393457269104739 * 0.46791393457269104739 ),
        IntegrationPointType( -0.9324695142031520278123, -0.2386191860831969086305, 0.6612093864662645136614, 0.1713244923791703450403 * 0.46791393457269104739 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.9324695142031520278123, -0.2386191860831969086305, 0.9324695142031520278123, 0.1713244923791703450403 * 0.46791393457269104739 * 0.1713244923791703450403 ),

        IntegrationPointType( -0.9324695142031520278123, 0.2386191860831969086305, -0.9324695142031520278123, 0.1713244923791703450403 * 0.46791393457269104739 * 0.1713244923791703450403 ),
        IntegrationPointType( -0.9324695142031520278123, 0.2386191860831969086305, -0.6612093864662645136614, 0.1713244923791703450403 * 0.46791393457269104739 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.9324695142031520278123, 0.2386191860831969086305, -0.2386191860831969086305, 0.1713244923791703450403 * 0.46791393457269104739 * 0.46791393457269104739 ),
        IntegrationPointType( -0.9324695142031520278123, 0.2386191860831969086305, 0.2386191860831969086305, 0.1713244923791703450403 * 0.46791393457269104739 * 0.46791393457269104739 ),
        IntegrationPointType( -0.9324695142031520278123, 0.2386191860831969086305, 0.6612093864662645136614, 0.1713244923791703450403 * 0.46791393457269104739 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.9324695142031520278123, 0.2386191860831969086305, 0.9324695142031520278123, 0.1713244923791703450403 * 0.46791393457269104739 * 0.1713244923791703450403 ),

        IntegrationPointType( -0.9324695142031520278123, 0.6612093864662645136614, -0.9324695142031520278123, 0.1713244923791703450403 * 0.3607615730481386075698 * 0.1713244923791703450403 ),
        IntegrationPointType( -0.9324695142031520278123, 0.6612093864662645136614, -0.6612093864662645136614, 0.1713244923791703450403 * 0.3607615730481386075698 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.9324695142031520278123, 0.6612093864662645136614, -0.2386191860831969086305, 0.1713244923791703450403 * 0.3607615730481386075698 * 0.46791393457269104739 ),
        IntegrationPointType( -0.9324695142031520278123, 0.6612093864662645136614, 0.2386191860831969086305, 0.1713244923791703450403 * 0.3607615730481386075698 * 0.46791393457269104739 ),
        IntegrationPointType( -0.9324695142031520278123, 0.6612093864662645136614, 0.6612093864662645136614, 0.1713244923791703450403 * 0.3607615730481386075698 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.9324695142031520278123, 0.6612093864662645136614, 0.9324695142031520278123, 0.1713244923791703450403 * 0.3607615730481386075698 * 0.1713244923791703450403 ),

        IntegrationPointType( -0.9324695142031520278123, 0.9324695142031520278123, -0.9324695142031520278123, 0.1713244923791703450403 * 0.1713244923791703450403 * 0.1713244923791703450403 ),
        IntegrationPointType( -0.9324695142031520278123, 0.9324695142031520278123, -0.6612093864662645136614, 0.1713244923791703450403 * 0.1713244923791703450403 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.9324695142031520278123, 0.9324695142031520278123, -0.2386191860831969086305, 0.1713244923791703450403 * 0.1713244923791703450403 * 0.46791393457269104739 ),
        IntegrationPointType( -0.9324695142031520278123, 0.9324695142031520278123, 0.2386191860831969086305, 0.1713244923791703450403 * 0.1713244923791703450403 * 0.46791393457269104739 ),
        IntegrationPointType( -0.9324695142031520278123, 0.9324695142031520278123, 0.6612093864662645136614, 0.1713244923791703450403 * 0.1713244923791703450403 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.9324695142031520278123, 0.9324695142031520278123, 0.9324695142031520278123, 0.1713244923791703450403 * 0.1713244923791703450403 * 0.1713244923791703450403 ),

        IntegrationPointType( -0.6612093864662645136614, -0.9324695142031520278123, -0.9324695142031520278123, 0.3607615730481386075698 * 0.1713244923791703450403 * 0.1713244923791703450403 ),
        IntegrationPointType( -0.6612093864662645136614, -0.9324695142031520278123, -0.6612093864662645136614, 0.3607615730481386075698 * 0.1713244923791703450403 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.6612093864662645136614, -0.9324695142031520278123, -0.2386191860831969086305, 0.3607615730481386075698 * 0.1713244923791703450403 * 0.46791393457269104739 ),
        IntegrationPointType( -0.6612093864662645136614, -0.9324695142031520278123, 0.2386191860831969086305, 0.3607615730481386075698 * 0.1713244923791703450403 * 0.46791393457269104739 ),
        IntegrationPointType( -0.6612093864662645136614, -0.9324695142031520278123, 0.6612093864662645136614, 0.3607615730481386075698 * 0.1713244923791703450403 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.6612093864662645136614, -0.9324695142031520278123, 0.9324695142031520278123, 0.3607615730481386075698 * 0.1713244923791703450403 * 0.1713244923791703450403 ),

        IntegrationPointType( -0.6612093864662645136614, -0.6612093864662645136614, -0.9324695142031520278123, 0.3607615730481386075698 * 0.3607615730481386075698 * 0.1713244923791703450403 ),
        IntegrationPointType( -0.6612093864662645136614, -0.6612093864662645136614, -0.6612093864662645136614, 0.3607615730481386075698 * 0.3607615730481386075698 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.6612093864662645136614, -0.6612093864662645136614, -0.2386191860831969086305, 0.3607615730481386075698 * 0.3607615730481386075698 * 0.46791393457269104739 ),
        IntegrationPointType( -0.6612093864662645136614, -0.6612093864662645136614, 0.2386191860831969086305, 0.3607615730481386075698 * 0.3607615730481386075698 * 0.46791393457269104739 ),
        IntegrationPointType( -0.6612093864662645136614, -0.6612093864662645136614, 0.6612093864662645136614, 0.3607615730481386075698 * 0.3607615730481386075698 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.6612093864662645136614, -0.6612093864662645136614, 0.9324695142031520278123, 0.3607615730481386075698 * 0.3607615730481386075698 * 0.1713244923791703450403 ),

        IntegrationPointType( -0.6612093864662645136614, -0.2386191860831969086305, -0.9324695142031520278123, 0.3607615730481386075698 * 0.46791393457269104739 * 0.1713244923791703450403 ),
        IntegrationPointType( -0.6612093864662645136614, -0.2386191860831969086305, -0.6612093864662645136614, 0.3607615730481386075698 * 0.46791393457269104739 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.6612093864662645136614, -0.2386191860831969086305, -0.2386191860831969086305, 0.3607615730481386075698 * 0.46791393457269104739 * 0.46791393457269104739 ),
        IntegrationPointType( -0.6612093864662645136614, -0.2386191860831969086305, 0.2386191860831969086305, 0.3607615730481386075698 * 0.46791393457269104739 * 0.46791393457269104739 ),
        IntegrationPointType( -0.6612093864662645136614, -0.2386191860831969086305, 0.6612093864662645136614, 0.3607615730481386075698 * 0.46791393457269104739 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.6612093864662645136614, -0.2386191860831969086305, 0.9324695142031520278123, 0.3607615730481386075698 * 0.46791393457269104739 * 0.1713244923791703450403 ),

        IntegrationPointType( -0.6612093864662645136614, 0.2386191860831969086305, -0.9324695142031520278123, 0.3607615730481386075698 * 0.46791393457269104739 * 0.1713244923791703450403 ),
        IntegrationPointType( -0.6612093864662645136614, 0.2386191860831969086305, -0.6612093864662645136614, 0.3607615730481386075698 * 0.46791393457269104739 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.6612093864662645136614, 0.2386191860831969086305, -0.2386191860831969086305, 0.3607615730481386075698 * 0.46791393457269104739 * 0.46791393457269104739 ),
        IntegrationPointType( -0.6612093864662645136614, 0.2386191860831969086305, 0.2386191860831969086305, 0.3607615730481386075698 * 0.46791393457269104739 * 0.46791393457269104739 ),
        IntegrationPointType( -0.6612093864662645136614, 0.2386191860831969086305, 0.6612093864662645136614, 0.3607615730481386075698 * 0.46791393457269104739 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.6612093864662645136614, 0.2386191860831969086305, 0.9324695142031520278123, 0.3607615730481386075698 * 0.46791393457269104739 * 0.1713244923791703450403 ),

        IntegrationPointType( -0.6612093864662645136614, 0.6612093864662645136614, -0.9324695142031520278123, 0.3607615730481386075698 * 0.3607615730481386075698 * 0.1713244923791703450403 ),
        IntegrationPointType( -0.6612093864662645136614, 0.6612093864662645136614, -0.6612093864662645136614, 0.3607615730481386075698 * 0.3607615730481386075698 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.6612093864662645136614, 0.6612093864662645136614, -0.2386191860831969086305, 0.3607615730481386075698 * 0.3607615730481386075698 * 0.46791393457269104739 ),
        IntegrationPointType( -0.6612093864662645136614, 0.6612093864662645136614, 0.2386191860831969086305, 0.3607615730481386075698 * 0.3607615730481386075698 * 0.46791393457269104739 ),
        IntegrationPointType( -0.6612093864662645136614, 0.6612093864662645136614, 0.6612093864662645136614, 0.3607615730481386075698 * 0.3607615730481386075698 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.6612093864662645136614, 0.6612093864662645136614, 0.9324695142031520278123, 0.3607615730481386075698 * 0.3607615730481386075698 * 0.1713244923791703450403 ),

        IntegrationPointType( -0.6612093864662645136614, 0.9324695142031520278123, -0.9324695142031520278123, 0.3607615730481386075698 * 0.1713244923791703450403 * 0.1713244923791703450403 ),
        IntegrationPointType( -0.6612093864662645136614, 0.9324695142031520278123, -0.6612093864662645136614, 0.3607615730481386075698 * 0.1713244923791703450403 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.6612093864662645136614, 0.9324695142031520278123, -0.2386191860831969086305, 0.3607615730481386075698 * 0.1713244923791703450403 * 0.46791393457269104739 ),
        IntegrationPointType( -0.6612093864662645136614, 0.9324695142031520278123, 0.2386191860831969086305, 0.3607615730481386075698 * 0.1713244923791703450403 * 0.46791393457269104739 ),
        IntegrationPointType( -0.6612093864662645136614, 0.9324695142031520278123, 0.6612093864662645136614, 0.3607615730481386075698 * 0.1713244923791703450403 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.6612093864662645136614, 0.9324695142031520278123, 0.9324695142031520278123, 0.3607615730481386075698 * 0.1713244923791703450403 * 0.1713244923791703450403 ),

        IntegrationPointType( -0.2386191860831969086305, -0.9324695142031520278123, -0.9324695142031520278123, 0.46791393457269104739 * 0.1713244923791703450403 * 0.1713244923791703450403 ),
        IntegrationPointType( -0.2386191860831969086305, -0.9324695142031520278123, -0.6612093864662645136614, 0.46791393457269104739 * 0.1713244923791703450403 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.2386191860831969086305, -0.9324695142031520278123, -0.2386191860831969086305, 0.46791393457269104739 * 0.1713244923791703450403 * 0.46791393457269104739 ),
        IntegrationPointType( -0.2386191860831969086305, -0.9324695142031520278123, 0.2386191860831969086305, 0.46791393457269104739 * 0.1713244923791703450403 * 0.46791393457269104739 ),
        IntegrationPointType( -0.2386191860831969086305, -0.9324695142031520278123, 0.6612093864662645136614, 0.46791393457269104739 * 0.1713244923791703450403 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.2386191860831969086305, -0.9324695142031520278123, 0.9324695142031520278123, 0.46791393457269104739 * 0.1713244923791703450403 * 0.1713244923791703450403 ),

        IntegrationPointType( -0.2386191860831969086305, -0.6612093864662645136614, -0.9324695142031520278123, 0.46791393457269104739 * 0.3607615730481386075698 * 0.1713244923791703450403 ),
        IntegrationPointType( -0.2386191860831969086305, -0.6612093864662645136614, -0.6612093864662645136614, 0.46791393457269104739 * 0.3607615730481386075698 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.2386191860831969086305, -0.6612093864662645136614, -0.2386191860831969086305, 0.46791393457269104739 * 0.3607615730481386075698 * 0.46791393457269104739 ),
        IntegrationPointType( -0.2386191860831969086305, -0.6612093864662645136614, 0.2386191860831969086305, 0.46791393457269104739 * 0.3607615730481386075698 * 0.46791393457269104739 ),
        IntegrationPointType( -0.2386191860831969086305, -0.6612093864662645136614, 0.6612093864662645136614, 0.46791393457269104739 * 0.3607615730481386075698 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.2386191860831969086305, -0.6612093864662645136614, 0.9324695142031520278123, 0.46791393457269104739 * 0.3607615730481386075698 * 0.1713244923791703450403 ),

        IntegrationPointType( -0.2386191860831969086305, -0.2386191860831969086305, -0.9324695142031520278123, 0.46791393457269104739 * 0.46791393457269104739 * 0.1713244923791703450403 ),
        IntegrationPointType( -0.2386191860831969086305, -0.2386191860831969086305, -0.6612093864662645136614, 0.46791393457269104739 * 0.46791393457269104739 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.2386191860831969086305, -0.2386191860831969086305, -0.2386191860831969086305, 0.46791393457269104739 * 0.46791393457269104739 * 0.46791393457269104739 ),
        IntegrationPointType( -0.2386191860831969086305, -0.2386191860831969086305, 0.2386191860831969086305, 0.46791393457269104739 * 0.46791393457269104739 * 0.46791393457269104739 ),
        IntegrationPointType( -0.2386191860831969086305, -0.2386191860831969086305, 0.6612093864662645136614, 0.46791393457269104739 * 0.46791393457269104739 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.2386191860831969086305, -0.2386191860831969086305, 0.9324695142031520278123, 0.46791393457269104739 * 0.46791393457269104739 * 0.1713244923791703450403 ),

        IntegrationPointType( -0.2386191860831969086305, 0.2386191860831969086305, -0.9324695142031520278123, 0.46791393457269104739 * 0.46791393457269104739 * 0.1713244923791703450403 ),
        IntegrationPointType( -0.2386191860831969086305, 0.2386191860831969086305, -0.6612093864662645136614, 0.46791393457269104739 * 0.46791393457269104739 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.2386191860831969086305, 0.2386191860831969086305, -0.2386191860831969086305, 0.46791393457269104739 * 0.46791393457269104739 * 0.46791393457269104739 ),
        IntegrationPointType( -0.2386191860831969086305, 0.2386191860831969086305, 0.2386191860831969086305, 0.46791393457269104739 * 0.46791393457269104739 * 0.46791393457269104739 ),
        IntegrationPointType( -0.2386191860831969086305, 0.2386191860831969086305, 0.6612093864662645136614, 0.46791393457269104739 * 0.46791393457269104739 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.2386191860831969086305, 0.2386191860831969086305, 0.9324695142031520278123, 0.46791393457269104739 * 0.46791393457269104739 * 0.1713244923791703450403 ),

        IntegrationPointType( -0.2386191860831969086305, 0.6612093864662645136614, -0.9324695142031520278123, 0.46791393457269104739 * 0.3607615730481386075698 * 0.1713244923791703450403 ),
        IntegrationPointType( -0.2386191860831969086305, 0.6612093864662645136614, -0.6612093864662645136614, 0.46791393457269104739 * 0.3607615730481386075698 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.2386191860831969086305, 0.6612093864662645136614, -0.2386191860831969086305, 0.46791393457269104739 * 0.3607615730481386075698 * 0.46791393457269104739 ),
        IntegrationPointType( -0.2386191860831969086305, 0.6612093864662645136614, 0.2386191860831969086305, 0.46791393457269104739 * 0.3607615730481386075698 * 0.46791393457269104739 ),
        IntegrationPointType( -0.2386191860831969086305, 0.6612093864662645136614, 0.6612093864662645136614, 0.46791393457269104739 * 0.3607615730481386075698 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.2386191860831969086305, 0.6612093864662645136614, 0.9324695142031520278123, 0.46791393457269104739 * 0.3607615730481386075698 * 0.1713244923791703450403 ),

        IntegrationPointType( -0.2386191860831969086305, 0.9324695142031520278123, -0.9324695142031520278123, 0.46791393457269104739 * 0.1713244923791703450403 * 0.1713244923791703450403 ),
        IntegrationPointType( -0.2386191860831969086305, 0.9324695142031520278123, -0.6612093864662645136614, 0.46791393457269104739 * 0.1713244923791703450403 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.2386191860831969086305, 0.9324695142031520278123, -0.2386191860831969086305, 0.46791393457269104739 * 0.1713244923791703450403 * 0.46791393457269104739 ),
        IntegrationPointType( -0.2386191860831969086305, 0.9324695142031520278123, 0.2386191860831969086305, 0.46791393457269104739 * 0.1713244923791703450403 * 0.46791393457269104739 ),
        IntegrationPointType( -0.2386191860831969086305, 0.9324695142031520278123, 0.6612093864662645136614, 0.46791393457269104739 * 0.1713244923791703450403 * 0.3607615730481386075698 ),
        IntegrationPointType( -0.2386191860831969086305, 0.9324695142031520278123, 0.9324695142031520278123, 0.46791393457269104739 * 0.1713244923791703450403 * 0.1713244923791703450403 ),

        IntegrationPointType( 0.2386191860831969086305, -0.9324695142031520278123, -0.9324695142031520278123, 0.46791393457269104739 * 0.1713244923791703450403 * 0.1713244923791703450403 ),
        IntegrationPointType( 0.2386191860831969086305, -0.9324695142031520278123, -0.6612093864662645136614, 0.46791393457269104739 * 0.1713244923791703450403 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.2386191860831969086305, -0.9324695142031520278123, -0.2386191860831969086305, 0.46791393457269104739 * 0.1713244923791703450403 * 0.46791393457269104739 ),
        IntegrationPointType( 0.2386191860831969086305, -0.9324695142031520278123, 0.2386191860831969086305, 0.46791393457269104739 * 0.1713244923791703450403 * 0.46791393457269104739 ),
        IntegrationPointType( 0.2386191860831969086305, -0.9324695142031520278123, 0.6612093864662645136614, 0.46791393457269104739 * 0.1713244923791703450403 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.2386191860831969086305, -0.9324695142031520278123, 0.9324695142031520278123, 0.46791393457269104739 * 0.1713244923791703450403 * 0.1713244923791703450403 ),

        IntegrationPointType( 0.2386191860831969086305, -0.6612093864662645136614, -0.9324695142031520278123, 0.46791393457269104739 * 0.3607615730481386075698 * 0.1713244923791703450403 ),
        IntegrationPointType( 0.2386191860831969086305, -0.6612093864662645136614, -0.6612093864662645136614, 0.46791393457269104739 * 0.3607615730481386075698 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.2386191860831969086305, -0.6612093864662645136614, -0.2386191860831969086305, 0.46791393457269104739 * 0.3607615730481386075698 * 0.46791393457269104739 ),
        IntegrationPointType( 0.2386191860831969086305, -0.6612093864662645136614, 0.2386191860831969086305, 0.46791393457269104739 * 0.3607615730481386075698 * 0.46791393457269104739 ),
        IntegrationPointType( 0.2386191860831969086305, -0.6612093864662645136614, 0.6612093864662645136614, 0.46791393457269104739 * 0.3607615730481386075698 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.2386191860831969086305, -0.6612093864662645136614, 0.9324695142031520278123, 0.46791393457269104739 * 0.3607615730481386075698 * 0.1713244923791703450403 ),

        IntegrationPointType( 0.2386191860831969086305, -0.2386191860831969086305, -0.9324695142031520278123, 0.46791393457269104739 * 0.46791393457269104739 * 0.1713244923791703450403 ),
        IntegrationPointType( 0.2386191860831969086305, -0.2386191860831969086305, -0.6612093864662645136614, 0.46791393457269104739 * 0.46791393457269104739 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.2386191860831969086305, -0.2386191860831969086305, -0.2386191860831969086305, 0.46791393457269104739 * 0.46791393457269104739 * 0.46791393457269104739 ),
        IntegrationPointType( 0.2386191860831969086305, -0.2386191860831969086305, 0.2386191860831969086305, 0.46791393457269104739 * 0.46791393457269104739 * 0.46791393457269104739 ),
        IntegrationPointType( 0.2386191860831969086305, -0.2386191860831969086305, 0.6612093864662645136614, 0.46791393457269104739 * 0.46791393457269104739 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.2386191860831969086305, -0.2386191860831969086305, 0.9324695142031520278123, 0.46791393457269104739 * 0.46791393457269104739 * 0.1713244923791703450403 ),

        IntegrationPointType( 0.2386191860831969086305, 0.2386191860831969086305, -0.9324695142031520278123, 0.46791393457269104739 * 0.46791393457269104739 * 0.1713244923791703450403 ),
        IntegrationPointType( 0.2386191860831969086305, 0.2386191860831969086305, -0.6612093864662645136614, 0.46791393457269104739 * 0.46791393457269104739 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.2386191860831969086305, 0.2386191860831969086305, -0.2386191860831969086305, 0.46791393457269104739 * 0.46791393457269104739 * 0.46791393457269104739 ),
        IntegrationPointType( 0.2386191860831969086305, 0.2386191860831969086305, 0.2386191860831969086305, 0.46791393457269104739 * 0.46791393457269104739 * 0.46791393457269104739 ),
        IntegrationPointType( 0.2386191860831969086305, 0.2386191860831969086305, 0.6612093864662645136614, 0.46791393457269104739 * 0.46791393457269104739 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.2386191860831969086305, 0.2386191860831969086305, 0.9324695142031520278123, 0.46791393457269104739 * 0.46791393457269104739 * 0.1713244923791703450403 ),

        IntegrationPointType( 0.2386191860831969086305, 0.6612093864662645136614, -0.9324695142031520278123, 0.46791393457269104739 * 0.3607615730481386075698 * 0.1713244923791703450403 ),
        IntegrationPointType( 0.2386191860831969086305, 0.6612093864662645136614, -0.6612093864662645136614, 0.46791393457269104739 * 0.3607615730481386075698 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.2386191860831969086305, 0.6612093864662645136614, -0.2386191860831969086305, 0.46791393457269104739 * 0.3607615730481386075698 * 0.46791393457269104739 ),
        IntegrationPointType( 0.2386191860831969086305, 0.6612093864662645136614, 0.2386191860831969086305, 0.46791393457269104739 * 0.3607615730481386075698 * 0.46791393457269104739 ),
        IntegrationPointType( 0.2386191860831969086305, 0.6612093864662645136614, 0.6612093864662645136614, 0.46791393457269104739 * 0.3607615730481386075698 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.2386191860831969086305, 0.6612093864662645136614, 0.9324695142031520278123, 0.46791393457269104739 * 0.3607615730481386075698 * 0.1713244923791703450403 ),

        IntegrationPointType( 0.2386191860831969086305, 0.9324695142031520278123, -0.9324695142031520278123, 0.46791393457269104739 * 0.1713244923791703450403 * 0.1713244923791703450403 ),
        IntegrationPointType( 0.2386191860831969086305, 0.9324695142031520278123, -0.6612093864662645136614, 0.46791393457269104739 * 0.1713244923791703450403 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.2386191860831969086305, 0.9324695142031520278123, -0.2386191860831969086305, 0.46791393457269104739 * 0.1713244923791703450403 * 0.46791393457269104739 ),
        IntegrationPointType( 0.2386191860831969086305, 0.9324695142031520278123, 0.2386191860831969086305, 0.46791393457269104739 * 0.1713244923791703450403 * 0.46791393457269104739 ),
        IntegrationPointType( 0.2386191860831969086305, 0.9324695142031520278123, 0.6612093864662645136614, 0.46791393457269104739 * 0.1713244923791703450403 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.2386191860831969086305, 0.9324695142031520278123, 0.9324695142031520278123, 0.46791393457269104739 * 0.1713244923791703450403 * 0.1713244923791703450403 ),

        IntegrationPointType( 0.6612093864662645136614, -0.9324695142031520278123, -0.9324695142031520278123, 0.3607615730481386075698 * 0.1713244923791703450403 * 0.1713244923791703450403 ),
        IntegrationPointType( 0.6612093864662645136614, -0.9324695142031520278123, -0.6612093864662645136614, 0.3607615730481386075698 * 0.1713244923791703450403 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.6612093864662645136614, -0.9324695142031520278123, -0.2386191860831969086305, 0.3607615730481386075698 * 0.1713244923791703450403 * 0.46791393457269104739 ),
        IntegrationPointType( 0.6612093864662645136614, -0.9324695142031520278123, 0.2386191860831969086305, 0.3607615730481386075698 * 0.1713244923791703450403 * 0.46791393457269104739 ),
        IntegrationPointType( 0.6612093864662645136614, -0.9324695142031520278123, 0.6612093864662645136614, 0.3607615730481386075698 * 0.1713244923791703450403 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.6612093864662645136614, -0.9324695142031520278123, 0.9324695142031520278123, 0.3607615730481386075698 * 0.1713244923791703450403 * 0.1713244923791703450403 ),

        IntegrationPointType( 0.6612093864662645136614, -0.6612093864662645136614, -0.9324695142031520278123, 0.3607615730481386075698 * 0.3607615730481386075698 * 0.1713244923791703450403 ),
        IntegrationPointType( 0.6612093864662645136614, -0.6612093864662645136614, -0.6612093864662645136614, 0.3607615730481386075698 * 0.3607615730481386075698 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.6612093864662645136614, -0.6612093864662645136614, -0.2386191860831969086305, 0.3607615730481386075698 * 0.3607615730481386075698 * 0.46791393457269104739 ),
        IntegrationPointType( 0.6612093864662645136614, -0.6612093864662645136614, 0.2386191860831969086305, 0.3607615730481386075698 * 0.3607615730481386075698 * 0.46791393457269104739 ),
        IntegrationPointType( 0.6612093864662645136614, -0.6612093864662645136614, 0.6612093864662645136614, 0.3607615730481386075698 * 0.3607615730481386075698 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.6612093864662645136614, -0.6612093864662645136614, 0.9324695142031520278123, 0.3607615730481386075698 * 0.3607615730481386075698 * 0.1713244923791703450403 ),

        IntegrationPointType( 0.6612093864662645136614, -0.2386191860831969086305, -0.9324695142031520278123, 0.3607615730481386075698 * 0.46791393457269104739 * 0.1713244923791703450403 ),
        IntegrationPointType( 0.6612093864662645136614, -0.2386191860831969086305, -0.6612093864662645136614, 0.3607615730481386075698 * 0.46791393457269104739 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.6612093864662645136614, -0.2386191860831969086305, -0.2386191860831969086305, 0.3607615730481386075698 * 0.46791393457269104739 * 0.46791393457269104739 ),
        IntegrationPointType( 0.6612093864662645136614, -0.2386191860831969086305, 0.2386191860831969086305, 0.3607615730481386075698 * 0.46791393457269104739 * 0.46791393457269104739 ),
        IntegrationPointType( 0.6612093864662645136614, -0.2386191860831969086305, 0.6612093864662645136614, 0.3607615730481386075698 * 0.46791393457269104739 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.6612093864662645136614, -0.2386191860831969086305, 0.9324695142031520278123, 0.3607615730481386075698 * 0.46791393457269104739 * 0.1713244923791703450403 ),

        IntegrationPointType( 0.6612093864662645136614, 0.2386191860831969086305, -0.9324695142031520278123, 0.3607615730481386075698 * 0.46791393457269104739 * 0.1713244923791703450403 ),
        IntegrationPointType( 0.6612093864662645136614, 0.2386191860831969086305, -0.6612093864662645136614, 0.3607615730481386075698 * 0.46791393457269104739 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.6612093864662645136614, 0.2386191860831969086305, -0.2386191860831969086305, 0.3607615730481386075698 * 0.46791393457269104739 * 0.46791393457269104739 ),
        IntegrationPointType( 0.6612093864662645136614, 0.2386191860831969086305, 0.2386191860831969086305, 0.3607615730481386075698 * 0.46791393457269104739 * 0.46791393457269104739 ),
        IntegrationPointType( 0.6612093864662645136614, 0.2386191860831969086305, 0.6612093864662645136614, 0.3607615730481386075698 * 0.46791393457269104739 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.6612093864662645136614, 0.2386191860831969086305, 0.9324695142031520278123, 0.3607615730481386075698 * 0.46791393457269104739 * 0.1713244923791703450403 ),

        IntegrationPointType( 0.6612093864662645136614, 0.6612093864662645136614, -0.9324695142031520278123, 0.3607615730481386075698 * 0.3607615730481386075698 * 0.1713244923791703450403 ),
        IntegrationPointType( 0.6612093864662645136614, 0.6612093864662645136614, -0.6612093864662645136614, 0.3607615730481386075698 * 0.3607615730481386075698 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.6612093864662645136614, 0.6612093864662645136614, -0.2386191860831969086305, 0.3607615730481386075698 * 0.3607615730481386075698 * 0.46791393457269104739 ),
        IntegrationPointType( 0.6612093864662645136614, 0.6612093864662645136614, 0.2386191860831969086305, 0.3607615730481386075698 * 0.3607615730481386075698 * 0.46791393457269104739 ),
        IntegrationPointType( 0.6612093864662645136614, 0.6612093864662645136614, 0.6612093864662645136614, 0.3607615730481386075698 * 0.3607615730481386075698 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.6612093864662645136614, 0.6612093864662645136614, 0.9324695142031520278123, 0.3607615730481386075698 * 0.3607615730481386075698 * 0.1713244923791703450403 ),

        IntegrationPointType( 0.6612093864662645136614, 0.9324695142031520278123, -0.9324695142031520278123, 0.3607615730481386075698 * 0.1713244923791703450403 * 0.1713244923791703450403 ),
        IntegrationPointType( 0.6612093864662645136614, 0.9324695142031520278123, -0.6612093864662645136614, 0.3607615730481386075698 * 0.1713244923791703450403 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.6612093864662645136614, 0.9324695142031520278123, -0.2386191860831969086305, 0.3607615730481386075698 * 0.1713244923791703450403 * 0.46791393457269104739 ),
        IntegrationPointType( 0.6612093864662645136614, 0.9324695142031520278123, 0.2386191860831969086305, 0.3607615730481386075698 * 0.1713244923791703450403 * 0.46791393457269104739 ),
        IntegrationPointType( 0.6612093864662645136614, 0.9324695142031520278123, 0.6612093864662645136614, 0.3607615730481386075698 * 0.1713244923791703450403 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.6612093864662645136614, 0.9324695142031520278123, 0.9324695142031520278123, 0.3607615730481386075698 * 0.1713244923791703450403 * 0.1713244923791703450403 ),

        IntegrationPointType( 0.9324695142031520278123, -0.9324695142031520278123, -0.9324695142031520278123, 0.1713244923791703450403 * 0.1713244923791703450403 * 0.1713244923791703450403 ),
        IntegrationPointType( 0.9324695142031520278123, -0.9324695142031520278123, -0.6612093864662645136614, 0.1713244923791703450403 * 0.1713244923791703450403 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.9324695142031520278123, -0.9324695142031520278123, -0.2386191860831969086305, 0.1713244923791703450403 * 0.1713244923791703450403 * 0.46791393457269104739 ),
        IntegrationPointType( 0.9324695142031520278123, -0.9324695142031520278123, 0.2386191860831969086305, 0.1713244923791703450403 * 0.1713244923791703450403 * 0.46791393457269104739 ),
        IntegrationPointType( 0.9324695142031520278123, -0.9324695142031520278123, 0.6612093864662645136614, 0.1713244923791703450403 * 0.1713244923791703450403 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.9324695142031520278123, -0.9324695142031520278123, 0.9324695142031520278123, 0.1713244923791703450403 * 0.1713244923791703450403 * 0.1713244923791703450403 ),

        IntegrationPointType( 0.9324695142031520278123, -0.6612093864662645136614, -0.9324695142031520278123, 0.1713244923791703450403 * 0.3607615730481386075698 * 0.1713244923791703450403 ),
        IntegrationPointType( 0.9324695142031520278123, -0.6612093864662645136614, -0.6612093864662645136614, 0.1713244923791703450403 * 0.3607615730481386075698 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.9324695142031520278123, -0.6612093864662645136614, -0.2386191860831969086305, 0.1713244923791703450403 * 0.3607615730481386075698 * 0.46791393457269104739 ),
        IntegrationPointType( 0.9324695142031520278123, -0.6612093864662645136614, 0.2386191860831969086305, 0.1713244923791703450403 * 0.3607615730481386075698 * 0.46791393457269104739 ),
        IntegrationPointType( 0.9324695142031520278123, -0.6612093864662645136614, 0.6612093864662645136614, 0.1713244923791703450403 * 0.3607615730481386075698 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.9324695142031520278123, -0.6612093864662645136614, 0.9324695142031520278123, 0.1713244923791703450403 * 0.3607615730481386075698 * 0.1713244923791703450403 ),

        IntegrationPointType( 0.9324695142031520278123, -0.2386191860831969086305, -0.9324695142031520278123, 0.1713244923791703450403 * 0.46791393457269104739 * 0.1713244923791703450403 ),
        IntegrationPointType( 0.9324695142031520278123, -0.2386191860831969086305, -0.6612093864662645136614, 0.1713244923791703450403 * 0.46791393457269104739 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.9324695142031520278123, -0.2386191860831969086305, -0.2386191860831969086305, 0.1713244923791703450403 * 0.46791393457269104739 * 0.46791393457269104739 ),
        IntegrationPointType( 0.9324695142031520278123, -0.2386191860831969086305, 0.2386191860831969086305, 0.1713244923791703450403 * 0.46791393457269104739 * 0.46791393457269104739 ),
        IntegrationPointType( 0.9324695142031520278123, -0.2386191860831969086305, 0.6612093864662645136614, 0.1713244923791703450403 * 0.46791393457269104739 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.9324695142031520278123, -0.2386191860831969086305, 0.9324695142031520278123, 0.1713244923791703450403 * 0.46791393457269104739 * 0.1713244923791703450403 ),

        IntegrationPointType( 0.9324695142031520278123, 0.2386191860831969086305, -0.9324695142031520278123, 0.1713244923791703450403 * 0.46791393457269104739 * 0.1713244923791703450403 ),
        IntegrationPointType( 0.9324695142031520278123, 0.2386191860831969086305, -0.6612093864662645136614, 0.1713244923791703450403 * 0.46791393457269104739 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.9324695142031520278123, 0.2386191860831969086305, -0.2386191860831969086305, 0.1713244923791703450403 * 0.46791393457269104739 * 0.46791393457269104739 ),
        IntegrationPointType( 0.9324695142031520278123, 0.2386191860831969086305, 0.2386191860831969086305, 0.1713244923791703450403 * 0.46791393457269104739 * 0.46791393457269104739 ),
        IntegrationPointType( 0.9324695142031520278123, 0.2386191860831969086305, 0.6612093864662645136614, 0.1713244923791703450403 * 0.46791393457269104739 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.9324695142031520278123, 0.2386191860831969086305, 0.9324695142031520278123, 0.1713244923791703450403 * 0.46791393457269104739 * 0.1713244923791703450403 ),

        IntegrationPointType( 0.9324695142031520278123, 0.6612093864662645136614, -0.9324695142031520278123, 0.1713244923791703450403 * 0.3607615730481386075698 * 0.1713244923791703450403 ),
        IntegrationPointType( 0.9324695142031520278123, 0.6612093864662645136614, -0.6612093864662645136614, 0.1713244923791703450403 * 0.3607615730481386075698 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.9324695142031520278123, 0.6612093864662645136614, -0.2386191860831969086305, 0.1713244923791703450403 * 0.3607615730481386075698 * 0.46791393457269104739 ),
        IntegrationPointType( 0.9324695142031520278123, 0.6612093864662645136614, 0.2386191860831969086305, 0.1713244923791703450403 * 0.3607615730481386075698 * 0.46791393457269104739 ),
        IntegrationPointType( 0.9324695142031520278123, 0.6612093864662645136614, 0.6612093864662645136614, 0.1713244923791703450403 * 0.3607615730481386075698 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.9324695142031520278123, 0.6612093864662645136614, 0.9324695142031520278123, 0.1713244923791703450403 * 0.3607615730481386075698 * 0.1713244923791703450403 ),

        IntegrationPointType( 0.9324695142031520278123, 0.9324695142031520278123, -0.9324695142031520278123, 0.1713244923791703450403 * 0.1713244923791703450403 * 0.1713244923791703450403 ),
        IntegrationPointType( 0.9324695142031520278123, 0.9324695142031520278123, -0.6612093864662645136614, 0.1713244923791703450403 * 0.1713244923791703450403 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.9324695142031520278123, 0.9324695142031520278123, -0.2386191860831969086305, 0.1713244923791703450403 * 0.1713244923791703450403 * 0.46791393457269104739 ),
        IntegrationPointType( 0.9324695142031520278123, 0.9324695142031520278123, 0.2386191860831969086305, 0.1713244923791703450403 * 0.1713244923791703450403 * 0.46791393457269104739 ),
        IntegrationPointType( 0.9324695142031520278123, 0.9324695142031520278123, 0.6612093864662645136614, 0.1713244923791703450403 * 0.1713244923791703450403 * 0.3607615730481386075698 ),
        IntegrationPointType( 0.9324695142031520278123, 0.9324695142031520278123, 0.9324695142031520278123, 0.1713244923791703450403 * 0.1713244923791703450403 * 0.1713244923791703450403 ),

    }
};

HexahedronGaussLegendreIntegrationPoints7::IntegrationPointsArrayType
HexahedronGaussLegendreIntegrationPoints7::msIntegrationPoints =
{
    {
        IntegrationPointType( -0.9491079123427585245262, -0.9491079123427585245262, -0.9491079123427585245262, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.1294849661688696932706 ),
        IntegrationPointType( -0.9491079123427585245262, -0.9491079123427585245262, -0.7415311855993944398639, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.9491079123427585245262, -0.9491079123427585245262, -0.4058451513773971669066, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( -0.9491079123427585245262, -0.9491079123427585245262, 0, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.4179591836734693877551 ),
        IntegrationPointType( -0.9491079123427585245262, -0.9491079123427585245262, 0.4058451513773971669066, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( -0.9491079123427585245262, -0.9491079123427585245262, 0.7415311855993944398639, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.9491079123427585245262, -0.9491079123427585245262, 0.9491079123427585245262, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.1294849661688696932706 ),

        IntegrationPointType( -0.9491079123427585245262, -0.7415311855993944398639, -0.9491079123427585245262, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.1294849661688696932706 ),
        IntegrationPointType( -0.9491079123427585245262, -0.7415311855993944398639, -0.7415311855993944398639, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.9491079123427585245262, -0.7415311855993944398639, -0.4058451513773971669066, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( -0.9491079123427585245262, -0.7415311855993944398639, 0, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.4179591836734693877551 ),
        IntegrationPointType( -0.9491079123427585245262, -0.7415311855993944398639, 0.4058451513773971669066, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( -0.9491079123427585245262, -0.7415311855993944398639, 0.7415311855993944398639, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.9491079123427585245262, -0.7415311855993944398639, 0.9491079123427585245262, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.1294849661688696932706 ),

        IntegrationPointType( -0.9491079123427585245262, -0.4058451513773971669066, -0.9491079123427585245262, 0.1294849661688696932706 * 0.38183005050511894495 * 0.1294849661688696932706 ),
        IntegrationPointType( -0.9491079123427585245262, -0.4058451513773971669066, -0.7415311855993944398639, 0.1294849661688696932706 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.9491079123427585245262, -0.4058451513773971669066, -0.4058451513773971669066, 0.1294849661688696932706 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( -0.9491079123427585245262, -0.4058451513773971669066, 0, 0.1294849661688696932706 * 0.38183005050511894495 * 0.4179591836734693877551 ),
        IntegrationPointType( -0.9491079123427585245262, -0.4058451513773971669066, 0.4058451513773971669066, 0.1294849661688696932706 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( -0.9491079123427585245262, -0.4058451513773971669066, 0.7415311855993944398639, 0.1294849661688696932706 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.9491079123427585245262, -0.4058451513773971669066, 0.9491079123427585245262, 0.1294849661688696932706 * 0.38183005050511894495 * 0.1294849661688696932706 ),

        IntegrationPointType( -0.9491079123427585245262, 0, -0.9491079123427585245262, 0.1294849661688696932706 * 0.4179591836734693877551 * 0.1294849661688696932706 ),
        IntegrationPointType( -0.9491079123427585245262, 0, -0.7415311855993944398639, 0.1294849661688696932706 * 0.4179591836734693877551 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.9491079123427585245262, 0, -0.4058451513773971669066, 0.1294849661688696932706 * 0.4179591836734693877551 * 0.38183005050511894495 ),
        IntegrationPointType( -0.9491079123427585245262, 0, 0, 0.1294849661688696932706 * 0.4179591836734693877551 * 0.4179591836734693877551 ),
        IntegrationPointType( -0.9491079123427585245262, 0, 0.4058451513773971669066, 0.1294849661688696932706 * 0.4179591836734693877551 * 0.38183005050511894495 ),
        IntegrationPointType( -0.9491079123427585245262, 0, 0.7415311855993944398639, 0.1294849661688696932706 * 0.4179591836734693877551 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.9491079123427585245262, 0, 0.9491079123427585245262, 0.1294849661688696932706 * 0.4179591836734693877551 * 0.1294849661688696932706 ),

        IntegrationPointType( -0.9491079123427585245262, 0.4058451513773971669066, -0.9491079123427585245262, 0.1294849661688696932706 * 0.38183005050511894495 * 0.1294849661688696932706 ),
        IntegrationPointType( -0.9491079123427585245262, 0.4058451513773971669066, -0.7415311855993944398639, 0.1294849661688696932706 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.9491079123427585245262, 0.4058451513773971669066, -0.4058451513773971669066, 0.1294849661688696932706 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( -0.9491079123427585245262, 0.4058451513773971669066, 0, 0.1294849661688696932706 * 0.38183005050511894495 * 0.4179591836734693877551 ),
        IntegrationPointType( -0.9491079123427585245262, 0.4058451513773971669066, 0.4058451513773971669066, 0.1294849661688696932706 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( -0.9491079123427585245262, 0.4058451513773971669066, 0.7415311855993944398639, 0.1294849661688696932706 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.9491079123427585245262, 0.4058451513773971669066, 0.9491079123427585245262, 0.1294849661688696932706 * 0.38183005050511894495 * 0.1294849661688696932706 ),

        IntegrationPointType( -0.9491079123427585245262, 0.7415311855993944398639, -0.9491079123427585245262, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.1294849661688696932706 ),
        IntegrationPointType( -0.9491079123427585245262, 0.7415311855993944398639, -0.7415311855993944398639, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.9491079123427585245262, 0.7415311855993944398639, -0.4058451513773971669066, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( -0.9491079123427585245262, 0.7415311855993944398639, 0, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.4179591836734693877551 ),
        IntegrationPointType( -0.9491079123427585245262, 0.7415311855993944398639, 0.4058451513773971669066, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( -0.9491079123427585245262, 0.7415311855993944398639, 0.7415311855993944398639, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.9491079123427585245262, 0.7415311855993944398639, 0.9491079123427585245262, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.1294849661688696932706 ),

        IntegrationPointType( -0.9491079123427585245262, 0.9491079123427585245262, -0.9491079123427585245262, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.1294849661688696932706 ),
        IntegrationPointType( -0.9491079123427585245262, 0.9491079123427585245262, -0.7415311855993944398639, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.9491079123427585245262, 0.9491079123427585245262, -0.4058451513773971669066, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( -0.9491079123427585245262, 0.9491079123427585245262, 0, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.4179591836734693877551 ),
        IntegrationPointType( -0.9491079123427585245262, 0.9491079123427585245262, 0.4058451513773971669066, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( -0.9491079123427585245262, 0.9491079123427585245262, 0.7415311855993944398639, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.9491079123427585245262, 0.9491079123427585245262, 0.9491079123427585245262, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.1294849661688696932706 ),

        IntegrationPointType( -0.7415311855993944398639, -0.9491079123427585245262, -0.9491079123427585245262, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.1294849661688696932706 ),
        IntegrationPointType( -0.7415311855993944398639, -0.9491079123427585245262, -0.7415311855993944398639, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.7415311855993944398639, -0.9491079123427585245262, -0.4058451513773971669066, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( -0.7415311855993944398639, -0.9491079123427585245262, 0, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.4179591836734693877551 ),
        IntegrationPointType( -0.7415311855993944398639, -0.9491079123427585245262, 0.4058451513773971669066, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( -0.7415311855993944398639, -0.9491079123427585245262, 0.7415311855993944398639, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.7415311855993944398639, -0.9491079123427585245262, 0.9491079123427585245262, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.1294849661688696932706 ),

        IntegrationPointType( -0.7415311855993944398639, -0.7415311855993944398639, -0.9491079123427585245262, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.1294849661688696932706 ),
        IntegrationPointType( -0.7415311855993944398639, -0.7415311855993944398639, -0.7415311855993944398639, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.7415311855993944398639, -0.7415311855993944398639, -0.4058451513773971669066, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( -0.7415311855993944398639, -0.7415311855993944398639, 0, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.4179591836734693877551 ),
        IntegrationPointType( -0.7415311855993944398639, -0.7415311855993944398639, 0.4058451513773971669066, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( -0.7415311855993944398639, -0.7415311855993944398639, 0.7415311855993944398639, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.7415311855993944398639, -0.7415311855993944398639, 0.9491079123427585245262, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.1294849661688696932706 ),

        IntegrationPointType( -0.7415311855993944398639, -0.4058451513773971669066, -0.9491079123427585245262, 0.2797053914892766679015 * 0.38183005050511894495 * 0.1294849661688696932706 ),
        IntegrationPointType( -0.7415311855993944398639, -0.4058451513773971669066, -0.7415311855993944398639, 0.2797053914892766679015 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.7415311855993944398639, -0.4058451513773971669066, -0.4058451513773971669066, 0.2797053914892766679015 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( -0.7415311855993944398639, -0.4058451513773971669066, 0, 0.2797053914892766679015 * 0.38183005050511894495 * 0.4179591836734693877551 ),
        IntegrationPointType( -0.7415311855993944398639, -0.4058451513773971669066, 0.4058451513773971669066, 0.2797053914892766679015 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( -0.7415311855993944398639, -0.4058451513773971669066, 0.7415311855993944398639, 0.2797053914892766679015 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.7415311855993944398639, -0.4058451513773971669066, 0.9491079123427585245262, 0.2797053914892766679015 * 0.38183005050511894495 * 0.1294849661688696932706 ),

        IntegrationPointType( -0.7415311855993944398639, 0, -0.9491079123427585245262, 0.2797053914892766679015 * 0.4179591836734693877551 * 0.1294849661688696932706 ),
        IntegrationPointType( -0.7415311855993944398639, 0, -0.7415311855993944398639, 0.2797053914892766679015 * 0.4179591836734693877551 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.7415311855993944398639, 0, -0.4058451513773971669066, 0.2797053914892766679015 * 0.4179591836734693877551 * 0.38183005050511894495 ),
        IntegrationPointType( -0.7415311855993944398639, 0, 0, 0.2797053914892766679015 * 0.4179591836734693877551 * 0.4179591836734693877551 ),
        IntegrationPointType( -0.7415311855993944398639, 0, 0.4058451513773971669066, 0.2797053914892766679015 * 0.4179591836734693877551 * 0.38183005050511894495 ),
        IntegrationPointType( -0.7415311855993944398639, 0, 0.7415311855993944398639, 0.2797053914892766679015 * 0.4179591836734693877551 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.7415311855993944398639, 0, 0.9491079123427585245262, 0.2797053914892766679015 * 0.4179591836734693877551 * 0.1294849661688696932706 ),

        IntegrationPointType( -0.7415311855993944398639, 0.4058451513773971669066, -0.9491079123427585245262, 0.2797053914892766679015 * 0.38183005050511894495 * 0.1294849661688696932706 ),
        IntegrationPointType( -0.7415311855993944398639, 0.4058451513773971669066, -0.7415311855993944398639, 0.2797053914892766679015 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.7415311855993944398639, 0.4058451513773971669066, -0.4058451513773971669066, 0.2797053914892766679015 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( -0.7415311855993944398639, 0.4058451513773971669066, 0, 0.2797053914892766679015 * 0.38183005050511894495 * 0.4179591836734693877551 ),
        IntegrationPointType( -0.7415311855993944398639, 0.4058451513773971669066, 0.4058451513773971669066, 0.2797053914892766679015 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( -0.7415311855993944398639, 0.4058451513773971669066, 0.7415311855993944398639, 0.2797053914892766679015 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.7415311855993944398639, 0.4058451513773971669066, 0.9491079123427585245262, 0.2797053914892766679015 * 0.38183005050511894495 * 0.1294849661688696932706 ),

        IntegrationPointType( -0.7415311855993944398639, 0.7415311855993944398639, -0.9491079123427585245262, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.1294849661688696932706 ),
        IntegrationPointType( -0.7415311855993944398639, 0.7415311855993944398639, -0.7415311855993944398639, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.7415311855993944398639, 0.7415311855993944398639, -0.4058451513773971669066, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( -0.7415311855993944398639, 0.7415311855993944398639, 0, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.4179591836734693877551 ),
        IntegrationPointType( -0.7415311855993944398639, 0.7415311855993944398639, 0.4058451513773971669066, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( -0.7415311855993944398639, 0.7415311855993944398639, 0.7415311855993944398639, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.7415311855993944398639, 0.7415311855993944398639, 0.9491079123427585245262, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.1294849661688696932706 ),

        IntegrationPointType( -0.7415311855993944398639, 0.9491079123427585245262, -0.9491079123427585245262, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.1294849661688696932706 ),
        IntegrationPointType( -0.7415311855993944398639, 0.9491079123427585245262, -0.7415311855993944398639, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.7415311855993944398639, 0.9491079123427585245262, -0.4058451513773971669066, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( -0.7415311855993944398639, 0.9491079123427585245262, 0, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.4179591836734693877551 ),
        IntegrationPointType( -0.7415311855993944398639, 0.9491079123427585245262, 0.4058451513773971669066, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( -0.7415311855993944398639, 0.9491079123427585245262, 0.7415311855993944398639, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.7415311855993944398639, 0.9491079123427585245262, 0.9491079123427585245262, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.1294849661688696932706 ),

        IntegrationPointType( -0.4058451513773971669066, -0.9491079123427585245262, -0.9491079123427585245262, 0.38183005050511894495 * 0.1294849661688696932706 * 0.1294849661688696932706 ),
        IntegrationPointType( -0.4058451513773971669066, -0.9491079123427585245262, -0.7415311855993944398639, 0.38183005050511894495 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.4058451513773971669066, -0.9491079123427585245262, -0.4058451513773971669066, 0.38183005050511894495 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( -0.4058451513773971669066, -0.9491079123427585245262, 0, 0.38183005050511894495 * 0.1294849661688696932706 * 0.4179591836734693877551 ),
        IntegrationPointType( -0.4058451513773971669066, -0.9491079123427585245262, 0.4058451513773971669066, 0.38183005050511894495 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( -0.4058451513773971669066, -0.9491079123427585245262, 0.7415311855993944398639, 0.38183005050511894495 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.4058451513773971669066, -0.9491079123427585245262, 0.9491079123427585245262, 0.38183005050511894495 * 0.1294849661688696932706 * 0.1294849661688696932706 ),

        IntegrationPointType( -0.4058451513773971669066, -0.7415311855993944398639, -0.9491079123427585245262, 0.38183005050511894495 * 0.2797053914892766679015 * 0.1294849661688696932706 ),
        IntegrationPointType( -0.4058451513773971669066, -0.7415311855993944398639, -0.7415311855993944398639, 0.38183005050511894495 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.4058451513773971669066, -0.7415311855993944398639, -0.4058451513773971669066, 0.38183005050511894495 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( -0.4058451513773971669066, -0.7415311855993944398639, 0, 0.38183005050511894495 * 0.2797053914892766679015 * 0.4179591836734693877551 ),
        IntegrationPointType( -0.4058451513773971669066, -0.7415311855993944398639, 0.4058451513773971669066, 0.38183005050511894495 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( -0.4058451513773971669066, -0.7415311855993944398639, 0.7415311855993944398639, 0.38183005050511894495 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.4058451513773971669066, -0.7415311855993944398639, 0.9491079123427585245262, 0.38183005050511894495 * 0.2797053914892766679015 * 0.1294849661688696932706 ),

        IntegrationPointType( -0.4058451513773971669066, -0.4058451513773971669066, -0.9491079123427585245262, 0.38183005050511894495 * 0.38183005050511894495 * 0.1294849661688696932706 ),
        IntegrationPointType( -0.4058451513773971669066, -0.4058451513773971669066, -0.7415311855993944398639, 0.38183005050511894495 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.4058451513773971669066, -0.4058451513773971669066, -0.4058451513773971669066, 0.38183005050511894495 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( -0.4058451513773971669066, -0.4058451513773971669066, 0, 0.38183005050511894495 * 0.38183005050511894495 * 0.4179591836734693877551 ),
        IntegrationPointType( -0.4058451513773971669066, -0.4058451513773971669066, 0.4058451513773971669066, 0.38183005050511894495 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( -0.4058451513773971669066, -0.4058451513773971669066, 0.7415311855993944398639, 0.38183005050511894495 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.4058451513773971669066, -0.4058451513773971669066, 0.9491079123427585245262, 0.38183005050511894495 * 0.38183005050511894495 * 0.1294849661688696932706 ),

        IntegrationPointType( -0.4058451513773971669066, 0, -0.9491079123427585245262, 0.38183005050511894495 * 0.4179591836734693877551 * 0.1294849661688696932706 ),
        IntegrationPointType( -0.4058451513773971669066, 0, -0.7415311855993944398639, 0.38183005050511894495 * 0.4179591836734693877551 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.4058451513773971669066, 0, -0.4058451513773971669066, 0.38183005050511894495 * 0.4179591836734693877551 * 0.38183005050511894495 ),
        IntegrationPointType( -0.4058451513773971669066, 0, 0, 0.38183005050511894495 * 0.4179591836734693877551 * 0.4179591836734693877551 ),
        IntegrationPointType( -0.4058451513773971669066, 0, 0.4058451513773971669066, 0.38183005050511894495 * 0.4179591836734693877551 * 0.38183005050511894495 ),
        IntegrationPointType( -0.4058451513773971669066, 0, 0.7415311855993944398639, 0.38183005050511894495 * 0.4179591836734693877551 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.4058451513773971669066, 0, 0.9491079123427585245262, 0.38183005050511894495 * 0.4179591836734693877551 * 0.1294849661688696932706 ),

        IntegrationPointType( -0.4058451513773971669066, 0.4058451513773971669066, -0.9491079123427585245262, 0.38183005050511894495 * 0.38183005050511894495 * 0.1294849661688696932706 ),
        IntegrationPointType( -0.4058451513773971669066, 0.4058451513773971669066, -0.7415311855993944398639, 0.38183005050511894495 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.4058451513773971669066, 0.4058451513773971669066, -0.4058451513773971669066, 0.38183005050511894495 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( -0.4058451513773971669066, 0.4058451513773971669066, 0, 0.38183005050511894495 * 0.38183005050511894495 * 0.4179591836734693877551 ),
        IntegrationPointType( -0.4058451513773971669066, 0.4058451513773971669066, 0.4058451513773971669066, 0.38183005050511894495 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( -0.4058451513773971669066, 0.4058451513773971669066, 0.7415311855993944398639, 0.38183005050511894495 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.4058451513773971669066, 0.4058451513773971669066, 0.9491079123427585245262, 0.38183005050511894495 * 0.38183005050511894495 * 0.1294849661688696932706 ),

        IntegrationPointType( -0.4058451513773971669066, 0.7415311855993944398639, -0.9491079123427585245262, 0.38183005050511894495 * 0.2797053914892766679015 * 0.1294849661688696932706 ),
        IntegrationPointType( -0.4058451513773971669066, 0.7415311855993944398639, -0.7415311855993944398639, 0.38183005050511894495 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.4058451513773971669066, 0.7415311855993944398639, -0.4058451513773971669066, 0.38183005050511894495 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( -0.4058451513773971669066, 0.7415311855993944398639, 0, 0.38183005050511894495 * 0.2797053914892766679015 * 0.4179591836734693877551 ),
        IntegrationPointType( -0.4058451513773971669066, 0.7415311855993944398639, 0.4058451513773971669066, 0.38183005050511894495 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( -0.4058451513773971669066, 0.7415311855993944398639, 0.7415311855993944398639, 0.38183005050511894495 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.4058451513773971669066, 0.7415311855993944398639, 0.9491079123427585245262, 0.38183005050511894495 * 0.2797053914892766679015 * 0.1294849661688696932706 ),

        IntegrationPointType( -0.4058451513773971669066, 0.9491079123427585245262, -0.9491079123427585245262, 0.38183005050511894495 * 0.1294849661688696932706 * 0.1294849661688696932706 ),
        IntegrationPointType( -0.4058451513773971669066, 0.9491079123427585245262, -0.7415311855993944398639, 0.38183005050511894495 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.4058451513773971669066, 0.9491079123427585245262, -0.4058451513773971669066, 0.38183005050511894495 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( -0.4058451513773971669066, 0.9491079123427585245262, 0, 0.38183005050511894495 * 0.1294849661688696932706 * 0.4179591836734693877551 ),
        IntegrationPointType( -0.4058451513773971669066, 0.9491079123427585245262, 0.4058451513773971669066, 0.38183005050511894495 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( -0.4058451513773971669066, 0.9491079123427585245262, 0.7415311855993944398639, 0.38183005050511894495 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( -0.4058451513773971669066, 0.9491079123427585245262, 0.9491079123427585245262, 0.38183005050511894495 * 0.1294849661688696932706 * 0.1294849661688696932706 ),

        IntegrationPointType( 0, -0.9491079123427585245262, -0.9491079123427585245262, 0.4179591836734693877551 * 0.1294849661688696932706 * 0.1294849661688696932706 ),
        IntegrationPointType( 0, -0.9491079123427585245262, -0.7415311855993944398639, 0.4179591836734693877551 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( 0, -0.9491079123427585245262, -0.4058451513773971669066, 0.4179591836734693877551 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( 0, -0.9491079123427585245262, 0, 0.4179591836734693877551 * 0.1294849661688696932706 * 0.4179591836734693877551 ),
        IntegrationPointType( 0, -0.9491079123427585245262, 0.4058451513773971669066, 0.4179591836734693877551 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( 0, -0.9491079123427585245262, 0.7415311855993944398639, 0.4179591836734693877551 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( 0, -0.9491079123427585245262, 0.9491079123427585245262, 0.4179591836734693877551 * 0.1294849661688696932706 * 0.1294849661688696932706 ),

        IntegrationPointType( 0, -0.7415311855993944398639, -0.9491079123427585245262, 0.4179591836734693877551 * 0.2797053914892766679015 * 0.1294849661688696932706 ),
        IntegrationPointType( 0, -0.7415311855993944398639, -0.7415311855993944398639, 0.4179591836734693877551 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( 0, -0.7415311855993944398639, -0.4058451513773971669066, 0.4179591836734693877551 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( 0, -0.7415311855993944398639, 0, 0.4179591836734693877551 * 0.2797053914892766679015 * 0.4179591836734693877551 ),
        IntegrationPointType( 0, -0.7415311855993944398639, 0.4058451513773971669066, 0.4179591836734693877551 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( 0, -0.7415311855993944398639, 0.7415311855993944398639, 0.4179591836734693877551 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( 0, -0.7415311855993944398639, 0.9491079123427585245262, 0.4179591836734693877551 * 0.2797053914892766679015 * 0.1294849661688696932706 ),

        IntegrationPointType( 0, -0.4058451513773971669066, -0.9491079123427585245262, 0.4179591836734693877551 * 0.38183005050511894495 * 0.1294849661688696932706 ),
        IntegrationPointType( 0, -0.4058451513773971669066, -0.7415311855993944398639, 0.4179591836734693877551 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( 0, -0.4058451513773971669066, -0.4058451513773971669066, 0.4179591836734693877551 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( 0, -0.4058451513773971669066, 0, 0.4179591836734693877551 * 0.38183005050511894495 * 0.4179591836734693877551 ),
        IntegrationPointType( 0, -0.4058451513773971669066, 0.4058451513773971669066, 0.4179591836734693877551 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( 0, -0.4058451513773971669066, 0.7415311855993944398639, 0.4179591836734693877551 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( 0, -0.4058451513773971669066, 0.9491079123427585245262, 0.4179591836734693877551 * 0.38183005050511894495 * 0.1294849661688696932706 ),

        IntegrationPointType( 0, 0, -0.9491079123427585245262, 0.4179591836734693877551 * 0.4179591836734693877551 * 0.1294849661688696932706 ),
        IntegrationPointType( 0, 0, -0.7415311855993944398639, 0.4179591836734693877551 * 0.4179591836734693877551 * 0.2797053914892766679015 ),
        IntegrationPointType( 0, 0, -0.4058451513773971669066, 0.4179591836734693877551 * 0.4179591836734693877551 * 0.38183005050511894495 ),
        IntegrationPointType( 0, 0, 0, 0.4179591836734693877551 * 0.4179591836734693877551 * 0.4179591836734693877551 ),
        IntegrationPointType( 0, 0, 0.4058451513773971669066, 0.4179591836734693877551 * 0.4179591836734693877551 * 0.38183005050511894495 ),
        IntegrationPointType( 0, 0, 0.7415311855993944398639, 0.4179591836734693877551 * 0.4179591836734693877551 * 0.2797053914892766679015 ),
        IntegrationPointType( 0, 0, 0.9491079123427585245262, 0.4179591836734693877551 * 0.4179591836734693877551 * 0.1294849661688696932706 ),

        IntegrationPointType( 0, 0.4058451513773971669066, -0.9491079123427585245262, 0.4179591836734693877551 * 0.38183005050511894495 * 0.1294849661688696932706 ),
        IntegrationPointType( 0, 0.4058451513773971669066, -0.7415311855993944398639, 0.4179591836734693877551 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( 0, 0.4058451513773971669066, -0.4058451513773971669066, 0.4179591836734693877551 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( 0, 0.4058451513773971669066, 0, 0.4179591836734693877551 * 0.38183005050511894495 * 0.4179591836734693877551 ),
        IntegrationPointType( 0, 0.4058451513773971669066, 0.4058451513773971669066, 0.4179591836734693877551 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( 0, 0.4058451513773971669066, 0.7415311855993944398639, 0.4179591836734693877551 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( 0, 0.4058451513773971669066, 0.9491079123427585245262, 0.4179591836734693877551 * 0.38183005050511894495 * 0.1294849661688696932706 ),

        IntegrationPointType( 0, 0.7415311855993944398639, -0.9491079123427585245262, 0.4179591836734693877551 * 0.2797053914892766679015 * 0.1294849661688696932706 ),
        IntegrationPointType( 0, 0.7415311855993944398639, -0.7415311855993944398639, 0.4179591836734693877551 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( 0, 0.7415311855993944398639, -0.4058451513773971669066, 0.4179591836734693877551 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( 0, 0.7415311855993944398639, 0, 0.4179591836734693877551 * 0.2797053914892766679015 * 0.4179591836734693877551 ),
        IntegrationPointType( 0, 0.7415311855993944398639, 0.4058451513773971669066, 0.4179591836734693877551 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( 0, 0.7415311855993944398639, 0.7415311855993944398639, 0.4179591836734693877551 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( 0, 0.7415311855993944398639, 0.9491079123427585245262, 0.4179591836734693877551 * 0.2797053914892766679015 * 0.1294849661688696932706 ),

        IntegrationPointType( 0, 0.9491079123427585245262, -0.9491079123427585245262, 0.4179591836734693877551 * 0.1294849661688696932706 * 0.1294849661688696932706 ),
        IntegrationPointType( 0, 0.9491079123427585245262, -0.7415311855993944398639, 0.4179591836734693877551 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( 0, 0.9491079123427585245262, -0.4058451513773971669066, 0.4179591836734693877551 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( 0, 0.9491079123427585245262, 0, 0.4179591836734693877551 * 0.1294849661688696932706 * 0.4179591836734693877551 ),
        IntegrationPointType( 0, 0.9491079123427585245262, 0.4058451513773971669066, 0.4179591836734693877551 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( 0, 0.9491079123427585245262, 0.7415311855993944398639, 0.4179591836734693877551 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( 0, 0.9491079123427585245262, 0.9491079123427585245262, 0.4179591836734693877551 * 0.1294849661688696932706 * 0.1294849661688696932706 ),

        IntegrationPointType( 0.4058451513773971669066, -0.9491079123427585245262, -0.9491079123427585245262, 0.38183005050511894495 * 0.1294849661688696932706 * 0.1294849661688696932706 ),
        IntegrationPointType( 0.4058451513773971669066, -0.9491079123427585245262, -0.7415311855993944398639, 0.38183005050511894495 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.4058451513773971669066, -0.9491079123427585245262, -0.4058451513773971669066, 0.38183005050511894495 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( 0.4058451513773971669066, -0.9491079123427585245262, 0, 0.38183005050511894495 * 0.1294849661688696932706 * 0.4179591836734693877551 ),
        IntegrationPointType( 0.4058451513773971669066, -0.9491079123427585245262, 0.4058451513773971669066, 0.38183005050511894495 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( 0.4058451513773971669066, -0.9491079123427585245262, 0.7415311855993944398639, 0.38183005050511894495 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.4058451513773971669066, -0.9491079123427585245262, 0.9491079123427585245262, 0.38183005050511894495 * 0.1294849661688696932706 * 0.1294849661688696932706 ),

        IntegrationPointType( 0.4058451513773971669066, -0.7415311855993944398639, -0.9491079123427585245262, 0.38183005050511894495 * 0.2797053914892766679015 * 0.1294849661688696932706 ),
        IntegrationPointType( 0.4058451513773971669066, -0.7415311855993944398639, -0.7415311855993944398639, 0.38183005050511894495 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.4058451513773971669066, -0.7415311855993944398639, -0.4058451513773971669066, 0.38183005050511894495 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( 0.4058451513773971669066, -0.7415311855993944398639, 0, 0.38183005050511894495 * 0.2797053914892766679015 * 0.4179591836734693877551 ),
        IntegrationPointType( 0.4058451513773971669066, -0.7415311855993944398639, 0.4058451513773971669066, 0.38183005050511894495 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( 0.4058451513773971669066, -0.7415311855993944398639, 0.7415311855993944398639, 0.38183005050511894495 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.4058451513773971669066, -0.7415311855993944398639, 0.9491079123427585245262, 0.38183005050511894495 * 0.2797053914892766679015 * 0.1294849661688696932706 ),

        IntegrationPointType( 0.4058451513773971669066, -0.4058451513773971669066, -0.9491079123427585245262, 0.38183005050511894495 * 0.38183005050511894495 * 0.1294849661688696932706 ),
        IntegrationPointType( 0.4058451513773971669066, -0.4058451513773971669066, -0.7415311855993944398639, 0.38183005050511894495 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.4058451513773971669066, -0.4058451513773971669066, -0.4058451513773971669066, 0.38183005050511894495 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( 0.4058451513773971669066, -0.4058451513773971669066, 0, 0.38183005050511894495 * 0.38183005050511894495 * 0.4179591836734693877551 ),
        IntegrationPointType( 0.4058451513773971669066, -0.4058451513773971669066, 0.4058451513773971669066, 0.38183005050511894495 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( 0.4058451513773971669066, -0.4058451513773971669066, 0.7415311855993944398639, 0.38183005050511894495 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.4058451513773971669066, -0.4058451513773971669066, 0.9491079123427585245262, 0.38183005050511894495 * 0.38183005050511894495 * 0.1294849661688696932706 ),

        IntegrationPointType( 0.4058451513773971669066, 0, -0.9491079123427585245262, 0.38183005050511894495 * 0.4179591836734693877551 * 0.1294849661688696932706 ),
        IntegrationPointType( 0.4058451513773971669066, 0, -0.7415311855993944398639, 0.38183005050511894495 * 0.4179591836734693877551 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.4058451513773971669066, 0, -0.4058451513773971669066, 0.38183005050511894495 * 0.4179591836734693877551 * 0.38183005050511894495 ),
        IntegrationPointType( 0.4058451513773971669066, 0, 0, 0.38183005050511894495 * 0.4179591836734693877551 * 0.4179591836734693877551 ),
        IntegrationPointType( 0.4058451513773971669066, 0, 0.4058451513773971669066, 0.38183005050511894495 * 0.4179591836734693877551 * 0.38183005050511894495 ),
        IntegrationPointType( 0.4058451513773971669066, 0, 0.7415311855993944398639, 0.38183005050511894495 * 0.4179591836734693877551 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.4058451513773971669066, 0, 0.9491079123427585245262, 0.38183005050511894495 * 0.4179591836734693877551 * 0.1294849661688696932706 ),

        IntegrationPointType( 0.4058451513773971669066, 0.4058451513773971669066, -0.9491079123427585245262, 0.38183005050511894495 * 0.38183005050511894495 * 0.1294849661688696932706 ),
        IntegrationPointType( 0.4058451513773971669066, 0.4058451513773971669066, -0.7415311855993944398639, 0.38183005050511894495 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.4058451513773971669066, 0.4058451513773971669066, -0.4058451513773971669066, 0.38183005050511894495 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( 0.4058451513773971669066, 0.4058451513773971669066, 0, 0.38183005050511894495 * 0.38183005050511894495 * 0.4179591836734693877551 ),
        IntegrationPointType( 0.4058451513773971669066, 0.4058451513773971669066, 0.4058451513773971669066, 0.38183005050511894495 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( 0.4058451513773971669066, 0.4058451513773971669066, 0.7415311855993944398639, 0.38183005050511894495 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.4058451513773971669066, 0.4058451513773971669066, 0.9491079123427585245262, 0.38183005050511894495 * 0.38183005050511894495 * 0.1294849661688696932706 ),

        IntegrationPointType( 0.4058451513773971669066, 0.7415311855993944398639, -0.9491079123427585245262, 0.38183005050511894495 * 0.2797053914892766679015 * 0.1294849661688696932706 ),
        IntegrationPointType( 0.4058451513773971669066, 0.7415311855993944398639, -0.7415311855993944398639, 0.38183005050511894495 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.4058451513773971669066, 0.7415311855993944398639, -0.4058451513773971669066, 0.38183005050511894495 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( 0.4058451513773971669066, 0.7415311855993944398639, 0, 0.38183005050511894495 * 0.2797053914892766679015 * 0.4179591836734693877551 ),
        IntegrationPointType( 0.4058451513773971669066, 0.7415311855993944398639, 0.4058451513773971669066, 0.38183005050511894495 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( 0.4058451513773971669066, 0.7415311855993944398639, 0.7415311855993944398639, 0.38183005050511894495 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.4058451513773971669066, 0.7415311855993944398639, 0.9491079123427585245262, 0.38183005050511894495 * 0.2797053914892766679015 * 0.1294849661688696932706 ),

        IntegrationPointType( 0.4058451513773971669066, 0.9491079123427585245262, -0.9491079123427585245262, 0.38183005050511894495 * 0.1294849661688696932706 * 0.1294849661688696932706 ),
        IntegrationPointType( 0.4058451513773971669066, 0.9491079123427585245262, -0.7415311855993944398639, 0.38183005050511894495 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.4058451513773971669066, 0.9491079123427585245262, -0.4058451513773971669066, 0.38183005050511894495 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( 0.4058451513773971669066, 0.9491079123427585245262, 0, 0.38183005050511894495 * 0.1294849661688696932706 * 0.4179591836734693877551 ),
        IntegrationPointType( 0.4058451513773971669066, 0.9491079123427585245262, 0.4058451513773971669066, 0.38183005050511894495 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( 0.4058451513773971669066, 0.9491079123427585245262, 0.7415311855993944398639, 0.38183005050511894495 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.4058451513773971669066, 0.9491079123427585245262, 0.9491079123427585245262, 0.38183005050511894495 * 0.1294849661688696932706 * 0.1294849661688696932706 ),

        IntegrationPointType( 0.7415311855993944398639, -0.9491079123427585245262, -0.9491079123427585245262, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.1294849661688696932706 ),
        IntegrationPointType( 0.7415311855993944398639, -0.9491079123427585245262, -0.7415311855993944398639, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.7415311855993944398639, -0.9491079123427585245262, -0.4058451513773971669066, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( 0.7415311855993944398639, -0.9491079123427585245262, 0, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.4179591836734693877551 ),
        IntegrationPointType( 0.7415311855993944398639, -0.9491079123427585245262, 0.4058451513773971669066, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( 0.7415311855993944398639, -0.9491079123427585245262, 0.7415311855993944398639, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.7415311855993944398639, -0.9491079123427585245262, 0.9491079123427585245262, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.1294849661688696932706 ),

        IntegrationPointType( 0.7415311855993944398639, -0.7415311855993944398639, -0.9491079123427585245262, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.1294849661688696932706 ),
        IntegrationPointType( 0.7415311855993944398639, -0.7415311855993944398639, -0.7415311855993944398639, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.7415311855993944398639, -0.7415311855993944398639, -0.4058451513773971669066, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( 0.7415311855993944398639, -0.7415311855993944398639, 0, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.4179591836734693877551 ),
        IntegrationPointType( 0.7415311855993944398639, -0.7415311855993944398639, 0.4058451513773971669066, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( 0.7415311855993944398639, -0.7415311855993944398639, 0.7415311855993944398639, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.7415311855993944398639, -0.7415311855993944398639, 0.9491079123427585245262, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.1294849661688696932706 ),

        IntegrationPointType( 0.7415311855993944398639, -0.4058451513773971669066, -0.9491079123427585245262, 0.2797053914892766679015 * 0.38183005050511894495 * 0.1294849661688696932706 ),
        IntegrationPointType( 0.7415311855993944398639, -0.4058451513773971669066, -0.7415311855993944398639, 0.2797053914892766679015 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.7415311855993944398639, -0.4058451513773971669066, -0.4058451513773971669066, 0.2797053914892766679015 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( 0.7415311855993944398639, -0.4058451513773971669066, 0, 0.2797053914892766679015 * 0.38183005050511894495 * 0.4179591836734693877551 ),
        IntegrationPointType( 0.7415311855993944398639, -0.4058451513773971669066, 0.4058451513773971669066, 0.2797053914892766679015 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( 0.7415311855993944398639, -0.4058451513773971669066, 0.7415311855993944398639, 0.2797053914892766679015 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.7415311855993944398639, -0.4058451513773971669066, 0.9491079123427585245262, 0.2797053914892766679015 * 0.38183005050511894495 * 0.1294849661688696932706 ),

        IntegrationPointType( 0.7415311855993944398639, 0, -0.9491079123427585245262, 0.2797053914892766679015 * 0.4179591836734693877551 * 0.1294849661688696932706 ),
        IntegrationPointType( 0.7415311855993944398639, 0, -0.7415311855993944398639, 0.2797053914892766679015 * 0.4179591836734693877551 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.7415311855993944398639, 0, -0.4058451513773971669066, 0.2797053914892766679015 * 0.4179591836734693877551 * 0.38183005050511894495 ),
        IntegrationPointType( 0.7415311855993944398639, 0, 0, 0.2797053914892766679015 * 0.4179591836734693877551 * 0.4179591836734693877551 ),
        IntegrationPointType( 0.7415311855993944398639, 0, 0.4058451513773971669066, 0.2797053914892766679015 * 0.4179591836734693877551 * 0.38183005050511894495 ),
        IntegrationPointType( 0.7415311855993944398639, 0, 0.7415311855993944398639, 0.2797053914892766679015 * 0.4179591836734693877551 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.7415311855993944398639, 0, 0.9491079123427585245262, 0.2797053914892766679015 * 0.4179591836734693877551 * 0.1294849661688696932706 ),

        IntegrationPointType( 0.7415311855993944398639, 0.4058451513773971669066, -0.9491079123427585245262, 0.2797053914892766679015 * 0.38183005050511894495 * 0.1294849661688696932706 ),
        IntegrationPointType( 0.7415311855993944398639, 0.4058451513773971669066, -0.7415311855993944398639, 0.2797053914892766679015 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.7415311855993944398639, 0.4058451513773971669066, -0.4058451513773971669066, 0.2797053914892766679015 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( 0.7415311855993944398639, 0.4058451513773971669066, 0, 0.2797053914892766679015 * 0.38183005050511894495 * 0.4179591836734693877551 ),
        IntegrationPointType( 0.7415311855993944398639, 0.4058451513773971669066, 0.4058451513773971669066, 0.2797053914892766679015 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( 0.7415311855993944398639, 0.4058451513773971669066, 0.7415311855993944398639, 0.2797053914892766679015 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.7415311855993944398639, 0.4058451513773971669066, 0.9491079123427585245262, 0.2797053914892766679015 * 0.38183005050511894495 * 0.1294849661688696932706 ),

        IntegrationPointType( 0.7415311855993944398639, 0.7415311855993944398639, -0.9491079123427585245262, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.1294849661688696932706 ),
        IntegrationPointType( 0.7415311855993944398639, 0.7415311855993944398639, -0.7415311855993944398639, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.7415311855993944398639, 0.7415311855993944398639, -0.4058451513773971669066, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( 0.7415311855993944398639, 0.7415311855993944398639, 0, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.4179591836734693877551 ),
        IntegrationPointType( 0.7415311855993944398639, 0.7415311855993944398639, 0.4058451513773971669066, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( 0.7415311855993944398639, 0.7415311855993944398639, 0.7415311855993944398639, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.7415311855993944398639, 0.7415311855993944398639, 0.9491079123427585245262, 0.2797053914892766679015 * 0.2797053914892766679015 * 0.1294849661688696932706 ),

        IntegrationPointType( 0.7415311855993944398639, 0.9491079123427585245262, -0.9491079123427585245262, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.1294849661688696932706 ),
        IntegrationPointType( 0.7415311855993944398639, 0.9491079123427585245262, -0.7415311855993944398639, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.7415311855993944398639, 0.9491079123427585245262, -0.4058451513773971669066, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( 0.7415311855993944398639, 0.9491079123427585245262, 0, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.4179591836734693877551 ),
        IntegrationPointType( 0.7415311855993944398639, 0.9491079123427585245262, 0.4058451513773971669066, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( 0.7415311855993944398639, 0.9491079123427585245262, 0.7415311855993944398639, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.7415311855993944398639, 0.9491079123427585245262, 0.9491079123427585245262, 0.2797053914892766679015 * 0.1294849661688696932706 * 0.1294849661688696932706 ),

        IntegrationPointType( 0.9491079123427585245262, -0.9491079123427585245262, -0.9491079123427585245262, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.1294849661688696932706 ),
        IntegrationPointType( 0.9491079123427585245262, -0.9491079123427585245262, -0.7415311855993944398639, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.9491079123427585245262, -0.9491079123427585245262, -0.4058451513773971669066, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( 0.9491079123427585245262, -0.9491079123427585245262, 0, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.4179591836734693877551 ),
        IntegrationPointType( 0.9491079123427585245262, -0.9491079123427585245262, 0.4058451513773971669066, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( 0.9491079123427585245262, -0.9491079123427585245262, 0.7415311855993944398639, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.9491079123427585245262, -0.9491079123427585245262, 0.9491079123427585245262, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.1294849661688696932706 ),

        IntegrationPointType( 0.9491079123427585245262, -0.7415311855993944398639, -0.9491079123427585245262, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.1294849661688696932706 ),
        IntegrationPointType( 0.9491079123427585245262, -0.7415311855993944398639, -0.7415311855993944398639, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.9491079123427585245262, -0.7415311855993944398639, -0.4058451513773971669066, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( 0.9491079123427585245262, -0.7415311855993944398639, 0, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.4179591836734693877551 ),
        IntegrationPointType( 0.9491079123427585245262, -0.7415311855993944398639, 0.4058451513773971669066, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( 0.9491079123427585245262, -0.7415311855993944398639, 0.7415311855993944398639, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.9491079123427585245262, -0.7415311855993944398639, 0.9491079123427585245262, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.1294849661688696932706 ),

        IntegrationPointType( 0.9491079123427585245262, -0.4058451513773971669066, -0.9491079123427585245262, 0.1294849661688696932706 * 0.38183005050511894495 * 0.1294849661688696932706 ),
        IntegrationPointType( 0.9491079123427585245262, -0.4058451513773971669066, -0.7415311855993944398639, 0.1294849661688696932706 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.9491079123427585245262, -0.4058451513773971669066, -0.4058451513773971669066, 0.1294849661688696932706 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( 0.9491079123427585245262, -0.4058451513773971669066, 0, 0.1294849661688696932706 * 0.38183005050511894495 * 0.4179591836734693877551 ),
        IntegrationPointType( 0.9491079123427585245262, -0.4058451513773971669066, 0.4058451513773971669066, 0.1294849661688696932706 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( 0.9491079123427585245262, -0.4058451513773971669066, 0.7415311855993944398639, 0.1294849661688696932706 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.9491079123427585245262, -0.4058451513773971669066, 0.9491079123427585245262, 0.1294849661688696932706 * 0.38183005050511894495 * 0.1294849661688696932706 ),

        IntegrationPointType( 0.9491079123427585245262, 0, -0.9491079123427585245262, 0.1294849661688696932706 * 0.4179591836734693877551 * 0.1294849661688696932706 ),
        IntegrationPointType( 0.9491079123427585245262, 0, -0.7415311855993944398639, 0.1294849661688696932706 * 0.4179591836734693877551 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.9491079123427585245262, 0, -0.4058451513773971669066, 0.1294849661688696932706 * 0.4179591836734693877551 * 0.38183005050511894495 ),
        IntegrationPointType( 0.9491079123427585245262, 0, 0, 0.1294849661688696932706 * 0.4179591836734693877551 * 0.4179591836734693877551 ),
        IntegrationPointType( 0.9491079123427585245262, 0, 0.4058451513773971669066, 0.1294849661688696932706 * 0.4179591836734693877551 * 0.38183005050511894495 ),
        IntegrationPointType( 0.9491079123427585245262, 0, 0.7415311855993944398639, 0.1294849661688696932706 * 0.4179591836734693877551 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.9491079123427585245262, 0, 0.9491079123427585245262, 0.1294849661688696932706 * 0.4179591836734693877551 * 0.1294849661688696932706 ),

        IntegrationPointType( 0.9491079123427585245262, 0.4058451513773971669066, -0.9491079123427585245262, 0.1294849661688696932706 * 0.38183005050511894495 * 0.1294849661688696932706 ),
        IntegrationPointType( 0.9491079123427585245262, 0.4058451513773971669066, -0.7415311855993944398639, 0.1294849661688696932706 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.9491079123427585245262, 0.4058451513773971669066, -0.4058451513773971669066, 0.1294849661688696932706 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( 0.9491079123427585245262, 0.4058451513773971669066, 0, 0.1294849661688696932706 * 0.38183005050511894495 * 0.4179591836734693877551 ),
        IntegrationPointType( 0.9491079123427585245262, 0.4058451513773971669066, 0.4058451513773971669066, 0.1294849661688696932706 * 0.38183005050511894495 * 0.38183005050511894495 ),
        IntegrationPointType( 0.9491079123427585245262, 0.4058451513773971669066, 0.7415311855993944398639, 0.1294849661688696932706 * 0.38183005050511894495 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.9491079123427585245262, 0.4058451513773971669066, 0.9491079123427585245262, 0.1294849661688696932706 * 0.38183005050511894495 * 0.1294849661688696932706 ),

        IntegrationPointType( 0.9491079123427585245262, 0.7415311855993944398639, -0.9491079123427585245262, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.1294849661688696932706 ),
        IntegrationPointType( 0.9491079123427585245262, 0.7415311855993944398639, -0.7415311855993944398639, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.9491079123427585245262, 0.7415311855993944398639, -0.4058451513773971669066, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( 0.9491079123427585245262, 0.7415311855993944398639, 0, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.4179591836734693877551 ),
        IntegrationPointType( 0.9491079123427585245262, 0.7415311855993944398639, 0.4058451513773971669066, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.38183005050511894495 ),
        IntegrationPointType( 0.9491079123427585245262, 0.7415311855993944398639, 0.7415311855993944398639, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.9491079123427585245262, 0.7415311855993944398639, 0.9491079123427585245262, 0.1294849661688696932706 * 0.2797053914892766679015 * 0.1294849661688696932706 ),

        IntegrationPointType( 0.9491079123427585245262, 0.9491079123427585245262, -0.9491079123427585245262, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.1294849661688696932706 ),
        IntegrationPointType( 0.9491079123427585245262, 0.9491079123427585245262, -0.7415311855993944398639, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.9491079123427585245262, 0.9491079123427585245262, -0.4058451513773971669066, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( 0.9491079123427585245262, 0.9491079123427585245262, 0, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.4179591836734693877551 ),
        IntegrationPointType( 0.9491079123427585245262, 0.9491079123427585245262, 0.4058451513773971669066, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.38183005050511894495 ),
        IntegrationPointType( 0.9491079123427585245262, 0.9491079123427585245262, 0.7415311855993944398639, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.2797053914892766679015 ),
        IntegrationPointType( 0.9491079123427585245262, 0.9491079123427585245262, 0.9491079123427585245262, 0.1294849661688696932706 * 0.1294849661688696932706 * 0.1294849661688696932706 ),

    }
};

HexahedronGaussLegendreIntegrationPoints8::IntegrationPointsArrayType
HexahedronGaussLegendreIntegrationPoints8::msIntegrationPoints =
{
    {
        IntegrationPointType( -0.9602898564975362316836, -0.9602898564975362316836, -0.9602898564975362316836, 0.101228536290376259153 * 0.101228536290376259153 * 0.101228536290376259153 ),
        IntegrationPointType( -0.9602898564975362316836, -0.9602898564975362316836, -0.7966664774136267395916, 0.101228536290376259153 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( -0.9602898564975362316836, -0.9602898564975362316836, -0.5255324099163289858177, 0.101228536290376259153 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( -0.9602898564975362316836, -0.9602898564975362316836, -0.1834346424956498049395, 0.101228536290376259153 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( -0.9602898564975362316836, -0.9602898564975362316836, 0.1834346424956498049395, 0.101228536290376259153 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( -0.9602898564975362316836, -0.9602898564975362316836, 0.5255324099163289858177, 0.101228536290376259153 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( -0.9602898564975362316836, -0.9602898564975362316836, 0.7966664774136267395916, 0.101228536290376259153 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( -0.9602898564975362316836, -0.9602898564975362316836, 0.9602898564975362316836, 0.101228536290376259153 * 0.101228536290376259153 * 0.101228536290376259153 ),

        IntegrationPointType( -0.9602898564975362316836, -0.7966664774136267395916, -0.9602898564975362316836, 0.101228536290376259153 * 0.222381034453374470544 * 0.101228536290376259153 ),
        IntegrationPointType( -0.9602898564975362316836, -0.7966664774136267395916, -0.7966664774136267395916, 0.101228536290376259153 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( -0.9602898564975362316836, -0.7966664774136267395916, -0.5255324099163289858177, 0.101228536290376259153 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( -0.9602898564975362316836, -0.7966664774136267395916, -0.1834346424956498049395, 0.101228536290376259153 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( -0.9602898564975362316836, -0.7966664774136267395916, 0.1834346424956498049395, 0.101228536290376259153 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( -0.9602898564975362316836, -0.7966664774136267395916, 0.5255324099163289858177, 0.101228536290376259153 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( -0.9602898564975362316836, -0.7966664774136267395916, 0.7966664774136267395916, 0.101228536290376259153 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( -0.9602898564975362316836, -0.7966664774136267395916, 0.9602898564975362316836, 0.101228536290376259153 * 0.222381034453374470544 * 0.101228536290376259153 ),

        IntegrationPointType( -0.9602898564975362316836, -0.5255324099163289858177, -0.9602898564975362316836, 0.101228536290376259153 * 0.313706645877887287338 * 0.101228536290376259153 ),
        IntegrationPointType( -0.9602898564975362316836, -0.5255324099163289858177, -0.7966664774136267395916, 0.101228536290376259153 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( -0.9602898564975362316836, -0.5255324099163289858177, -0.5255324099163289858177, 0.101228536290376259153 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( -0.9602898564975362316836, -0.5255324099163289858177, -0.1834346424956498049395, 0.101228536290376259153 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( -0.9602898564975362316836, -0.5255324099163289858177, 0.1834346424956498049395, 0.101228536290376259153 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( -0.9602898564975362316836, -0.5255324099163289858177, 0.5255324099163289858177, 0.101228536290376259153 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( -0.9602898564975362316836, -0.5255324099163289858177, 0.7966664774136267395916, 0.101228536290376259153 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( -0.9602898564975362316836, -0.5255324099163289858177, 0.9602898564975362316836, 0.101228536290376259153 * 0.313706645877887287338 * 0.101228536290376259153 ),

        IntegrationPointType( -0.9602898564975362316836, -0.1834346424956498049395, -0.9602898564975362316836, 0.101228536290376259153 * 0.362683783378361982965 * 0.101228536290376259153 ),
        IntegrationPointType( -0.9602898564975362316836, -0.1834346424956498049395, -0.7966664774136267395916, 0.101228536290376259153 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( -0.9602898564975362316836, -0.1834346424956498049395, -0.5255324099163289858177, 0.101228536290376259153 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( -0.9602898564975362316836, -0.1834346424956498049395, -0.1834346424956498049395, 0.101228536290376259153 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( -0.9602898564975362316836, -0.1834346424956498049395, 0.1834346424956498049395, 0.101228536290376259153 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( -0.9602898564975362316836, -0.1834346424956498049395, 0.5255324099163289858177, 0.101228536290376259153 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( -0.9602898564975362316836, -0.1834346424956498049395, 0.7966664774136267395916, 0.101228536290376259153 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( -0.9602898564975362316836, -0.1834346424956498049395, 0.9602898564975362316836, 0.101228536290376259153 * 0.362683783378361982965 * 0.101228536290376259153 ),

        IntegrationPointType( -0.9602898564975362316836, 0.1834346424956498049395, -0.9602898564975362316836, 0.101228536290376259153 * 0.362683783378361982965 * 0.101228536290376259153 ),
        IntegrationPointType( -0.9602898564975362316836, 0.1834346424956498049395, -0.7966664774136267395916, 0.101228536290376259153 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( -0.9602898564975362316836, 0.1834346424956498049395, -0.5255324099163289858177, 0.101228536290376259153 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( -0.9602898564975362316836, 0.1834346424956498049395, -0.1834346424956498049395, 0.101228536290376259153 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( -0.9602898564975362316836, 0.1834346424956498049395, 0.1834346424956498049395, 0.101228536290376259153 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( -0.9602898564975362316836, 0.1834346424956498049395, 0.5255324099163289858177, 0.101228536290376259153 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( -0.9602898564975362316836, 0.1834346424956498049395, 0.7966664774136267395916, 0.101228536290376259153 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( -0.9602898564975362316836, 0.1834346424956498049395, 0.9602898564975362316836, 0.101228536290376259153 * 0.362683783378361982965 * 0.101228536290376259153 ),

        IntegrationPointType( -0.9602898564975362316836, 0.5255324099163289858177, -0.9602898564975362316836, 0.101228536290376259153 * 0.313706645877887287338 * 0.101228536290376259153 ),
        IntegrationPointType( -0.9602898564975362316836, 0.5255324099163289858177, -0.7966664774136267395916, 0.101228536290376259153 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( -0.9602898564975362316836, 0.5255324099163289858177, -0.5255324099163289858177, 0.101228536290376259153 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( -0.9602898564975362316836, 0.5255324099163289858177, -0.1834346424956498049395, 0.101228536290376259153 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( -0.9602898564975362316836, 0.5255324099163289858177, 0.1834346424956498049395, 0.101228536290376259153 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( -0.9602898564975362316836, 0.5255324099163289858177, 0.5255324099163289858177, 0.101228536290376259153 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( -0.9602898564975362316836, 0.5255324099163289858177, 0.7966664774136267395916, 0.101228536290376259153 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( -0.9602898564975362316836, 0.5255324099163289858177, 0.9602898564975362316836, 0.101228536290376259153 * 0.313706645877887287338 * 0.101228536290376259153 ),

        IntegrationPointType( -0.9602898564975362316836, 0.7966664774136267395916, -0.9602898564975362316836, 0.101228536290376259153 * 0.222381034453374470544 * 0.101228536290376259153 ),
        IntegrationPointType( -0.9602898564975362316836, 0.7966664774136267395916, -0.7966664774136267395916, 0.101228536290376259153 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( -0.9602898564975362316836, 0.7966664774136267395916, -0.5255324099163289858177, 0.101228536290376259153 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( -0.9602898564975362316836, 0.7966664774136267395916, -0.1834346424956498049395, 0.101228536290376259153 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( -0.9602898564975362316836, 0.7966664774136267395916, 0.1834346424956498049395, 0.101228536290376259153 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( -0.9602898564975362316836, 0.7966664774136267395916, 0.5255324099163289858177, 0.101228536290376259153 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( -0.9602898564975362316836, 0.7966664774136267395916, 0.7966664774136267395916, 0.101228536290376259153 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( -0.9602898564975362316836, 0.7966664774136267395916, 0.9602898564975362316836, 0.101228536290376259153 * 0.222381034453374470544 * 0.101228536290376259153 ),

        IntegrationPointType( -0.9602898564975362316836, 0.9602898564975362316836, -0.9602898564975362316836, 0.101228536290376259153 * 0.101228536290376259153 * 0.101228536290376259153 ),
        IntegrationPointType( -0.9602898564975362316836, 0.9602898564975362316836, -0.7966664774136267395916, 0.101228536290376259153 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( -0.9602898564975362316836, 0.9602898564975362316836, -0.5255324099163289858177, 0.101228536290376259153 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( -0.9602898564975362316836, 0.9602898564975362316836, -0.1834346424956498049395, 0.101228536290376259153 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( -0.9602898564975362316836, 0.9602898564975362316836, 0.1834346424956498049395, 0.101228536290376259153 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( -0.9602898564975362316836, 0.9602898564975362316836, 0.5255324099163289858177, 0.101228536290376259153 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( -0.9602898564975362316836, 0.9602898564975362316836, 0.7966664774136267395916, 0.101228536290376259153 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( -0.9602898564975362316836, 0.9602898564975362316836, 0.9602898564975362316836, 0.101228536290376259153 * 0.101228536290376259153 * 0.101228536290376259153 ),

        IntegrationPointType( -0.7966664774136267395916, -0.9602898564975362316836, -0.9602898564975362316836, 0.222381034453374470544 * 0.101228536290376259153 * 0.101228536290376259153 ),
        IntegrationPointType( -0.7966664774136267395916, -0.9602898564975362316836, -0.7966664774136267395916, 0.222381034453374470544 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( -0.7966664774136267395916, -0.9602898564975362316836, -0.5255324099163289858177, 0.222381034453374470544 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( -0.7966664774136267395916, -0.9602898564975362316836, -0.1834346424956498049395, 0.222381034453374470544 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( -0.7966664774136267395916, -0.9602898564975362316836, 0.1834346424956498049395, 0.222381034453374470544 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( -0.7966664774136267395916, -0.9602898564975362316836, 0.5255324099163289858177, 0.222381034453374470544 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( -0.7966664774136267395916, -0.9602898564975362316836, 0.7966664774136267395916, 0.222381034453374470544 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( -0.7966664774136267395916, -0.9602898564975362316836, 0.9602898564975362316836, 0.222381034453374470544 * 0.101228536290376259153 * 0.101228536290376259153 ),

        IntegrationPointType( -0.7966664774136267395916, -0.7966664774136267395916, -0.9602898564975362316836, 0.222381034453374470544 * 0.222381034453374470544 * 0.101228536290376259153 ),
        IntegrationPointType( -0.7966664774136267395916, -0.7966664774136267395916, -0.7966664774136267395916, 0.222381034453374470544 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( -0.7966664774136267395916, -0.7966664774136267395916, -0.5255324099163289858177, 0.222381034453374470544 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( -0.7966664774136267395916, -0.7966664774136267395916, -0.1834346424956498049395, 0.222381034453374470544 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( -0.7966664774136267395916, -0.7966664774136267395916, 0.1834346424956498049395, 0.222381034453374470544 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( -0.7966664774136267395916, -0.7966664774136267395916, 0.5255324099163289858177, 0.222381034453374470544 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( -0.7966664774136267395916, -0.7966664774136267395916, 0.7966664774136267395916, 0.222381034453374470544 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( -0.7966664774136267395916, -0.7966664774136267395916, 0.9602898564975362316836, 0.222381034453374470544 * 0.222381034453374470544 * 0.101228536290376259153 ),

        IntegrationPointType( -0.7966664774136267395916, -0.5255324099163289858177, -0.9602898564975362316836, 0.222381034453374470544 * 0.313706645877887287338 * 0.101228536290376259153 ),
        IntegrationPointType( -0.7966664774136267395916, -0.5255324099163289858177, -0.7966664774136267395916, 0.222381034453374470544 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( -0.7966664774136267395916, -0.5255324099163289858177, -0.5255324099163289858177, 0.222381034453374470544 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( -0.7966664774136267395916, -0.5255324099163289858177, -0.1834346424956498049395, 0.222381034453374470544 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( -0.7966664774136267395916, -0.5255324099163289858177, 0.1834346424956498049395, 0.222381034453374470544 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( -0.7966664774136267395916, -0.5255324099163289858177, 0.5255324099163289858177, 0.222381034453374470544 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( -0.7966664774136267395916, -0.5255324099163289858177, 0.7966664774136267395916, 0.222381034453374470544 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( -0.7966664774136267395916, -0.5255324099163289858177, 0.9602898564975362316836, 0.222381034453374470544 * 0.313706645877887287338 * 0.101228536290376259153 ),

        IntegrationPointType( -0.7966664774136267395916, -0.1834346424956498049395, -0.9602898564975362316836, 0.222381034453374470544 * 0.362683783378361982965 * 0.101228536290376259153 ),
        IntegrationPointType( -0.7966664774136267395916, -0.1834346424956498049395, -0.7966664774136267395916, 0.222381034453374470544 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( -0.7966664774136267395916, -0.1834346424956498049395, -0.5255324099163289858177, 0.222381034453374470544 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( -0.7966664774136267395916, -0.1834346424956498049395, -0.1834346424956498049395, 0.222381034453374470544 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( -0.7966664774136267395916, -0.1834346424956498049395, 0.1834346424956498049395, 0.222381034453374470544 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( -0.7966664774136267395916, -0.1834346424956498049395, 0.5255324099163289858177, 0.222381034453374470544 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( -0.7966664774136267395916, -0.1834346424956498049395, 0.7966664774136267395916, 0.222381034453374470544 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( -0.7966664774136267395916, -0.1834346424956498049395, 0.9602898564975362316836, 0.222381034453374470544 * 0.362683783378361982965 * 0.101228536290376259153 ),

        IntegrationPointType( -0.7966664774136267395916, 0.1834346424956498049395, -0.9602898564975362316836, 0.222381034453374470544 * 0.362683783378361982965 * 0.101228536290376259153 ),
        IntegrationPointType( -0.7966664774136267395916, 0.1834346424956498049395, -0.7966664774136267395916, 0.222381034453374470544 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( -0.7966664774136267395916, 0.1834346424956498049395, -0.5255324099163289858177, 0.222381034453374470544 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( -0.7966664774136267395916, 0.1834346424956498049395, -0.1834346424956498049395, 0.222381034453374470544 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( -0.7966664774136267395916, 0.1834346424956498049395, 0.1834346424956498049395, 0.222381034453374470544 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( -0.7966664774136267395916, 0.1834346424956498049395, 0.5255324099163289858177, 0.222381034453374470544 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( -0.7966664774136267395916, 0.1834346424956498049395, 0.7966664774136267395916, 0.222381034453374470544 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( -0.7966664774136267395916, 0.1834346424956498049395, 0.9602898564975362316836, 0.222381034453374470544 * 0.362683783378361982965 * 0.101228536290376259153 ),

        IntegrationPointType( -0.7966664774136267395916, 0.5255324099163289858177, -0.9602898564975362316836, 0.222381034453374470544 * 0.313706645877887287338 * 0.101228536290376259153 ),
        IntegrationPointType( -0.7966664774136267395916, 0.5255324099163289858177, -0.7966664774136267395916, 0.222381034453374470544 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( -0.7966664774136267395916, 0.5255324099163289858177, -0.5255324099163289858177, 0.222381034453374470544 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( -0.7966664774136267395916, 0.5255324099163289858177, -0.1834346424956498049395, 0.222381034453374470544 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( -0.7966664774136267395916, 0.5255324099163289858177, 0.1834346424956498049395, 0.222381034453374470544 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( -0.7966664774136267395916, 0.5255324099163289858177, 0.5255324099163289858177, 0.222381034453374470544 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( -0.7966664774136267395916, 0.5255324099163289858177, 0.7966664774136267395916, 0.222381034453374470544 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( -0.7966664774136267395916, 0.5255324099163289858177, 0.9602898564975362316836, 0.222381034453374470544 * 0.313706645877887287338 * 0.101228536290376259153 ),

        IntegrationPointType( -0.7966664774136267395916, 0.7966664774136267395916, -0.9602898564975362316836, 0.222381034453374470544 * 0.222381034453374470544 * 0.101228536290376259153 ),
        IntegrationPointType( -0.7966664774136267395916, 0.7966664774136267395916, -0.7966664774136267395916, 0.222381034453374470544 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( -0.7966664774136267395916, 0.7966664774136267395916, -0.5255324099163289858177, 0.222381034453374470544 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( -0.7966664774136267395916, 0.7966664774136267395916, -0.1834346424956498049395, 0.222381034453374470544 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( -0.7966664774136267395916, 0.7966664774136267395916, 0.1834346424956498049395, 0.222381034453374470544 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( -0.7966664774136267395916, 0.7966664774136267395916, 0.5255324099163289858177, 0.222381034453374470544 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( -0.7966664774136267395916, 0.7966664774136267395916, 0.7966664774136267395916, 0.222381034453374470544 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( -0.7966664774136267395916, 0.7966664774136267395916, 0.9602898564975362316836, 0.222381034453374470544 * 0.222381034453374470544 * 0.101228536290376259153 ),

        IntegrationPointType( -0.7966664774136267395916, 0.9602898564975362316836, -0.9602898564975362316836, 0.222381034453374470544 * 0.101228536290376259153 * 0.101228536290376259153 ),
        IntegrationPointType( -0.7966664774136267395916, 0.9602898564975362316836, -0.7966664774136267395916, 0.222381034453374470544 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( -0.7966664774136267395916, 0.9602898564975362316836, -0.5255324099163289858177, 0.222381034453374470544 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( -0.7966664774136267395916, 0.9602898564975362316836, -0.1834346424956498049395, 0.222381034453374470544 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( -0.7966664774136267395916, 0.9602898564975362316836, 0.1834346424956498049395, 0.222381034453374470544 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( -0.7966664774136267395916, 0.9602898564975362316836, 0.5255324099163289858177, 0.222381034453374470544 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( -0.7966664774136267395916, 0.9602898564975362316836, 0.7966664774136267395916, 0.222381034453374470544 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( -0.7966664774136267395916, 0.9602898564975362316836, 0.9602898564975362316836, 0.222381034453374470544 * 0.101228536290376259153 * 0.101228536290376259153 ),

        IntegrationPointType( -0.5255324099163289858177, -0.9602898564975362316836, -0.9602898564975362316836, 0.313706645877887287338 * 0.101228536290376259153 * 0.101228536290376259153 ),
        IntegrationPointType( -0.5255324099163289858177, -0.9602898564975362316836, -0.7966664774136267395916, 0.313706645877887287338 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( -0.5255324099163289858177, -0.9602898564975362316836, -0.5255324099163289858177, 0.313706645877887287338 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( -0.5255324099163289858177, -0.9602898564975362316836, -0.1834346424956498049395, 0.313706645877887287338 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( -0.5255324099163289858177, -0.9602898564975362316836, 0.1834346424956498049395, 0.313706645877887287338 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( -0.5255324099163289858177, -0.9602898564975362316836, 0.5255324099163289858177, 0.313706645877887287338 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( -0.5255324099163289858177, -0.9602898564975362316836, 0.7966664774136267395916, 0.313706645877887287338 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( -0.5255324099163289858177, -0.9602898564975362316836, 0.9602898564975362316836, 0.313706645877887287338 * 0.101228536290376259153 * 0.101228536290376259153 ),

        IntegrationPointType( -0.5255324099163289858177, -0.7966664774136267395916, -0.9602898564975362316836, 0.313706645877887287338 * 0.222381034453374470544 * 0.101228536290376259153 ),
        IntegrationPointType( -0.5255324099163289858177, -0.7966664774136267395916, -0.7966664774136267395916, 0.313706645877887287338 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( -0.5255324099163289858177, -0.7966664774136267395916, -0.5255324099163289858177, 0.313706645877887287338 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( -0.5255324099163289858177, -0.7966664774136267395916, -0.1834346424956498049395, 0.313706645877887287338 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( -0.5255324099163289858177, -0.7966664774136267395916, 0.1834346424956498049395, 0.313706645877887287338 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( -0.5255324099163289858177, -0.7966664774136267395916, 0.5255324099163289858177, 0.313706645877887287338 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( -0.5255324099163289858177, -0.7966664774136267395916, 0.7966664774136267395916, 0.313706645877887287338 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( -0.5255324099163289858177, -0.7966664774136267395916, 0.9602898564975362316836, 0.313706645877887287338 * 0.222381034453374470544 * 0.101228536290376259153 ),

        IntegrationPointType( -0.5255324099163289858177, -0.5255324099163289858177, -0.9602898564975362316836, 0.313706645877887287338 * 0.313706645877887287338 * 0.101228536290376259153 ),
        IntegrationPointType( -0.5255324099163289858177, -0.5255324099163289858177, -0.7966664774136267395916, 0.313706645877887287338 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( -0.5255324099163289858177, -0.5255324099163289858177, -0.5255324099163289858177, 0.313706645877887287338 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( -0.5255324099163289858177, -0.5255324099163289858177, -0.1834346424956498049395, 0.313706645877887287338 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( -0.5255324099163289858177, -0.5255324099163289858177, 0.1834346424956498049395, 0.313706645877887287338 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( -0.5255324099163289858177, -0.5255324099163289858177, 0.5255324099163289858177, 0.313706645877887287338 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( -0.5255324099163289858177, -0.5255324099163289858177, 0.7966664774136267395916, 0.313706645877887287338 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( -0.5255324099163289858177, -0.5255324099163289858177, 0.9602898564975362316836, 0.313706645877887287338 * 0.313706645877887287338 * 0.101228536290376259153 ),

        IntegrationPointType( -0.5255324099163289858177, -0.1834346424956498049395, -0.9602898564975362316836, 0.313706645877887287338 * 0.362683783378361982965 * 0.101228536290376259153 ),
        IntegrationPointType( -0.5255324099163289858177, -0.1834346424956498049395, -0.7966664774136267395916, 0.313706645877887287338 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( -0.5255324099163289858177, -0.1834346424956498049395, -0.5255324099163289858177, 0.313706645877887287338 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( -0.5255324099163289858177, -0.1834346424956498049395, -0.1834346424956498049395, 0.313706645877887287338 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( -0.5255324099163289858177, -0.1834346424956498049395, 0.1834346424956498049395, 0.313706645877887287338 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( -0.5255324099163289858177, -0.1834346424956498049395, 0.5255324099163289858177, 0.313706645877887287338 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( -0.5255324099163289858177, -0.1834346424956498049395, 0.7966664774136267395916, 0.313706645877887287338 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( -0.5255324099163289858177, -0.1834346424956498049395, 0.9602898564975362316836, 0.313706645877887287338 * 0.362683783378361982965 * 0.101228536290376259153 ),

        IntegrationPointType( -0.5255324099163289858177, 0.1834346424956498049395, -0.9602898564975362316836, 0.313706645877887287338 * 0.362683783378361982965 * 0.101228536290376259153 ),
        IntegrationPointType( -0.5255324099163289858177, 0.1834346424956498049395, -0.7966664774136267395916, 0.313706645877887287338 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( -0.5255324099163289858177, 0.1834346424956498049395, -0.5255324099163289858177, 0.313706645877887287338 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( -0.5255324099163289858177, 0.1834346424956498049395, -0.1834346424956498049395, 0.313706645877887287338 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( -0.5255324099163289858177, 0.1834346424956498049395, 0.1834346424956498049395, 0.313706645877887287338 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( -0.5255324099163289858177, 0.1834346424956498049395, 0.5255324099163289858177, 0.313706645877887287338 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( -0.5255324099163289858177, 0.1834346424956498049395, 0.7966664774136267395916, 0.313706645877887287338 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( -0.5255324099163289858177, 0.1834346424956498049395, 0.9602898564975362316836, 0.313706645877887287338 * 0.362683783378361982965 * 0.101228536290376259153 ),

        IntegrationPointType( -0.5255324099163289858177, 0.5255324099163289858177, -0.9602898564975362316836, 0.313706645877887287338 * 0.313706645877887287338 * 0.101228536290376259153 ),
        IntegrationPointType( -0.5255324099163289858177, 0.5255324099163289858177, -0.7966664774136267395916, 0.313706645877887287338 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( -0.5255324099163289858177, 0.5255324099163289858177, -0.5255324099163289858177, 0.313706645877887287338 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( -0.5255324099163289858177, 0.5255324099163289858177, -0.1834346424956498049395, 0.313706645877887287338 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( -0.5255324099163289858177, 0.5255324099163289858177, 0.1834346424956498049395, 0.313706645877887287338 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( -0.5255324099163289858177, 0.5255324099163289858177, 0.5255324099163289858177, 0.313706645877887287338 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( -0.5255324099163289858177, 0.5255324099163289858177, 0.7966664774136267395916, 0.313706645877887287338 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( -0.5255324099163289858177, 0.5255324099163289858177, 0.9602898564975362316836, 0.313706645877887287338 * 0.313706645877887287338 * 0.101228536290376259153 ),

        IntegrationPointType( -0.5255324099163289858177, 0.7966664774136267395916, -0.9602898564975362316836, 0.313706645877887287338 * 0.222381034453374470544 * 0.101228536290376259153 ),
        IntegrationPointType( -0.5255324099163289858177, 0.7966664774136267395916, -0.7966664774136267395916, 0.313706645877887287338 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( -0.5255324099163289858177, 0.7966664774136267395916, -0.5255324099163289858177, 0.313706645877887287338 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( -0.5255324099163289858177, 0.7966664774136267395916, -0.1834346424956498049395, 0.313706645877887287338 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( -0.5255324099163289858177, 0.7966664774136267395916, 0.1834346424956498049395, 0.313706645877887287338 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( -0.5255324099163289858177, 0.7966664774136267395916, 0.5255324099163289858177, 0.313706645877887287338 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( -0.5255324099163289858177, 0.7966664774136267395916, 0.7966664774136267395916, 0.313706645877887287338 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( -0.5255324099163289858177, 0.7966664774136267395916, 0.9602898564975362316836, 0.313706645877887287338 * 0.222381034453374470544 * 0.101228536290376259153 ),

        IntegrationPointType( -0.5255324099163289858177, 0.9602898564975362316836, -0.9602898564975362316836, 0.313706645877887287338 * 0.101228536290376259153 * 0.101228536290376259153 ),
        IntegrationPointType( -0.5255324099163289858177, 0.9602898564975362316836, -0.7966664774136267395916, 0.313706645877887287338 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( -0.5255324099163289858177, 0.9602898564975362316836, -0.5255324099163289858177, 0.313706645877887287338 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( -0.5255324099163289858177, 0.9602898564975362316836, -0.1834346424956498049395, 0.313706645877887287338 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( -0.5255324099163289858177, 0.9602898564975362316836, 0.1834346424956498049395, 0.313706645877887287338 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( -0.5255324099163289858177, 0.9602898564975362316836, 0.5255324099163289858177, 0.313706645877887287338 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( -0.5255324099163289858177, 0.9602898564975362316836, 0.7966664774136267395916, 0.313706645877887287338 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( -0.5255324099163289858177, 0.9602898564975362316836, 0.9602898564975362316836, 0.313706645877887287338 * 0.101228536290376259153 * 0.101228536290376259153 ),

        IntegrationPointType( -0.1834346424956498049395, -0.9602898564975362316836, -0.9602898564975362316836, 0.362683783378361982965 * 0.101228536290376259153 * 0.101228536290376259153 ),
        IntegrationPointType( -0.1834346424956498049395, -0.9602898564975362316836, -0.7966664774136267395916, 0.362683783378361982965 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( -0.1834346424956498049395, -0.9602898564975362316836, -0.5255324099163289858177, 0.362683783378361982965 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( -0.1834346424956498049395, -0.9602898564975362316836, -0.1834346424956498049395, 0.362683783378361982965 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( -0.1834346424956498049395, -0.9602898564975362316836, 0.1834346424956498049395, 0.362683783378361982965 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( -0.1834346424956498049395, -0.9602898564975362316836, 0.5255324099163289858177, 0.362683783378361982965 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( -0.1834346424956498049395, -0.9602898564975362316836, 0.7966664774136267395916, 0.362683783378361982965 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( -0.1834346424956498049395, -0.9602898564975362316836, 0.9602898564975362316836, 0.362683783378361982965 * 0.101228536290376259153 * 0.101228536290376259153 ),

        IntegrationPointType( -0.1834346424956498049395, -0.7966664774136267395916, -0.9602898564975362316836, 0.362683783378361982965 * 0.222381034453374470544 * 0.101228536290376259153 ),
        IntegrationPointType( -0.1834346424956498049395, -0.7966664774136267395916, -0.7966664774136267395916, 0.362683783378361982965 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( -0.1834346424956498049395, -0.7966664774136267395916, -0.5255324099163289858177, 0.362683783378361982965 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( -0.1834346424956498049395, -0.7966664774136267395916, -0.1834346424956498049395, 0.362683783378361982965 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( -0.1834346424956498049395, -0.7966664774136267395916, 0.1834346424956498049395, 0.362683783378361982965 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( -0.1834346424956498049395, -0.7966664774136267395916, 0.5255324099163289858177, 0.362683783378361982965 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( -0.1834346424956498049395, -0.7966664774136267395916, 0.7966664774136267395916, 0.362683783378361982965 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( -0.1834346424956498049395, -0.7966664774136267395916, 0.9602898564975362316836, 0.362683783378361982965 * 0.222381034453374470544 * 0.101228536290376259153 ),

        IntegrationPointType( -0.1834346424956498049395, -0.5255324099163289858177, -0.9602898564975362316836, 0.362683783378361982965 * 0.313706645877887287338 * 0.101228536290376259153 ),
        IntegrationPointType( -0.1834346424956498049395, -0.5255324099163289858177, -0.7966664774136267395916, 0.362683783378361982965 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( -0.1834346424956498049395, -0.5255324099163289858177, -0.5255324099163289858177, 0.362683783378361982965 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( -0.1834346424956498049395, -0.5255324099163289858177, -0.1834346424956498049395, 0.362683783378361982965 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( -0.1834346424956498049395, -0.5255324099163289858177, 0.1834346424956498049395, 0.362683783378361982965 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( -0.1834346424956498049395, -0.5255324099163289858177, 0.5255324099163289858177, 0.362683783378361982965 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( -0.1834346424956498049395, -0.5255324099163289858177, 0.7966664774136267395916, 0.362683783378361982965 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( -0.1834346424956498049395, -0.5255324099163289858177, 0.9602898564975362316836, 0.362683783378361982965 * 0.313706645877887287338 * 0.101228536290376259153 ),

        IntegrationPointType( -0.1834346424956498049395, -0.1834346424956498049395, -0.9602898564975362316836, 0.362683783378361982965 * 0.362683783378361982965 * 0.101228536290376259153 ),
        IntegrationPointType( -0.1834346424956498049395, -0.1834346424956498049395, -0.7966664774136267395916, 0.362683783378361982965 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( -0.1834346424956498049395, -0.1834346424956498049395, -0.5255324099163289858177, 0.362683783378361982965 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( -0.1834346424956498049395, -0.1834346424956498049395, -0.1834346424956498049395, 0.362683783378361982965 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( -0.1834346424956498049395, -0.1834346424956498049395, 0.1834346424956498049395, 0.362683783378361982965 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( -0.1834346424956498049395, -0.1834346424956498049395, 0.5255324099163289858177, 0.362683783378361982965 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( -0.1834346424956498049395, -0.1834346424956498049395, 0.7966664774136267395916, 0.362683783378361982965 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( -0.1834346424956498049395, -0.1834346424956498049395, 0.9602898564975362316836, 0.362683783378361982965 * 0.362683783378361982965 * 0.101228536290376259153 ),

        IntegrationPointType( -0.1834346424956498049395, 0.1834346424956498049395, -0.9602898564975362316836, 0.362683783378361982965 * 0.362683783378361982965 * 0.101228536290376259153 ),
        IntegrationPointType( -0.1834346424956498049395, 0.1834346424956498049395, -0.7966664774136267395916, 0.362683783378361982965 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( -0.1834346424956498049395, 0.1834346424956498049395, -0.5255324099163289858177, 0.362683783378361982965 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( -0.1834346424956498049395, 0.1834346424956498049395, -0.1834346424956498049395, 0.362683783378361982965 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( -0.1834346424956498049395, 0.1834346424956498049395, 0.1834346424956498049395, 0.362683783378361982965 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( -0.1834346424956498049395, 0.1834346424956498049395, 0.5255324099163289858177, 0.362683783378361982965 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( -0.1834346424956498049395, 0.1834346424956498049395, 0.7966664774136267395916, 0.362683783378361982965 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( -0.1834346424956498049395, 0.1834346424956498049395, 0.9602898564975362316836, 0.362683783378361982965 * 0.362683783378361982965 * 0.101228536290376259153 ),

        IntegrationPointType( -0.1834346424956498049395, 0.5255324099163289858177, -0.9602898564975362316836, 0.362683783378361982965 * 0.313706645877887287338 * 0.101228536290376259153 ),
        IntegrationPointType( -0.1834346424956498049395, 0.5255324099163289858177, -0.7966664774136267395916, 0.362683783378361982965 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( -0.1834346424956498049395, 0.5255324099163289858177, -0.5255324099163289858177, 0.362683783378361982965 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( -0.1834346424956498049395, 0.5255324099163289858177, -0.1834346424956498049395, 0.362683783378361982965 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( -0.1834346424956498049395, 0.5255324099163289858177, 0.1834346424956498049395, 0.362683783378361982965 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( -0.1834346424956498049395, 0.5255324099163289858177, 0.5255324099163289858177, 0.362683783378361982965 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( -0.1834346424956498049395, 0.5255324099163289858177, 0.7966664774136267395916, 0.362683783378361982965 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( -0.1834346424956498049395, 0.5255324099163289858177, 0.9602898564975362316836, 0.362683783378361982965 * 0.313706645877887287338 * 0.101228536290376259153 ),

        IntegrationPointType( -0.1834346424956498049395, 0.7966664774136267395916, -0.9602898564975362316836, 0.362683783378361982965 * 0.222381034453374470544 * 0.101228536290376259153 ),
        IntegrationPointType( -0.1834346424956498049395, 0.7966664774136267395916, -0.7966664774136267395916, 0.362683783378361982965 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( -0.1834346424956498049395, 0.7966664774136267395916, -0.5255324099163289858177, 0.362683783378361982965 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( -0.1834346424956498049395, 0.7966664774136267395916, -0.1834346424956498049395, 0.362683783378361982965 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( -0.1834346424956498049395, 0.7966664774136267395916, 0.1834346424956498049395, 0.362683783378361982965 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( -0.1834346424956498049395, 0.7966664774136267395916, 0.5255324099163289858177, 0.362683783378361982965 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( -0.1834346424956498049395, 0.7966664774136267395916, 0.7966664774136267395916, 0.362683783378361982965 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( -0.1834346424956498049395, 0.7966664774136267395916, 0.9602898564975362316836, 0.362683783378361982965 * 0.222381034453374470544 * 0.101228536290376259153 ),

        IntegrationPointType( -0.1834346424956498049395, 0.9602898564975362316836, -0.9602898564975362316836, 0.362683783378361982965 * 0.101228536290376259153 * 0.101228536290376259153 ),
        IntegrationPointType( -0.1834346424956498049395, 0.9602898564975362316836, -0.7966664774136267395916, 0.362683783378361982965 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( -0.1834346424956498049395, 0.9602898564975362316836, -0.5255324099163289858177, 0.362683783378361982965 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( -0.1834346424956498049395, 0.9602898564975362316836, -0.1834346424956498049395, 0.362683783378361982965 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( -0.1834346424956498049395, 0.9602898564975362316836, 0.1834346424956498049395, 0.362683783378361982965 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( -0.1834346424956498049395, 0.9602898564975362316836, 0.5255324099163289858177, 0.362683783378361982965 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( -0.1834346424956498049395, 0.9602898564975362316836, 0.7966664774136267395916, 0.362683783378361982965 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( -0.1834346424956498049395, 0.9602898564975362316836, 0.9602898564975362316836, 0.362683783378361982965 * 0.101228536290376259153 * 0.101228536290376259153 ),

        IntegrationPointType( 0.1834346424956498049395, -0.9602898564975362316836, -0.9602898564975362316836, 0.362683783378361982965 * 0.101228536290376259153 * 0.101228536290376259153 ),
        IntegrationPointType( 0.1834346424956498049395, -0.9602898564975362316836, -0.7966664774136267395916, 0.362683783378361982965 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( 0.1834346424956498049395, -0.9602898564975362316836, -0.5255324099163289858177, 0.362683783378361982965 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( 0.1834346424956498049395, -0.9602898564975362316836, -0.1834346424956498049395, 0.362683783378361982965 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( 0.1834346424956498049395, -0.9602898564975362316836, 0.1834346424956498049395, 0.362683783378361982965 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( 0.1834346424956498049395, -0.9602898564975362316836, 0.5255324099163289858177, 0.362683783378361982965 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( 0.1834346424956498049395, -0.9602898564975362316836, 0.7966664774136267395916, 0.362683783378361982965 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( 0.1834346424956498049395, -0.9602898564975362316836, 0.9602898564975362316836, 0.362683783378361982965 * 0.101228536290376259153 * 0.101228536290376259153 ),

        IntegrationPointType( 0.1834346424956498049395, -0.7966664774136267395916, -0.9602898564975362316836, 0.362683783378361982965 * 0.222381034453374470544 * 0.101228536290376259153 ),
        IntegrationPointType( 0.1834346424956498049395, -0.7966664774136267395916, -0.7966664774136267395916, 0.362683783378361982965 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( 0.1834346424956498049395, -0.7966664774136267395916, -0.5255324099163289858177, 0.362683783378361982965 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( 0.1834346424956498049395, -0.7966664774136267395916, -0.1834346424956498049395, 0.362683783378361982965 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( 0.1834346424956498049395, -0.7966664774136267395916, 0.1834346424956498049395, 0.362683783378361982965 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( 0.1834346424956498049395, -0.7966664774136267395916, 0.5255324099163289858177, 0.362683783378361982965 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( 0.1834346424956498049395, -0.7966664774136267395916, 0.7966664774136267395916, 0.362683783378361982965 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( 0.1834346424956498049395, -0.7966664774136267395916, 0.9602898564975362316836, 0.362683783378361982965 * 0.222381034453374470544 * 0.101228536290376259153 ),

        IntegrationPointType( 0.1834346424956498049395, -0.5255324099163289858177, -0.9602898564975362316836, 0.362683783378361982965 * 0.313706645877887287338 * 0.101228536290376259153 ),
        IntegrationPointType( 0.1834346424956498049395, -0.5255324099163289858177, -0.7966664774136267395916, 0.362683783378361982965 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( 0.1834346424956498049395, -0.5255324099163289858177, -0.5255324099163289858177, 0.362683783378361982965 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( 0.1834346424956498049395, -0.5255324099163289858177, -0.1834346424956498049395, 0.362683783378361982965 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( 0.1834346424956498049395, -0.5255324099163289858177, 0.1834346424956498049395, 0.362683783378361982965 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( 0.1834346424956498049395, -0.5255324099163289858177, 0.5255324099163289858177, 0.362683783378361982965 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( 0.1834346424956498049395, -0.5255324099163289858177, 0.7966664774136267395916, 0.362683783378361982965 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( 0.1834346424956498049395, -0.5255324099163289858177, 0.9602898564975362316836, 0.362683783378361982965 * 0.313706645877887287338 * 0.101228536290376259153 ),

        IntegrationPointType( 0.1834346424956498049395, -0.1834346424956498049395, -0.9602898564975362316836, 0.362683783378361982965 * 0.362683783378361982965 * 0.101228536290376259153 ),
        IntegrationPointType( 0.1834346424956498049395, -0.1834346424956498049395, -0.7966664774136267395916, 0.362683783378361982965 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( 0.1834346424956498049395, -0.1834346424956498049395, -0.5255324099163289858177, 0.362683783378361982965 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( 0.1834346424956498049395, -0.1834346424956498049395, -0.1834346424956498049395, 0.362683783378361982965 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( 0.1834346424956498049395, -0.1834346424956498049395, 0.1834346424956498049395, 0.362683783378361982965 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( 0.1834346424956498049395, -0.1834346424956498049395, 0.5255324099163289858177, 0.362683783378361982965 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( 0.1834346424956498049395, -0.1834346424956498049395, 0.7966664774136267395916, 0.362683783378361982965 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( 0.1834346424956498049395, -0.1834346424956498049395, 0.9602898564975362316836, 0.362683783378361982965 * 0.362683783378361982965 * 0.101228536290376259153 ),

        IntegrationPointType( 0.1834346424956498049395, 0.1834346424956498049395, -0.9602898564975362316836, 0.362683783378361982965 * 0.362683783378361982965 * 0.101228536290376259153 ),
        IntegrationPointType( 0.1834346424956498049395, 0.1834346424956498049395, -0.7966664774136267395916, 0.362683783378361982965 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( 0.1834346424956498049395, 0.1834346424956498049395, -0.5255324099163289858177, 0.362683783378361982965 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( 0.1834346424956498049395, 0.1834346424956498049395, -0.1834346424956498049395, 0.362683783378361982965 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( 0.1834346424956498049395, 0.1834346424956498049395, 0.1834346424956498049395, 0.362683783378361982965 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( 0.1834346424956498049395, 0.1834346424956498049395, 0.5255324099163289858177, 0.362683783378361982965 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( 0.1834346424956498049395, 0.1834346424956498049395, 0.7966664774136267395916, 0.362683783378361982965 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( 0.1834346424956498049395, 0.1834346424956498049395, 0.9602898564975362316836, 0.362683783378361982965 * 0.362683783378361982965 * 0.101228536290376259153 ),

        IntegrationPointType( 0.1834346424956498049395, 0.5255324099163289858177, -0.9602898564975362316836, 0.362683783378361982965 * 0.313706645877887287338 * 0.101228536290376259153 ),
        IntegrationPointType( 0.1834346424956498049395, 0.5255324099163289858177, -0.7966664774136267395916, 0.362683783378361982965 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( 0.1834346424956498049395, 0.5255324099163289858177, -0.5255324099163289858177, 0.362683783378361982965 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( 0.1834346424956498049395, 0.5255324099163289858177, -0.1834346424956498049395, 0.362683783378361982965 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( 0.1834346424956498049395, 0.5255324099163289858177, 0.1834346424956498049395, 0.362683783378361982965 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( 0.1834346424956498049395, 0.5255324099163289858177, 0.5255324099163289858177, 0.362683783378361982965 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( 0.1834346424956498049395, 0.5255324099163289858177, 0.7966664774136267395916, 0.362683783378361982965 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( 0.1834346424956498049395, 0.5255324099163289858177, 0.9602898564975362316836, 0.362683783378361982965 * 0.313706645877887287338 * 0.101228536290376259153 ),

        IntegrationPointType( 0.1834346424956498049395, 0.7966664774136267395916, -0.9602898564975362316836, 0.362683783378361982965 * 0.222381034453374470544 * 0.101228536290376259153 ),
        IntegrationPointType( 0.1834346424956498049395, 0.7966664774136267395916, -0.7966664774136267395916, 0.362683783378361982965 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( 0.1834346424956498049395, 0.7966664774136267395916, -0.5255324099163289858177, 0.362683783378361982965 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( 0.1834346424956498049395, 0.7966664774136267395916, -0.1834346424956498049395, 0.362683783378361982965 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( 0.1834346424956498049395, 0.7966664774136267395916, 0.1834346424956498049395, 0.362683783378361982965 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( 0.1834346424956498049395, 0.7966664774136267395916, 0.5255324099163289858177, 0.362683783378361982965 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( 0.1834346424956498049395, 0.7966664774136267395916, 0.7966664774136267395916, 0.362683783378361982965 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( 0.1834346424956498049395, 0.7966664774136267395916, 0.9602898564975362316836, 0.362683783378361982965 * 0.222381034453374470544 * 0.101228536290376259153 ),

        IntegrationPointType( 0.1834346424956498049395, 0.9602898564975362316836, -0.9602898564975362316836, 0.362683783378361982965 * 0.101228536290376259153 * 0.101228536290376259153 ),
        IntegrationPointType( 0.1834346424956498049395, 0.9602898564975362316836, -0.7966664774136267395916, 0.362683783378361982965 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( 0.1834346424956498049395, 0.9602898564975362316836, -0.5255324099163289858177, 0.362683783378361982965 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( 0.1834346424956498049395, 0.9602898564975362316836, -0.1834346424956498049395, 0.362683783378361982965 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( 0.1834346424956498049395, 0.9602898564975362316836, 0.1834346424956498049395, 0.362683783378361982965 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( 0.1834346424956498049395, 0.9602898564975362316836, 0.5255324099163289858177, 0.362683783378361982965 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( 0.1834346424956498049395, 0.9602898564975362316836, 0.7966664774136267395916, 0.362683783378361982965 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( 0.1834346424956498049395, 0.9602898564975362316836, 0.9602898564975362316836, 0.362683783378361982965 * 0.101228536290376259153 * 0.101228536290376259153 ),

        IntegrationPointType( 0.5255324099163289858177, -0.9602898564975362316836, -0.9602898564975362316836, 0.313706645877887287338 * 0.101228536290376259153 * 0.101228536290376259153 ),
        IntegrationPointType( 0.5255324099163289858177, -0.9602898564975362316836, -0.7966664774136267395916, 0.313706645877887287338 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( 0.5255324099163289858177, -0.9602898564975362316836, -0.5255324099163289858177, 0.313706645877887287338 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( 0.5255324099163289858177, -0.9602898564975362316836, -0.1834346424956498049395, 0.313706645877887287338 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( 0.5255324099163289858177, -0.9602898564975362316836, 0.1834346424956498049395, 0.313706645877887287338 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( 0.5255324099163289858177, -0.9602898564975362316836, 0.5255324099163289858177, 0.313706645877887287338 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( 0.5255324099163289858177, -0.9602898564975362316836, 0.7966664774136267395916, 0.313706645877887287338 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( 0.5255324099163289858177, -0.9602898564975362316836, 0.9602898564975362316836, 0.313706645877887287338 * 0.101228536290376259153 * 0.101228536290376259153 ),

        IntegrationPointType( 0.5255324099163289858177, -0.7966664774136267395916, -0.9602898564975362316836, 0.313706645877887287338 * 0.222381034453374470544 * 0.101228536290376259153 ),
        IntegrationPointType( 0.5255324099163289858177, -0.7966664774136267395916, -0.7966664774136267395916, 0.313706645877887287338 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( 0.5255324099163289858177, -0.7966664774136267395916, -0.5255324099163289858177, 0.313706645877887287338 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( 0.5255324099163289858177, -0.7966664774136267395916, -0.1834346424956498049395, 0.313706645877887287338 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( 0.5255324099163289858177, -0.7966664774136267395916, 0.1834346424956498049395, 0.313706645877887287338 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( 0.5255324099163289858177, -0.7966664774136267395916, 0.5255324099163289858177, 0.313706645877887287338 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( 0.5255324099163289858177, -0.7966664774136267395916, 0.7966664774136267395916, 0.313706645877887287338 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( 0.5255324099163289858177, -0.7966664774136267395916, 0.9602898564975362316836, 0.313706645877887287338 * 0.222381034453374470544 * 0.101228536290376259153 ),

        IntegrationPointType( 0.5255324099163289858177, -0.5255324099163289858177, -0.9602898564975362316836, 0.313706645877887287338 * 0.313706645877887287338 * 0.101228536290376259153 ),
        IntegrationPointType( 0.5255324099163289858177, -0.5255324099163289858177, -0.7966664774136267395916, 0.313706645877887287338 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( 0.5255324099163289858177, -0.5255324099163289858177, -0.5255324099163289858177, 0.313706645877887287338 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( 0.5255324099163289858177, -0.5255324099163289858177, -0.1834346424956498049395, 0.313706645877887287338 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( 0.5255324099163289858177, -0.5255324099163289858177, 0.1834346424956498049395, 0.313706645877887287338 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( 0.5255324099163289858177, -0.5255324099163289858177, 0.5255324099163289858177, 0.313706645877887287338 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( 0.5255324099163289858177, -0.5255324099163289858177, 0.7966664774136267395916, 0.313706645877887287338 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( 0.5255324099163289858177, -0.5255324099163289858177, 0.9602898564975362316836, 0.313706645877887287338 * 0.313706645877887287338 * 0.101228536290376259153 ),

        IntegrationPointType( 0.5255324099163289858177, -0.1834346424956498049395, -0.9602898564975362316836, 0.313706645877887287338 * 0.362683783378361982965 * 0.101228536290376259153 ),
        IntegrationPointType( 0.5255324099163289858177, -0.1834346424956498049395, -0.7966664774136267395916, 0.313706645877887287338 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( 0.5255324099163289858177, -0.1834346424956498049395, -0.5255324099163289858177, 0.313706645877887287338 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( 0.5255324099163289858177, -0.1834346424956498049395, -0.1834346424956498049395, 0.313706645877887287338 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( 0.5255324099163289858177, -0.1834346424956498049395, 0.1834346424956498049395, 0.313706645877887287338 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( 0.5255324099163289858177, -0.1834346424956498049395, 0.5255324099163289858177, 0.313706645877887287338 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( 0.5255324099163289858177, -0.1834346424956498049395, 0.7966664774136267395916, 0.313706645877887287338 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( 0.5255324099163289858177, -0.1834346424956498049395, 0.9602898564975362316836, 0.313706645877887287338 * 0.362683783378361982965 * 0.101228536290376259153 ),

        IntegrationPointType( 0.5255324099163289858177, 0.1834346424956498049395, -0.9602898564975362316836, 0.313706645877887287338 * 0.362683783378361982965 * 0.101228536290376259153 ),
        IntegrationPointType( 0.5255324099163289858177, 0.1834346424956498049395, -0.7966664774136267395916, 0.313706645877887287338 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( 0.5255324099163289858177, 0.1834346424956498049395, -0.5255324099163289858177, 0.313706645877887287338 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( 0.5255324099163289858177, 0.1834346424956498049395, -0.1834346424956498049395, 0.313706645877887287338 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( 0.5255324099163289858177, 0.1834346424956498049395, 0.1834346424956498049395, 0.313706645877887287338 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( 0.5255324099163289858177, 0.1834346424956498049395, 0.5255324099163289858177, 0.313706645877887287338 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( 0.5255324099163289858177, 0.1834346424956498049395, 0.7966664774136267395916, 0.313706645877887287338 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( 0.5255324099163289858177, 0.1834346424956498049395, 0.9602898564975362316836, 0.313706645877887287338 * 0.362683783378361982965 * 0.101228536290376259153 ),

        IntegrationPointType( 0.5255324099163289858177, 0.5255324099163289858177, -0.9602898564975362316836, 0.313706645877887287338 * 0.313706645877887287338 * 0.101228536290376259153 ),
        IntegrationPointType( 0.5255324099163289858177, 0.5255324099163289858177, -0.7966664774136267395916, 0.313706645877887287338 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( 0.5255324099163289858177, 0.5255324099163289858177, -0.5255324099163289858177, 0.313706645877887287338 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( 0.5255324099163289858177, 0.5255324099163289858177, -0.1834346424956498049395, 0.313706645877887287338 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( 0.5255324099163289858177, 0.5255324099163289858177, 0.1834346424956498049395, 0.313706645877887287338 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( 0.5255324099163289858177, 0.5255324099163289858177, 0.5255324099163289858177, 0.313706645877887287338 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( 0.5255324099163289858177, 0.5255324099163289858177, 0.7966664774136267395916, 0.313706645877887287338 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( 0.5255324099163289858177, 0.5255324099163289858177, 0.9602898564975362316836, 0.313706645877887287338 * 0.313706645877887287338 * 0.101228536290376259153 ),

        IntegrationPointType( 0.5255324099163289858177, 0.7966664774136267395916, -0.9602898564975362316836, 0.313706645877887287338 * 0.222381034453374470544 * 0.101228536290376259153 ),
        IntegrationPointType( 0.5255324099163289858177, 0.7966664774136267395916, -0.7966664774136267395916, 0.313706645877887287338 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( 0.5255324099163289858177, 0.7966664774136267395916, -0.5255324099163289858177, 0.313706645877887287338 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( 0.5255324099163289858177, 0.7966664774136267395916, -0.1834346424956498049395, 0.313706645877887287338 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( 0.5255324099163289858177, 0.7966664774136267395916, 0.1834346424956498049395, 0.313706645877887287338 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( 0.5255324099163289858177, 0.7966664774136267395916, 0.5255324099163289858177, 0.313706645877887287338 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( 0.5255324099163289858177, 0.7966664774136267395916, 0.7966664774136267395916, 0.313706645877887287338 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( 0.5255324099163289858177, 0.7966664774136267395916, 0.9602898564975362316836, 0.313706645877887287338 * 0.222381034453374470544 * 0.101228536290376259153 ),

        IntegrationPointType( 0.5255324099163289858177, 0.9602898564975362316836, -0.9602898564975362316836, 0.313706645877887287338 * 0.101228536290376259153 * 0.101228536290376259153 ),
        IntegrationPointType( 0.5255324099163289858177, 0.9602898564975362316836, -0.7966664774136267395916, 0.313706645877887287338 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( 0.5255324099163289858177, 0.9602898564975362316836, -0.5255324099163289858177, 0.313706645877887287338 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( 0.5255324099163289858177, 0.9602898564975362316836, -0.1834346424956498049395, 0.313706645877887287338 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( 0.5255324099163289858177, 0.9602898564975362316836, 0.1834346424956498049395, 0.313706645877887287338 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( 0.5255324099163289858177, 0.9602898564975362316836, 0.5255324099163289858177, 0.313706645877887287338 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( 0.5255324099163289858177, 0.9602898564975362316836, 0.7966664774136267395916, 0.313706645877887287338 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( 0.5255324099163289858177, 0.9602898564975362316836, 0.9602898564975362316836, 0.313706645877887287338 * 0.101228536290376259153 * 0.101228536290376259153 ),

        IntegrationPointType( 0.7966664774136267395916, -0.9602898564975362316836, -0.9602898564975362316836, 0.222381034453374470544 * 0.101228536290376259153 * 0.101228536290376259153 ),
        IntegrationPointType( 0.7966664774136267395916, -0.9602898564975362316836, -0.7966664774136267395916, 0.222381034453374470544 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( 0.7966664774136267395916, -0.9602898564975362316836, -0.5255324099163289858177, 0.222381034453374470544 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( 0.7966664774136267395916, -0.9602898564975362316836, -0.1834346424956498049395, 0.222381034453374470544 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( 0.7966664774136267395916, -0.9602898564975362316836, 0.1834346424956498049395, 0.222381034453374470544 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( 0.7966664774136267395916, -0.9602898564975362316836, 0.5255324099163289858177, 0.222381034453374470544 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( 0.7966664774136267395916, -0.9602898564975362316836, 0.7966664774136267395916, 0.222381034453374470544 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( 0.7966664774136267395916, -0.9602898564975362316836, 0.9602898564975362316836, 0.222381034453374470544 * 0.101228536290376259153 * 0.101228536290376259153 ),

        IntegrationPointType( 0.7966664774136267395916, -0.7966664774136267395916, -0.9602898564975362316836, 0.222381034453374470544 * 0.222381034453374470544 * 0.101228536290376259153 ),
        IntegrationPointType( 0.7966664774136267395916, -0.7966664774136267395916, -0.7966664774136267395916, 0.222381034453374470544 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( 0.7966664774136267395916, -0.7966664774136267395916, -0.5255324099163289858177, 0.222381034453374470544 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( 0.7966664774136267395916, -0.7966664774136267395916, -0.1834346424956498049395, 0.222381034453374470544 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( 0.7966664774136267395916, -0.7966664774136267395916, 0.1834346424956498049395, 0.222381034453374470544 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( 0.7966664774136267395916, -0.7966664774136267395916, 0.5255324099163289858177, 0.222381034453374470544 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( 0.7966664774136267395916, -0.7966664774136267395916, 0.7966664774136267395916, 0.222381034453374470544 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( 0.7966664774136267395916, -0.7966664774136267395916, 0.9602898564975362316836, 0.222381034453374470544 * 0.222381034453374470544 * 0.101228536290376259153 ),

        IntegrationPointType( 0.7966664774136267395916, -0.5255324099163289858177, -0.9602898564975362316836, 0.222381034453374470544 * 0.313706645877887287338 * 0.101228536290376259153 ),
        IntegrationPointType( 0.7966664774136267395916, -0.5255324099163289858177, -0.7966664774136267395916, 0.222381034453374470544 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( 0.7966664774136267395916, -0.5255324099163289858177, -0.5255324099163289858177, 0.222381034453374470544 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( 0.7966664774136267395916, -0.5255324099163289858177, -0.1834346424956498049395, 0.222381034453374470544 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( 0.7966664774136267395916, -0.5255324099163289858177, 0.1834346424956498049395, 0.222381034453374470544 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( 0.7966664774136267395916, -0.5255324099163289858177, 0.5255324099163289858177, 0.222381034453374470544 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( 0.7966664774136267395916, -0.5255324099163289858177, 0.7966664774136267395916, 0.222381034453374470544 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( 0.7966664774136267395916, -0.5255324099163289858177, 0.9602898564975362316836, 0.222381034453374470544 * 0.313706645877887287338 * 0.101228536290376259153 ),

        IntegrationPointType( 0.7966664774136267395916, -0.1834346424956498049395, -0.9602898564975362316836, 0.222381034453374470544 * 0.362683783378361982965 * 0.101228536290376259153 ),
        IntegrationPointType( 0.7966664774136267395916, -0.1834346424956498049395, -0.7966664774136267395916, 0.222381034453374470544 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( 0.7966664774136267395916, -0.1834346424956498049395, -0.5255324099163289858177, 0.222381034453374470544 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( 0.7966664774136267395916, -0.1834346424956498049395, -0.1834346424956498049395, 0.222381034453374470544 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( 0.7966664774136267395916, -0.1834346424956498049395, 0.1834346424956498049395, 0.222381034453374470544 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( 0.7966664774136267395916, -0.1834346424956498049395, 0.5255324099163289858177, 0.222381034453374470544 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( 0.7966664774136267395916, -0.1834346424956498049395, 0.7966664774136267395916, 0.222381034453374470544 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( 0.7966664774136267395916, -0.1834346424956498049395, 0.9602898564975362316836, 0.222381034453374470544 * 0.362683783378361982965 * 0.101228536290376259153 ),

        IntegrationPointType( 0.7966664774136267395916, 0.1834346424956498049395, -0.9602898564975362316836, 0.222381034453374470544 * 0.362683783378361982965 * 0.101228536290376259153 ),
        IntegrationPointType( 0.7966664774136267395916, 0.1834346424956498049395, -0.7966664774136267395916, 0.222381034453374470544 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( 0.7966664774136267395916, 0.1834346424956498049395, -0.5255324099163289858177, 0.222381034453374470544 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( 0.7966664774136267395916, 0.1834346424956498049395, -0.1834346424956498049395, 0.222381034453374470544 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( 0.7966664774136267395916, 0.1834346424956498049395, 0.1834346424956498049395, 0.222381034453374470544 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( 0.7966664774136267395916, 0.1834346424956498049395, 0.5255324099163289858177, 0.222381034453374470544 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( 0.7966664774136267395916, 0.1834346424956498049395, 0.7966664774136267395916, 0.222381034453374470544 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( 0.7966664774136267395916, 0.1834346424956498049395, 0.9602898564975362316836, 0.222381034453374470544 * 0.362683783378361982965 * 0.101228536290376259153 ),

        IntegrationPointType( 0.7966664774136267395916, 0.5255324099163289858177, -0.9602898564975362316836, 0.222381034453374470544 * 0.313706645877887287338 * 0.101228536290376259153 ),
        IntegrationPointType( 0.7966664774136267395916, 0.5255324099163289858177, -0.7966664774136267395916, 0.222381034453374470544 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( 0.7966664774136267395916, 0.5255324099163289858177, -0.5255324099163289858177, 0.222381034453374470544 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( 0.7966664774136267395916, 0.5255324099163289858177, -0.1834346424956498049395, 0.222381034453374470544 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( 0.7966664774136267395916, 0.5255324099163289858177, 0.1834346424956498049395, 0.222381034453374470544 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( 0.7966664774136267395916, 0.5255324099163289858177, 0.5255324099163289858177, 0.222381034453374470544 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( 0.7966664774136267395916, 0.5255324099163289858177, 0.7966664774136267395916, 0.222381034453374470544 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( 0.7966664774136267395916, 0.5255324099163289858177, 0.9602898564975362316836, 0.222381034453374470544 * 0.313706645877887287338 * 0.101228536290376259153 ),

        IntegrationPointType( 0.7966664774136267395916, 0.7966664774136267395916, -0.9602898564975362316836, 0.222381034453374470544 * 0.222381034453374470544 * 0.101228536290376259153 ),
        IntegrationPointType( 0.7966664774136267395916, 0.7966664774136267395916, -0.7966664774136267395916, 0.222381034453374470544 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( 0.7966664774136267395916, 0.7966664774136267395916, -0.5255324099163289858177, 0.222381034453374470544 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( 0.7966664774136267395916, 0.7966664774136267395916, -0.1834346424956498049395, 0.222381034453374470544 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( 0.7966664774136267395916, 0.7966664774136267395916, 0.1834346424956498049395, 0.222381034453374470544 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( 0.7966664774136267395916, 0.7966664774136267395916, 0.5255324099163289858177, 0.222381034453374470544 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( 0.7966664774136267395916, 0.7966664774136267395916, 0.7966664774136267395916, 0.222381034453374470544 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( 0.7966664774136267395916, 0.7966664774136267395916, 0.9602898564975362316836, 0.222381034453374470544 * 0.222381034453374470544 * 0.101228536290376259153 ),

        IntegrationPointType( 0.7966664774136267395916, 0.9602898564975362316836, -0.9602898564975362316836, 0.222381034453374470544 * 0.101228536290376259153 * 0.101228536290376259153 ),
        IntegrationPointType( 0.7966664774136267395916, 0.9602898564975362316836, -0.7966664774136267395916, 0.222381034453374470544 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( 0.7966664774136267395916, 0.9602898564975362316836, -0.5255324099163289858177, 0.222381034453374470544 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( 0.7966664774136267395916, 0.9602898564975362316836, -0.1834346424956498049395, 0.222381034453374470544 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( 0.7966664774136267395916, 0.9602898564975362316836, 0.1834346424956498049395, 0.222381034453374470544 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( 0.7966664774136267395916, 0.9602898564975362316836, 0.5255324099163289858177, 0.222381034453374470544 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( 0.7966664774136267395916, 0.9602898564975362316836, 0.7966664774136267395916, 0.222381034453374470544 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( 0.7966664774136267395916, 0.9602898564975362316836, 0.9602898564975362316836, 0.222381034453374470544 * 0.101228536290376259153 * 0.101228536290376259153 ),

        IntegrationPointType( 0.9602898564975362316836, -0.9602898564975362316836, -0.9602898564975362316836, 0.101228536290376259153 * 0.101228536290376259153 * 0.101228536290376259153 ),
        IntegrationPointType( 0.9602898564975362316836, -0.9602898564975362316836, -0.7966664774136267395916, 0.101228536290376259153 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( 0.9602898564975362316836, -0.9602898564975362316836, -0.5255324099163289858177, 0.101228536290376259153 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( 0.9602898564975362316836, -0.9602898564975362316836, -0.1834346424956498049395, 0.101228536290376259153 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( 0.9602898564975362316836, -0.9602898564975362316836, 0.1834346424956498049395, 0.101228536290376259153 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( 0.9602898564975362316836, -0.9602898564975362316836, 0.5255324099163289858177, 0.101228536290376259153 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( 0.9602898564975362316836, -0.9602898564975362316836, 0.7966664774136267395916, 0.101228536290376259153 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( 0.9602898564975362316836, -0.9602898564975362316836, 0.9602898564975362316836, 0.101228536290376259153 * 0.101228536290376259153 * 0.101228536290376259153 ),

        IntegrationPointType( 0.9602898564975362316836, -0.7966664774136267395916, -0.9602898564975362316836, 0.101228536290376259153 * 0.222381034453374470544 * 0.101228536290376259153 ),
        IntegrationPointType( 0.9602898564975362316836, -0.7966664774136267395916, -0.7966664774136267395916, 0.101228536290376259153 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( 0.9602898564975362316836, -0.7966664774136267395916, -0.5255324099163289858177, 0.101228536290376259153 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( 0.9602898564975362316836, -0.7966664774136267395916, -0.1834346424956498049395, 0.101228536290376259153 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( 0.9602898564975362316836, -0.7966664774136267395916, 0.1834346424956498049395, 0.101228536290376259153 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( 0.9602898564975362316836, -0.7966664774136267395916, 0.5255324099163289858177, 0.101228536290376259153 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( 0.9602898564975362316836, -0.7966664774136267395916, 0.7966664774136267395916, 0.101228536290376259153 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( 0.9602898564975362316836, -0.7966664774136267395916, 0.9602898564975362316836, 0.101228536290376259153 * 0.222381034453374470544 * 0.101228536290376259153 ),

        IntegrationPointType( 0.9602898564975362316836, -0.5255324099163289858177, -0.9602898564975362316836, 0.101228536290376259153 * 0.313706645877887287338 * 0.101228536290376259153 ),
        IntegrationPointType( 0.9602898564975362316836, -0.5255324099163289858177, -0.7966664774136267395916, 0.101228536290376259153 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( 0.9602898564975362316836, -0.5255324099163289858177, -0.5255324099163289858177, 0.101228536290376259153 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( 0.9602898564975362316836, -0.5255324099163289858177, -0.1834346424956498049395, 0.101228536290376259153 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( 0.9602898564975362316836, -0.5255324099163289858177, 0.1834346424956498049395, 0.101228536290376259153 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( 0.9602898564975362316836, -0.5255324099163289858177, 0.5255324099163289858177, 0.101228536290376259153 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( 0.9602898564975362316836, -0.5255324099163289858177, 0.7966664774136267395916, 0.101228536290376259153 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( 0.9602898564975362316836, -0.5255324099163289858177, 0.9602898564975362316836, 0.101228536290376259153 * 0.313706645877887287338 * 0.101228536290376259153 ),

        IntegrationPointType( 0.9602898564975362316836, -0.1834346424956498049395, -0.9602898564975362316836, 0.101228536290376259153 * 0.362683783378361982965 * 0.101228536290376259153 ),
        IntegrationPointType( 0.9602898564975362316836, -0.1834346424956498049395, -0.7966664774136267395916, 0.101228536290376259153 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( 0.9602898564975362316836, -0.1834346424956498049395, -0.5255324099163289858177, 0.101228536290376259153 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( 0.9602898564975362316836, -0.1834346424956498049395, -0.1834346424956498049395, 0.101228536290376259153 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( 0.9602898564975362316836, -0.1834346424956498049395, 0.1834346424956498049395, 0.101228536290376259153 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( 0.9602898564975362316836, -0.1834346424956498049395, 0.5255324099163289858177, 0.101228536290376259153 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( 0.9602898564975362316836, -0.1834346424956498049395, 0.7966664774136267395916, 0.101228536290376259153 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( 0.9602898564975362316836, -0.1834346424956498049395, 0.9602898564975362316836, 0.101228536290376259153 * 0.362683783378361982965 * 0.101228536290376259153 ),

        IntegrationPointType( 0.9602898564975362316836, 0.1834346424956498049395, -0.9602898564975362316836, 0.101228536290376259153 * 0.362683783378361982965 * 0.101228536290376259153 ),
        IntegrationPointType( 0.9602898564975362316836, 0.1834346424956498049395, -0.7966664774136267395916, 0.101228536290376259153 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( 0.9602898564975362316836, 0.1834346424956498049395, -0.5255324099163289858177, 0.101228536290376259153 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( 0.9602898564975362316836, 0.1834346424956498049395, -0.1834346424956498049395, 0.101228536290376259153 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( 0.9602898564975362316836, 0.1834346424956498049395, 0.1834346424956498049395, 0.101228536290376259153 * 0.362683783378361982965 * 0.362683783378361982965 ),
        IntegrationPointType( 0.9602898564975362316836, 0.1834346424956498049395, 0.5255324099163289858177, 0.101228536290376259153 * 0.362683783378361982965 * 0.313706645877887287338 ),
        IntegrationPointType( 0.9602898564975362316836, 0.1834346424956498049395, 0.7966664774136267395916, 0.101228536290376259153 * 0.362683783378361982965 * 0.222381034453374470544 ),
        IntegrationPointType( 0.9602898564975362316836, 0.1834346424956498049395, 0.9602898564975362316836, 0.101228536290376259153 * 0.362683783378361982965 * 0.101228536290376259153 ),

        IntegrationPointType( 0.9602898564975362316836, 0.5255324099163289858177, -0.9602898564975362316836, 0.101228536290376259153 * 0.313706645877887287338 * 0.101228536290376259153 ),
        IntegrationPointType( 0.9602898564975362316836, 0.5255324099163289858177, -0.7966664774136267395916, 0.101228536290376259153 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( 0.9602898564975362316836, 0.5255324099163289858177, -0.5255324099163289858177, 0.101228536290376259153 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( 0.9602898564975362316836, 0.5255324099163289858177, -0.1834346424956498049395, 0.101228536290376259153 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( 0.9602898564975362316836, 0.5255324099163289858177, 0.1834346424956498049395, 0.101228536290376259153 * 0.313706645877887287338 * 0.362683783378361982965 ),
        IntegrationPointType( 0.9602898564975362316836, 0.5255324099163289858177, 0.5255324099163289858177, 0.101228536290376259153 * 0.313706645877887287338 * 0.313706645877887287338 ),
        IntegrationPointType( 0.9602898564975362316836, 0.5255324099163289858177, 0.7966664774136267395916, 0.101228536290376259153 * 0.313706645877887287338 * 0.222381034453374470544 ),
        IntegrationPointType( 0.9602898564975362316836, 0.5255324099163289858177, 0.9602898564975362316836, 0.101228536290376259153 * 0.313706645877887287338 * 0.101228536290376259153 ),

        IntegrationPointType( 0.9602898564975362316836, 0.7966664774136267395916, -0.9602898564975362316836, 0.101228536290376259153 * 0.222381034453374470544 * 0.101228536290376259153 ),
        IntegrationPointType( 0.9602898564975362316836, 0.7966664774136267395916, -0.7966664774136267395916, 0.101228536290376259153 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( 0.9602898564975362316836, 0.7966664774136267395916, -0.5255324099163289858177, 0.101228536290376259153 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( 0.9602898564975362316836, 0.7966664774136267395916, -0.1834346424956498049395, 0.101228536290376259153 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( 0.9602898564975362316836, 0.7966664774136267395916, 0.1834346424956498049395, 0.101228536290376259153 * 0.222381034453374470544 * 0.362683783378361982965 ),
        IntegrationPointType( 0.9602898564975362316836, 0.7966664774136267395916, 0.5255324099163289858177, 0.101228536290376259153 * 0.222381034453374470544 * 0.313706645877887287338 ),
        IntegrationPointType( 0.9602898564975362316836, 0.7966664774136267395916, 0.7966664774136267395916, 0.101228536290376259153 * 0.222381034453374470544 * 0.222381034453374470544 ),
        IntegrationPointType( 0.9602898564975362316836, 0.7966664774136267395916, 0.9602898564975362316836, 0.101228536290376259153 * 0.222381034453374470544 * 0.101228536290376259153 ),

        IntegrationPointType( 0.9602898564975362316836, 0.9602898564975362316836, -0.9602898564975362316836, 0.101228536290376259153 * 0.101228536290376259153 * 0.101228536290376259153 ),
        IntegrationPointType( 0.9602898564975362316836, 0.9602898564975362316836, -0.7966664774136267395916, 0.101228536290376259153 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( 0.9602898564975362316836, 0.9602898564975362316836, -0.5255324099163289858177, 0.101228536290376259153 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( 0.9602898564975362316836, 0.9602898564975362316836, -0.1834346424956498049395, 0.101228536290376259153 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( 0.9602898564975362316836, 0.9602898564975362316836, 0.1834346424956498049395, 0.101228536290376259153 * 0.101228536290376259153 * 0.362683783378361982965 ),
        IntegrationPointType( 0.9602898564975362316836, 0.9602898564975362316836, 0.5255324099163289858177, 0.101228536290376259153 * 0.101228536290376259153 * 0.313706645877887287338 ),
        IntegrationPointType( 0.9602898564975362316836, 0.9602898564975362316836, 0.7966664774136267395916, 0.101228536290376259153 * 0.101228536290376259153 * 0.222381034453374470544 ),
        IntegrationPointType( 0.9602898564975362316836, 0.9602898564975362316836, 0.9602898564975362316836, 0.101228536290376259153 * 0.101228536290376259153 * 0.101228536290376259153 ),

    }
};

//Gauss-Lobatto

HexahedronGaussLobattoIntegrationPoints1::IntegrationPointsArrayType
HexahedronGaussLobattoIntegrationPoints1::msIntegrationPoints =
{
	{
		IntegrationPointType( 0.00 , 0.00, 0.00, 8.00 ),
	}
};

HexahedronGaussLobattoIntegrationPoints2::IntegrationPointsArrayType
HexahedronGaussLobattoIntegrationPoints2::msIntegrationPoints =
{
	{
		IntegrationPointType( -1.00 , -1.00, -1.00, 1.00 ),
		IntegrationPointType(  1.00 , -1.00, -1.00, 1.00 ),
		IntegrationPointType(  1.00 ,  1.00, -1.00, 1.00 ),
		IntegrationPointType( -1.00 ,  1.00, -1.00, 1.00 ),
		IntegrationPointType( -1.00 , -1.00,  1.00, 1.00 ),
		IntegrationPointType(  1.00 , -1.00,  1.00, 1.00 ),
		IntegrationPointType(  1.00 ,  1.00,  1.00, 1.00 ),
		IntegrationPointType( -1.00 ,  1.00,  1.00, 1.00 )
	}
};

HexahedronGaussLobattoIntegrationPoints3::IntegrationPointsArrayType
HexahedronGaussLobattoIntegrationPoints3::msIntegrationPoints =
{
    {
        IntegrationPointType( -1.00, -1.00, -1.00, 1.00/3.00 * 1.00/3.00 * 1.00/3.00 ),
        IntegrationPointType( -1.00, -1.00, 0.00, 1.00/3.00 * 1.00/3.00 * 4.00/3.00 ),
        IntegrationPointType( -1.00, -1.00, 1.00, 1.00/3.00 * 1.00/3.00 * 1.00/3.00 ),

        IntegrationPointType( -1.00, 0.00, -1.00, 1.00/3.00 * 4.00/3.00 * 1.00/3.00 ),
        IntegrationPointType( -1.00, 0.00, 0.00, 1.00/3.00 * 4.00/3.00 * 4.00/3.00 ),
        IntegrationPointType( -1.00, 0.00, 1.00, 1.00/3.00 * 4.00/3.00 * 1.00/3.00 ),

        IntegrationPointType( -1.00, 1.00, -1.00, 1.00/3.00 * 1.00/3.00 * 1.00/3.00 ),
        IntegrationPointType( -1.00, 1.00, 0.00, 1.00/3.00 * 1.00/3.00 * 4.00/3.00 ),
        IntegrationPointType( -1.00, 1.00, 1.00, 1.00/3.00 * 1.00/3.00 * 1.00/3.00 ),

        IntegrationPointType( 0.00, -1.00, -1.00, 4.00/3.00 * 1.00/3.00 * 1.00/3.00 ),
        IntegrationPointType( 0.00, -1.00, 0.00, 4.00/3.00 * 1.00/3.00 * 4.00/3.00 ),
        IntegrationPointType( 0.00, -1.00, 1.00, 4.00/3.00 * 1.00/3.00 * 1.00/3.00 ),

        IntegrationPointType( 0.00, 0.00, -1.00, 4.00/3.00 * 4.00/3.00 * 1.00/3.00 ),
        IntegrationPointType( 0.00, 0.00, 0.00, 4.00/3.00 * 4.00/3.00 * 4.00/3.00 ),
        IntegrationPointType( 0.00, 0.00, 1.00, 4.00/3.00 * 4.00/3.00 * 1.00/3.00 ),

        IntegrationPointType( 0.00, 1.00, -1.00, 4.00/3.00 * 1.00/3.00 * 1.00/3.00 ),
        IntegrationPointType( 0.00, 1.00, 0.00, 4.00/3.00 * 1.00/3.00 * 4.00/3.00 ),
        IntegrationPointType( 0.00, 1.00, 1.00, 4.00/3.00 * 1.00/3.00 * 1.00/3.00 ),

        IntegrationPointType( 1.00, -1.00, -1.00, 1.00/3.00 * 1.00/3.00 * 1.00/3.00 ),
        IntegrationPointType( 1.00, -1.00, 0.00, 1.00/3.00 * 1.00/3.00 * 4.00/3.00 ),
        IntegrationPointType( 1.00, -1.00, 1.00, 1.00/3.00 * 1.00/3.00 * 1.00/3.00 ),

        IntegrationPointType( 1.00, 0.00, -1.00, 1.00/3.00 * 4.00/3.00 * 1.00/3.00 ),
        IntegrationPointType( 1.00, 0.00, 0.00, 1.00/3.00 * 4.00/3.00 * 4.00/3.00 ),
        IntegrationPointType( 1.00, 0.00, 1.00, 1.00/3.00 * 4.00/3.00 * 1.00/3.00 ),

        IntegrationPointType( 1.00, 1.00, -1.00, 1.00/3.00 * 1.00/3.00 * 1.00/3.00 ),
        IntegrationPointType( 1.00, 1.00, 0.00, 1.00/3.00 * 1.00/3.00 * 4.00/3.00 ),
        IntegrationPointType( 1.00, 1.00, 1.00, 1.00/3.00 * 1.00/3.00 * 1.00/3.00 ),

    }
};

HexahedronGaussLobattoIntegrationPoints4::IntegrationPointsArrayType
HexahedronGaussLobattoIntegrationPoints4::msIntegrationPoints =
{
    {
        IntegrationPointType( -1.00, -1.00, -1.00, 1.00/6.00 * 1.00/6.00 * 1.00/6.00 ),
        IntegrationPointType( -1.00, -1.00, -std::sqrt(5.00)/5.00, 1.00/6.00 * 1.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( -1.00, -1.00, std::sqrt(5.00)/5.00, 1.00/6.00 * 1.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( -1.00, -1.00, 1.00, 1.00/6.00 * 1.00/6.00 * 1.00/6.00 ),

        IntegrationPointType( -1.00, -std::sqrt(5.00)/5.00, -1.00, 1.00/6.00 * 5.00/6.00 * 1.00/6.00 ),
        IntegrationPointType( -1.00, -std::sqrt(5.00)/5.00, -std::sqrt(5.00)/5.00, 1.00/6.00 * 5.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( -1.00, -std::sqrt(5.00)/5.00, std::sqrt(5.00)/5.00, 1.00/6.00 * 5.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( -1.00, -std::sqrt(5.00)/5.00, 1.00, 1.00/6.00 * 5.00/6.00 * 1.00/6.00 ),

        IntegrationPointType( -1.00, std::sqrt(5.00)/5.00, -1.00, 1.00/6.00 * 5.00/6.00 * 1.00/6.00 ),
        IntegrationPointType( -1.00, std::sqrt(5.00)/5.00, -std::sqrt(5.00)/5.00, 1.00/6.00 * 5.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( -1.00, std::sqrt(5.00)/5.00, std::sqrt(5.00)/5.00, 1.00/6.00 * 5.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( -1.00, std::sqrt(5.00)/5.00, 1.00, 1.00/6.00 * 5.00/6.00 * 1.00/6.00 ),

        IntegrationPointType( -1.00, 1.00, -1.00, 1.00/6.00 * 1.00/6.00 * 1.00/6.00 ),
        IntegrationPointType( -1.00, 1.00, -std::sqrt(5.00)/5.00, 1.00/6.00 * 1.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( -1.00, 1.00, std::sqrt(5.00)/5.00, 1.00/6.00 * 1.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( -1.00, 1.00, 1.00, 1.00/6.00 * 1.00/6.00 * 1.00/6.00 ),

        IntegrationPointType( -std::sqrt(5.00)/5.00, -1.00, -1.00, 5.00/6.00 * 1.00/6.00 * 1.00/6.00 ),
        IntegrationPointType( -std::sqrt(5.00)/5.00, -1.00, -std::sqrt(5.00)/5.00, 5.00/6.00 * 1.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( -std::sqrt(5.00)/5.00, -1.00, std::sqrt(5.00)/5.00, 5.00/6.00 * 1.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( -std::sqrt(5.00)/5.00, -1.00, 1.00, 5.00/6.00 * 1.00/6.00 * 1.00/6.00 ),

        IntegrationPointType( -std::sqrt(5.00)/5.00, -std::sqrt(5.00)/5.00, -1.00, 5.00/6.00 * 5.00/6.00 * 1.00/6.00 ),
        IntegrationPointType( -std::sqrt(5.00)/5.00, -std::sqrt(5.00)/5.00, -std::sqrt(5.00)/5.00, 5.00/6.00 * 5.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( -std::sqrt(5.00)/5.00, -std::sqrt(5.00)/5.00, std::sqrt(5.00)/5.00, 5.00/6.00 * 5.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( -std::sqrt(5.00)/5.00, -std::sqrt(5.00)/5.00, 1.00, 5.00/6.00 * 5.00/6.00 * 1.00/6.00 ),

        IntegrationPointType( -std::sqrt(5.00)/5.00, std::sqrt(5.00)/5.00, -1.00, 5.00/6.00 * 5.00/6.00 * 1.00/6.00 ),
        IntegrationPointType( -std::sqrt(5.00)/5.00, std::sqrt(5.00)/5.00, -std::sqrt(5.00)/5.00, 5.00/6.00 * 5.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( -std::sqrt(5.00)/5.00, std::sqrt(5.00)/5.00, std::sqrt(5.00)/5.00, 5.00/6.00 * 5.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( -std::sqrt(5.00)/5.00, std::sqrt(5.00)/5.00, 1.00, 5.00/6.00 * 5.00/6.00 * 1.00/6.00 ),

        IntegrationPointType( -std::sqrt(5.00)/5.00, 1.00, -1.00, 5.00/6.00 * 1.00/6.00 * 1.00/6.00 ),
        IntegrationPointType( -std::sqrt(5.00)/5.00, 1.00, -std::sqrt(5.00)/5.00, 5.00/6.00 * 1.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( -std::sqrt(5.00)/5.00, 1.00, std::sqrt(5.00)/5.00, 5.00/6.00 * 1.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( -std::sqrt(5.00)/5.00, 1.00, 1.00, 5.00/6.00 * 1.00/6.00 * 1.00/6.00 ),

        IntegrationPointType( std::sqrt(5.00)/5.00, -1.00, -1.00, 5.00/6.00 * 1.00/6.00 * 1.00/6.00 ),
        IntegrationPointType( std::sqrt(5.00)/5.00, -1.00, -std::sqrt(5.00)/5.00, 5.00/6.00 * 1.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( std::sqrt(5.00)/5.00, -1.00, std::sqrt(5.00)/5.00, 5.00/6.00 * 1.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( std::sqrt(5.00)/5.00, -1.00, 1.00, 5.00/6.00 * 1.00/6.00 * 1.00/6.00 ),

        IntegrationPointType( std::sqrt(5.00)/5.00, -std::sqrt(5.00)/5.00, -1.00, 5.00/6.00 * 5.00/6.00 * 1.00/6.00 ),
        IntegrationPointType( std::sqrt(5.00)/5.00, -std::sqrt(5.00)/5.00, -std::sqrt(5.00)/5.00, 5.00/6.00 * 5.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( std::sqrt(5.00)/5.00, -std::sqrt(5.00)/5.00, std::sqrt(5.00)/5.00, 5.00/6.00 * 5.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( std::sqrt(5.00)/5.00, -std::sqrt(5.00)/5.00, 1.00, 5.00/6.00 * 5.00/6.00 * 1.00/6.00 ),

        IntegrationPointType( std::sqrt(5.00)/5.00, std::sqrt(5.00)/5.00, -1.00, 5.00/6.00 * 5.00/6.00 * 1.00/6.00 ),
        IntegrationPointType( std::sqrt(5.00)/5.00, std::sqrt(5.00)/5.00, -std::sqrt(5.00)/5.00, 5.00/6.00 * 5.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( std::sqrt(5.00)/5.00, std::sqrt(5.00)/5.00, std::sqrt(5.00)/5.00, 5.00/6.00 * 5.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( std::sqrt(5.00)/5.00, std::sqrt(5.00)/5.00, 1.00, 5.00/6.00 * 5.00/6.00 * 1.00/6.00 ),

        IntegrationPointType( std::sqrt(5.00)/5.00, 1.00, -1.00, 5.00/6.00 * 1.00/6.00 * 1.00/6.00 ),
        IntegrationPointType( std::sqrt(5.00)/5.00, 1.00, -std::sqrt(5.00)/5.00, 5.00/6.00 * 1.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( std::sqrt(5.00)/5.00, 1.00, std::sqrt(5.00)/5.00, 5.00/6.00 * 1.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( std::sqrt(5.00)/5.00, 1.00, 1.00, 5.00/6.00 * 1.00/6.00 * 1.00/6.00 ),

        IntegrationPointType( 1.00, -1.00, -1.00, 1.00/6.00 * 1.00/6.00 * 1.00/6.00 ),
        IntegrationPointType( 1.00, -1.00, -std::sqrt(5.00)/5.00, 1.00/6.00 * 1.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( 1.00, -1.00, std::sqrt(5.00)/5.00, 1.00/6.00 * 1.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( 1.00, -1.00, 1.00, 1.00/6.00 * 1.00/6.00 * 1.00/6.00 ),

        IntegrationPointType( 1.00, -std::sqrt(5.00)/5.00, -1.00, 1.00/6.00 * 5.00/6.00 * 1.00/6.00 ),
        IntegrationPointType( 1.00, -std::sqrt(5.00)/5.00, -std::sqrt(5.00)/5.00, 1.00/6.00 * 5.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( 1.00, -std::sqrt(5.00)/5.00, std::sqrt(5.00)/5.00, 1.00/6.00 * 5.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( 1.00, -std::sqrt(5.00)/5.00, 1.00, 1.00/6.00 * 5.00/6.00 * 1.00/6.00 ),

        IntegrationPointType( 1.00, std::sqrt(5.00)/5.00, -1.00, 1.00/6.00 * 5.00/6.00 * 1.00/6.00 ),
        IntegrationPointType( 1.00, std::sqrt(5.00)/5.00, -std::sqrt(5.00)/5.00, 1.00/6.00 * 5.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( 1.00, std::sqrt(5.00)/5.00, std::sqrt(5.00)/5.00, 1.00/6.00 * 5.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( 1.00, std::sqrt(5.00)/5.00, 1.00, 1.00/6.00 * 5.00/6.00 * 1.00/6.00 ),

        IntegrationPointType( 1.00, 1.00, -1.00, 1.00/6.00 * 1.00/6.00 * 1.00/6.00 ),
        IntegrationPointType( 1.00, 1.00, -std::sqrt(5.00)/5.00, 1.00/6.00 * 1.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( 1.00, 1.00, std::sqrt(5.00)/5.00, 1.00/6.00 * 1.00/6.00 * 5.00/6.00 ),
        IntegrationPointType( 1.00, 1.00, 1.00, 1.00/6.00 * 1.00/6.00 * 1.00/6.00 ),

    }
};

HexahedronGaussLobattoIntegrationPoints5::IntegrationPointsArrayType
HexahedronGaussLobattoIntegrationPoints5::msIntegrationPoints =
{
    {
        IntegrationPointType( -1.00, -1.00, -1.00, 1.00/10.00 * 1.00/10.00 * 1.00/10.00 ),
        IntegrationPointType( -1.00, -1.00, -std::sqrt(21.00)/7.00, 1.00/10.00 * 1.00/10.00 * 49.00/90.00 ),
        IntegrationPointType( -1.00, -1.00, 0.00, 1.00/10.00 * 1.00/10.00 * 32.00/45.00 ),
        IntegrationPointType( -1.00, -1.00, std::sqrt(21.00)/7.00, 1.00/10.00 * 1.00/10.00 * 49.00/90.00 ),
        IntegrationPointType( -1.00, -1.00, 1.00, 1.00/10.00 * 1.00/10.00 * 1.00/10.00 ),

        IntegrationPointType( -1.00, -std::sqrt(21.00)/7.00, -1.00, 1.00/10.00 * 49.00/90.00 * 1.00/10.00 ),
        IntegrationPointType( -1.00, -std::sqrt(21.00)/7.00, -std::sqrt(21.00)/7.00, 1.00/10.00 * 49.00/90.00 * 49.00/90.00 ),
        IntegrationPointType( -1.00, -std::sqrt(21.00)/7.00, 0.00, 1.00/10.00 * 49.00/90.00 * 32.00/45.00 ),
        IntegrationPointType( -1.00, -std::sqrt(21.00)/7.00, std::sqrt(21.00)/7.00, 1.00/10.00 * 49.00/90.00 * 49.00/90.00 ),
        IntegrationPointType( -1.00, -std::sqrt(21.00)/7.00, 1.00, 1.00/10.00 * 49.00/90.00 * 1.00/10.00 ),

        IntegrationPointType( -1.00, 0.00, -1.00, 1.00/10.00 * 32.00/45.00 * 1.00/10.00 ),
        IntegrationPointType( -1.00, 0.00, -std::sqrt(21.00)/7.00, 1.00/10.00 * 32.00/45.00 * 49.00/90.00 ),
        IntegrationPointType( -1.00, 0.00, 0.00, 1.00/10.00 * 32.00/45.00 * 32.00/45.00 ),
        IntegrationPointType( -1.00, 0.00, std::sqrt(21.00)/7.00, 1.00/10.00 * 32.00/45.00 * 49.00/90.00 ),
        IntegrationPointType( -1.00, 0.00, 1.00, 1.00/10.00 * 32.00/45.00 * 1.00/10.00 ),

        IntegrationPointType( -1.00, std::sqrt(21.00)/7.00, -1.00, 1.00/10.00 * 49.00/90.00 * 1.00/10.00 ),
        IntegrationPointType( -1.00, std::sqrt(21.00)/7.00, -std::sqrt(21.00)/7.00, 1.00/10.00 * 49.00/90.00 * 49.00/90.00 ),
        IntegrationPointType( -1.00, std::sqrt(21.00)/7.00, 0.00, 1.00/10.00 * 49.00/90.00 * 32.00/45.00 ),
        IntegrationPointType( -1.00, std::sqrt(21.00)/7.00, std::sqrt(21.00)/7.00, 1.00/10.00 * 49.00/90.00 * 49.00/90.00 ),
        IntegrationPointType( -1.00, std::sqrt(21.00)/7.00, 1.00, 1.00/10.00 * 49.00/90.00 * 1.00/10.00 ),

        IntegrationPointType( -1.00, 1.00, -1.00, 1.00/10.00 * 1.00/10.00 * 1.00/10.00 ),
        IntegrationPointType( -1.00, 1.00, -std::sqrt(21.00)/7.00, 1.00/10.00 * 1.00/10.00 * 49.00/90.00 ),
        IntegrationPointType( -1.00, 1.00, 0.00, 1.00/10.00 * 1.00/10.00 * 32.00/45.00 ),
        IntegrationPointType( -1.00, 1.00, std::sqrt(21.00)/7.00, 1.00/10.00 * 1.00/10.00 * 49.00/90.00 ),
        IntegrationPointType( -1.00, 1.00, 1.00, 1.00/10.00 * 1.00/10.00 * 1.00/10.00 ),

        IntegrationPointType( -std::sqrt(21.00)/7.00, -1.00, -1.00, 49.00/90.00 * 1.00/10.00 * 1.00/10.00 ),
        IntegrationPointType( -std::sqrt(21.00)/7.00, -1.00, -std::sqrt(21.00)/7.00, 49.00/90.00 * 1.00/10.00 * 49.00/90.00 ),
        IntegrationPointType( -std::sqrt(21.00)/7.00, -1.00, 0.00, 49.00/90.00 * 1.00/10.00 * 32.00/45.00 ),
        IntegrationPointType( -std::sqrt(21.00)/7.00, -1.00, std::sqrt(21.00)/7.00, 49.00/90.00 * 1.00/10.00 * 49.00/90.00 ),
        IntegrationPointType( -std::sqrt(21.00)/7.00, -1.00, 1.00, 49.00/90.00 * 1.00/10.00 * 1.00/10.00 ),

        IntegrationPointType( -std::sqrt(21.00)/7.00, -std::sqrt(21.00)/7.00, -1.00, 49.00/90.00 * 49.00/90.00 * 1.00/10.00 ),
        IntegrationPointType( -std::sqrt(21.00)/7.00, -std::sqrt(21.00)/7.00, -std::sqrt(21.00)/7.00, 49.00/90.00 * 49.00/90.00 * 49.00/90.00 ),
        IntegrationPointType( -std::sqrt(21.00)/7.00, -std::sqrt(21.00)/7.00, 0.00, 49.00/90.00 * 49.00/90.00 * 32.00/45.00 ),
        IntegrationPointType( -std::sqrt(21.00)/7.00, -std::sqrt(21.00)/7.00, std::sqrt(21.00)/7.00, 49.00/90.00 * 49.00/90.00 * 49.00/90.00 ),
        IntegrationPointType( -std::sqrt(21.00)/7.00, -std::sqrt(21.00)/7.00, 1.00, 49.00/90.00 * 49.00/90.00 * 1.00/10.00 ),

        IntegrationPointType( -std::sqrt(21.00)/7.00, 0.00, -1.00, 49.00/90.00 * 32.00/45.00 * 1.00/10.00 ),
        IntegrationPointType( -std::sqrt(21.00)/7.00, 0.00, -std::sqrt(21.00)/7.00, 49.00/90.00 * 32.00/45.00 * 49.00/90.00 ),
        IntegrationPointType( -std::sqrt(21.00)/7.00, 0.00, 0.00, 49.00/90.00 * 32.00/45.00 * 32.00/45.00 ),
        IntegrationPointType( -std::sqrt(21.00)/7.00, 0.00, std::sqrt(21.00)/7.00, 49.00/90.00 * 32.00/45.00 * 49.00/90.00 ),
        IntegrationPointType( -std::sqrt(21.00)/7.00, 0.00, 1.00, 49.00/90.00 * 32.00/45.00 * 1.00/10.00 ),

        IntegrationPointType( -std::sqrt(21.00)/7.00, std::sqrt(21.00)/7.00, -1.00, 49.00/90.00 * 49.00/90.00 * 1.00/10.00 ),
        IntegrationPointType( -std::sqrt(21.00)/7.00, std::sqrt(21.00)/7.00, -std::sqrt(21.00)/7.00, 49.00/90.00 * 49.00/90.00 * 49.00/90.00 ),
        IntegrationPointType( -std::sqrt(21.00)/7.00, std::sqrt(21.00)/7.00, 0.00, 49.00/90.00 * 49.00/90.00 * 32.00/45.00 ),
        IntegrationPointType( -std::sqrt(21.00)/7.00, std::sqrt(21.00)/7.00, std::sqrt(21.00)/7.00, 49.00/90.00 * 49.00/90.00 * 49.00/90.00 ),
        IntegrationPointType( -std::sqrt(21.00)/7.00, std::sqrt(21.00)/7.00, 1.00, 49.00/90.00 * 49.00/90.00 * 1.00/10.00 ),

        IntegrationPointType( -std::sqrt(21.00)/7.00, 1.00, -1.00, 49.00/90.00 * 1.00/10.00 * 1.00/10.00 ),
        IntegrationPointType( -std::sqrt(21.00)/7.00, 1.00, -std::sqrt(21.00)/7.00, 49.00/90.00 * 1.00/10.00 * 49.00/90.00 ),
        IntegrationPointType( -std::sqrt(21.00)/7.00, 1.00, 0.00, 49.00/90.00 * 1.00/10.00 * 32.00/45.00 ),
        IntegrationPointType( -std::sqrt(21.00)/7.00, 1.00, std::sqrt(21.00)/7.00, 49.00/90.00 * 1.00/10.00 * 49.00/90.00 ),
        IntegrationPointType( -std::sqrt(21.00)/7.00, 1.00, 1.00, 49.00/90.00 * 1.00/10.00 * 1.00/10.00 ),

        IntegrationPointType( 0.00, -1.00, -1.00, 32.00/45.00 * 1.00/10.00 * 1.00/10.00 ),
        IntegrationPointType( 0.00, -1.00, -std::sqrt(21.00)/7.00, 32.00/45.00 * 1.00/10.00 * 49.00/90.00 ),
        IntegrationPointType( 0.00, -1.00, 0.00, 32.00/45.00 * 1.00/10.00 * 32.00/45.00 ),
        IntegrationPointType( 0.00, -1.00, std::sqrt(21.00)/7.00, 32.00/45.00 * 1.00/10.00 * 49.00/90.00 ),
        IntegrationPointType( 0.00, -1.00, 1.00, 32.00/45.00 * 1.00/10.00 * 1.00/10.00 ),

        IntegrationPointType( 0.00, -std::sqrt(21.00)/7.00, -1.00, 32.00/45.00 * 49.00/90.00 * 1.00/10.00 ),
        IntegrationPointType( 0.00, -std::sqrt(21.00)/7.00, -std::sqrt(21.00)/7.00, 32.00/45.00 * 49.00/90.00 * 49.00/90.00 ),
        IntegrationPointType( 0.00, -std::sqrt(21.00)/7.00, 0.00, 32.00/45.00 * 49.00/90.00 * 32.00/45.00 ),
        IntegrationPointType( 0.00, -std::sqrt(21.00)/7.00, std::sqrt(21.00)/7.00, 32.00/45.00 * 49.00/90.00 * 49.00/90.00 ),
        IntegrationPointType( 0.00, -std::sqrt(21.00)/7.00, 1.00, 32.00/45.00 * 49.00/90.00 * 1.00/10.00 ),

        IntegrationPointType( 0.00, 0.00, -1.00, 32.00/45.00 * 32.00/45.00 * 1.00/10.00 ),
        IntegrationPointType( 0.00, 0.00, -std::sqrt(21.00)/7.00, 32.00/45.00 * 32.00/45.00 * 49.00/90.00 ),
        IntegrationPointType( 0.00, 0.00, 0.00, 32.00/45.00 * 32.00/45.00 * 32.00/45.00 ),
        IntegrationPointType( 0.00, 0.00, std::sqrt(21.00)/7.00, 32.00/45.00 * 32.00/45.00 * 49.00/90.00 ),
        IntegrationPointType( 0.00, 0.00, 1.00, 32.00/45.00 * 32.00/45.00 * 1.00/10.00 ),

        IntegrationPointType( 0.00, std::sqrt(21.00)/7.00, -1.00, 32.00/45.00 * 49.00/90.00 * 1.00/10.00 ),
        IntegrationPointType( 0.00, std::sqrt(21.00)/7.00, -std::sqrt(21.00)/7.00, 32.00/45.00 * 49.00/90.00 * 49.00/90.00 ),
        IntegrationPointType( 0.00, std::sqrt(21.00)/7.00, 0.00, 32.00/45.00 * 49.00/90.00 * 32.00/45.00 ),
        IntegrationPointType( 0.00, std::sqrt(21.00)/7.00, std::sqrt(21.00)/7.00, 32.00/45.00 * 49.00/90.00 * 49.00/90.00 ),
        IntegrationPointType( 0.00, std::sqrt(21.00)/7.00, 1.00, 32.00/45.00 * 49.00/90.00 * 1.00/10.00 ),

        IntegrationPointType( 0.00, 1.00, -1.00, 32.00/45.00 * 1.00/10.00 * 1.00/10.00 ),
        IntegrationPointType( 0.00, 1.00, -std::sqrt(21.00)/7.00, 32.00/45.00 * 1.00/10.00 * 49.00/90.00 ),
        IntegrationPointType( 0.00, 1.00, 0.00, 32.00/45.00 * 1.00/10.00 * 32.00/45.00 ),
        IntegrationPointType( 0.00, 1.00, std::sqrt(21.00)/7.00, 32.00/45.00 * 1.00/10.00 * 49.00/90.00 ),
        IntegrationPointType( 0.00, 1.00, 1.00, 32.00/45.00 * 1.00/10.00 * 1.00/10.00 ),

        IntegrationPointType( std::sqrt(21.00)/7.00, -1.00, -1.00, 49.00/90.00 * 1.00/10.00 * 1.00/10.00 ),
        IntegrationPointType( std::sqrt(21.00)/7.00, -1.00, -std::sqrt(21.00)/7.00, 49.00/90.00 * 1.00/10.00 * 49.00/90.00 ),
        IntegrationPointType( std::sqrt(21.00)/7.00, -1.00, 0.00, 49.00/90.00 * 1.00/10.00 * 32.00/45.00 ),
        IntegrationPointType( std::sqrt(21.00)/7.00, -1.00, std::sqrt(21.00)/7.00, 49.00/90.00 * 1.00/10.00 * 49.00/90.00 ),
        IntegrationPointType( std::sqrt(21.00)/7.00, -1.00, 1.00, 49.00/90.00 * 1.00/10.00 * 1.00/10.00 ),

        IntegrationPointType( std::sqrt(21.00)/7.00, -std::sqrt(21.00)/7.00, -1.00, 49.00/90.00 * 49.00/90.00 * 1.00/10.00 ),
        IntegrationPointType( std::sqrt(21.00)/7.00, -std::sqrt(21.00)/7.00, -std::sqrt(21.00)/7.00, 49.00/90.00 * 49.00/90.00 * 49.00/90.00 ),
        IntegrationPointType( std::sqrt(21.00)/7.00, -std::sqrt(21.00)/7.00, 0.00, 49.00/90.00 * 49.00/90.00 * 32.00/45.00 ),
        IntegrationPointType( std::sqrt(21.00)/7.00, -std::sqrt(21.00)/7.00, std::sqrt(21.00)/7.00, 49.00/90.00 * 49.00/90.00 * 49.00/90.00 ),
        IntegrationPointType( std::sqrt(21.00)/7.00, -std::sqrt(21.00)/7.00, 1.00, 49.00/90.00 * 49.00/90.00 * 1.00/10.00 ),

        IntegrationPointType( std::sqrt(21.00)/7.00, 0.00, -1.00, 49.00/90.00 * 32.00/45.00 * 1.00/10.00 ),
        IntegrationPointType( std::sqrt(21.00)/7.00, 0.00, -std::sqrt(21.00)/7.00, 49.00/90.00 * 32.00/45.00 * 49.00/90.00 ),
        IntegrationPointType( std::sqrt(21.00)/7.00, 0.00, 0.00, 49.00/90.00 * 32.00/45.00 * 32.00/45.00 ),
        IntegrationPointType( std::sqrt(21.00)/7.00, 0.00, std::sqrt(21.00)/7.00, 49.00/90.00 * 32.00/45.00 * 49.00/90.00 ),
        IntegrationPointType( std::sqrt(21.00)/7.00, 0.00, 1.00, 49.00/90.00 * 32.00/45.00 * 1.00/10.00 ),

        IntegrationPointType( std::sqrt(21.00)/7.00, std::sqrt(21.00)/7.00, -1.00, 49.00/90.00 * 49.00/90.00 * 1.00/10.00 ),
        IntegrationPointType( std::sqrt(21.00)/7.00, std::sqrt(21.00)/7.00, -std::sqrt(21.00)/7.00, 49.00/90.00 * 49.00/90.00 * 49.00/90.00 ),
        IntegrationPointType( std::sqrt(21.00)/7.00, std::sqrt(21.00)/7.00, 0.00, 49.00/90.00 * 49.00/90.00 * 32.00/45.00 ),
        IntegrationPointType( std::sqrt(21.00)/7.00, std::sqrt(21.00)/7.00, std::sqrt(21.00)/7.00, 49.00/90.00 * 49.00/90.00 * 49.00/90.00 ),
        IntegrationPointType( std::sqrt(21.00)/7.00, std::sqrt(21.00)/7.00, 1.00, 49.00/90.00 * 49.00/90.00 * 1.00/10.00 ),

        IntegrationPointType( std::sqrt(21.00)/7.00, 1.00, -1.00, 49.00/90.00 * 1.00/10.00 * 1.00/10.00 ),
        IntegrationPointType( std::sqrt(21.00)/7.00, 1.00, -std::sqrt(21.00)/7.00, 49.00/90.00 * 1.00/10.00 * 49.00/90.00 ),
        IntegrationPointType( std::sqrt(21.00)/7.00, 1.00, 0.00, 49.00/90.00 * 1.00/10.00 * 32.00/45.00 ),
        IntegrationPointType( std::sqrt(21.00)/7.00, 1.00, std::sqrt(21.00)/7.00, 49.00/90.00 * 1.00/10.00 * 49.00/90.00 ),
        IntegrationPointType( std::sqrt(21.00)/7.00, 1.00, 1.00, 49.00/90.00 * 1.00/10.00 * 1.00/10.00 ),

        IntegrationPointType( 1.00, -1.00, -1.00, 1.00/10.00 * 1.00/10.00 * 1.00/10.00 ),
        IntegrationPointType( 1.00, -1.00, -std::sqrt(21.00)/7.00, 1.00/10.00 * 1.00/10.00 * 49.00/90.00 ),
        IntegrationPointType( 1.00, -1.00, 0.00, 1.00/10.00 * 1.00/10.00 * 32.00/45.00 ),
        IntegrationPointType( 1.00, -1.00, std::sqrt(21.00)/7.00, 1.00/10.00 * 1.00/10.00 * 49.00/90.00 ),
        IntegrationPointType( 1.00, -1.00, 1.00, 1.00/10.00 * 1.00/10.00 * 1.00/10.00 ),

        IntegrationPointType( 1.00, -std::sqrt(21.00)/7.00, -1.00, 1.00/10.00 * 49.00/90.00 * 1.00/10.00 ),
        IntegrationPointType( 1.00, -std::sqrt(21.00)/7.00, -std::sqrt(21.00)/7.00, 1.00/10.00 * 49.00/90.00 * 49.00/90.00 ),
        IntegrationPointType( 1.00, -std::sqrt(21.00)/7.00, 0.00, 1.00/10.00 * 49.00/90.00 * 32.00/45.00 ),
        IntegrationPointType( 1.00, -std::sqrt(21.00)/7.00, std::sqrt(21.00)/7.00, 1.00/10.00 * 49.00/90.00 * 49.00/90.00 ),
        IntegrationPointType( 1.00, -std::sqrt(21.00)/7.00, 1.00, 1.00/10.00 * 49.00/90.00 * 1.00/10.00 ),

        IntegrationPointType( 1.00, 0.00, -1.00, 1.00/10.00 * 32.00/45.00 * 1.00/10.00 ),
        IntegrationPointType( 1.00, 0.00, -std::sqrt(21.00)/7.00, 1.00/10.00 * 32.00/45.00 * 49.00/90.00 ),
        IntegrationPointType( 1.00, 0.00, 0.00, 1.00/10.00 * 32.00/45.00 * 32.00/45.00 ),
        IntegrationPointType( 1.00, 0.00, std::sqrt(21.00)/7.00, 1.00/10.00 * 32.00/45.00 * 49.00/90.00 ),
        IntegrationPointType( 1.00, 0.00, 1.00, 1.00/10.00 * 32.00/45.00 * 1.00/10.00 ),

        IntegrationPointType( 1.00, std::sqrt(21.00)/7.00, -1.00, 1.00/10.00 * 49.00/90.00 * 1.00/10.00 ),
        IntegrationPointType( 1.00, std::sqrt(21.00)/7.00, -std::sqrt(21.00)/7.00, 1.00/10.00 * 49.00/90.00 * 49.00/90.00 ),
        IntegrationPointType( 1.00, std::sqrt(21.00)/7.00, 0.00, 1.00/10.00 * 49.00/90.00 * 32.00/45.00 ),
        IntegrationPointType( 1.00, std::sqrt(21.00)/7.00, std::sqrt(21.00)/7.00, 1.00/10.00 * 49.00/90.00 * 49.00/90.00 ),
        IntegrationPointType( 1.00, std::sqrt(21.00)/7.00, 1.00, 1.00/10.00 * 49.00/90.00 * 1.00/10.00 ),

        IntegrationPointType( 1.00, 1.00, -1.00, 1.00/10.00 * 1.00/10.00 * 1.00/10.00 ),
        IntegrationPointType( 1.00, 1.00, -std::sqrt(21.00)/7.00, 1.00/10.00 * 1.00/10.00 * 49.00/90.00 ),
        IntegrationPointType( 1.00, 1.00, 0.00, 1.00/10.00 * 1.00/10.00 * 32.00/45.00 ),
        IntegrationPointType( 1.00, 1.00, std::sqrt(21.00)/7.00, 1.00/10.00 * 1.00/10.00 * 49.00/90.00 ),
        IntegrationPointType( 1.00, 1.00, 1.00, 1.00/10.00 * 1.00/10.00 * 1.00/10.00 ),

    }
};

HexahedronGaussLobattoIntegrationPoints6::IntegrationPointsArrayType
HexahedronGaussLobattoIntegrationPoints6::msIntegrationPoints =
{
    {
        IntegrationPointType( -1.00, -1.00, -1.00, 1.00/15.00 * 1.00/15.00 * 1.00/15.00 ),
        IntegrationPointType( -1.00, -1.00, -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -1.00, -1.00, -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -1.00, -1.00, std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -1.00, -1.00, std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -1.00, -1.00, 1.00, 1.00/15.00 * 1.00/15.00 * 1.00/15.00 ),

        IntegrationPointType( -1.00, -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -1.00, 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 ),
        IntegrationPointType( -1.00, -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -1.00, -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -1.00, -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -1.00, -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -1.00, -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00, 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 ),

        IntegrationPointType( -1.00, -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -1.00, 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 ),
        IntegrationPointType( -1.00, -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -1.00, -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -1.00, -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -1.00, -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -1.00, -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00, 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 ),

        IntegrationPointType( -1.00, std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -1.00, 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 ),
        IntegrationPointType( -1.00, std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -1.00, std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -1.00, std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -1.00, std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -1.00, std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00, 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 ),

        IntegrationPointType( -1.00, std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -1.00, 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 ),
        IntegrationPointType( -1.00, std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -1.00, std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -1.00, std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -1.00, std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -1.00, std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00, 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 ),

        IntegrationPointType( -1.00, 1.00, -1.00, 1.00/15.00 * 1.00/15.00 * 1.00/15.00 ),
        IntegrationPointType( -1.00, 1.00, -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -1.00, 1.00, -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -1.00, 1.00, std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -1.00, 1.00, std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -1.00, 1.00, 1.00, 1.00/15.00 * 1.00/15.00 * 1.00/15.00 ),

        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -1.00, -1.00, (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 * 1.00/15.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -1.00, -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -1.00, -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -1.00, std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -1.00, std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -1.00, 1.00, (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 * 1.00/15.00 ),

        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -1.00, (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00, (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 ),

        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -1.00, (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00, (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 ),

        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -1.00, (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00, (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 ),

        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -1.00, (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00, (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 ),

        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00, -1.00, (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 * 1.00/15.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00, -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00, -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00, std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00, std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00, 1.00, (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 * 1.00/15.00 ),

        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -1.00, -1.00, (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 * 1.00/15.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -1.00, -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -1.00, -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -1.00, std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -1.00, std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -1.00, 1.00, (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 * 1.00/15.00 ),

        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -1.00, (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00, (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 ),

        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -1.00, (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00, (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 ),

        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -1.00, (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00, (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 ),

        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -1.00, (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00, (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 ),

        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00, -1.00, (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 * 1.00/15.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00, -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00, -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00, std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00, std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00, 1.00, (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 * 1.00/15.00 ),

        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -1.00, -1.00, (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 * 1.00/15.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -1.00, -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -1.00, -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -1.00, std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -1.00, std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -1.00, 1.00, (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 * 1.00/15.00 ),

        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -1.00, (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00, (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 ),

        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -1.00, (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00, (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 ),

        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -1.00, (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00, (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 ),

        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -1.00, (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00, (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 ),

        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00, -1.00, (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 * 1.00/15.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00, -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00, -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00, std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00, std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00, 1.00, (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 * 1.00/15.00 ),

        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -1.00, -1.00, (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 * 1.00/15.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -1.00, -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -1.00, -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -1.00, std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -1.00, std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -1.00, 1.00, (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 * 1.00/15.00 ),

        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -1.00, (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00, (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 ),

        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -1.00, (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00, (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 ),

        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -1.00, (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00, (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 ),

        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -1.00, (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00, (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 ),

        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00, -1.00, (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 * 1.00/15.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00, -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00, -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00, std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00, std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00, 1.00, (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 * 1.00/15.00 ),

        IntegrationPointType( 1.00, -1.00, -1.00, 1.00/15.00 * 1.00/15.00 * 1.00/15.00 ),
        IntegrationPointType( 1.00, -1.00, -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( 1.00, -1.00, -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( 1.00, -1.00, std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( 1.00, -1.00, std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( 1.00, -1.00, 1.00, 1.00/15.00 * 1.00/15.00 * 1.00/15.00 ),

        IntegrationPointType( 1.00, -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -1.00, 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 ),
        IntegrationPointType( 1.00, -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( 1.00, -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( 1.00, -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( 1.00, -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( 1.00, -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00, 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 ),

        IntegrationPointType( 1.00, -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -1.00, 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 ),
        IntegrationPointType( 1.00, -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( 1.00, -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( 1.00, -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( 1.00, -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( 1.00, -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00, 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 ),

        IntegrationPointType( 1.00, std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -1.00, 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 ),
        IntegrationPointType( 1.00, std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( 1.00, std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( 1.00, std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( 1.00, std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( 1.00, std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00, 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 * 1.00/15.00 ),

        IntegrationPointType( 1.00, std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -1.00, 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 ),
        IntegrationPointType( 1.00, std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( 1.00, std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( 1.00, std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( 1.00, std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( 1.00, std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00, 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 * 1.00/15.00 ),

        IntegrationPointType( 1.00, 1.00, -1.00, 1.00/15.00 * 1.00/15.00 * 1.00/15.00 ),
        IntegrationPointType( 1.00, 1.00, -std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( 1.00, 1.00, -std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( 1.00, 1.00, std::sqrt((7.00-2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * 1.00/15.00 * (14.00+std::sqrt(7.00))/30.00 ),
        IntegrationPointType( 1.00, 1.00, std::sqrt((7.00+2.00*std::sqrt(7.00))/21.00), 1.00/15.00 * 1.00/15.00 * (14.00-std::sqrt(7.00))/30.00 ),
        IntegrationPointType( 1.00, 1.00, 1.00, 1.00/15.00 * 1.00/15.00 * 1.00/15.00 ),

    }
};

HexahedronGaussLobattoIntegrationPoints7::IntegrationPointsArrayType
HexahedronGaussLobattoIntegrationPoints7::msIntegrationPoints =
{
    {
        IntegrationPointType( -1.00, -1.00, -1.00, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),
        IntegrationPointType( -1.00, -1.00, -0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -1.00, -1.00, -0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -1.00, -1.00, 0.00, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.487619047619047619047619047619 ),
        IntegrationPointType( -1.00, -1.00, 0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -1.00, -1.00, 0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -1.00, -1.00, 1.00, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),

        IntegrationPointType( -1.00, -0.830223896278566929872032213967, -1.00, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),
        IntegrationPointType( -1.00, -0.830223896278566929872032213967, -0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -1.00, -0.830223896278566929872032213967, -0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -1.00, -0.830223896278566929872032213967, 0.00, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.487619047619047619047619047619 ),
        IntegrationPointType( -1.00, -0.830223896278566929872032213967, 0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -1.00, -0.830223896278566929872032213967, 0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -1.00, -0.830223896278566929872032213967, 1.00, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),

        IntegrationPointType( -1.00, -0.468848793470714213803771881909, -1.00, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),
        IntegrationPointType( -1.00, -0.468848793470714213803771881909, -0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -1.00, -0.468848793470714213803771881909, -0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -1.00, -0.468848793470714213803771881909, 0.00, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.487619047619047619047619047619 ),
        IntegrationPointType( -1.00, -0.468848793470714213803771881909, 0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -1.00, -0.468848793470714213803771881909, 0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -1.00, -0.468848793470714213803771881909, 1.00, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),

        IntegrationPointType( -1.00, 0.00, -1.00, 0.047619047619047619047619047619 * 0.487619047619047619047619047619 * 0.047619047619047619047619047619 ),
        IntegrationPointType( -1.00, 0.00, -0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.487619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -1.00, 0.00, -0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.487619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -1.00, 0.00, 0.00, 0.047619047619047619047619047619 * 0.487619047619047619047619047619 * 0.487619047619047619047619047619 ),
        IntegrationPointType( -1.00, 0.00, 0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.487619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -1.00, 0.00, 0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.487619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -1.00, 0.00, 1.00, 0.047619047619047619047619047619 * 0.487619047619047619047619047619 * 0.047619047619047619047619047619 ),

        IntegrationPointType( -1.00, 0.468848793470714213803771881909, -1.00, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),
        IntegrationPointType( -1.00, 0.468848793470714213803771881909, -0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -1.00, 0.468848793470714213803771881909, -0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -1.00, 0.468848793470714213803771881909, 0.00, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.487619047619047619047619047619 ),
        IntegrationPointType( -1.00, 0.468848793470714213803771881909, 0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -1.00, 0.468848793470714213803771881909, 0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -1.00, 0.468848793470714213803771881909, 1.00, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),

        IntegrationPointType( -1.00, 0.830223896278566929872032213967, -1.00, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),
        IntegrationPointType( -1.00, 0.830223896278566929872032213967, -0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -1.00, 0.830223896278566929872032213967, -0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -1.00, 0.830223896278566929872032213967, 0.00, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.487619047619047619047619047619 ),
        IntegrationPointType( -1.00, 0.830223896278566929872032213967, 0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -1.00, 0.830223896278566929872032213967, 0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -1.00, 0.830223896278566929872032213967, 1.00, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),

        IntegrationPointType( -1.00, 1.00, -1.00, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),
        IntegrationPointType( -1.00, 1.00, -0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -1.00, 1.00, -0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -1.00, 1.00, 0.00, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.487619047619047619047619047619 ),
        IntegrationPointType( -1.00, 1.00, 0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -1.00, 1.00, 0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -1.00, 1.00, 1.00, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),

        IntegrationPointType( -0.830223896278566929872032213967, -1.00, -1.00, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),
        IntegrationPointType( -0.830223896278566929872032213967, -1.00, -0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.830223896278566929872032213967, -1.00, -0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.830223896278566929872032213967, -1.00, 0.00, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.487619047619047619047619047619 ),
        IntegrationPointType( -0.830223896278566929872032213967, -1.00, 0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.830223896278566929872032213967, -1.00, 0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.830223896278566929872032213967, -1.00, 1.00, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),

        IntegrationPointType( -0.830223896278566929872032213967, -0.830223896278566929872032213967, -1.00, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),
        IntegrationPointType( -0.830223896278566929872032213967, -0.830223896278566929872032213967, -0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.830223896278566929872032213967, -0.830223896278566929872032213967, -0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.830223896278566929872032213967, -0.830223896278566929872032213967, 0.00, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.487619047619047619047619047619 ),
        IntegrationPointType( -0.830223896278566929872032213967, -0.830223896278566929872032213967, 0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.830223896278566929872032213967, -0.830223896278566929872032213967, 0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.830223896278566929872032213967, -0.830223896278566929872032213967, 1.00, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),

        IntegrationPointType( -0.830223896278566929872032213967, -0.468848793470714213803771881909, -1.00, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),
        IntegrationPointType( -0.830223896278566929872032213967, -0.468848793470714213803771881909, -0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.830223896278566929872032213967, -0.468848793470714213803771881909, -0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.830223896278566929872032213967, -0.468848793470714213803771881909, 0.00, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.487619047619047619047619047619 ),
        IntegrationPointType( -0.830223896278566929872032213967, -0.468848793470714213803771881909, 0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.830223896278566929872032213967, -0.468848793470714213803771881909, 0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.830223896278566929872032213967, -0.468848793470714213803771881909, 1.00, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),

        IntegrationPointType( -0.830223896278566929872032213967, 0.00, -1.00, 0.27682604736156594801070040629 * 0.487619047619047619047619047619 * 0.047619047619047619047619047619 ),
        IntegrationPointType( -0.830223896278566929872032213967, 0.00, -0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.487619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.830223896278566929872032213967, 0.00, -0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.487619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.830223896278566929872032213967, 0.00, 0.00, 0.27682604736156594801070040629 * 0.487619047619047619047619047619 * 0.487619047619047619047619047619 ),
        IntegrationPointType( -0.830223896278566929872032213967, 0.00, 0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.487619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.830223896278566929872032213967, 0.00, 0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.487619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.830223896278566929872032213967, 0.00, 1.00, 0.27682604736156594801070040629 * 0.487619047619047619047619047619 * 0.047619047619047619047619047619 ),

        IntegrationPointType( -0.830223896278566929872032213967, 0.468848793470714213803771881909, -1.00, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),
        IntegrationPointType( -0.830223896278566929872032213967, 0.468848793470714213803771881909, -0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.830223896278566929872032213967, 0.468848793470714213803771881909, -0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.830223896278566929872032213967, 0.468848793470714213803771881909, 0.00, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.487619047619047619047619047619 ),
        IntegrationPointType( -0.830223896278566929872032213967, 0.468848793470714213803771881909, 0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.830223896278566929872032213967, 0.468848793470714213803771881909, 0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.830223896278566929872032213967, 0.468848793470714213803771881909, 1.00, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),

        IntegrationPointType( -0.830223896278566929872032213967, 0.830223896278566929872032213967, -1.00, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),
        IntegrationPointType( -0.830223896278566929872032213967, 0.830223896278566929872032213967, -0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.830223896278566929872032213967, 0.830223896278566929872032213967, -0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.830223896278566929872032213967, 0.830223896278566929872032213967, 0.00, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.487619047619047619047619047619 ),
        IntegrationPointType( -0.830223896278566929872032213967, 0.830223896278566929872032213967, 0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.830223896278566929872032213967, 0.830223896278566929872032213967, 0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.830223896278566929872032213967, 0.830223896278566929872032213967, 1.00, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),

        IntegrationPointType( -0.830223896278566929872032213967, 1.00, -1.00, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),
        IntegrationPointType( -0.830223896278566929872032213967, 1.00, -0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.830223896278566929872032213967, 1.00, -0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.830223896278566929872032213967, 1.00, 0.00, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.487619047619047619047619047619 ),
        IntegrationPointType( -0.830223896278566929872032213967, 1.00, 0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.830223896278566929872032213967, 1.00, 0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.830223896278566929872032213967, 1.00, 1.00, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),

        IntegrationPointType( -0.468848793470714213803771881909, -1.00, -1.00, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),
        IntegrationPointType( -0.468848793470714213803771881909, -1.00, -0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.468848793470714213803771881909, -1.00, -0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.468848793470714213803771881909, -1.00, 0.00, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.487619047619047619047619047619 ),
        IntegrationPointType( -0.468848793470714213803771881909, -1.00, 0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.468848793470714213803771881909, -1.00, 0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.468848793470714213803771881909, -1.00, 1.00, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),

        IntegrationPointType( -0.468848793470714213803771881909, -0.830223896278566929872032213967, -1.00, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),
        IntegrationPointType( -0.468848793470714213803771881909, -0.830223896278566929872032213967, -0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.468848793470714213803771881909, -0.830223896278566929872032213967, -0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.468848793470714213803771881909, -0.830223896278566929872032213967, 0.00, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.487619047619047619047619047619 ),
        IntegrationPointType( -0.468848793470714213803771881909, -0.830223896278566929872032213967, 0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.468848793470714213803771881909, -0.830223896278566929872032213967, 0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.468848793470714213803771881909, -0.830223896278566929872032213967, 1.00, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),

        IntegrationPointType( -0.468848793470714213803771881909, -0.468848793470714213803771881909, -1.00, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),
        IntegrationPointType( -0.468848793470714213803771881909, -0.468848793470714213803771881909, -0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.468848793470714213803771881909, -0.468848793470714213803771881909, -0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.468848793470714213803771881909, -0.468848793470714213803771881909, 0.00, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.487619047619047619047619047619 ),
        IntegrationPointType( -0.468848793470714213803771881909, -0.468848793470714213803771881909, 0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.468848793470714213803771881909, -0.468848793470714213803771881909, 0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.468848793470714213803771881909, -0.468848793470714213803771881909, 1.00, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),

        IntegrationPointType( -0.468848793470714213803771881909, 0.00, -1.00, 0.431745381209862623417871022281 * 0.487619047619047619047619047619 * 0.047619047619047619047619047619 ),
        IntegrationPointType( -0.468848793470714213803771881909, 0.00, -0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.487619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.468848793470714213803771881909, 0.00, -0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.487619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.468848793470714213803771881909, 0.00, 0.00, 0.431745381209862623417871022281 * 0.487619047619047619047619047619 * 0.487619047619047619047619047619 ),
        IntegrationPointType( -0.468848793470714213803771881909, 0.00, 0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.487619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.468848793470714213803771881909, 0.00, 0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.487619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.468848793470714213803771881909, 0.00, 1.00, 0.431745381209862623417871022281 * 0.487619047619047619047619047619 * 0.047619047619047619047619047619 ),

        IntegrationPointType( -0.468848793470714213803771881909, 0.468848793470714213803771881909, -1.00, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),
        IntegrationPointType( -0.468848793470714213803771881909, 0.468848793470714213803771881909, -0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.468848793470714213803771881909, 0.468848793470714213803771881909, -0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.468848793470714213803771881909, 0.468848793470714213803771881909, 0.00, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.487619047619047619047619047619 ),
        IntegrationPointType( -0.468848793470714213803771881909, 0.468848793470714213803771881909, 0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.468848793470714213803771881909, 0.468848793470714213803771881909, 0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.468848793470714213803771881909, 0.468848793470714213803771881909, 1.00, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),

        IntegrationPointType( -0.468848793470714213803771881909, 0.830223896278566929872032213967, -1.00, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),
        IntegrationPointType( -0.468848793470714213803771881909, 0.830223896278566929872032213967, -0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.468848793470714213803771881909, 0.830223896278566929872032213967, -0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.468848793470714213803771881909, 0.830223896278566929872032213967, 0.00, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.487619047619047619047619047619 ),
        IntegrationPointType( -0.468848793470714213803771881909, 0.830223896278566929872032213967, 0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.468848793470714213803771881909, 0.830223896278566929872032213967, 0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.468848793470714213803771881909, 0.830223896278566929872032213967, 1.00, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),

        IntegrationPointType( -0.468848793470714213803771881909, 1.00, -1.00, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),
        IntegrationPointType( -0.468848793470714213803771881909, 1.00, -0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.468848793470714213803771881909, 1.00, -0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.468848793470714213803771881909, 1.00, 0.00, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.487619047619047619047619047619 ),
        IntegrationPointType( -0.468848793470714213803771881909, 1.00, 0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( -0.468848793470714213803771881909, 1.00, 0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( -0.468848793470714213803771881909, 1.00, 1.00, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 0.00, -1.00, -1.00, 0.487619047619047619047619047619 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 0.00, -1.00, -0.830223896278566929872032213967, 0.487619047619047619047619047619 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.00, -1.00, -0.468848793470714213803771881909, 0.487619047619047619047619047619 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.00, -1.00, 0.00, 0.487619047619047619047619047619 * 0.047619047619047619047619047619 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 0.00, -1.00, 0.468848793470714213803771881909, 0.487619047619047619047619047619 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.00, -1.00, 0.830223896278566929872032213967, 0.487619047619047619047619047619 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.00, -1.00, 1.00, 0.487619047619047619047619047619 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 0.00, -0.830223896278566929872032213967, -1.00, 0.487619047619047619047619047619 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 0.00, -0.830223896278566929872032213967, -0.830223896278566929872032213967, 0.487619047619047619047619047619 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.00, -0.830223896278566929872032213967, -0.468848793470714213803771881909, 0.487619047619047619047619047619 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.00, -0.830223896278566929872032213967, 0.00, 0.487619047619047619047619047619 * 0.27682604736156594801070040629 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 0.00, -0.830223896278566929872032213967, 0.468848793470714213803771881909, 0.487619047619047619047619047619 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.00, -0.830223896278566929872032213967, 0.830223896278566929872032213967, 0.487619047619047619047619047619 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.00, -0.830223896278566929872032213967, 1.00, 0.487619047619047619047619047619 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 0.00, -0.468848793470714213803771881909, -1.00, 0.487619047619047619047619047619 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 0.00, -0.468848793470714213803771881909, -0.830223896278566929872032213967, 0.487619047619047619047619047619 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.00, -0.468848793470714213803771881909, -0.468848793470714213803771881909, 0.487619047619047619047619047619 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.00, -0.468848793470714213803771881909, 0.00, 0.487619047619047619047619047619 * 0.431745381209862623417871022281 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 0.00, -0.468848793470714213803771881909, 0.468848793470714213803771881909, 0.487619047619047619047619047619 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.00, -0.468848793470714213803771881909, 0.830223896278566929872032213967, 0.487619047619047619047619047619 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.00, -0.468848793470714213803771881909, 1.00, 0.487619047619047619047619047619 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 0.00, 0.00, -1.00, 0.487619047619047619047619047619 * 0.487619047619047619047619047619 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 0.00, 0.00, -0.830223896278566929872032213967, 0.487619047619047619047619047619 * 0.487619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.00, 0.00, -0.468848793470714213803771881909, 0.487619047619047619047619047619 * 0.487619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.00, 0.00, 0.00, 0.487619047619047619047619047619 * 0.487619047619047619047619047619 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 0.00, 0.00, 0.468848793470714213803771881909, 0.487619047619047619047619047619 * 0.487619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.00, 0.00, 0.830223896278566929872032213967, 0.487619047619047619047619047619 * 0.487619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.00, 0.00, 1.00, 0.487619047619047619047619047619 * 0.487619047619047619047619047619 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 0.00, 0.468848793470714213803771881909, -1.00, 0.487619047619047619047619047619 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 0.00, 0.468848793470714213803771881909, -0.830223896278566929872032213967, 0.487619047619047619047619047619 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.00, 0.468848793470714213803771881909, -0.468848793470714213803771881909, 0.487619047619047619047619047619 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.00, 0.468848793470714213803771881909, 0.00, 0.487619047619047619047619047619 * 0.431745381209862623417871022281 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 0.00, 0.468848793470714213803771881909, 0.468848793470714213803771881909, 0.487619047619047619047619047619 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.00, 0.468848793470714213803771881909, 0.830223896278566929872032213967, 0.487619047619047619047619047619 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.00, 0.468848793470714213803771881909, 1.00, 0.487619047619047619047619047619 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 0.00, 0.830223896278566929872032213967, -1.00, 0.487619047619047619047619047619 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 0.00, 0.830223896278566929872032213967, -0.830223896278566929872032213967, 0.487619047619047619047619047619 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.00, 0.830223896278566929872032213967, -0.468848793470714213803771881909, 0.487619047619047619047619047619 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.00, 0.830223896278566929872032213967, 0.00, 0.487619047619047619047619047619 * 0.27682604736156594801070040629 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 0.00, 0.830223896278566929872032213967, 0.468848793470714213803771881909, 0.487619047619047619047619047619 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.00, 0.830223896278566929872032213967, 0.830223896278566929872032213967, 0.487619047619047619047619047619 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.00, 0.830223896278566929872032213967, 1.00, 0.487619047619047619047619047619 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 0.00, 1.00, -1.00, 0.487619047619047619047619047619 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 0.00, 1.00, -0.830223896278566929872032213967, 0.487619047619047619047619047619 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.00, 1.00, -0.468848793470714213803771881909, 0.487619047619047619047619047619 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.00, 1.00, 0.00, 0.487619047619047619047619047619 * 0.047619047619047619047619047619 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 0.00, 1.00, 0.468848793470714213803771881909, 0.487619047619047619047619047619 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.00, 1.00, 0.830223896278566929872032213967, 0.487619047619047619047619047619 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.00, 1.00, 1.00, 0.487619047619047619047619047619 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 0.468848793470714213803771881909, -1.00, -1.00, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 0.468848793470714213803771881909, -1.00, -0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.468848793470714213803771881909, -1.00, -0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.468848793470714213803771881909, -1.00, 0.00, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 0.468848793470714213803771881909, -1.00, 0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.468848793470714213803771881909, -1.00, 0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.468848793470714213803771881909, -1.00, 1.00, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 0.468848793470714213803771881909, -0.830223896278566929872032213967, -1.00, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 0.468848793470714213803771881909, -0.830223896278566929872032213967, -0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.468848793470714213803771881909, -0.830223896278566929872032213967, -0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.468848793470714213803771881909, -0.830223896278566929872032213967, 0.00, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 0.468848793470714213803771881909, -0.830223896278566929872032213967, 0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.468848793470714213803771881909, -0.830223896278566929872032213967, 0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.468848793470714213803771881909, -0.830223896278566929872032213967, 1.00, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 0.468848793470714213803771881909, -0.468848793470714213803771881909, -1.00, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 0.468848793470714213803771881909, -0.468848793470714213803771881909, -0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.468848793470714213803771881909, -0.468848793470714213803771881909, -0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.468848793470714213803771881909, -0.468848793470714213803771881909, 0.00, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 0.468848793470714213803771881909, -0.468848793470714213803771881909, 0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.468848793470714213803771881909, -0.468848793470714213803771881909, 0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.468848793470714213803771881909, -0.468848793470714213803771881909, 1.00, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 0.468848793470714213803771881909, 0.00, -1.00, 0.431745381209862623417871022281 * 0.487619047619047619047619047619 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 0.468848793470714213803771881909, 0.00, -0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.487619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.468848793470714213803771881909, 0.00, -0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.487619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.468848793470714213803771881909, 0.00, 0.00, 0.431745381209862623417871022281 * 0.487619047619047619047619047619 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 0.468848793470714213803771881909, 0.00, 0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.487619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.468848793470714213803771881909, 0.00, 0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.487619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.468848793470714213803771881909, 0.00, 1.00, 0.431745381209862623417871022281 * 0.487619047619047619047619047619 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 0.468848793470714213803771881909, 0.468848793470714213803771881909, -1.00, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 0.468848793470714213803771881909, 0.468848793470714213803771881909, -0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.468848793470714213803771881909, 0.468848793470714213803771881909, -0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.468848793470714213803771881909, 0.468848793470714213803771881909, 0.00, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 0.468848793470714213803771881909, 0.468848793470714213803771881909, 0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.468848793470714213803771881909, 0.468848793470714213803771881909, 0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.468848793470714213803771881909, 0.468848793470714213803771881909, 1.00, 0.431745381209862623417871022281 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 0.468848793470714213803771881909, 0.830223896278566929872032213967, -1.00, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 0.468848793470714213803771881909, 0.830223896278566929872032213967, -0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.468848793470714213803771881909, 0.830223896278566929872032213967, -0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.468848793470714213803771881909, 0.830223896278566929872032213967, 0.00, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 0.468848793470714213803771881909, 0.830223896278566929872032213967, 0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.468848793470714213803771881909, 0.830223896278566929872032213967, 0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.468848793470714213803771881909, 0.830223896278566929872032213967, 1.00, 0.431745381209862623417871022281 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 0.468848793470714213803771881909, 1.00, -1.00, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 0.468848793470714213803771881909, 1.00, -0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.468848793470714213803771881909, 1.00, -0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.468848793470714213803771881909, 1.00, 0.00, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 0.468848793470714213803771881909, 1.00, 0.468848793470714213803771881909, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.468848793470714213803771881909, 1.00, 0.830223896278566929872032213967, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.468848793470714213803771881909, 1.00, 1.00, 0.431745381209862623417871022281 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 0.830223896278566929872032213967, -1.00, -1.00, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 0.830223896278566929872032213967, -1.00, -0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.830223896278566929872032213967, -1.00, -0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.830223896278566929872032213967, -1.00, 0.00, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 0.830223896278566929872032213967, -1.00, 0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.830223896278566929872032213967, -1.00, 0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.830223896278566929872032213967, -1.00, 1.00, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 0.830223896278566929872032213967, -0.830223896278566929872032213967, -1.00, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 0.830223896278566929872032213967, -0.830223896278566929872032213967, -0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.830223896278566929872032213967, -0.830223896278566929872032213967, -0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.830223896278566929872032213967, -0.830223896278566929872032213967, 0.00, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 0.830223896278566929872032213967, -0.830223896278566929872032213967, 0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.830223896278566929872032213967, -0.830223896278566929872032213967, 0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.830223896278566929872032213967, -0.830223896278566929872032213967, 1.00, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 0.830223896278566929872032213967, -0.468848793470714213803771881909, -1.00, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 0.830223896278566929872032213967, -0.468848793470714213803771881909, -0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.830223896278566929872032213967, -0.468848793470714213803771881909, -0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.830223896278566929872032213967, -0.468848793470714213803771881909, 0.00, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 0.830223896278566929872032213967, -0.468848793470714213803771881909, 0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.830223896278566929872032213967, -0.468848793470714213803771881909, 0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.830223896278566929872032213967, -0.468848793470714213803771881909, 1.00, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 0.830223896278566929872032213967, 0.00, -1.00, 0.27682604736156594801070040629 * 0.487619047619047619047619047619 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 0.830223896278566929872032213967, 0.00, -0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.487619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.830223896278566929872032213967, 0.00, -0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.487619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.830223896278566929872032213967, 0.00, 0.00, 0.27682604736156594801070040629 * 0.487619047619047619047619047619 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 0.830223896278566929872032213967, 0.00, 0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.487619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.830223896278566929872032213967, 0.00, 0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.487619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.830223896278566929872032213967, 0.00, 1.00, 0.27682604736156594801070040629 * 0.487619047619047619047619047619 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 0.830223896278566929872032213967, 0.468848793470714213803771881909, -1.00, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 0.830223896278566929872032213967, 0.468848793470714213803771881909, -0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.830223896278566929872032213967, 0.468848793470714213803771881909, -0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.830223896278566929872032213967, 0.468848793470714213803771881909, 0.00, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 0.830223896278566929872032213967, 0.468848793470714213803771881909, 0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.830223896278566929872032213967, 0.468848793470714213803771881909, 0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.830223896278566929872032213967, 0.468848793470714213803771881909, 1.00, 0.27682604736156594801070040629 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 0.830223896278566929872032213967, 0.830223896278566929872032213967, -1.00, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 0.830223896278566929872032213967, 0.830223896278566929872032213967, -0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.830223896278566929872032213967, 0.830223896278566929872032213967, -0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.830223896278566929872032213967, 0.830223896278566929872032213967, 0.00, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 0.830223896278566929872032213967, 0.830223896278566929872032213967, 0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.830223896278566929872032213967, 0.830223896278566929872032213967, 0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.830223896278566929872032213967, 0.830223896278566929872032213967, 1.00, 0.27682604736156594801070040629 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 0.830223896278566929872032213967, 1.00, -1.00, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 0.830223896278566929872032213967, 1.00, -0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.830223896278566929872032213967, 1.00, -0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.830223896278566929872032213967, 1.00, 0.00, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 0.830223896278566929872032213967, 1.00, 0.468848793470714213803771881909, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 0.830223896278566929872032213967, 1.00, 0.830223896278566929872032213967, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 0.830223896278566929872032213967, 1.00, 1.00, 0.27682604736156594801070040629 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 1.00, -1.00, -1.00, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 1.00, -1.00, -0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 1.00, -1.00, -0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 1.00, -1.00, 0.00, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 1.00, -1.00, 0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 1.00, -1.00, 0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 1.00, -1.00, 1.00, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 1.00, -0.830223896278566929872032213967, -1.00, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 1.00, -0.830223896278566929872032213967, -0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 1.00, -0.830223896278566929872032213967, -0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 1.00, -0.830223896278566929872032213967, 0.00, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 1.00, -0.830223896278566929872032213967, 0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 1.00, -0.830223896278566929872032213967, 0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 1.00, -0.830223896278566929872032213967, 1.00, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 1.00, -0.468848793470714213803771881909, -1.00, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 1.00, -0.468848793470714213803771881909, -0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 1.00, -0.468848793470714213803771881909, -0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 1.00, -0.468848793470714213803771881909, 0.00, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 1.00, -0.468848793470714213803771881909, 0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 1.00, -0.468848793470714213803771881909, 0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 1.00, -0.468848793470714213803771881909, 1.00, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 1.00, 0.00, -1.00, 0.047619047619047619047619047619 * 0.487619047619047619047619047619 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 1.00, 0.00, -0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.487619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 1.00, 0.00, -0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.487619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 1.00, 0.00, 0.00, 0.047619047619047619047619047619 * 0.487619047619047619047619047619 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 1.00, 0.00, 0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.487619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 1.00, 0.00, 0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.487619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 1.00, 0.00, 1.00, 0.047619047619047619047619047619 * 0.487619047619047619047619047619 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 1.00, 0.468848793470714213803771881909, -1.00, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 1.00, 0.468848793470714213803771881909, -0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 1.00, 0.468848793470714213803771881909, -0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 1.00, 0.468848793470714213803771881909, 0.00, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 1.00, 0.468848793470714213803771881909, 0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 1.00, 0.468848793470714213803771881909, 0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 1.00, 0.468848793470714213803771881909, 1.00, 0.047619047619047619047619047619 * 0.431745381209862623417871022281 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 1.00, 0.830223896278566929872032213967, -1.00, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 1.00, 0.830223896278566929872032213967, -0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 1.00, 0.830223896278566929872032213967, -0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 1.00, 0.830223896278566929872032213967, 0.00, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 1.00, 0.830223896278566929872032213967, 0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 1.00, 0.830223896278566929872032213967, 0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 1.00, 0.830223896278566929872032213967, 1.00, 0.047619047619047619047619047619 * 0.27682604736156594801070040629 * 0.047619047619047619047619047619 ),

        IntegrationPointType( 1.00, 1.00, -1.00, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),
        IntegrationPointType( 1.00, 1.00, -0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 1.00, 1.00, -0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 1.00, 1.00, 0.00, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.487619047619047619047619047619 ),
        IntegrationPointType( 1.00, 1.00, 0.468848793470714213803771881909, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.431745381209862623417871022281 ),
        IntegrationPointType( 1.00, 1.00, 0.830223896278566929872032213967, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.27682604736156594801070040629 ),
        IntegrationPointType( 1.00, 1.00, 1.00, 0.047619047619047619047619047619 * 0.047619047619047619047619047619 * 0.047619047619047619047619047619 ),

    }
};

HexahedronGaussLobattoIntegrationPoints8::IntegrationPointsArrayType
HexahedronGaussLobattoIntegrationPoints8::msIntegrationPoints =
{
    {
        IntegrationPointType( -1.00, -1.00, -1.00, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -1.00, -1.00, -0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -1.00, -1.00, -0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -1.00, -1.00, -0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -1.00, -1.00, 0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -1.00, -1.00, 0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -1.00, -1.00, 0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -1.00, -1.00, 1.00, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -1.00, -0.871740148509606615337445761221, -1.00, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -1.00, -0.871740148509606615337445761221, -0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -1.00, -0.871740148509606615337445761221, -0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -1.00, -0.871740148509606615337445761221, -0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -1.00, -0.871740148509606615337445761221, 0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -1.00, -0.871740148509606615337445761221, 0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -1.00, -0.871740148509606615337445761221, 0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -1.00, -0.871740148509606615337445761221, 1.00, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -1.00, -0.591700181433142302144510731398, -1.00, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -1.00, -0.591700181433142302144510731398, -0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -1.00, -0.591700181433142302144510731398, -0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -1.00, -0.591700181433142302144510731398, -0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -1.00, -0.591700181433142302144510731398, 0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -1.00, -0.591700181433142302144510731398, 0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -1.00, -0.591700181433142302144510731398, 0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -1.00, -0.591700181433142302144510731398, 1.00, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -1.00, -0.209299217902478868768657260345, -1.00, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -1.00, -0.209299217902478868768657260345, -0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -1.00, -0.209299217902478868768657260345, -0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -1.00, -0.209299217902478868768657260345, -0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -1.00, -0.209299217902478868768657260345, 0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -1.00, -0.209299217902478868768657260345, 0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -1.00, -0.209299217902478868768657260345, 0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -1.00, -0.209299217902478868768657260345, 1.00, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -1.00, 0.209299217902478868768657260345, -1.00, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -1.00, 0.209299217902478868768657260345, -0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -1.00, 0.209299217902478868768657260345, -0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -1.00, 0.209299217902478868768657260345, -0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -1.00, 0.209299217902478868768657260345, 0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -1.00, 0.209299217902478868768657260345, 0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -1.00, 0.209299217902478868768657260345, 0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -1.00, 0.209299217902478868768657260345, 1.00, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -1.00, 0.591700181433142302144510731398, -1.00, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -1.00, 0.591700181433142302144510731398, -0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -1.00, 0.591700181433142302144510731398, -0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -1.00, 0.591700181433142302144510731398, -0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -1.00, 0.591700181433142302144510731398, 0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -1.00, 0.591700181433142302144510731398, 0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -1.00, 0.591700181433142302144510731398, 0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -1.00, 0.591700181433142302144510731398, 1.00, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -1.00, 0.871740148509606615337445761221, -1.00, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -1.00, 0.871740148509606615337445761221, -0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -1.00, 0.871740148509606615337445761221, -0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -1.00, 0.871740148509606615337445761221, -0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -1.00, 0.871740148509606615337445761221, 0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -1.00, 0.871740148509606615337445761221, 0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -1.00, 0.871740148509606615337445761221, 0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -1.00, 0.871740148509606615337445761221, 1.00, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -1.00, 1.00, -1.00, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -1.00, 1.00, -0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -1.00, 1.00, -0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -1.00, 1.00, -0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -1.00, 1.00, 0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -1.00, 1.00, 0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -1.00, 1.00, 0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -1.00, 1.00, 1.00, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -0.871740148509606615337445761221, -1.00, -1.00, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -0.871740148509606615337445761221, -1.00, -0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.871740148509606615337445761221, -1.00, -0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.871740148509606615337445761221, -1.00, -0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.871740148509606615337445761221, -1.00, 0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.871740148509606615337445761221, -1.00, 0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.871740148509606615337445761221, -1.00, 0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.871740148509606615337445761221, -1.00, 1.00, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -0.871740148509606615337445761221, -0.871740148509606615337445761221, -1.00, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -0.871740148509606615337445761221, -0.871740148509606615337445761221, -0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.871740148509606615337445761221, -0.871740148509606615337445761221, -0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.871740148509606615337445761221, -0.871740148509606615337445761221, -0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.871740148509606615337445761221, -0.871740148509606615337445761221, 0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.871740148509606615337445761221, -0.871740148509606615337445761221, 0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.871740148509606615337445761221, -0.871740148509606615337445761221, 0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.871740148509606615337445761221, -0.871740148509606615337445761221, 1.00, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -0.871740148509606615337445761221, -0.591700181433142302144510731398, -1.00, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -0.871740148509606615337445761221, -0.591700181433142302144510731398, -0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.871740148509606615337445761221, -0.591700181433142302144510731398, -0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.871740148509606615337445761221, -0.591700181433142302144510731398, -0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.871740148509606615337445761221, -0.591700181433142302144510731398, 0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.871740148509606615337445761221, -0.591700181433142302144510731398, 0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.871740148509606615337445761221, -0.591700181433142302144510731398, 0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.871740148509606615337445761221, -0.591700181433142302144510731398, 1.00, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -0.871740148509606615337445761221, -0.209299217902478868768657260345, -1.00, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -0.871740148509606615337445761221, -0.209299217902478868768657260345, -0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.871740148509606615337445761221, -0.209299217902478868768657260345, -0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.871740148509606615337445761221, -0.209299217902478868768657260345, -0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.871740148509606615337445761221, -0.209299217902478868768657260345, 0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.871740148509606615337445761221, -0.209299217902478868768657260345, 0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.871740148509606615337445761221, -0.209299217902478868768657260345, 0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.871740148509606615337445761221, -0.209299217902478868768657260345, 1.00, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -0.871740148509606615337445761221, 0.209299217902478868768657260345, -1.00, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -0.871740148509606615337445761221, 0.209299217902478868768657260345, -0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.871740148509606615337445761221, 0.209299217902478868768657260345, -0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.871740148509606615337445761221, 0.209299217902478868768657260345, -0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.871740148509606615337445761221, 0.209299217902478868768657260345, 0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.871740148509606615337445761221, 0.209299217902478868768657260345, 0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.871740148509606615337445761221, 0.209299217902478868768657260345, 0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.871740148509606615337445761221, 0.209299217902478868768657260345, 1.00, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -0.871740148509606615337445761221, 0.591700181433142302144510731398, -1.00, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -0.871740148509606615337445761221, 0.591700181433142302144510731398, -0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.871740148509606615337445761221, 0.591700181433142302144510731398, -0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.871740148509606615337445761221, 0.591700181433142302144510731398, -0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.871740148509606615337445761221, 0.591700181433142302144510731398, 0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.871740148509606615337445761221, 0.591700181433142302144510731398, 0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.871740148509606615337445761221, 0.591700181433142302144510731398, 0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.871740148509606615337445761221, 0.591700181433142302144510731398, 1.00, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -0.871740148509606615337445761221, 0.871740148509606615337445761221, -1.00, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -0.871740148509606615337445761221, 0.871740148509606615337445761221, -0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.871740148509606615337445761221, 0.871740148509606615337445761221, -0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.871740148509606615337445761221, 0.871740148509606615337445761221, -0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.871740148509606615337445761221, 0.871740148509606615337445761221, 0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.871740148509606615337445761221, 0.871740148509606615337445761221, 0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.871740148509606615337445761221, 0.871740148509606615337445761221, 0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.871740148509606615337445761221, 0.871740148509606615337445761221, 1.00, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -0.871740148509606615337445761221, 1.00, -1.00, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -0.871740148509606615337445761221, 1.00, -0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.871740148509606615337445761221, 1.00, -0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.871740148509606615337445761221, 1.00, -0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.871740148509606615337445761221, 1.00, 0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.871740148509606615337445761221, 1.00, 0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.871740148509606615337445761221, 1.00, 0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.871740148509606615337445761221, 1.00, 1.00, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -0.591700181433142302144510731398, -1.00, -1.00, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -0.591700181433142302144510731398, -1.00, -0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.591700181433142302144510731398, -1.00, -0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.591700181433142302144510731398, -1.00, -0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.591700181433142302144510731398, -1.00, 0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.591700181433142302144510731398, -1.00, 0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.591700181433142302144510731398, -1.00, 0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.591700181433142302144510731398, -1.00, 1.00, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -0.591700181433142302144510731398, -0.871740148509606615337445761221, -1.00, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -0.591700181433142302144510731398, -0.871740148509606615337445761221, -0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.591700181433142302144510731398, -0.871740148509606615337445761221, -0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.591700181433142302144510731398, -0.871740148509606615337445761221, -0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.591700181433142302144510731398, -0.871740148509606615337445761221, 0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.591700181433142302144510731398, -0.871740148509606615337445761221, 0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.591700181433142302144510731398, -0.871740148509606615337445761221, 0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.591700181433142302144510731398, -0.871740148509606615337445761221, 1.00, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -0.591700181433142302144510731398, -0.591700181433142302144510731398, -1.00, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -0.591700181433142302144510731398, -0.591700181433142302144510731398, -0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.591700181433142302144510731398, -0.591700181433142302144510731398, -0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.591700181433142302144510731398, -0.591700181433142302144510731398, -0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.591700181433142302144510731398, -0.591700181433142302144510731398, 0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.591700181433142302144510731398, -0.591700181433142302144510731398, 0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.591700181433142302144510731398, -0.591700181433142302144510731398, 0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.591700181433142302144510731398, -0.591700181433142302144510731398, 1.00, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -0.591700181433142302144510731398, -0.209299217902478868768657260345, -1.00, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -0.591700181433142302144510731398, -0.209299217902478868768657260345, -0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.591700181433142302144510731398, -0.209299217902478868768657260345, -0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.591700181433142302144510731398, -0.209299217902478868768657260345, -0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.591700181433142302144510731398, -0.209299217902478868768657260345, 0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.591700181433142302144510731398, -0.209299217902478868768657260345, 0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.591700181433142302144510731398, -0.209299217902478868768657260345, 0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.591700181433142302144510731398, -0.209299217902478868768657260345, 1.00, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -0.591700181433142302144510731398, 0.209299217902478868768657260345, -1.00, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -0.591700181433142302144510731398, 0.209299217902478868768657260345, -0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.591700181433142302144510731398, 0.209299217902478868768657260345, -0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.591700181433142302144510731398, 0.209299217902478868768657260345, -0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.591700181433142302144510731398, 0.209299217902478868768657260345, 0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.591700181433142302144510731398, 0.209299217902478868768657260345, 0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.591700181433142302144510731398, 0.209299217902478868768657260345, 0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.591700181433142302144510731398, 0.209299217902478868768657260345, 1.00, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -0.591700181433142302144510731398, 0.591700181433142302144510731398, -1.00, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -0.591700181433142302144510731398, 0.591700181433142302144510731398, -0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.591700181433142302144510731398, 0.591700181433142302144510731398, -0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.591700181433142302144510731398, 0.591700181433142302144510731398, -0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.591700181433142302144510731398, 0.591700181433142302144510731398, 0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.591700181433142302144510731398, 0.591700181433142302144510731398, 0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.591700181433142302144510731398, 0.591700181433142302144510731398, 0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.591700181433142302144510731398, 0.591700181433142302144510731398, 1.00, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -0.591700181433142302144510731398, 0.871740148509606615337445761221, -1.00, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -0.591700181433142302144510731398, 0.871740148509606615337445761221, -0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.591700181433142302144510731398, 0.871740148509606615337445761221, -0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.591700181433142302144510731398, 0.871740148509606615337445761221, -0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.591700181433142302144510731398, 0.871740148509606615337445761221, 0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.591700181433142302144510731398, 0.871740148509606615337445761221, 0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.591700181433142302144510731398, 0.871740148509606615337445761221, 0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.591700181433142302144510731398, 0.871740148509606615337445761221, 1.00, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -0.591700181433142302144510731398, 1.00, -1.00, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -0.591700181433142302144510731398, 1.00, -0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.591700181433142302144510731398, 1.00, -0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.591700181433142302144510731398, 1.00, -0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.591700181433142302144510731398, 1.00, 0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.591700181433142302144510731398, 1.00, 0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.591700181433142302144510731398, 1.00, 0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.591700181433142302144510731398, 1.00, 1.00, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -0.209299217902478868768657260345, -1.00, -1.00, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -0.209299217902478868768657260345, -1.00, -0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.209299217902478868768657260345, -1.00, -0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.209299217902478868768657260345, -1.00, -0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.209299217902478868768657260345, -1.00, 0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.209299217902478868768657260345, -1.00, 0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.209299217902478868768657260345, -1.00, 0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.209299217902478868768657260345, -1.00, 1.00, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -0.209299217902478868768657260345, -0.871740148509606615337445761221, -1.00, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -0.209299217902478868768657260345, -0.871740148509606615337445761221, -0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.209299217902478868768657260345, -0.871740148509606615337445761221, -0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.209299217902478868768657260345, -0.871740148509606615337445761221, -0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.209299217902478868768657260345, -0.871740148509606615337445761221, 0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.209299217902478868768657260345, -0.871740148509606615337445761221, 0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.209299217902478868768657260345, -0.871740148509606615337445761221, 0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.209299217902478868768657260345, -0.871740148509606615337445761221, 1.00, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -0.209299217902478868768657260345, -0.591700181433142302144510731398, -1.00, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -0.209299217902478868768657260345, -0.591700181433142302144510731398, -0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.209299217902478868768657260345, -0.591700181433142302144510731398, -0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.209299217902478868768657260345, -0.591700181433142302144510731398, -0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.209299217902478868768657260345, -0.591700181433142302144510731398, 0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.209299217902478868768657260345, -0.591700181433142302144510731398, 0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.209299217902478868768657260345, -0.591700181433142302144510731398, 0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.209299217902478868768657260345, -0.591700181433142302144510731398, 1.00, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -0.209299217902478868768657260345, -0.209299217902478868768657260345, -1.00, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -0.209299217902478868768657260345, -0.209299217902478868768657260345, -0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.209299217902478868768657260345, -0.209299217902478868768657260345, -0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.209299217902478868768657260345, -0.209299217902478868768657260345, -0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.209299217902478868768657260345, -0.209299217902478868768657260345, 0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.209299217902478868768657260345, -0.209299217902478868768657260345, 0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.209299217902478868768657260345, -0.209299217902478868768657260345, 0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.209299217902478868768657260345, -0.209299217902478868768657260345, 1.00, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -0.209299217902478868768657260345, 0.209299217902478868768657260345, -1.00, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -0.209299217902478868768657260345, 0.209299217902478868768657260345, -0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.209299217902478868768657260345, 0.209299217902478868768657260345, -0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.209299217902478868768657260345, 0.209299217902478868768657260345, -0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.209299217902478868768657260345, 0.209299217902478868768657260345, 0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.209299217902478868768657260345, 0.209299217902478868768657260345, 0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.209299217902478868768657260345, 0.209299217902478868768657260345, 0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.209299217902478868768657260345, 0.209299217902478868768657260345, 1.00, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -0.209299217902478868768657260345, 0.591700181433142302144510731398, -1.00, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -0.209299217902478868768657260345, 0.591700181433142302144510731398, -0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.209299217902478868768657260345, 0.591700181433142302144510731398, -0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.209299217902478868768657260345, 0.591700181433142302144510731398, -0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.209299217902478868768657260345, 0.591700181433142302144510731398, 0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.209299217902478868768657260345, 0.591700181433142302144510731398, 0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.209299217902478868768657260345, 0.591700181433142302144510731398, 0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.209299217902478868768657260345, 0.591700181433142302144510731398, 1.00, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -0.209299217902478868768657260345, 0.871740148509606615337445761221, -1.00, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -0.209299217902478868768657260345, 0.871740148509606615337445761221, -0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.209299217902478868768657260345, 0.871740148509606615337445761221, -0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.209299217902478868768657260345, 0.871740148509606615337445761221, -0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.209299217902478868768657260345, 0.871740148509606615337445761221, 0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.209299217902478868768657260345, 0.871740148509606615337445761221, 0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.209299217902478868768657260345, 0.871740148509606615337445761221, 0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.209299217902478868768657260345, 0.871740148509606615337445761221, 1.00, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( -0.209299217902478868768657260345, 1.00, -1.00, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( -0.209299217902478868768657260345, 1.00, -0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.209299217902478868768657260345, 1.00, -0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.209299217902478868768657260345, 1.00, -0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.209299217902478868768657260345, 1.00, 0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( -0.209299217902478868768657260345, 1.00, 0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( -0.209299217902478868768657260345, 1.00, 0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( -0.209299217902478868768657260345, 1.00, 1.00, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 0.209299217902478868768657260345, -1.00, -1.00, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 0.209299217902478868768657260345, -1.00, -0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.209299217902478868768657260345, -1.00, -0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.209299217902478868768657260345, -1.00, -0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.209299217902478868768657260345, -1.00, 0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.209299217902478868768657260345, -1.00, 0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.209299217902478868768657260345, -1.00, 0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.209299217902478868768657260345, -1.00, 1.00, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 0.209299217902478868768657260345, -0.871740148509606615337445761221, -1.00, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 0.209299217902478868768657260345, -0.871740148509606615337445761221, -0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.209299217902478868768657260345, -0.871740148509606615337445761221, -0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.209299217902478868768657260345, -0.871740148509606615337445761221, -0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.209299217902478868768657260345, -0.871740148509606615337445761221, 0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.209299217902478868768657260345, -0.871740148509606615337445761221, 0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.209299217902478868768657260345, -0.871740148509606615337445761221, 0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.209299217902478868768657260345, -0.871740148509606615337445761221, 1.00, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 0.209299217902478868768657260345, -0.591700181433142302144510731398, -1.00, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 0.209299217902478868768657260345, -0.591700181433142302144510731398, -0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.209299217902478868768657260345, -0.591700181433142302144510731398, -0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.209299217902478868768657260345, -0.591700181433142302144510731398, -0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.209299217902478868768657260345, -0.591700181433142302144510731398, 0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.209299217902478868768657260345, -0.591700181433142302144510731398, 0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.209299217902478868768657260345, -0.591700181433142302144510731398, 0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.209299217902478868768657260345, -0.591700181433142302144510731398, 1.00, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 0.209299217902478868768657260345, -0.209299217902478868768657260345, -1.00, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 0.209299217902478868768657260345, -0.209299217902478868768657260345, -0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.209299217902478868768657260345, -0.209299217902478868768657260345, -0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.209299217902478868768657260345, -0.209299217902478868768657260345, -0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.209299217902478868768657260345, -0.209299217902478868768657260345, 0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.209299217902478868768657260345, -0.209299217902478868768657260345, 0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.209299217902478868768657260345, -0.209299217902478868768657260345, 0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.209299217902478868768657260345, -0.209299217902478868768657260345, 1.00, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 0.209299217902478868768657260345, 0.209299217902478868768657260345, -1.00, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 0.209299217902478868768657260345, 0.209299217902478868768657260345, -0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.209299217902478868768657260345, 0.209299217902478868768657260345, -0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.209299217902478868768657260345, 0.209299217902478868768657260345, -0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.209299217902478868768657260345, 0.209299217902478868768657260345, 0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.209299217902478868768657260345, 0.209299217902478868768657260345, 0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.209299217902478868768657260345, 0.209299217902478868768657260345, 0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.209299217902478868768657260345, 0.209299217902478868768657260345, 1.00, 0.4124587946587038815670529714 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 0.209299217902478868768657260345, 0.591700181433142302144510731398, -1.00, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 0.209299217902478868768657260345, 0.591700181433142302144510731398, -0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.209299217902478868768657260345, 0.591700181433142302144510731398, -0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.209299217902478868768657260345, 0.591700181433142302144510731398, -0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.209299217902478868768657260345, 0.591700181433142302144510731398, 0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.209299217902478868768657260345, 0.591700181433142302144510731398, 0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.209299217902478868768657260345, 0.591700181433142302144510731398, 0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.209299217902478868768657260345, 0.591700181433142302144510731398, 1.00, 0.4124587946587038815670529714 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 0.209299217902478868768657260345, 0.871740148509606615337445761221, -1.00, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 0.209299217902478868768657260345, 0.871740148509606615337445761221, -0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.209299217902478868768657260345, 0.871740148509606615337445761221, -0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.209299217902478868768657260345, 0.871740148509606615337445761221, -0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.209299217902478868768657260345, 0.871740148509606615337445761221, 0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.209299217902478868768657260345, 0.871740148509606615337445761221, 0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.209299217902478868768657260345, 0.871740148509606615337445761221, 0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.209299217902478868768657260345, 0.871740148509606615337445761221, 1.00, 0.4124587946587038815670529714 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 0.209299217902478868768657260345, 1.00, -1.00, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 0.209299217902478868768657260345, 1.00, -0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.209299217902478868768657260345, 1.00, -0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.209299217902478868768657260345, 1.00, -0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.209299217902478868768657260345, 1.00, 0.209299217902478868768657260345, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.209299217902478868768657260345, 1.00, 0.591700181433142302144510731398, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.209299217902478868768657260345, 1.00, 0.871740148509606615337445761221, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.209299217902478868768657260345, 1.00, 1.00, 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 0.591700181433142302144510731398, -1.00, -1.00, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 0.591700181433142302144510731398, -1.00, -0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.591700181433142302144510731398, -1.00, -0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.591700181433142302144510731398, -1.00, -0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.591700181433142302144510731398, -1.00, 0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.591700181433142302144510731398, -1.00, 0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.591700181433142302144510731398, -1.00, 0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.591700181433142302144510731398, -1.00, 1.00, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 0.591700181433142302144510731398, -0.871740148509606615337445761221, -1.00, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 0.591700181433142302144510731398, -0.871740148509606615337445761221, -0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.591700181433142302144510731398, -0.871740148509606615337445761221, -0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.591700181433142302144510731398, -0.871740148509606615337445761221, -0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.591700181433142302144510731398, -0.871740148509606615337445761221, 0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.591700181433142302144510731398, -0.871740148509606615337445761221, 0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.591700181433142302144510731398, -0.871740148509606615337445761221, 0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.591700181433142302144510731398, -0.871740148509606615337445761221, 1.00, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 0.591700181433142302144510731398, -0.591700181433142302144510731398, -1.00, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 0.591700181433142302144510731398, -0.591700181433142302144510731398, -0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.591700181433142302144510731398, -0.591700181433142302144510731398, -0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.591700181433142302144510731398, -0.591700181433142302144510731398, -0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.591700181433142302144510731398, -0.591700181433142302144510731398, 0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.591700181433142302144510731398, -0.591700181433142302144510731398, 0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.591700181433142302144510731398, -0.591700181433142302144510731398, 0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.591700181433142302144510731398, -0.591700181433142302144510731398, 1.00, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 0.591700181433142302144510731398, -0.209299217902478868768657260345, -1.00, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 0.591700181433142302144510731398, -0.209299217902478868768657260345, -0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.591700181433142302144510731398, -0.209299217902478868768657260345, -0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.591700181433142302144510731398, -0.209299217902478868768657260345, -0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.591700181433142302144510731398, -0.209299217902478868768657260345, 0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.591700181433142302144510731398, -0.209299217902478868768657260345, 0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.591700181433142302144510731398, -0.209299217902478868768657260345, 0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.591700181433142302144510731398, -0.209299217902478868768657260345, 1.00, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 0.591700181433142302144510731398, 0.209299217902478868768657260345, -1.00, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 0.591700181433142302144510731398, 0.209299217902478868768657260345, -0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.591700181433142302144510731398, 0.209299217902478868768657260345, -0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.591700181433142302144510731398, 0.209299217902478868768657260345, -0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.591700181433142302144510731398, 0.209299217902478868768657260345, 0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.591700181433142302144510731398, 0.209299217902478868768657260345, 0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.591700181433142302144510731398, 0.209299217902478868768657260345, 0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.591700181433142302144510731398, 0.209299217902478868768657260345, 1.00, 0.34112269248350436476424067711 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 0.591700181433142302144510731398, 0.591700181433142302144510731398, -1.00, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 0.591700181433142302144510731398, 0.591700181433142302144510731398, -0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.591700181433142302144510731398, 0.591700181433142302144510731398, -0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.591700181433142302144510731398, 0.591700181433142302144510731398, -0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.591700181433142302144510731398, 0.591700181433142302144510731398, 0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.591700181433142302144510731398, 0.591700181433142302144510731398, 0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.591700181433142302144510731398, 0.591700181433142302144510731398, 0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.591700181433142302144510731398, 0.591700181433142302144510731398, 1.00, 0.34112269248350436476424067711 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 0.591700181433142302144510731398, 0.871740148509606615337445761221, -1.00, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 0.591700181433142302144510731398, 0.871740148509606615337445761221, -0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.591700181433142302144510731398, 0.871740148509606615337445761221, -0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.591700181433142302144510731398, 0.871740148509606615337445761221, -0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.591700181433142302144510731398, 0.871740148509606615337445761221, 0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.591700181433142302144510731398, 0.871740148509606615337445761221, 0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.591700181433142302144510731398, 0.871740148509606615337445761221, 0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.591700181433142302144510731398, 0.871740148509606615337445761221, 1.00, 0.34112269248350436476424067711 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 0.591700181433142302144510731398, 1.00, -1.00, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 0.591700181433142302144510731398, 1.00, -0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.591700181433142302144510731398, 1.00, -0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.591700181433142302144510731398, 1.00, -0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.591700181433142302144510731398, 1.00, 0.209299217902478868768657260345, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.591700181433142302144510731398, 1.00, 0.591700181433142302144510731398, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.591700181433142302144510731398, 1.00, 0.871740148509606615337445761221, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.591700181433142302144510731398, 1.00, 1.00, 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 0.871740148509606615337445761221, -1.00, -1.00, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 0.871740148509606615337445761221, -1.00, -0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.871740148509606615337445761221, -1.00, -0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.871740148509606615337445761221, -1.00, -0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.871740148509606615337445761221, -1.00, 0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.871740148509606615337445761221, -1.00, 0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.871740148509606615337445761221, -1.00, 0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.871740148509606615337445761221, -1.00, 1.00, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 0.871740148509606615337445761221, -0.871740148509606615337445761221, -1.00, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 0.871740148509606615337445761221, -0.871740148509606615337445761221, -0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.871740148509606615337445761221, -0.871740148509606615337445761221, -0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.871740148509606615337445761221, -0.871740148509606615337445761221, -0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.871740148509606615337445761221, -0.871740148509606615337445761221, 0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.871740148509606615337445761221, -0.871740148509606615337445761221, 0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.871740148509606615337445761221, -0.871740148509606615337445761221, 0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.871740148509606615337445761221, -0.871740148509606615337445761221, 1.00, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 0.871740148509606615337445761221, -0.591700181433142302144510731398, -1.00, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 0.871740148509606615337445761221, -0.591700181433142302144510731398, -0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.871740148509606615337445761221, -0.591700181433142302144510731398, -0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.871740148509606615337445761221, -0.591700181433142302144510731398, -0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.871740148509606615337445761221, -0.591700181433142302144510731398, 0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.871740148509606615337445761221, -0.591700181433142302144510731398, 0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.871740148509606615337445761221, -0.591700181433142302144510731398, 0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.871740148509606615337445761221, -0.591700181433142302144510731398, 1.00, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 0.871740148509606615337445761221, -0.209299217902478868768657260345, -1.00, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 0.871740148509606615337445761221, -0.209299217902478868768657260345, -0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.871740148509606615337445761221, -0.209299217902478868768657260345, -0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.871740148509606615337445761221, -0.209299217902478868768657260345, -0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.871740148509606615337445761221, -0.209299217902478868768657260345, 0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.871740148509606615337445761221, -0.209299217902478868768657260345, 0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.871740148509606615337445761221, -0.209299217902478868768657260345, 0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.871740148509606615337445761221, -0.209299217902478868768657260345, 1.00, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 0.871740148509606615337445761221, 0.209299217902478868768657260345, -1.00, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 0.871740148509606615337445761221, 0.209299217902478868768657260345, -0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.871740148509606615337445761221, 0.209299217902478868768657260345, -0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.871740148509606615337445761221, 0.209299217902478868768657260345, -0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.871740148509606615337445761221, 0.209299217902478868768657260345, 0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.871740148509606615337445761221, 0.209299217902478868768657260345, 0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.871740148509606615337445761221, 0.209299217902478868768657260345, 0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.871740148509606615337445761221, 0.209299217902478868768657260345, 1.00, 0.21070422714350603938299206578 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 0.871740148509606615337445761221, 0.591700181433142302144510731398, -1.00, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 0.871740148509606615337445761221, 0.591700181433142302144510731398, -0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.871740148509606615337445761221, 0.591700181433142302144510731398, -0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.871740148509606615337445761221, 0.591700181433142302144510731398, -0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.871740148509606615337445761221, 0.591700181433142302144510731398, 0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.871740148509606615337445761221, 0.591700181433142302144510731398, 0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.871740148509606615337445761221, 0.591700181433142302144510731398, 0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.871740148509606615337445761221, 0.591700181433142302144510731398, 1.00, 0.21070422714350603938299206578 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 0.871740148509606615337445761221, 0.871740148509606615337445761221, -1.00, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 0.871740148509606615337445761221, 0.871740148509606615337445761221, -0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.871740148509606615337445761221, 0.871740148509606615337445761221, -0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.871740148509606615337445761221, 0.871740148509606615337445761221, -0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.871740148509606615337445761221, 0.871740148509606615337445761221, 0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.871740148509606615337445761221, 0.871740148509606615337445761221, 0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.871740148509606615337445761221, 0.871740148509606615337445761221, 0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.871740148509606615337445761221, 0.871740148509606615337445761221, 1.00, 0.21070422714350603938299206578 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 0.871740148509606615337445761221, 1.00, -1.00, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 0.871740148509606615337445761221, 1.00, -0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.871740148509606615337445761221, 1.00, -0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.871740148509606615337445761221, 1.00, -0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.871740148509606615337445761221, 1.00, 0.209299217902478868768657260345, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 0.871740148509606615337445761221, 1.00, 0.591700181433142302144510731398, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 0.871740148509606615337445761221, 1.00, 0.871740148509606615337445761221, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 0.871740148509606615337445761221, 1.00, 1.00, 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 1.00, -1.00, -1.00, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 1.00, -1.00, -0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 1.00, -1.00, -0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 1.00, -1.00, -0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 1.00, -1.00, 0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 1.00, -1.00, 0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 1.00, -1.00, 0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 1.00, -1.00, 1.00, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 1.00, -0.871740148509606615337445761221, -1.00, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 1.00, -0.871740148509606615337445761221, -0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 1.00, -0.871740148509606615337445761221, -0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 1.00, -0.871740148509606615337445761221, -0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 1.00, -0.871740148509606615337445761221, 0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 1.00, -0.871740148509606615337445761221, 0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 1.00, -0.871740148509606615337445761221, 0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 1.00, -0.871740148509606615337445761221, 1.00, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 1.00, -0.591700181433142302144510731398, -1.00, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 1.00, -0.591700181433142302144510731398, -0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 1.00, -0.591700181433142302144510731398, -0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 1.00, -0.591700181433142302144510731398, -0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 1.00, -0.591700181433142302144510731398, 0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 1.00, -0.591700181433142302144510731398, 0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 1.00, -0.591700181433142302144510731398, 0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 1.00, -0.591700181433142302144510731398, 1.00, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 1.00, -0.209299217902478868768657260345, -1.00, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 1.00, -0.209299217902478868768657260345, -0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 1.00, -0.209299217902478868768657260345, -0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 1.00, -0.209299217902478868768657260345, -0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 1.00, -0.209299217902478868768657260345, 0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 1.00, -0.209299217902478868768657260345, 0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 1.00, -0.209299217902478868768657260345, 0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 1.00, -0.209299217902478868768657260345, 1.00, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 1.00, 0.209299217902478868768657260345, -1.00, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 1.00, 0.209299217902478868768657260345, -0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 1.00, 0.209299217902478868768657260345, -0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 1.00, 0.209299217902478868768657260345, -0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 1.00, 0.209299217902478868768657260345, 0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 1.00, 0.209299217902478868768657260345, 0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 1.00, 0.209299217902478868768657260345, 0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 1.00, 0.209299217902478868768657260345, 1.00, 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 1.00, 0.591700181433142302144510731398, -1.00, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 1.00, 0.591700181433142302144510731398, -0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 1.00, 0.591700181433142302144510731398, -0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 1.00, 0.591700181433142302144510731398, -0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 1.00, 0.591700181433142302144510731398, 0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 1.00, 0.591700181433142302144510731398, 0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 1.00, 0.591700181433142302144510731398, 0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 1.00, 0.591700181433142302144510731398, 1.00, 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 1.00, 0.871740148509606615337445761221, -1.00, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 1.00, 0.871740148509606615337445761221, -0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 1.00, 0.871740148509606615337445761221, -0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 1.00, 0.871740148509606615337445761221, -0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 1.00, 0.871740148509606615337445761221, 0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 1.00, 0.871740148509606615337445761221, 0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 1.00, 0.871740148509606615337445761221, 0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 1.00, 0.871740148509606615337445761221, 1.00, 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 * 0.0357142857142857142857142857143 ),

        IntegrationPointType( 1.00, 1.00, -1.00, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),
        IntegrationPointType( 1.00, 1.00, -0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 1.00, 1.00, -0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 1.00, 1.00, -0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 1.00, 1.00, 0.209299217902478868768657260345, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.4124587946587038815670529714 ),
        IntegrationPointType( 1.00, 1.00, 0.591700181433142302144510731398, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.34112269248350436476424067711 ),
        IntegrationPointType( 1.00, 1.00, 0.871740148509606615337445761221, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.21070422714350603938299206578 ),
        IntegrationPointType( 1.00, 1.00, 1.00, 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 * 0.0357142857142857142857142857143 ),

    }
};

HexahedronGaussLobattoIntegrationPoints9::IntegrationPointsArrayType
HexahedronGaussLobattoIntegrationPoints9::msIntegrationPoints =
{
    {
        IntegrationPointType( -1.00, -1.00, -1.00, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -1.00, -1.00, -0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -1.00, -1.00, -0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -1.00, -1.00, -0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -1.00, -1.00, 0.00, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -1.00, -1.00, 0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -1.00, -1.00, 0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -1.00, -1.00, 0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -1.00, -1.00, 1.00, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -1.00, -0.899757995411460157312345244418, -1.00, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -1.00, -0.899757995411460157312345244418, -0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -1.00, -0.899757995411460157312345244418, -0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -1.00, -0.899757995411460157312345244418, -0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -1.00, -0.899757995411460157312345244418, 0.00, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -1.00, -0.899757995411460157312345244418, 0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -1.00, -0.899757995411460157312345244418, 0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -1.00, -0.899757995411460157312345244418, 0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -1.00, -0.899757995411460157312345244418, 1.00, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -1.00, -0.677186279510737753445885427091, -1.00, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -1.00, -0.677186279510737753445885427091, -0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -1.00, -0.677186279510737753445885427091, -0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -1.00, -0.677186279510737753445885427091, -0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -1.00, -0.677186279510737753445885427091, 0.00, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -1.00, -0.677186279510737753445885427091, 0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -1.00, -0.677186279510737753445885427091, 0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -1.00, -0.677186279510737753445885427091, 0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -1.00, -0.677186279510737753445885427091, 1.00, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -1.00, -0.363117463826178158710752068709, -1.00, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -1.00, -0.363117463826178158710752068709, -0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -1.00, -0.363117463826178158710752068709, -0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -1.00, -0.363117463826178158710752068709, -0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -1.00, -0.363117463826178158710752068709, 0.00, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -1.00, -0.363117463826178158710752068709, 0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -1.00, -0.363117463826178158710752068709, 0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -1.00, -0.363117463826178158710752068709, 0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -1.00, -0.363117463826178158710752068709, 1.00, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -1.00, 0.00, -1.00, 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -1.00, 0.00, -0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -1.00, 0.00, -0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -1.00, 0.00, -0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -1.00, 0.00, 0.00, 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -1.00, 0.00, 0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -1.00, 0.00, 0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -1.00, 0.00, 0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -1.00, 0.00, 1.00, 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -1.00, 0.363117463826178158710752068709, -1.00, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -1.00, 0.363117463826178158710752068709, -0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -1.00, 0.363117463826178158710752068709, -0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -1.00, 0.363117463826178158710752068709, -0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -1.00, 0.363117463826178158710752068709, 0.00, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -1.00, 0.363117463826178158710752068709, 0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -1.00, 0.363117463826178158710752068709, 0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -1.00, 0.363117463826178158710752068709, 0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -1.00, 0.363117463826178158710752068709, 1.00, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -1.00, 0.677186279510737753445885427091, -1.00, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -1.00, 0.677186279510737753445885427091, -0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -1.00, 0.677186279510737753445885427091, -0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -1.00, 0.677186279510737753445885427091, -0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -1.00, 0.677186279510737753445885427091, 0.00, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -1.00, 0.677186279510737753445885427091, 0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -1.00, 0.677186279510737753445885427091, 0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -1.00, 0.677186279510737753445885427091, 0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -1.00, 0.677186279510737753445885427091, 1.00, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -1.00, 0.899757995411460157312345244418, -1.00, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -1.00, 0.899757995411460157312345244418, -0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -1.00, 0.899757995411460157312345244418, -0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -1.00, 0.899757995411460157312345244418, -0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -1.00, 0.899757995411460157312345244418, 0.00, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -1.00, 0.899757995411460157312345244418, 0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -1.00, 0.899757995411460157312345244418, 0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -1.00, 0.899757995411460157312345244418, 0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -1.00, 0.899757995411460157312345244418, 1.00, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -1.00, 1.00, -1.00, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -1.00, 1.00, -0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -1.00, 1.00, -0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -1.00, 1.00, -0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -1.00, 1.00, 0.00, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -1.00, 1.00, 0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -1.00, 1.00, 0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -1.00, 1.00, 0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -1.00, 1.00, 1.00, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -0.899757995411460157312345244418, -1.00, -1.00, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -0.899757995411460157312345244418, -1.00, -0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.899757995411460157312345244418, -1.00, -0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.899757995411460157312345244418, -1.00, -0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.899757995411460157312345244418, -1.00, 0.00, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -0.899757995411460157312345244418, -1.00, 0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.899757995411460157312345244418, -1.00, 0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.899757995411460157312345244418, -1.00, 0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.899757995411460157312345244418, -1.00, 1.00, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -0.899757995411460157312345244418, -0.899757995411460157312345244418, -1.00, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -0.899757995411460157312345244418, -0.899757995411460157312345244418, -0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.899757995411460157312345244418, -0.899757995411460157312345244418, -0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.899757995411460157312345244418, -0.899757995411460157312345244418, -0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.899757995411460157312345244418, -0.899757995411460157312345244418, 0.00, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -0.899757995411460157312345244418, -0.899757995411460157312345244418, 0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.899757995411460157312345244418, -0.899757995411460157312345244418, 0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.899757995411460157312345244418, -0.899757995411460157312345244418, 0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.899757995411460157312345244418, -0.899757995411460157312345244418, 1.00, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -0.899757995411460157312345244418, -0.677186279510737753445885427091, -1.00, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -0.899757995411460157312345244418, -0.677186279510737753445885427091, -0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.899757995411460157312345244418, -0.677186279510737753445885427091, -0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.899757995411460157312345244418, -0.677186279510737753445885427091, -0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.899757995411460157312345244418, -0.677186279510737753445885427091, 0.00, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -0.899757995411460157312345244418, -0.677186279510737753445885427091, 0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.899757995411460157312345244418, -0.677186279510737753445885427091, 0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.899757995411460157312345244418, -0.677186279510737753445885427091, 0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.899757995411460157312345244418, -0.677186279510737753445885427091, 1.00, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -0.899757995411460157312345244418, -0.363117463826178158710752068709, -1.00, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -0.899757995411460157312345244418, -0.363117463826178158710752068709, -0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.899757995411460157312345244418, -0.363117463826178158710752068709, -0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.899757995411460157312345244418, -0.363117463826178158710752068709, -0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.899757995411460157312345244418, -0.363117463826178158710752068709, 0.00, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -0.899757995411460157312345244418, -0.363117463826178158710752068709, 0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.899757995411460157312345244418, -0.363117463826178158710752068709, 0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.899757995411460157312345244418, -0.363117463826178158710752068709, 0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.899757995411460157312345244418, -0.363117463826178158710752068709, 1.00, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -0.899757995411460157312345244418, 0.00, -1.00, 0.16549536156080552504633972003 * 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.00, -0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.371519274376417233560090702948 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.00, -0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.371519274376417233560090702948 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.00, -0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.371519274376417233560090702948 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.00, 0.00, 0.16549536156080552504633972003 * 0.371519274376417233560090702948 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.00, 0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.371519274376417233560090702948 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.00, 0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.371519274376417233560090702948 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.00, 0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.371519274376417233560090702948 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.00, 1.00, 0.16549536156080552504633972003 * 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -0.899757995411460157312345244418, 0.363117463826178158710752068709, -1.00, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.363117463826178158710752068709, -0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.363117463826178158710752068709, -0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.363117463826178158710752068709, -0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.363117463826178158710752068709, 0.00, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.363117463826178158710752068709, 0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.363117463826178158710752068709, 0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.363117463826178158710752068709, 0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.363117463826178158710752068709, 1.00, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -0.899757995411460157312345244418, 0.677186279510737753445885427091, -1.00, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.677186279510737753445885427091, -0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.677186279510737753445885427091, -0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.677186279510737753445885427091, -0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.677186279510737753445885427091, 0.00, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.677186279510737753445885427091, 0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.677186279510737753445885427091, 0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.677186279510737753445885427091, 0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.677186279510737753445885427091, 1.00, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -0.899757995411460157312345244418, 0.899757995411460157312345244418, -1.00, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.899757995411460157312345244418, -0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.899757995411460157312345244418, -0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.899757995411460157312345244418, -0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.899757995411460157312345244418, 0.00, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.899757995411460157312345244418, 0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.899757995411460157312345244418, 0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.899757995411460157312345244418, 0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.899757995411460157312345244418, 0.899757995411460157312345244418, 1.00, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -0.899757995411460157312345244418, 1.00, -1.00, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -0.899757995411460157312345244418, 1.00, -0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.899757995411460157312345244418, 1.00, -0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.899757995411460157312345244418, 1.00, -0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.899757995411460157312345244418, 1.00, 0.00, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -0.899757995411460157312345244418, 1.00, 0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.899757995411460157312345244418, 1.00, 0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.899757995411460157312345244418, 1.00, 0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.899757995411460157312345244418, 1.00, 1.00, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -0.677186279510737753445885427091, -1.00, -1.00, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -0.677186279510737753445885427091, -1.00, -0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.677186279510737753445885427091, -1.00, -0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.677186279510737753445885427091, -1.00, -0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.677186279510737753445885427091, -1.00, 0.00, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -0.677186279510737753445885427091, -1.00, 0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.677186279510737753445885427091, -1.00, 0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.677186279510737753445885427091, -1.00, 0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.677186279510737753445885427091, -1.00, 1.00, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -0.677186279510737753445885427091, -0.899757995411460157312345244418, -1.00, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -0.677186279510737753445885427091, -0.899757995411460157312345244418, -0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.677186279510737753445885427091, -0.899757995411460157312345244418, -0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.677186279510737753445885427091, -0.899757995411460157312345244418, -0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.677186279510737753445885427091, -0.899757995411460157312345244418, 0.00, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -0.677186279510737753445885427091, -0.899757995411460157312345244418, 0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.677186279510737753445885427091, -0.899757995411460157312345244418, 0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.677186279510737753445885427091, -0.899757995411460157312345244418, 0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.677186279510737753445885427091, -0.899757995411460157312345244418, 1.00, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -0.677186279510737753445885427091, -0.677186279510737753445885427091, -1.00, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -0.677186279510737753445885427091, -0.677186279510737753445885427091, -0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.677186279510737753445885427091, -0.677186279510737753445885427091, -0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.677186279510737753445885427091, -0.677186279510737753445885427091, -0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.677186279510737753445885427091, -0.677186279510737753445885427091, 0.00, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -0.677186279510737753445885427091, -0.677186279510737753445885427091, 0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.677186279510737753445885427091, -0.677186279510737753445885427091, 0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.677186279510737753445885427091, -0.677186279510737753445885427091, 0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.677186279510737753445885427091, -0.677186279510737753445885427091, 1.00, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -0.677186279510737753445885427091, -0.363117463826178158710752068709, -1.00, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -0.677186279510737753445885427091, -0.363117463826178158710752068709, -0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.677186279510737753445885427091, -0.363117463826178158710752068709, -0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.677186279510737753445885427091, -0.363117463826178158710752068709, -0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.677186279510737753445885427091, -0.363117463826178158710752068709, 0.00, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -0.677186279510737753445885427091, -0.363117463826178158710752068709, 0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.677186279510737753445885427091, -0.363117463826178158710752068709, 0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.677186279510737753445885427091, -0.363117463826178158710752068709, 0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.677186279510737753445885427091, -0.363117463826178158710752068709, 1.00, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -0.677186279510737753445885427091, 0.00, -1.00, 0.274538712500161735280705618579 * 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.00, -0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.371519274376417233560090702948 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.00, -0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.371519274376417233560090702948 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.00, -0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.371519274376417233560090702948 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.00, 0.00, 0.274538712500161735280705618579 * 0.371519274376417233560090702948 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.00, 0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.371519274376417233560090702948 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.00, 0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.371519274376417233560090702948 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.00, 0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.371519274376417233560090702948 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.00, 1.00, 0.274538712500161735280705618579 * 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -0.677186279510737753445885427091, 0.363117463826178158710752068709, -1.00, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.363117463826178158710752068709, -0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.363117463826178158710752068709, -0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.363117463826178158710752068709, -0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.363117463826178158710752068709, 0.00, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.363117463826178158710752068709, 0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.363117463826178158710752068709, 0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.363117463826178158710752068709, 0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.363117463826178158710752068709, 1.00, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -0.677186279510737753445885427091, 0.677186279510737753445885427091, -1.00, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.677186279510737753445885427091, -0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.677186279510737753445885427091, -0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.677186279510737753445885427091, -0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.677186279510737753445885427091, 0.00, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.677186279510737753445885427091, 0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.677186279510737753445885427091, 0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.677186279510737753445885427091, 0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.677186279510737753445885427091, 1.00, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -0.677186279510737753445885427091, 0.899757995411460157312345244418, -1.00, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.899757995411460157312345244418, -0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.899757995411460157312345244418, -0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.899757995411460157312345244418, -0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.899757995411460157312345244418, 0.00, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.899757995411460157312345244418, 0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.899757995411460157312345244418, 0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.899757995411460157312345244418, 0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.677186279510737753445885427091, 0.899757995411460157312345244418, 1.00, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -0.677186279510737753445885427091, 1.00, -1.00, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -0.677186279510737753445885427091, 1.00, -0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.677186279510737753445885427091, 1.00, -0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.677186279510737753445885427091, 1.00, -0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.677186279510737753445885427091, 1.00, 0.00, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -0.677186279510737753445885427091, 1.00, 0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.677186279510737753445885427091, 1.00, 0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.677186279510737753445885427091, 1.00, 0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.677186279510737753445885427091, 1.00, 1.00, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -0.363117463826178158710752068709, -1.00, -1.00, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -0.363117463826178158710752068709, -1.00, -0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.363117463826178158710752068709, -1.00, -0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.363117463826178158710752068709, -1.00, -0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.363117463826178158710752068709, -1.00, 0.00, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -0.363117463826178158710752068709, -1.00, 0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.363117463826178158710752068709, -1.00, 0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.363117463826178158710752068709, -1.00, 0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.363117463826178158710752068709, -1.00, 1.00, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -0.363117463826178158710752068709, -0.899757995411460157312345244418, -1.00, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -0.363117463826178158710752068709, -0.899757995411460157312345244418, -0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.363117463826178158710752068709, -0.899757995411460157312345244418, -0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.363117463826178158710752068709, -0.899757995411460157312345244418, -0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.363117463826178158710752068709, -0.899757995411460157312345244418, 0.00, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -0.363117463826178158710752068709, -0.899757995411460157312345244418, 0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.363117463826178158710752068709, -0.899757995411460157312345244418, 0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.363117463826178158710752068709, -0.899757995411460157312345244418, 0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.363117463826178158710752068709, -0.899757995411460157312345244418, 1.00, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -0.363117463826178158710752068709, -0.677186279510737753445885427091, -1.00, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -0.363117463826178158710752068709, -0.677186279510737753445885427091, -0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.363117463826178158710752068709, -0.677186279510737753445885427091, -0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.363117463826178158710752068709, -0.677186279510737753445885427091, -0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.363117463826178158710752068709, -0.677186279510737753445885427091, 0.00, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -0.363117463826178158710752068709, -0.677186279510737753445885427091, 0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.363117463826178158710752068709, -0.677186279510737753445885427091, 0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.363117463826178158710752068709, -0.677186279510737753445885427091, 0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.363117463826178158710752068709, -0.677186279510737753445885427091, 1.00, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -0.363117463826178158710752068709, -0.363117463826178158710752068709, -1.00, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -0.363117463826178158710752068709, -0.363117463826178158710752068709, -0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.363117463826178158710752068709, -0.363117463826178158710752068709, -0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.363117463826178158710752068709, -0.363117463826178158710752068709, -0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.363117463826178158710752068709, -0.363117463826178158710752068709, 0.00, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -0.363117463826178158710752068709, -0.363117463826178158710752068709, 0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.363117463826178158710752068709, -0.363117463826178158710752068709, 0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.363117463826178158710752068709, -0.363117463826178158710752068709, 0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.363117463826178158710752068709, -0.363117463826178158710752068709, 1.00, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -0.363117463826178158710752068709, 0.00, -1.00, 0.34642851097304634511513153214 * 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.00, -0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.371519274376417233560090702948 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.00, -0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.371519274376417233560090702948 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.00, -0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.371519274376417233560090702948 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.00, 0.00, 0.34642851097304634511513153214 * 0.371519274376417233560090702948 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.00, 0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.371519274376417233560090702948 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.00, 0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.371519274376417233560090702948 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.00, 0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.371519274376417233560090702948 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.00, 1.00, 0.34642851097304634511513153214 * 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -0.363117463826178158710752068709, 0.363117463826178158710752068709, -1.00, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.363117463826178158710752068709, -0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.363117463826178158710752068709, -0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.363117463826178158710752068709, -0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.363117463826178158710752068709, 0.00, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.363117463826178158710752068709, 0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.363117463826178158710752068709, 0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.363117463826178158710752068709, 0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.363117463826178158710752068709, 1.00, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -0.363117463826178158710752068709, 0.677186279510737753445885427091, -1.00, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.677186279510737753445885427091, -0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.677186279510737753445885427091, -0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.677186279510737753445885427091, -0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.677186279510737753445885427091, 0.00, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.677186279510737753445885427091, 0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.677186279510737753445885427091, 0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.677186279510737753445885427091, 0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.677186279510737753445885427091, 1.00, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -0.363117463826178158710752068709, 0.899757995411460157312345244418, -1.00, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.899757995411460157312345244418, -0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.899757995411460157312345244418, -0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.899757995411460157312345244418, -0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.899757995411460157312345244418, 0.00, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.899757995411460157312345244418, 0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.899757995411460157312345244418, 0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.899757995411460157312345244418, 0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.363117463826178158710752068709, 0.899757995411460157312345244418, 1.00, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( -0.363117463826178158710752068709, 1.00, -1.00, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( -0.363117463826178158710752068709, 1.00, -0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.363117463826178158710752068709, 1.00, -0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.363117463826178158710752068709, 1.00, -0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.363117463826178158710752068709, 1.00, 0.00, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 ),
        IntegrationPointType( -0.363117463826178158710752068709, 1.00, 0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( -0.363117463826178158710752068709, 1.00, 0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( -0.363117463826178158710752068709, 1.00, 0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( -0.363117463826178158710752068709, 1.00, 1.00, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.00, -1.00, -1.00, 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.00, -1.00, -0.899757995411460157312345244418, 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.00, -1.00, -0.677186279510737753445885427091, 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.00, -1.00, -0.363117463826178158710752068709, 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.00, -1.00, 0.00, 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.00, -1.00, 0.363117463826178158710752068709, 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.00, -1.00, 0.677186279510737753445885427091, 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.00, -1.00, 0.899757995411460157312345244418, 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.00, -1.00, 1.00, 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.00, -0.899757995411460157312345244418, -1.00, 0.371519274376417233560090702948 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.00, -0.899757995411460157312345244418, -0.899757995411460157312345244418, 0.371519274376417233560090702948 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.00, -0.899757995411460157312345244418, -0.677186279510737753445885427091, 0.371519274376417233560090702948 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.00, -0.899757995411460157312345244418, -0.363117463826178158710752068709, 0.371519274376417233560090702948 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.00, -0.899757995411460157312345244418, 0.00, 0.371519274376417233560090702948 * 0.16549536156080552504633972003 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.00, -0.899757995411460157312345244418, 0.363117463826178158710752068709, 0.371519274376417233560090702948 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.00, -0.899757995411460157312345244418, 0.677186279510737753445885427091, 0.371519274376417233560090702948 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.00, -0.899757995411460157312345244418, 0.899757995411460157312345244418, 0.371519274376417233560090702948 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.00, -0.899757995411460157312345244418, 1.00, 0.371519274376417233560090702948 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.00, -0.677186279510737753445885427091, -1.00, 0.371519274376417233560090702948 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.00, -0.677186279510737753445885427091, -0.899757995411460157312345244418, 0.371519274376417233560090702948 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.00, -0.677186279510737753445885427091, -0.677186279510737753445885427091, 0.371519274376417233560090702948 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.00, -0.677186279510737753445885427091, -0.363117463826178158710752068709, 0.371519274376417233560090702948 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.00, -0.677186279510737753445885427091, 0.00, 0.371519274376417233560090702948 * 0.274538712500161735280705618579 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.00, -0.677186279510737753445885427091, 0.363117463826178158710752068709, 0.371519274376417233560090702948 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.00, -0.677186279510737753445885427091, 0.677186279510737753445885427091, 0.371519274376417233560090702948 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.00, -0.677186279510737753445885427091, 0.899757995411460157312345244418, 0.371519274376417233560090702948 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.00, -0.677186279510737753445885427091, 1.00, 0.371519274376417233560090702948 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.00, -0.363117463826178158710752068709, -1.00, 0.371519274376417233560090702948 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.00, -0.363117463826178158710752068709, -0.899757995411460157312345244418, 0.371519274376417233560090702948 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.00, -0.363117463826178158710752068709, -0.677186279510737753445885427091, 0.371519274376417233560090702948 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.00, -0.363117463826178158710752068709, -0.363117463826178158710752068709, 0.371519274376417233560090702948 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.00, -0.363117463826178158710752068709, 0.00, 0.371519274376417233560090702948 * 0.34642851097304634511513153214 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.00, -0.363117463826178158710752068709, 0.363117463826178158710752068709, 0.371519274376417233560090702948 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.00, -0.363117463826178158710752068709, 0.677186279510737753445885427091, 0.371519274376417233560090702948 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.00, -0.363117463826178158710752068709, 0.899757995411460157312345244418, 0.371519274376417233560090702948 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.00, -0.363117463826178158710752068709, 1.00, 0.371519274376417233560090702948 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.00, 0.00, -1.00, 0.371519274376417233560090702948 * 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.00, 0.00, -0.899757995411460157312345244418, 0.371519274376417233560090702948 * 0.371519274376417233560090702948 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.00, 0.00, -0.677186279510737753445885427091, 0.371519274376417233560090702948 * 0.371519274376417233560090702948 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.00, 0.00, -0.363117463826178158710752068709, 0.371519274376417233560090702948 * 0.371519274376417233560090702948 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.00, 0.00, 0.00, 0.371519274376417233560090702948 * 0.371519274376417233560090702948 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.00, 0.00, 0.363117463826178158710752068709, 0.371519274376417233560090702948 * 0.371519274376417233560090702948 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.00, 0.00, 0.677186279510737753445885427091, 0.371519274376417233560090702948 * 0.371519274376417233560090702948 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.00, 0.00, 0.899757995411460157312345244418, 0.371519274376417233560090702948 * 0.371519274376417233560090702948 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.00, 0.00, 1.00, 0.371519274376417233560090702948 * 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.00, 0.363117463826178158710752068709, -1.00, 0.371519274376417233560090702948 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.00, 0.363117463826178158710752068709, -0.899757995411460157312345244418, 0.371519274376417233560090702948 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.00, 0.363117463826178158710752068709, -0.677186279510737753445885427091, 0.371519274376417233560090702948 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.00, 0.363117463826178158710752068709, -0.363117463826178158710752068709, 0.371519274376417233560090702948 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.00, 0.363117463826178158710752068709, 0.00, 0.371519274376417233560090702948 * 0.34642851097304634511513153214 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.00, 0.363117463826178158710752068709, 0.363117463826178158710752068709, 0.371519274376417233560090702948 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.00, 0.363117463826178158710752068709, 0.677186279510737753445885427091, 0.371519274376417233560090702948 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.00, 0.363117463826178158710752068709, 0.899757995411460157312345244418, 0.371519274376417233560090702948 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.00, 0.363117463826178158710752068709, 1.00, 0.371519274376417233560090702948 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.00, 0.677186279510737753445885427091, -1.00, 0.371519274376417233560090702948 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.00, 0.677186279510737753445885427091, -0.899757995411460157312345244418, 0.371519274376417233560090702948 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.00, 0.677186279510737753445885427091, -0.677186279510737753445885427091, 0.371519274376417233560090702948 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.00, 0.677186279510737753445885427091, -0.363117463826178158710752068709, 0.371519274376417233560090702948 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.00, 0.677186279510737753445885427091, 0.00, 0.371519274376417233560090702948 * 0.274538712500161735280705618579 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.00, 0.677186279510737753445885427091, 0.363117463826178158710752068709, 0.371519274376417233560090702948 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.00, 0.677186279510737753445885427091, 0.677186279510737753445885427091, 0.371519274376417233560090702948 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.00, 0.677186279510737753445885427091, 0.899757995411460157312345244418, 0.371519274376417233560090702948 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.00, 0.677186279510737753445885427091, 1.00, 0.371519274376417233560090702948 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.00, 0.899757995411460157312345244418, -1.00, 0.371519274376417233560090702948 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.00, 0.899757995411460157312345244418, -0.899757995411460157312345244418, 0.371519274376417233560090702948 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.00, 0.899757995411460157312345244418, -0.677186279510737753445885427091, 0.371519274376417233560090702948 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.00, 0.899757995411460157312345244418, -0.363117463826178158710752068709, 0.371519274376417233560090702948 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.00, 0.899757995411460157312345244418, 0.00, 0.371519274376417233560090702948 * 0.16549536156080552504633972003 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.00, 0.899757995411460157312345244418, 0.363117463826178158710752068709, 0.371519274376417233560090702948 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.00, 0.899757995411460157312345244418, 0.677186279510737753445885427091, 0.371519274376417233560090702948 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.00, 0.899757995411460157312345244418, 0.899757995411460157312345244418, 0.371519274376417233560090702948 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.00, 0.899757995411460157312345244418, 1.00, 0.371519274376417233560090702948 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.00, 1.00, -1.00, 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.00, 1.00, -0.899757995411460157312345244418, 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.00, 1.00, -0.677186279510737753445885427091, 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.00, 1.00, -0.363117463826178158710752068709, 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.00, 1.00, 0.00, 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.00, 1.00, 0.363117463826178158710752068709, 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.00, 1.00, 0.677186279510737753445885427091, 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.00, 1.00, 0.899757995411460157312345244418, 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.00, 1.00, 1.00, 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.363117463826178158710752068709, -1.00, -1.00, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.363117463826178158710752068709, -1.00, -0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.363117463826178158710752068709, -1.00, -0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.363117463826178158710752068709, -1.00, -0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.363117463826178158710752068709, -1.00, 0.00, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.363117463826178158710752068709, -1.00, 0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.363117463826178158710752068709, -1.00, 0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.363117463826178158710752068709, -1.00, 0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.363117463826178158710752068709, -1.00, 1.00, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.363117463826178158710752068709, -0.899757995411460157312345244418, -1.00, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.363117463826178158710752068709, -0.899757995411460157312345244418, -0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.363117463826178158710752068709, -0.899757995411460157312345244418, -0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.363117463826178158710752068709, -0.899757995411460157312345244418, -0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.363117463826178158710752068709, -0.899757995411460157312345244418, 0.00, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.363117463826178158710752068709, -0.899757995411460157312345244418, 0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.363117463826178158710752068709, -0.899757995411460157312345244418, 0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.363117463826178158710752068709, -0.899757995411460157312345244418, 0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.363117463826178158710752068709, -0.899757995411460157312345244418, 1.00, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.363117463826178158710752068709, -0.677186279510737753445885427091, -1.00, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.363117463826178158710752068709, -0.677186279510737753445885427091, -0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.363117463826178158710752068709, -0.677186279510737753445885427091, -0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.363117463826178158710752068709, -0.677186279510737753445885427091, -0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.363117463826178158710752068709, -0.677186279510737753445885427091, 0.00, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.363117463826178158710752068709, -0.677186279510737753445885427091, 0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.363117463826178158710752068709, -0.677186279510737753445885427091, 0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.363117463826178158710752068709, -0.677186279510737753445885427091, 0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.363117463826178158710752068709, -0.677186279510737753445885427091, 1.00, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.363117463826178158710752068709, -0.363117463826178158710752068709, -1.00, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.363117463826178158710752068709, -0.363117463826178158710752068709, -0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.363117463826178158710752068709, -0.363117463826178158710752068709, -0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.363117463826178158710752068709, -0.363117463826178158710752068709, -0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.363117463826178158710752068709, -0.363117463826178158710752068709, 0.00, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.363117463826178158710752068709, -0.363117463826178158710752068709, 0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.363117463826178158710752068709, -0.363117463826178158710752068709, 0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.363117463826178158710752068709, -0.363117463826178158710752068709, 0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.363117463826178158710752068709, -0.363117463826178158710752068709, 1.00, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.363117463826178158710752068709, 0.00, -1.00, 0.34642851097304634511513153214 * 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.00, -0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.371519274376417233560090702948 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.00, -0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.371519274376417233560090702948 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.00, -0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.371519274376417233560090702948 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.00, 0.00, 0.34642851097304634511513153214 * 0.371519274376417233560090702948 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.00, 0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.371519274376417233560090702948 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.00, 0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.371519274376417233560090702948 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.00, 0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.371519274376417233560090702948 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.00, 1.00, 0.34642851097304634511513153214 * 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.363117463826178158710752068709, 0.363117463826178158710752068709, -1.00, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.363117463826178158710752068709, -0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.363117463826178158710752068709, -0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.363117463826178158710752068709, -0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.363117463826178158710752068709, 0.00, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.363117463826178158710752068709, 0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.363117463826178158710752068709, 0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.363117463826178158710752068709, 0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.363117463826178158710752068709, 1.00, 0.34642851097304634511513153214 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.363117463826178158710752068709, 0.677186279510737753445885427091, -1.00, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.677186279510737753445885427091, -0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.677186279510737753445885427091, -0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.677186279510737753445885427091, -0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.677186279510737753445885427091, 0.00, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.677186279510737753445885427091, 0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.677186279510737753445885427091, 0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.677186279510737753445885427091, 0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.677186279510737753445885427091, 1.00, 0.34642851097304634511513153214 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.363117463826178158710752068709, 0.899757995411460157312345244418, -1.00, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.899757995411460157312345244418, -0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.899757995411460157312345244418, -0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.899757995411460157312345244418, -0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.899757995411460157312345244418, 0.00, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.899757995411460157312345244418, 0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.899757995411460157312345244418, 0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.899757995411460157312345244418, 0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.363117463826178158710752068709, 0.899757995411460157312345244418, 1.00, 0.34642851097304634511513153214 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.363117463826178158710752068709, 1.00, -1.00, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.363117463826178158710752068709, 1.00, -0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.363117463826178158710752068709, 1.00, -0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.363117463826178158710752068709, 1.00, -0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.363117463826178158710752068709, 1.00, 0.00, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.363117463826178158710752068709, 1.00, 0.363117463826178158710752068709, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.363117463826178158710752068709, 1.00, 0.677186279510737753445885427091, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.363117463826178158710752068709, 1.00, 0.899757995411460157312345244418, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.363117463826178158710752068709, 1.00, 1.00, 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.677186279510737753445885427091, -1.00, -1.00, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.677186279510737753445885427091, -1.00, -0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.677186279510737753445885427091, -1.00, -0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.677186279510737753445885427091, -1.00, -0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.677186279510737753445885427091, -1.00, 0.00, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.677186279510737753445885427091, -1.00, 0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.677186279510737753445885427091, -1.00, 0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.677186279510737753445885427091, -1.00, 0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.677186279510737753445885427091, -1.00, 1.00, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.677186279510737753445885427091, -0.899757995411460157312345244418, -1.00, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.677186279510737753445885427091, -0.899757995411460157312345244418, -0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.677186279510737753445885427091, -0.899757995411460157312345244418, -0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.677186279510737753445885427091, -0.899757995411460157312345244418, -0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.677186279510737753445885427091, -0.899757995411460157312345244418, 0.00, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.677186279510737753445885427091, -0.899757995411460157312345244418, 0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.677186279510737753445885427091, -0.899757995411460157312345244418, 0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.677186279510737753445885427091, -0.899757995411460157312345244418, 0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.677186279510737753445885427091, -0.899757995411460157312345244418, 1.00, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.677186279510737753445885427091, -0.677186279510737753445885427091, -1.00, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.677186279510737753445885427091, -0.677186279510737753445885427091, -0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.677186279510737753445885427091, -0.677186279510737753445885427091, -0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.677186279510737753445885427091, -0.677186279510737753445885427091, -0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.677186279510737753445885427091, -0.677186279510737753445885427091, 0.00, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.677186279510737753445885427091, -0.677186279510737753445885427091, 0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.677186279510737753445885427091, -0.677186279510737753445885427091, 0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.677186279510737753445885427091, -0.677186279510737753445885427091, 0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.677186279510737753445885427091, -0.677186279510737753445885427091, 1.00, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.677186279510737753445885427091, -0.363117463826178158710752068709, -1.00, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.677186279510737753445885427091, -0.363117463826178158710752068709, -0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.677186279510737753445885427091, -0.363117463826178158710752068709, -0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.677186279510737753445885427091, -0.363117463826178158710752068709, -0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.677186279510737753445885427091, -0.363117463826178158710752068709, 0.00, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.677186279510737753445885427091, -0.363117463826178158710752068709, 0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.677186279510737753445885427091, -0.363117463826178158710752068709, 0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.677186279510737753445885427091, -0.363117463826178158710752068709, 0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.677186279510737753445885427091, -0.363117463826178158710752068709, 1.00, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.677186279510737753445885427091, 0.00, -1.00, 0.274538712500161735280705618579 * 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.00, -0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.371519274376417233560090702948 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.00, -0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.371519274376417233560090702948 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.00, -0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.371519274376417233560090702948 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.00, 0.00, 0.274538712500161735280705618579 * 0.371519274376417233560090702948 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.00, 0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.371519274376417233560090702948 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.00, 0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.371519274376417233560090702948 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.00, 0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.371519274376417233560090702948 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.00, 1.00, 0.274538712500161735280705618579 * 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.677186279510737753445885427091, 0.363117463826178158710752068709, -1.00, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.363117463826178158710752068709, -0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.363117463826178158710752068709, -0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.363117463826178158710752068709, -0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.363117463826178158710752068709, 0.00, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.363117463826178158710752068709, 0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.363117463826178158710752068709, 0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.363117463826178158710752068709, 0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.363117463826178158710752068709, 1.00, 0.274538712500161735280705618579 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.677186279510737753445885427091, 0.677186279510737753445885427091, -1.00, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.677186279510737753445885427091, -0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.677186279510737753445885427091, -0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.677186279510737753445885427091, -0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.677186279510737753445885427091, 0.00, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.677186279510737753445885427091, 0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.677186279510737753445885427091, 0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.677186279510737753445885427091, 0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.677186279510737753445885427091, 1.00, 0.274538712500161735280705618579 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.677186279510737753445885427091, 0.899757995411460157312345244418, -1.00, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.899757995411460157312345244418, -0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.899757995411460157312345244418, -0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.899757995411460157312345244418, -0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.899757995411460157312345244418, 0.00, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.899757995411460157312345244418, 0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.899757995411460157312345244418, 0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.899757995411460157312345244418, 0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.677186279510737753445885427091, 0.899757995411460157312345244418, 1.00, 0.274538712500161735280705618579 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.677186279510737753445885427091, 1.00, -1.00, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.677186279510737753445885427091, 1.00, -0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.677186279510737753445885427091, 1.00, -0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.677186279510737753445885427091, 1.00, -0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.677186279510737753445885427091, 1.00, 0.00, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.677186279510737753445885427091, 1.00, 0.363117463826178158710752068709, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.677186279510737753445885427091, 1.00, 0.677186279510737753445885427091, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.677186279510737753445885427091, 1.00, 0.899757995411460157312345244418, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.677186279510737753445885427091, 1.00, 1.00, 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.899757995411460157312345244418, -1.00, -1.00, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.899757995411460157312345244418, -1.00, -0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.899757995411460157312345244418, -1.00, -0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.899757995411460157312345244418, -1.00, -0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.899757995411460157312345244418, -1.00, 0.00, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.899757995411460157312345244418, -1.00, 0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.899757995411460157312345244418, -1.00, 0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.899757995411460157312345244418, -1.00, 0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.899757995411460157312345244418, -1.00, 1.00, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.899757995411460157312345244418, -0.899757995411460157312345244418, -1.00, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.899757995411460157312345244418, -0.899757995411460157312345244418, -0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.899757995411460157312345244418, -0.899757995411460157312345244418, -0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.899757995411460157312345244418, -0.899757995411460157312345244418, -0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.899757995411460157312345244418, -0.899757995411460157312345244418, 0.00, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.899757995411460157312345244418, -0.899757995411460157312345244418, 0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.899757995411460157312345244418, -0.899757995411460157312345244418, 0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.899757995411460157312345244418, -0.899757995411460157312345244418, 0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.899757995411460157312345244418, -0.899757995411460157312345244418, 1.00, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.899757995411460157312345244418, -0.677186279510737753445885427091, -1.00, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.899757995411460157312345244418, -0.677186279510737753445885427091, -0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.899757995411460157312345244418, -0.677186279510737753445885427091, -0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.899757995411460157312345244418, -0.677186279510737753445885427091, -0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.899757995411460157312345244418, -0.677186279510737753445885427091, 0.00, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.899757995411460157312345244418, -0.677186279510737753445885427091, 0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.899757995411460157312345244418, -0.677186279510737753445885427091, 0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.899757995411460157312345244418, -0.677186279510737753445885427091, 0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.899757995411460157312345244418, -0.677186279510737753445885427091, 1.00, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.899757995411460157312345244418, -0.363117463826178158710752068709, -1.00, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.899757995411460157312345244418, -0.363117463826178158710752068709, -0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.899757995411460157312345244418, -0.363117463826178158710752068709, -0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.899757995411460157312345244418, -0.363117463826178158710752068709, -0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.899757995411460157312345244418, -0.363117463826178158710752068709, 0.00, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.899757995411460157312345244418, -0.363117463826178158710752068709, 0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.899757995411460157312345244418, -0.363117463826178158710752068709, 0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.899757995411460157312345244418, -0.363117463826178158710752068709, 0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.899757995411460157312345244418, -0.363117463826178158710752068709, 1.00, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.899757995411460157312345244418, 0.00, -1.00, 0.16549536156080552504633972003 * 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.00, -0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.371519274376417233560090702948 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.00, -0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.371519274376417233560090702948 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.00, -0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.371519274376417233560090702948 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.00, 0.00, 0.16549536156080552504633972003 * 0.371519274376417233560090702948 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.00, 0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.371519274376417233560090702948 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.00, 0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.371519274376417233560090702948 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.00, 0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.371519274376417233560090702948 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.00, 1.00, 0.16549536156080552504633972003 * 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.899757995411460157312345244418, 0.363117463826178158710752068709, -1.00, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.363117463826178158710752068709, -0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.363117463826178158710752068709, -0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.363117463826178158710752068709, -0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.363117463826178158710752068709, 0.00, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.363117463826178158710752068709, 0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.363117463826178158710752068709, 0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.363117463826178158710752068709, 0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.363117463826178158710752068709, 1.00, 0.16549536156080552504633972003 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.899757995411460157312345244418, 0.677186279510737753445885427091, -1.00, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.677186279510737753445885427091, -0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.677186279510737753445885427091, -0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.677186279510737753445885427091, -0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.677186279510737753445885427091, 0.00, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.677186279510737753445885427091, 0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.677186279510737753445885427091, 0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.677186279510737753445885427091, 0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.677186279510737753445885427091, 1.00, 0.16549536156080552504633972003 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.899757995411460157312345244418, 0.899757995411460157312345244418, -1.00, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.899757995411460157312345244418, -0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.899757995411460157312345244418, -0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.899757995411460157312345244418, -0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.899757995411460157312345244418, 0.00, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.899757995411460157312345244418, 0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.899757995411460157312345244418, 0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.899757995411460157312345244418, 0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.899757995411460157312345244418, 0.899757995411460157312345244418, 1.00, 0.16549536156080552504633972003 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 0.899757995411460157312345244418, 1.00, -1.00, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 0.899757995411460157312345244418, 1.00, -0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.899757995411460157312345244418, 1.00, -0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.899757995411460157312345244418, 1.00, -0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.899757995411460157312345244418, 1.00, 0.00, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 0.899757995411460157312345244418, 1.00, 0.363117463826178158710752068709, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 0.899757995411460157312345244418, 1.00, 0.677186279510737753445885427091, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 0.899757995411460157312345244418, 1.00, 0.899757995411460157312345244418, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 0.899757995411460157312345244418, 1.00, 1.00, 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 1.00, -1.00, -1.00, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 1.00, -1.00, -0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 1.00, -1.00, -0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 1.00, -1.00, -0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 1.00, -1.00, 0.00, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 1.00, -1.00, 0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 1.00, -1.00, 0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 1.00, -1.00, 0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 1.00, -1.00, 1.00, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 1.00, -0.899757995411460157312345244418, -1.00, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 1.00, -0.899757995411460157312345244418, -0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 1.00, -0.899757995411460157312345244418, -0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 1.00, -0.899757995411460157312345244418, -0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 1.00, -0.899757995411460157312345244418, 0.00, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 1.00, -0.899757995411460157312345244418, 0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 1.00, -0.899757995411460157312345244418, 0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 1.00, -0.899757995411460157312345244418, 0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 1.00, -0.899757995411460157312345244418, 1.00, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 1.00, -0.677186279510737753445885427091, -1.00, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 1.00, -0.677186279510737753445885427091, -0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 1.00, -0.677186279510737753445885427091, -0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 1.00, -0.677186279510737753445885427091, -0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 1.00, -0.677186279510737753445885427091, 0.00, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 1.00, -0.677186279510737753445885427091, 0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 1.00, -0.677186279510737753445885427091, 0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 1.00, -0.677186279510737753445885427091, 0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 1.00, -0.677186279510737753445885427091, 1.00, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 1.00, -0.363117463826178158710752068709, -1.00, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 1.00, -0.363117463826178158710752068709, -0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 1.00, -0.363117463826178158710752068709, -0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 1.00, -0.363117463826178158710752068709, -0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 1.00, -0.363117463826178158710752068709, 0.00, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 1.00, -0.363117463826178158710752068709, 0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 1.00, -0.363117463826178158710752068709, 0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 1.00, -0.363117463826178158710752068709, 0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 1.00, -0.363117463826178158710752068709, 1.00, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 1.00, 0.00, -1.00, 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 1.00, 0.00, -0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 1.00, 0.00, -0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 1.00, 0.00, -0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 1.00, 0.00, 0.00, 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 1.00, 0.00, 0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 1.00, 0.00, 0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 1.00, 0.00, 0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 1.00, 0.00, 1.00, 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 1.00, 0.363117463826178158710752068709, -1.00, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 1.00, 0.363117463826178158710752068709, -0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 1.00, 0.363117463826178158710752068709, -0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 1.00, 0.363117463826178158710752068709, -0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 1.00, 0.363117463826178158710752068709, 0.00, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 1.00, 0.363117463826178158710752068709, 0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 1.00, 0.363117463826178158710752068709, 0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 1.00, 0.363117463826178158710752068709, 0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 1.00, 0.363117463826178158710752068709, 1.00, 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 1.00, 0.677186279510737753445885427091, -1.00, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 1.00, 0.677186279510737753445885427091, -0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 1.00, 0.677186279510737753445885427091, -0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 1.00, 0.677186279510737753445885427091, -0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 1.00, 0.677186279510737753445885427091, 0.00, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 1.00, 0.677186279510737753445885427091, 0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 1.00, 0.677186279510737753445885427091, 0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 1.00, 0.677186279510737753445885427091, 0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 1.00, 0.677186279510737753445885427091, 1.00, 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 1.00, 0.899757995411460157312345244418, -1.00, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 1.00, 0.899757995411460157312345244418, -0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 1.00, 0.899757995411460157312345244418, -0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 1.00, 0.899757995411460157312345244418, -0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 1.00, 0.899757995411460157312345244418, 0.00, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 1.00, 0.899757995411460157312345244418, 0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 1.00, 0.899757995411460157312345244418, 0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 1.00, 0.899757995411460157312345244418, 0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 1.00, 0.899757995411460157312345244418, 1.00, 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 * 0.0277777777777777777777777777778 ),

        IntegrationPointType( 1.00, 1.00, -1.00, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),
        IntegrationPointType( 1.00, 1.00, -0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 1.00, 1.00, -0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 1.00, 1.00, -0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 1.00, 1.00, 0.00, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.371519274376417233560090702948 ),
        IntegrationPointType( 1.00, 1.00, 0.363117463826178158710752068709, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.34642851097304634511513153214 ),
        IntegrationPointType( 1.00, 1.00, 0.677186279510737753445885427091, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.274538712500161735280705618579 ),
        IntegrationPointType( 1.00, 1.00, 0.899757995411460157312345244418, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.16549536156080552504633972003 ),
        IntegrationPointType( 1.00, 1.00, 1.00, 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 * 0.0277777777777777777777777777778 ),

    }
};

HexahedronGaussLobattoIntegrationPoints10::IntegrationPointsArrayType
HexahedronGaussLobattoIntegrationPoints10::msIntegrationPoints =
{
    {
        IntegrationPointType( -1.00, -1.00, -1.00, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -1.00, -1.00, -0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -1.00, -1.00, -0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -1.00, -1.00, -0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -1.00, -1.00, -0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -1.00, -1.00, 0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -1.00, -1.00, 0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -1.00, -1.00, 0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -1.00, -1.00, 0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -1.00, -1.00, 1.00, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -1.00, -0.919533908166458813828932660822, -1.00, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -1.00, -0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -1.00, -0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -1.00, -0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -1.00, -0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -1.00, -0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -1.00, -0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -1.00, -0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -1.00, -0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -1.00, -0.919533908166458813828932660822, 1.00, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -1.00, -0.73877386510550507500310617486, -1.00, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -1.00, -0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -1.00, -0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -1.00, -0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -1.00, -0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -1.00, -0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -1.00, -0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -1.00, -0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -1.00, -0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -1.00, -0.73877386510550507500310617486, 1.00, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -1.00, -0.477924949810444495661175092731, -1.00, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -1.00, -0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -1.00, -0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -1.00, -0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -1.00, -0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -1.00, -0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -1.00, -0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -1.00, -0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -1.00, -0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -1.00, -0.477924949810444495661175092731, 1.00, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -1.00, -0.165278957666387024626219765958, -1.00, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -1.00, -0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -1.00, -0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -1.00, -0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -1.00, -0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -1.00, -0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -1.00, -0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -1.00, -0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -1.00, -0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -1.00, -0.165278957666387024626219765958, 1.00, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -1.00, 0.165278957666387024626219765958, -1.00, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -1.00, 0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -1.00, 0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -1.00, 0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -1.00, 0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -1.00, 0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -1.00, 0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -1.00, 0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -1.00, 0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -1.00, 0.165278957666387024626219765958, 1.00, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -1.00, 0.477924949810444495661175092731, -1.00, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -1.00, 0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -1.00, 0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -1.00, 0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -1.00, 0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -1.00, 0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -1.00, 0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -1.00, 0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -1.00, 0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -1.00, 0.477924949810444495661175092731, 1.00, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -1.00, 0.73877386510550507500310617486, -1.00, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -1.00, 0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -1.00, 0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -1.00, 0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -1.00, 0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -1.00, 0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -1.00, 0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -1.00, 0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -1.00, 0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -1.00, 0.73877386510550507500310617486, 1.00, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -1.00, 0.919533908166458813828932660822, -1.00, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -1.00, 0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -1.00, 0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -1.00, 0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -1.00, 0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -1.00, 0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -1.00, 0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -1.00, 0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -1.00, 0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -1.00, 0.919533908166458813828932660822, 1.00, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -1.00, 1.00, -1.00, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -1.00, 1.00, -0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -1.00, 1.00, -0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -1.00, 1.00, -0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -1.00, 1.00, -0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -1.00, 1.00, 0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -1.00, 1.00, 0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -1.00, 1.00, 0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -1.00, 1.00, 0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -1.00, 1.00, 1.00, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.919533908166458813828932660822, -1.00, -1.00, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.919533908166458813828932660822, -1.00, -0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.919533908166458813828932660822, -1.00, -0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.919533908166458813828932660822, -1.00, -0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.919533908166458813828932660822, -1.00, -0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.919533908166458813828932660822, -1.00, 0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.919533908166458813828932660822, -1.00, 0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.919533908166458813828932660822, -1.00, 0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.919533908166458813828932660822, -1.00, 0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.919533908166458813828932660822, -1.00, 1.00, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.919533908166458813828932660822, -0.919533908166458813828932660822, -1.00, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.919533908166458813828932660822, 1.00, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.919533908166458813828932660822, -0.73877386510550507500310617486, -1.00, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.73877386510550507500310617486, 1.00, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.919533908166458813828932660822, -0.477924949810444495661175092731, -1.00, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.477924949810444495661175092731, 1.00, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.919533908166458813828932660822, -0.165278957666387024626219765958, -1.00, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.919533908166458813828932660822, -0.165278957666387024626219765958, 1.00, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.919533908166458813828932660822, 0.165278957666387024626219765958, -1.00, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.165278957666387024626219765958, 1.00, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.919533908166458813828932660822, 0.477924949810444495661175092731, -1.00, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.477924949810444495661175092731, 1.00, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.919533908166458813828932660822, 0.73877386510550507500310617486, -1.00, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.73877386510550507500310617486, 1.00, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.919533908166458813828932660822, 0.919533908166458813828932660822, -1.00, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.919533908166458813828932660822, 0.919533908166458813828932660822, 1.00, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.919533908166458813828932660822, 1.00, -1.00, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.919533908166458813828932660822, 1.00, -0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.919533908166458813828932660822, 1.00, -0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.919533908166458813828932660822, 1.00, -0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.919533908166458813828932660822, 1.00, -0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.919533908166458813828932660822, 1.00, 0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.919533908166458813828932660822, 1.00, 0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.919533908166458813828932660822, 1.00, 0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.919533908166458813828932660822, 1.00, 0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.919533908166458813828932660822, 1.00, 1.00, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.73877386510550507500310617486, -1.00, -1.00, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.73877386510550507500310617486, -1.00, -0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.73877386510550507500310617486, -1.00, -0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.73877386510550507500310617486, -1.00, -0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.73877386510550507500310617486, -1.00, -0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.73877386510550507500310617486, -1.00, 0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.73877386510550507500310617486, -1.00, 0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.73877386510550507500310617486, -1.00, 0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.73877386510550507500310617486, -1.00, 0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.73877386510550507500310617486, -1.00, 1.00, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.73877386510550507500310617486, -0.919533908166458813828932660822, -1.00, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.919533908166458813828932660822, 1.00, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.73877386510550507500310617486, -0.73877386510550507500310617486, -1.00, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.73877386510550507500310617486, 1.00, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.73877386510550507500310617486, -0.477924949810444495661175092731, -1.00, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.477924949810444495661175092731, 1.00, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.73877386510550507500310617486, -0.165278957666387024626219765958, -1.00, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.73877386510550507500310617486, -0.165278957666387024626219765958, 1.00, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.73877386510550507500310617486, 0.165278957666387024626219765958, -1.00, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.165278957666387024626219765958, 1.00, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.73877386510550507500310617486, 0.477924949810444495661175092731, -1.00, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.477924949810444495661175092731, 1.00, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.73877386510550507500310617486, 0.73877386510550507500310617486, -1.00, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.73877386510550507500310617486, 1.00, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.73877386510550507500310617486, 0.919533908166458813828932660822, -1.00, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.73877386510550507500310617486, 0.919533908166458813828932660822, 1.00, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.73877386510550507500310617486, 1.00, -1.00, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.73877386510550507500310617486, 1.00, -0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.73877386510550507500310617486, 1.00, -0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.73877386510550507500310617486, 1.00, -0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.73877386510550507500310617486, 1.00, -0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.73877386510550507500310617486, 1.00, 0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.73877386510550507500310617486, 1.00, 0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.73877386510550507500310617486, 1.00, 0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.73877386510550507500310617486, 1.00, 0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.73877386510550507500310617486, 1.00, 1.00, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.477924949810444495661175092731, -1.00, -1.00, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.477924949810444495661175092731, -1.00, -0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.477924949810444495661175092731, -1.00, -0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.477924949810444495661175092731, -1.00, -0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.477924949810444495661175092731, -1.00, -0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.477924949810444495661175092731, -1.00, 0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.477924949810444495661175092731, -1.00, 0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.477924949810444495661175092731, -1.00, 0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.477924949810444495661175092731, -1.00, 0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.477924949810444495661175092731, -1.00, 1.00, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.477924949810444495661175092731, -0.919533908166458813828932660822, -1.00, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.919533908166458813828932660822, 1.00, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.477924949810444495661175092731, -0.73877386510550507500310617486, -1.00, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.73877386510550507500310617486, 1.00, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.477924949810444495661175092731, -0.477924949810444495661175092731, -1.00, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.477924949810444495661175092731, 1.00, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.477924949810444495661175092731, -0.165278957666387024626219765958, -1.00, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.477924949810444495661175092731, -0.165278957666387024626219765958, 1.00, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.477924949810444495661175092731, 0.165278957666387024626219765958, -1.00, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.165278957666387024626219765958, 1.00, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.477924949810444495661175092731, 0.477924949810444495661175092731, -1.00, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.477924949810444495661175092731, 1.00, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.477924949810444495661175092731, 0.73877386510550507500310617486, -1.00, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.73877386510550507500310617486, 1.00, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.477924949810444495661175092731, 0.919533908166458813828932660822, -1.00, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.477924949810444495661175092731, 0.919533908166458813828932660822, 1.00, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.477924949810444495661175092731, 1.00, -1.00, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.477924949810444495661175092731, 1.00, -0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.477924949810444495661175092731, 1.00, -0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.477924949810444495661175092731, 1.00, -0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.477924949810444495661175092731, 1.00, -0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.477924949810444495661175092731, 1.00, 0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.477924949810444495661175092731, 1.00, 0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.477924949810444495661175092731, 1.00, 0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.477924949810444495661175092731, 1.00, 0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.477924949810444495661175092731, 1.00, 1.00, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.165278957666387024626219765958, -1.00, -1.00, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.165278957666387024626219765958, -1.00, -0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.165278957666387024626219765958, -1.00, -0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.165278957666387024626219765958, -1.00, -0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.165278957666387024626219765958, -1.00, -0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.165278957666387024626219765958, -1.00, 0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.165278957666387024626219765958, -1.00, 0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.165278957666387024626219765958, -1.00, 0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.165278957666387024626219765958, -1.00, 0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.165278957666387024626219765958, -1.00, 1.00, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.165278957666387024626219765958, -0.919533908166458813828932660822, -1.00, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.919533908166458813828932660822, 1.00, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.165278957666387024626219765958, -0.73877386510550507500310617486, -1.00, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.73877386510550507500310617486, 1.00, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.165278957666387024626219765958, -0.477924949810444495661175092731, -1.00, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.477924949810444495661175092731, 1.00, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.165278957666387024626219765958, -0.165278957666387024626219765958, -1.00, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.165278957666387024626219765958, -0.165278957666387024626219765958, 1.00, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.165278957666387024626219765958, 0.165278957666387024626219765958, -1.00, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.165278957666387024626219765958, 1.00, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.165278957666387024626219765958, 0.477924949810444495661175092731, -1.00, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.477924949810444495661175092731, 1.00, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.165278957666387024626219765958, 0.73877386510550507500310617486, -1.00, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.73877386510550507500310617486, 1.00, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.165278957666387024626219765958, 0.919533908166458813828932660822, -1.00, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.165278957666387024626219765958, 0.919533908166458813828932660822, 1.00, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( -0.165278957666387024626219765958, 1.00, -1.00, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( -0.165278957666387024626219765958, 1.00, -0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.165278957666387024626219765958, 1.00, -0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.165278957666387024626219765958, 1.00, -0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.165278957666387024626219765958, 1.00, -0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.165278957666387024626219765958, 1.00, 0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( -0.165278957666387024626219765958, 1.00, 0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( -0.165278957666387024626219765958, 1.00, 0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( -0.165278957666387024626219765958, 1.00, 0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( -0.165278957666387024626219765958, 1.00, 1.00, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.165278957666387024626219765958, -1.00, -1.00, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.165278957666387024626219765958, -1.00, -0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.165278957666387024626219765958, -1.00, -0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.165278957666387024626219765958, -1.00, -0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.165278957666387024626219765958, -1.00, -0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.165278957666387024626219765958, -1.00, 0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.165278957666387024626219765958, -1.00, 0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.165278957666387024626219765958, -1.00, 0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.165278957666387024626219765958, -1.00, 0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.165278957666387024626219765958, -1.00, 1.00, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.165278957666387024626219765958, -0.919533908166458813828932660822, -1.00, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.919533908166458813828932660822, 1.00, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.165278957666387024626219765958, -0.73877386510550507500310617486, -1.00, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.73877386510550507500310617486, 1.00, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.165278957666387024626219765958, -0.477924949810444495661175092731, -1.00, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.477924949810444495661175092731, 1.00, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.165278957666387024626219765958, -0.165278957666387024626219765958, -1.00, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.165278957666387024626219765958, -0.165278957666387024626219765958, 1.00, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.165278957666387024626219765958, 0.165278957666387024626219765958, -1.00, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.165278957666387024626219765958, 1.00, 0.32753976118389745665651052792 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.165278957666387024626219765958, 0.477924949810444495661175092731, -1.00, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.477924949810444495661175092731, 1.00, 0.32753976118389745665651052792 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.165278957666387024626219765958, 0.73877386510550507500310617486, -1.00, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.73877386510550507500310617486, 1.00, 0.32753976118389745665651052792 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.165278957666387024626219765958, 0.919533908166458813828932660822, -1.00, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.165278957666387024626219765958, 0.919533908166458813828932660822, 1.00, 0.32753976118389745665651052792 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.165278957666387024626219765958, 1.00, -1.00, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.165278957666387024626219765958, 1.00, -0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.165278957666387024626219765958, 1.00, -0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.165278957666387024626219765958, 1.00, -0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.165278957666387024626219765958, 1.00, -0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.165278957666387024626219765958, 1.00, 0.165278957666387024626219765958, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.165278957666387024626219765958, 1.00, 0.477924949810444495661175092731, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.165278957666387024626219765958, 1.00, 0.73877386510550507500310617486, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.165278957666387024626219765958, 1.00, 0.919533908166458813828932660822, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.165278957666387024626219765958, 1.00, 1.00, 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.477924949810444495661175092731, -1.00, -1.00, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.477924949810444495661175092731, -1.00, -0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.477924949810444495661175092731, -1.00, -0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.477924949810444495661175092731, -1.00, -0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.477924949810444495661175092731, -1.00, -0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.477924949810444495661175092731, -1.00, 0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.477924949810444495661175092731, -1.00, 0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.477924949810444495661175092731, -1.00, 0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.477924949810444495661175092731, -1.00, 0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.477924949810444495661175092731, -1.00, 1.00, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.477924949810444495661175092731, -0.919533908166458813828932660822, -1.00, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.919533908166458813828932660822, 1.00, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.477924949810444495661175092731, -0.73877386510550507500310617486, -1.00, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.73877386510550507500310617486, 1.00, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.477924949810444495661175092731, -0.477924949810444495661175092731, -1.00, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.477924949810444495661175092731, 1.00, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.477924949810444495661175092731, -0.165278957666387024626219765958, -1.00, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.477924949810444495661175092731, -0.165278957666387024626219765958, 1.00, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.477924949810444495661175092731, 0.165278957666387024626219765958, -1.00, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.165278957666387024626219765958, 1.00, 0.29204268367968375787558225737 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.477924949810444495661175092731, 0.477924949810444495661175092731, -1.00, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.477924949810444495661175092731, 1.00, 0.29204268367968375787558225737 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.477924949810444495661175092731, 0.73877386510550507500310617486, -1.00, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.73877386510550507500310617486, 1.00, 0.29204268367968375787558225737 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.477924949810444495661175092731, 0.919533908166458813828932660822, -1.00, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.477924949810444495661175092731, 0.919533908166458813828932660822, 1.00, 0.29204268367968375787558225737 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.477924949810444495661175092731, 1.00, -1.00, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.477924949810444495661175092731, 1.00, -0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.477924949810444495661175092731, 1.00, -0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.477924949810444495661175092731, 1.00, -0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.477924949810444495661175092731, 1.00, -0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.477924949810444495661175092731, 1.00, 0.165278957666387024626219765958, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.477924949810444495661175092731, 1.00, 0.477924949810444495661175092731, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.477924949810444495661175092731, 1.00, 0.73877386510550507500310617486, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.477924949810444495661175092731, 1.00, 0.919533908166458813828932660822, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.477924949810444495661175092731, 1.00, 1.00, 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.73877386510550507500310617486, -1.00, -1.00, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.73877386510550507500310617486, -1.00, -0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.73877386510550507500310617486, -1.00, -0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.73877386510550507500310617486, -1.00, -0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.73877386510550507500310617486, -1.00, -0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.73877386510550507500310617486, -1.00, 0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.73877386510550507500310617486, -1.00, 0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.73877386510550507500310617486, -1.00, 0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.73877386510550507500310617486, -1.00, 0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.73877386510550507500310617486, -1.00, 1.00, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.73877386510550507500310617486, -0.919533908166458813828932660822, -1.00, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.919533908166458813828932660822, 1.00, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.73877386510550507500310617486, -0.73877386510550507500310617486, -1.00, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.73877386510550507500310617486, 1.00, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.73877386510550507500310617486, -0.477924949810444495661175092731, -1.00, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.477924949810444495661175092731, 1.00, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.73877386510550507500310617486, -0.165278957666387024626219765958, -1.00, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.73877386510550507500310617486, -0.165278957666387024626219765958, 1.00, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.73877386510550507500310617486, 0.165278957666387024626219765958, -1.00, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.165278957666387024626219765958, 1.00, 0.224889342063126452119457821731 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.73877386510550507500310617486, 0.477924949810444495661175092731, -1.00, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.477924949810444495661175092731, 1.00, 0.224889342063126452119457821731 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.73877386510550507500310617486, 0.73877386510550507500310617486, -1.00, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.73877386510550507500310617486, 1.00, 0.224889342063126452119457821731 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.73877386510550507500310617486, 0.919533908166458813828932660822, -1.00, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.73877386510550507500310617486, 0.919533908166458813828932660822, 1.00, 0.224889342063126452119457821731 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.73877386510550507500310617486, 1.00, -1.00, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.73877386510550507500310617486, 1.00, -0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.73877386510550507500310617486, 1.00, -0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.73877386510550507500310617486, 1.00, -0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.73877386510550507500310617486, 1.00, -0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.73877386510550507500310617486, 1.00, 0.165278957666387024626219765958, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.73877386510550507500310617486, 1.00, 0.477924949810444495661175092731, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.73877386510550507500310617486, 1.00, 0.73877386510550507500310617486, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.73877386510550507500310617486, 1.00, 0.919533908166458813828932660822, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.73877386510550507500310617486, 1.00, 1.00, 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.919533908166458813828932660822, -1.00, -1.00, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.919533908166458813828932660822, -1.00, -0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.919533908166458813828932660822, -1.00, -0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.919533908166458813828932660822, -1.00, -0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.919533908166458813828932660822, -1.00, -0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.919533908166458813828932660822, -1.00, 0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.919533908166458813828932660822, -1.00, 0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.919533908166458813828932660822, -1.00, 0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.919533908166458813828932660822, -1.00, 0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.919533908166458813828932660822, -1.00, 1.00, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.919533908166458813828932660822, -0.919533908166458813828932660822, -1.00, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.919533908166458813828932660822, 1.00, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.919533908166458813828932660822, -0.73877386510550507500310617486, -1.00, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.73877386510550507500310617486, 1.00, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.919533908166458813828932660822, -0.477924949810444495661175092731, -1.00, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.477924949810444495661175092731, 1.00, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.919533908166458813828932660822, -0.165278957666387024626219765958, -1.00, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.919533908166458813828932660822, -0.165278957666387024626219765958, 1.00, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.919533908166458813828932660822, 0.165278957666387024626219765958, -1.00, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.165278957666387024626219765958, 1.00, 0.133305990851070111126227170755 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.919533908166458813828932660822, 0.477924949810444495661175092731, -1.00, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.477924949810444495661175092731, 1.00, 0.133305990851070111126227170755 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.919533908166458813828932660822, 0.73877386510550507500310617486, -1.00, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.73877386510550507500310617486, 1.00, 0.133305990851070111126227170755 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.919533908166458813828932660822, 0.919533908166458813828932660822, -1.00, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.919533908166458813828932660822, 0.919533908166458813828932660822, 1.00, 0.133305990851070111126227170755 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 0.919533908166458813828932660822, 1.00, -1.00, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 0.919533908166458813828932660822, 1.00, -0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.919533908166458813828932660822, 1.00, -0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.919533908166458813828932660822, 1.00, -0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.919533908166458813828932660822, 1.00, -0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.919533908166458813828932660822, 1.00, 0.165278957666387024626219765958, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 0.919533908166458813828932660822, 1.00, 0.477924949810444495661175092731, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 0.919533908166458813828932660822, 1.00, 0.73877386510550507500310617486, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 0.919533908166458813828932660822, 1.00, 0.919533908166458813828932660822, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 0.919533908166458813828932660822, 1.00, 1.00, 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 1.00, -1.00, -1.00, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 1.00, -1.00, -0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 1.00, -1.00, -0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 1.00, -1.00, -0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 1.00, -1.00, -0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 1.00, -1.00, 0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 1.00, -1.00, 0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 1.00, -1.00, 0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 1.00, -1.00, 0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 1.00, -1.00, 1.00, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 1.00, -0.919533908166458813828932660822, -1.00, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 1.00, -0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 1.00, -0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 1.00, -0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 1.00, -0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 1.00, -0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 1.00, -0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 1.00, -0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 1.00, -0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 1.00, -0.919533908166458813828932660822, 1.00, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 1.00, -0.73877386510550507500310617486, -1.00, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 1.00, -0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 1.00, -0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 1.00, -0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 1.00, -0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 1.00, -0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 1.00, -0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 1.00, -0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 1.00, -0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 1.00, -0.73877386510550507500310617486, 1.00, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 1.00, -0.477924949810444495661175092731, -1.00, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 1.00, -0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 1.00, -0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 1.00, -0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 1.00, -0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 1.00, -0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 1.00, -0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 1.00, -0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 1.00, -0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 1.00, -0.477924949810444495661175092731, 1.00, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 1.00, -0.165278957666387024626219765958, -1.00, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 1.00, -0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 1.00, -0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 1.00, -0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 1.00, -0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 1.00, -0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 1.00, -0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 1.00, -0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 1.00, -0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 1.00, -0.165278957666387024626219765958, 1.00, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 1.00, 0.165278957666387024626219765958, -1.00, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 1.00, 0.165278957666387024626219765958, -0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 1.00, 0.165278957666387024626219765958, -0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 1.00, 0.165278957666387024626219765958, -0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 1.00, 0.165278957666387024626219765958, -0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 1.00, 0.165278957666387024626219765958, 0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 1.00, 0.165278957666387024626219765958, 0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 1.00, 0.165278957666387024626219765958, 0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 1.00, 0.165278957666387024626219765958, 0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 1.00, 0.165278957666387024626219765958, 1.00, 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 1.00, 0.477924949810444495661175092731, -1.00, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 1.00, 0.477924949810444495661175092731, -0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 1.00, 0.477924949810444495661175092731, -0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 1.00, 0.477924949810444495661175092731, -0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 1.00, 0.477924949810444495661175092731, -0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 1.00, 0.477924949810444495661175092731, 0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 1.00, 0.477924949810444495661175092731, 0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 1.00, 0.477924949810444495661175092731, 0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 1.00, 0.477924949810444495661175092731, 0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 1.00, 0.477924949810444495661175092731, 1.00, 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 1.00, 0.73877386510550507500310617486, -1.00, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 1.00, 0.73877386510550507500310617486, -0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 1.00, 0.73877386510550507500310617486, -0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 1.00, 0.73877386510550507500310617486, -0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 1.00, 0.73877386510550507500310617486, -0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 1.00, 0.73877386510550507500310617486, 0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 1.00, 0.73877386510550507500310617486, 0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 1.00, 0.73877386510550507500310617486, 0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 1.00, 0.73877386510550507500310617486, 0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 1.00, 0.73877386510550507500310617486, 1.00, 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 1.00, 0.919533908166458813828932660822, -1.00, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 1.00, 0.919533908166458813828932660822, -0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 1.00, 0.919533908166458813828932660822, -0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 1.00, 0.919533908166458813828932660822, -0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 1.00, 0.919533908166458813828932660822, -0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 1.00, 0.919533908166458813828932660822, 0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 1.00, 0.919533908166458813828932660822, 0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 1.00, 0.919533908166458813828932660822, 0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 1.00, 0.919533908166458813828932660822, 0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 1.00, 0.919533908166458813828932660822, 1.00, 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 * 0.0222222222222222222222222222222 ),

        IntegrationPointType( 1.00, 1.00, -1.00, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),
        IntegrationPointType( 1.00, 1.00, -0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 1.00, 1.00, -0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 1.00, 1.00, -0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 1.00, 1.00, -0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 1.00, 1.00, 0.165278957666387024626219765958, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.32753976118389745665651052792 ),
        IntegrationPointType( 1.00, 1.00, 0.477924949810444495661175092731, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.29204268367968375787558225737 ),
        IntegrationPointType( 1.00, 1.00, 0.73877386510550507500310617486, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.224889342063126452119457821731 ),
        IntegrationPointType( 1.00, 1.00, 0.919533908166458813828932660822, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.133305990851070111126227170755 ),
        IntegrationPointType( 1.00, 1.00, 1.00, 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 * 0.0222222222222222222222222222222 ),

    }
};

}
